      SUBROUTINE BECKINFO
C-----------------------------------------------------------------------
C     DONALD E. AMOS, MAY 1, 2002, FEBRUARY, 2006
C
C     EACH CALLABLE ROUTINE HAS A REFERENCE IN THE PROLOGUE TO AN
C     APPROPRIATE FOLDER WHICH DESCRIBES THE ANALYTICAL BASIS OF THE
C     CODE.
C
C     USAGE:
C
C     COMPILE THIS FILE AND LINK IT TO ANY PROGRAM NEEDING A SUBROUTINE
C     OR FUNCTION FROM THE FOLLOWING LIST:
C
C     CALLABLE SUBROUTINES:                               FOLDER    
C              INTEGI1(A,B,T,KODE,REL,ANSI1,IERR)         1,2,10
C              INTEGI2(A,B,T,KODE,ANSI2,IERR)               9
C              INTEGI9(A,B,T,KODE,ANSI9,IERR)               9
C              INTEGI3(A,B,C,T,ANSI3,IERR,KFORM)            7
C              INTEGI5(A,B,X,ANSI5,IERR)                    5
C              INTEGJ5(A,B,X,ANSJ5,IERR)                    5
C              INTEGV5(A,B,X,ANSV5,IERR)                    5
C              INTEGI6(A,B,T,KODE,ANSI6,IERR)             3,6,15
C              INTEGP(A,B,T,KODE,REL,PANS,IERR)             11
C              INTEGQ(A,B,T,REL,QANS,IERR)                  11
C              INTEGW3(A,B,T,KODE,REL,ANSW3,IERR)           10
C              INTEGI21(A,B,C,T,KODE,ANSI21,I21ERR,KFORM)   21
C              INTEGJ21(A,B,C,T,ANSJ21,J21ERR,KFORM)        21
C              INTEGI22(A,B,C,T,ANSI22,I22ERR,KFORM)        22
C              INTEGJ22(A,B,C,T,ANSJ22,J22ERR,KFORM)        22
C              INTEGI29(A,B,T,N0,NN,YN,IERR)                29 
C              INTEGS1(A,B,C,T,TOL,S1,IERR,KFORM)           21
C              INTEGS2(A,B,C,T,TOL,S2,IERR,KFORM)           22
C              GNSEQ(A,B,CAPT,M,REL,YN)                     21
C
C     CALLABLE DOUBLE PRECISION FUNCTIONS:
C              DVOFT(A,B,T,REL,IERR,KFORM)                  21
C              PHIZ(Z)                                      5
C-----------------------------------------------------------------------
      END
      SUBROUTINE INTEGI1(A,B,T,KODE,REL,ANSI1,IERR)
C
C     DONALD E. AMOS, JANUARY, 2003
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     INTEGI1 COMPUTES THE I1 INTEGRAL OF FOLDERS 1, 2, AND 10 USING
C     THE CLOSED FORM DERIVED IN FOLDER 10 ON KODE=1:
C
C         ANSI1=INT ON (T,INF) OF EXP(-A^2*W^2)*ERF(B*W)/W^2
C
C     AND THE COMPLEMENTARY FORM FOR KODE=2:
C
C         ANSI1=INT ON (T,INF) OF EXP(-A^2*W^2)*ERFC(B*W)/W^2
C
C     WHERE A.GE.0.0D0, B.GE.0.0D0, T.GT.0.0D0.
C
C     IERR=0 IS A  NORMAL RETURN
C     IERR=1 MEANS T.LE.0.0D0, ANSI1=0.0D0 RETURNED
C     IERR=2 MEANS ONE OR BOTH OF THE PARAMETERS A OR B IS NEGATIVE
C            OR KODE IS NOT 1 OR 2 ANSI1=0.0D0 RETURNED
C     IERR=4 IS AN UNDERFLOW AND ANSI1=0.0D0 IS RETURNED
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(1)
      DATA RTPI       /1.772453850905516D0/
      IERR=0
      IF (A.LT.0.0D0) THEN
        IERR=2
        ANSI1=0.0D0
        RETURN
      ENDIF
      IF (B.LT.0.0D0) THEN
        IERR=2
        ANSI1=0.0D0
        RETURN
      ENDIF
      IF (T.LE.0.0D0) THEN
        IERR=1
        ANSI1=0.0D0
        RETURN
      ENDIF
      IF((KODE.NE.1).AND.(KODE.NE.2))THEN
        IERR=2
        ANSI1=0.0D0
        RETURN
      ENDIF
      IF(B.EQ.0.0D0) THEN
        IF(KODE.EQ.1) THEN
          ANSI1=0.0D0
          RETURN
        ELSE
          IF(A.EQ.0.0D0) THEN
            ANSI1=1.0D0/T
            RETURN
          ENDIF
          AT=A*T
          KKODE=1
          CALL DIERFC(AT,KKODE,ANSIE,IERR)
          IF(IERR.EQ.2) IERR=4
          ANSI1=RTPI*ANSIE/T
          RETURN
        ENDIF
      ENDIF
      IF(A.EQ.0.0D0) THEN
C       B NOT ZERO HERE
        BT=B*T
        IF(BT.GT.670.0D0) THEN
          ANSI1=0.0D0
          IERR=4
          RETURN
        ENDIF
        ERFB=DRERF(BT,KODE,NZ)
        BT2=BT*BT
        N=1
        KKODE=1
        M=1
        TOL=REL
        NZ=0
        IERR=0
        CALL DEXINT(BT2, N, KKODE, M, TOL, EN, NZ, IERR)
        IF(KODE.EQ.1) THEN
          ANSI1=ERFB/T+B*EN(1)/RTPI
        ELSE
          ANSI1=ERFB/T-B*EN(1)/RTPI
        ENDIF
        RETURN
      ENDIF
      A2=A*A
      B2=B*B
      T2=T*T
      AT=A*T
      BT=B*T
      ARG=(A2+B2)*T2      
      N=1
      KKODE=1
      M=1
      TOL=REL
      NZ=0
      KERR=0
      CALL DEXINT(ARG, N, KKODE, M, TOL, EN, NZ, KERR)
      AT2=AT*AT
      DEX=DEXP(-AT2)
      ERFB=DRERF(BT,KODE,NZ)
      IF(KODE.EQ.1) THEN
C       THE MAGNITUDE OF DEX MUST BE AT LEAST 2 WORD LENGTHS GREATER
C       THAN UNDERFLOW TO GUARANTEE PROPER ARITHMETIC FOR KODE=1
        IF(AT2.GT.620.0D0) THEN
          ANSI1=0.0D0
          IERR=4
          RETURN
        ENDIF
        IF(A.GT.B) THEN
C         COMBINING THE LEADING ERF FUNCTIONS FROM J5 (FOLDER5) WITH THE
C         ERF(BT) FUNCTION FROM I1 IN FOLDER 10
          CALL GSERI5(B,A,T,GANS,JERR)
          KKODE=1
          CALL DIERFC(AT,KKODE,ANSIE,IERR)          
          TRMA=RTPI*ANSIE*ERFB/T
          TRM=TRMA+B*EN(1)/RTPI-(A+A)*GANS*B
        ELSE
          CALL INTEGJ5(A,B,T,ANSJ5,JERR)
          TRMA=DEX*ERFB/T
          TRM=TRMA+B*EN(1)/RTPI-(A2+A2)*ANSJ5
        ENDIF
      ELSE
C       THE ASYMPTOTICS FOR ALL TERMS ARE DOMINATED BY DEXP(-ARG).
C       HENCE THE UNDER FLOW TEST ON ARG.
        IF(ARG.GT.670.0D0) THEN
          ANSI1=0.0D0
          IERR=4
          RETURN
        ENDIF
        IF((A.GT.B).AND.(ARG.GT.3.0D0)) THEN
C         COMBINING THE LEADING ERFC FUNCTIONS FROM I5 (FOLDER5) WITH
C         THE ERFC(BT) FUNCTION FROM I1C IN FOLDER 24A
          KKODE=1
          CALL DIERFC(AT,KKODE,ANSIE,IERR)          
          TRMA=RTPI*ANSIE*ERFB/T
          CALL GSERI5(B,A,T,GANS,JERR)
          TRM=TRMA-B*EN(1)/RTPI+(A+A)*GANS*B
        ELSE
          CALL INTEGI5(A,B,T,ANSI5,KERR)
          TRMA=DEX*ERFB/T
          TRM=TRMA-B*EN(1)/RTPI-(A2+A2)*ANSI5
        ENDIF
      ENDIF
      ANSI1=TRM
      RETURN
      END
      SUBROUTINE INTEGI2(A,B,T,KODE,ANSI2,IERR)
C
C     DONALD E. AMOS   DECEMBER, 2005
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C     REF: FOLDER 9 , FOLDER 7
C
C     ON KODE=1, INTEGI2 COMPUTES THE INTEGRAL
C
C                ANSI2=INT ON (0,T) OF ERF(A*W)*ERF(B*W);
C
C     ON KODE=2, INTEGI2 COMPUTES  THE INTEGRAL
C
C                ANSI2=INT ON (T,INF) OF ERFC(A*W)*ERFC(B*W)
C
C     WHERE      A.GE.0.0D0, B.GE.0.0D0, AND T.GE.0.0D0
C                (A AND B NOT ZERO AT THE SAME TIME FOR KODE=2)
C
C     ERROR CONDITIONS:
C       
C     IERR=0 NORMAL RETURN, COMPUTATION COMPLETED
C     IERR=1 A IS NEGATIVE OR B IS NEGATIVE OR C IS NON-POSITIVE OR KODE
C            IS NOT 1 OR 2, OR T IS NEGATIVE, ANSI2=0.0D0 RETURNED
C     IERR=2 IS AN UNDERFLOW,  ANSI2=0.0D0 RETURNED
C     IERR=3 NO CONVERGENCE IN A SERIES 
C     IERR=4 KODE=2, A=0.0D0, AND B=0.0D0; THE INTEGRAL DOES NOT EXIST
C
C     CALLS ROUTINES: DRERF, DINERFC
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION Y(61), CA(62), CB(62)
      DATA RTPI, PI/ 1.772453850905516D0, 3.1415926535897932D0/
      IERR=0
      IF((A.LT.0.0D0).OR.(B.LT.0.0D0).OR.(T.LT.0.0D0)) THEN
        IERR=1
        ANSI2=0.0D0
        RETURN
      ENDIF
      IF((KODE.NE.1).AND.(KODE.NE.2)) THEN
        IERR=1
        ANSI2=0.0D0
        RETURN
      ENDIF
      IF(KODE.EQ.1) THEN
        IF((A.EQ.0.0D0).OR.(B.EQ.0.0D0).OR.(T.EQ.0.0D0)) THEN
          ANSI2=0.0D0
          RETURN
        ENDIF
      ELSE
        IF((A.EQ.0.0D0).AND.(B.EQ.0.0D0)) THEN
          IERR=4
          ANSI2=0.0D0
          RETURN
        ELSE
          IF(T.EQ.0.0D0) THEN
            DNUM=2.0D0/RTPI 
            ANSI2=DNUM/(A+B+DSQRT(A*A+B*B))
            RETURN
          ENDIF
          IF(MIN(A,B).EQ.0.0D0) THEN
            XB=MAX(A,B)
            BT=XB*T
            KKODE=1
            CALL DIERFC(BT,KKODE,ANSB,IERR)
            ANSI2=ANSB/XB
            RETURN
          ENDIF
        ENDIF
      ENDIF
      XA=MIN(A,B)
      XB=MAX(A,B)
      AT=XA*T
      BT=XB*T
      A2=XA*XA
      B2=XB*XB
      T2=T*T
      A2B2=DSQRT(A2+B2)
      IF(KODE.EQ.1) THEN
        XLIM=1.25D0
        IF((AT.LE.XLIM).AND.(BT.LE.XLIM).AND.(T.LE.1.0D0)) THEN
          SUM=1.0D0/3.0D0
          CA(1)=1.0D0
          CB(1)=1.0D0
          AK=2.0D0
          TKP1=5.0D0
          TKP3=5.0D0
          T2A=AT*AT
          T2B=BT*BT
          CA(2)=-T2A/3.0D0
          CB(2)=-T2B/3.0D0
          TA=-T2A
          TB=-T2B
          REL=0.5D-15
          DO 5 K=1,60
            KP1=K+1
            UK=0.0D0
            DO 2 M=1,KP1
              UK=UK+CA(M)*CB(KP1-M+1)
  2         CONTINUE
            TRM=UK/TKP3
            SUM=SUM+TRM
            IF(DABS(TRM).LT.REL*DABS(SUM)) GOTO 6
            TA=-TA*T2A/AK
            TB=-TB*T2B/AK
            CA(K+2)=TA/TKP1
            CB(K+2)=TB/TKP1
            AK=AK+1.0D0
            TKP3=TKP3+2.0D0
            TKP1=TKP1+2.0D0
  5       CONTINUE
          IERR=3
  6       CONTINUE
          COEF=4.0D0*AT*BT*T/PI
          ANSI2=COEF*SUM
          RETURN
        ENDIF
        ERFAT=DRERF(AT,KODE,NZ)
        ERFBT=DRERF(BT,KODE,NZ)
        IF(AT.LT.2.40D0) THEN
          S=T*ERFAT*ERFBT+DEXP(-B2*T2)*ERFAT/(XB*RTPI)
          ARG=0.5D0*AT*AT
          R1=XA/(XB+A2B2)
          R=R1/(XB*RTPI)
          SS=-2.0D0*(1.0D0/(XA*RTPI)+R)*DEXP(-ARG)*SINH(ARG)
          SS=SS-R*DEXP(-A2*T2)*ERFBT
          S=S+SS
          KKODE=2
          ERFAT=DRERF(AT,KKODE,NZ)
          ERFBT=DRERF(BT,KKODE,NZ)
          COEF=T*R*A2B2*DEXP(-AT*AT)
          N0=1
          KKODE=1
          N=60
          REL=0.5D-14
          CALL DINERFC(BT,N0,KKODE,N,REL,Y,NZ)
          SUM=0.0D0
          RAT=AT*R1
          PWR=-2.0D0
          DO 10 N=1,60
            TRM=PWR*Y(N)
            SUM=SUM+TRM
            IF(DABS(TRM).LE.REL*DABS(SUM)) GOTO 15
            PWR=-2.0D0*PWR*RAT
  10      CONTINUE
          IERR=3
  15      CONTINUE
          ANSI2=S+COEF*SUM
          RETURN
        ENDIF
        ARG=T*A2B2
        ERFABT=DRERF(ARG,KODE,NZ)
        S=T*ERFAT*ERFBT
        S=S+DEXP(-A2*T2)*ERFBT/(XA*RTPI)
        S=S+DEXP(-B2*T2)*ERFAT/(XB*RTPI)
        ANSI2=S-A2B2*ERFABT/(XA*XB*RTPI)
      ELSE
        EX=T2*(A2+B2)
        IF(EX.GT.670.0D0) THEN
          IERR=2
          ANSI2=0.0D0
          RETURN
        ENDIF
        XLIM=1.25D0
        IF((AT.LE.XLIM).AND.(BT.LE.XLIM).AND.(T.LE.1.0D0)) THEN
          SUMAB=1.0D0/3.0D0
          SUMA=1.0D0/2.0D0
          SUMB=1.0D0/2.0D0
          CA(1)=1.0D0
          CB(1)=1.0D0
          AK=2.0D0
          TKP1=5.0D0
          TKP2=4.0D0
          TKP3=5.0D0
          T2A=AT*AT
          T2B=BT*BT
          CA(2)=-T2A/3.0D0
          CB(2)=-T2B/3.0D0
          TA=-T2A
          TB=-T2B
          REL=0.5D-15
          DO 35 K=1,60
            KP1=K+1
            TRMA=CA(KP1)/TKP2
            TRMB=CB(KP1)/TKP2
            SUMA=SUMA+TRMA
            SUMB=SUMB+TRMB
            UK=0.0D0
            DO 32 M=1,KP1
              UK=UK+CA(M)*CB(KP1-M+1)
  32        CONTINUE
            TRMAB=UK/TKP3
            SUMAB=SUMAB+TRMAB
            IF(DABS(TRMAB).LT.REL*DABS(SUMAB)) THEN
              IF(DABS(TRMA).LT.REL*DABS(SUMA)) THEN
                IF(DABS(TRMB).LT.REL*DABS(SUMB)) GOTO 36
              ENDIF
            ENDIF
            TA=-TA*T2A/AK
            TB=-TB*T2B/AK
            CA(K+2)=TA/TKP1
            CB(K+2)=TB/TKP1
            AK=AK+1.0D0
            TKP3=TKP3+2.0D0
            TKP2=TKP2+2.0D0
            TKP1=TKP1+2.0D0
  35      CONTINUE
          IERR=3
  36      CONTINUE
          COEFA=2.0D0*AT*T/RTPI
          COEFB=2.0D0*BT*T/RTPI
          COEFAB=4.0D0*AT*BT*T/PI
          ANSA=COEFA*SUMA
          ANSB=COEFB*SUMB
          ANSAB=COEFAB*SUMAB
          DNUM=2.0D0/RTPI 
          ANST0=DNUM/(A+B+DSQRT(A*A+B*B))          
          ANSI2=ANST0-T+ANSA+ANSB-ANSAB
          RETURN
        ENDIF
        IF(AT.LE.2.40D0) THEN
          ERFAT=DRERF(AT,KODE,NZ)
          ERFBT=DRERF(BT,KODE,NZ)
          N0=1
          KKODE=1
          N=60
          REL=0.5D-14
          CALL DINERFC(BT,N0,KKODE,N,REL,Y,NZ)
          R1=XA/(XB+A2B2)
          R=R1/(XB*RTPI)
C         Y(1)=IERFC(BT)
          S=ERFAT*Y(1)/XB-EXP(-A2*T2)*ERFBT*R
          COEF=T*A2B2*R*DEXP(-AT*AT)
          SUM=0.0D0
          RAT=AT*R1
          PWR=-2.0D0
          DO 20 N=1,60
            TRM=PWR*Y(N)
            SUM=SUM+TRM
            IF(DABS(TRM).LE.REL*DABS(SUM)) GOTO 30
            PWR=-2.0D0*PWR*RAT
  20      CONTINUE
          IERR=3
  30      CONTINUE
          ANSI2=S-COEF*SUM
          RETURN
        ELSE
          ARG=T*A2B2
          KK0DE=1
          CALL DIERFC(ARG,KKODE,ANSAB,KERR)
          CALL DIERFC(AT,KKODE,ANSA,KERR)
          CALL DIERFC(BT,KKODE,ANSB,KERR)
          ANSI2=T*(ANSAB/RTPI-ANSA*ANSB)/(AT*BT)
        ENDIF
      ENDIF
      RETURN
      END
      SUBROUTINE INTEGI9(A,B,T,KODE,ANSI9,IERR)
C
C     DONALD E. AMOS   DECEMBER, 2005; APRIL, 2006  
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     REF. FOLDER 9
C
C     ON KODE=1, INTEGI9 COMPUTES THE INTEGRAL
C
C                ANSI9 =  INT ON (0,T) OF W*ERF(A*W)*ERF(B*W)
C
C     ON KODE=2, INTEGI9 COMPUTES THE COMPLEMENTARY INTEGRAL
C
C                ANSI9 =  INT ON (T,INF) OF W*ERFC(A*W)*ERFC(B*W)
C
C     FOR A.GE.0, B.GE.0, T.GE.0.
C
C     ERROR CONDITIONS
C     IERR=0 NORMAL RETURN, COMPUTATION COMPLETED
C     IERR=1 A IS NEGATIVE OR B IS NEGATIVE OR T IS NEGATIVE OR KODE
C            IS NOT 1 OR 2, ANSI9=0.0D0 RETURNED
C     IERR=2 IS AN UNDERFLOW,  ANSI9=0.0D0 RETURNED
C     IERR=3 NO CONVERGENCE IN A SERIES 
C     IERR=4 KODE=2, A=0.0D0, AND B=0.0D0; THE INTEGRAL DOES NOT EXIST
C
C     CALLS ROUTINES: DRERF, INERFC, DIERFC, INTEGI5, INTEGV5
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION CA(62),CB(62),Y(62)
      DATA RTPI, PI  /1.772453850905516D0, 3.1415926535897932D0/
      IERR=0
      IF((A.LT.0.0D0).OR.(B.LT.0.0D0).OR.(T.LT.0.0D0)) THEN
        IERR=1
        ANSI9=0.0D0
        RETURN
      ENDIF
      IF((KODE.NE.1).AND.(KODE.NE.2)) THEN
        IERR=1
        ANSI9=0.0D0
        RETURN
      ENDIF
      AT=A*T
      BT=B*T
      A2=A*A
      B2=B*B
      T2=T*T
      AT2=AT*AT
      BT2=BT*BT
      ARTPI=A*RTPI
      BRTPI=B*RTPI
      IF(KODE.EQ.1) THEN
        XA=MIN(A,B)
        IF((XA.EQ.0.0D0).OR.(T.EQ.0.0D0)) THEN
          ANSI9=0.0D0
          RETURN
        ENDIF
C       I9 BY POWER SERIES
        XLIM=1.25D0
        IF((AT.LE.XLIM).AND.(BT.LE.XLIM).AND.(T.LE.1.0D0)) THEN
          SUM=1.0D0/4.0D0
          CA(1)=1.0D0
          CB(1)=1.0D0
          AK=2.0D0
          TKP1=5.0D0
          TKP4=6.0D0
          T2A=AT*AT
          T2B=BT*BT
          CA(2)=-T2A/3.0D0
          CB(2)=-T2B/3.0D0
          TA=-T2A
          TB=-T2B
          REL=0.5D-15
          DO 5 K=1,60
            KP1=K+1
            UK=0.0D0
            DO 2 M=1,KP1
              UK=UK+CA(M)*CB(KP1-M+1)
  2         CONTINUE
            TRM=UK/TKP4
            SUM=SUM+TRM
            IF(DABS(TRM).LT.REL*DABS(SUM)) GOTO 6
            TA=-TA*T2A/AK
            TB=-TB*T2B/AK
            CA(K+2)=TA/TKP1
            CB(K+2)=TB/TKP1
            AK=AK+1.0D0
            TKP4=TKP4+2.0D0
            TKP1=TKP1+2.0D0
  5       CONTINUE
          IERR=3
  6       CONTINUE
          COEF=4.0D0*AT*BT*T2/PI
          ANSI9=COEF*SUM
          RETURN
        ENDIF
        XB=MAX(A,B)
C       I9 FOR A.LE.B AND XAT.LE.2.4
        XAT=XA*T
        IF(XAT.LE.2.4D0) THEN
          XBT=XB*T
          KKODE=1
          ERFAT=DRERF(XAT,KKODE,NZ)
          KKODE=1
          ERFBT=DRERF(XBT,KKODE,NZ)
          RTAB=DSQRT(A2+B2)
          XAT2=XAT*XAT
          EXA=DEXP(-XAT2)
          COEF=(XA/XB)*EXA/((XB+RTAB)*RTPI*RTAB)
          N0=1
          KKODE=1
          N=60
          REL=0.5D-14
          TOL=REL
          CALL DINERFC(XBT,N0,KKODE,N,TOL,Y,NZ)
          CALL INTEGV5(XB,XA,T,ANSV5,IERR5)
          XBT2=XBT*XBT
          EXB=DEXP(-XBT2)
          SUM=T2*ERFAT*ERFBT+(T*EXB*ERFAT-ANSV5)/(XB*RTPI)
          IF(XAT.LE.0.50D0) THEN
C           COMPUTE H1=(T*DEXP(-XAT2)-0.5D0*RTPI*ERF(XAT)/XA)/(XA*RTPI)
C           BY POWER SERIES TO RESOLVE THE INDETERMINANCY FOR XA TO ZERO
            S=-XAT/3.0D0
            AK=-XAT
            FAC=1.0D0
            TKP1=5.0D0
            REL=0.5D-15
            DO 60 K=2,50
              AK=-AK*XAT2/FAC
              TRM=AK/TKP1
              S=S+TRM
              IF(DABS(TRM).LE.REL*DABS(S)) GOTO 61
              FAC=FAC+1.0D0
              TKP1=TKP1+2.0D0
 60         CONTINUE
            IERR=3
 61         CONTINUE
            H1=2.0D0*T2*S/RTPI
          ELSE
            H1=(T*EXA-0.5D0*RTPI*ERFAT/XA)/(XA*RTPI)
          ENDIF
C         COMPUTE H2=(DATAN(RAB)-RAB)/(XA*XA*PI) BY POWER SERIES FOR
C         RAB.LE.0.5 TO RESOLVE THE INDETERMINANCY FOR XA TO ZERO
          RAB=XA/XB
          IF(RAB.LE.0.25D0) THEN
            RAB2=RAB*RAB
            S=-RAB/3.0D0
            AK=-RAB
            TKP1=5.0D0
            REL=0.5D-15
            DO 70 K=2,50
              AK=-AK*RAB2
              TRM=AK/TKP1
              S=S+TRM
              IF(DABS(TRM).LE.REL*DABS(S)) GOTO 71
              TKP1=TKP1+2.0D0
 70         CONTINUE
            IERR=3
 71         CONTINUE
            H2=S/(XB*XB*PI)
          ELSE
            H2=(DATAN(RAB)-RAB)/(XA*XA*PI)
          ENDIF
          SUM=SUM+H1+H2
          S=0.0D0
          RX=XA*XAT/(XB+RTAB)
          PX=-2.0D0
          AN=2.0D0
          REL=0.5D-15
          DO 45 N=2,60
            TRM=Y(N)*PX*AN
            S=S+TRM
            IF(DABS(TRM).LE.REL*DABS(S)) GOTO 46
            PX=-2.0D0*PX*RX
            AN=AN+1.0D0
  45      CONTINUE
C          WRITE(7,307)
          IERR=3
  46      CONTINUE
          WTRM=COEF*(-Y(1)+XBT*S)
C         ANSG9=G1(A,B,T)/A
          CALL GSERI9(XA,XB,T,ANSG9,IERRG)
          ANSI9=0.5D0*(SUM-WTRM-ANSG9)
          RETURN
        ENDIF
C       DEFAULT COMPUTATION FOR I9
        KKODE=1
        ERFA=DRERF(AT,KKODE,NZ)
        KKODE=1
        ERFB=DRERF(BT,KKODE,NZ)
        ARTPI=A*RTPI
        BRTPI=B*RTPI
        SUM=T*ERFA*ERFB
        SUM=SUM+DEXP(-AT2)*ERFB/(ARTPI)+DEXP(-BT2)*ERFA/(BRTPI)
        CALL INTEGV5(A,B,T,ANSAB,IERRB)
        CALL INTEGV5(B,A,T,ANSBA,IERRA)
        SUM=T*SUM-ANSAB/ARTPI
        SUM=SUM-ANSBA/BRTPI
        ARG=0.5D0*(AT2+BT2)
        IF(ARG.GT.18.0D0) THEN
          PHI=1.0D0
        ELSE
          PHI=2 0D0*DEXP(-ARG)*DSINH(ARG)
        ENDIF
        ANSI9=0.5D0*(SUM-PHI/(A*B*PI))
        RETURN
      ELSE
C       COMPUTE FOR I9C
        ARG=T2*(A2+B2)
        IF(ARG.GT.670.0D0) THEN
          ANSI9=0.0D0
          IERR=2
          RETURN
        ENDIF
        IF((A.EQ.0.0D0).AND.(B.EQ.0.0D0)) THEN
          ANSI9=0.0D0
          IERR=4
          RETURN
        ENDIF
        XA=MIN(A,B)
        IF(XA.EQ.0.0D0) THEN
          XB=MAX(A,B)
          XBT=XB*T
          N0=1
          KKODE=1
          N=2
          REL=0.5D-14
          CALL DINERFC(XBT,N0,KKODE,N,REL,Y,NZ)
          ANSI9=T*Y(1)/XB+Y(2)/(XB*XB)
          IF(NZ.NE.0) THEN
            ANSI9=0.0D0
            IERR=2
          ENDIF
          RETURN
        ENDIF
C       I9C BY POWER SERIES
        XLIM=1.25D0
        IF((AT.LE.XLIM).AND.(BT.LE.XLIM).AND.(T.LE.1.0D0)) THEN
          SUMAB=1.0D0/4.0D0
          SUMA=1.0D0/3.0D0
          SUMB=1.0D0/3.0D0
          CA(1)=1.0D0
          CB(1)=1.0D0
          AK=2.0D0
          TKP1=5.0D0
          TKP3=5.0D0
          TKP4=6.0D0
          T2A=AT*AT
          T2B=BT*BT
          CA(2)=-T2A/3.0D0
          CB(2)=-T2B/3.0D0
          TA=-T2A
          TB=-T2B
          REL=0.5D-15
          DO 35 K=1,60
            KP1=K+1
            TRMA=CA(KP1)/TKP3
            TRMB=CB(KP1)/TKP3
            SUMA=SUMA+TRMA
            SUMB=SUMB+TRMB
            UK=0.0D0
            DO 32 M=1,KP1
              UK=UK+CA(M)*CB(KP1-M+1)
  32        CONTINUE
            TRMAB=UK/TKP4
            SUMAB=SUMAB+TRMAB
            IF(DABS(TRMAB).LT.REL*DABS(SUMAB)) THEN
              IF(DABS(TRMA).LT.REL*DABS(SUMA)) THEN
                IF(DABS(TRMB).LT.REL*DABS(SUMB)) GOTO 36
              ENDIF
            ENDIF
            TA=-TA*T2A/AK
            TB=-TB*T2B/AK
            CA(K+2)=TA/TKP1
            CB(K+2)=TB/TKP1
            AK=AK+1.0D0
            TKP3=TKP3+2.0D0
            TKP4=TKP4+2.0D0
            TKP1=TKP1+2.0D0
  35      CONTINUE
          IERR=3
  36      CONTINUE
          COEFA=2.0D0*AT*T2/RTPI
          COEFB=2.0D0*BT*T2/RTPI
          COEFAB=4.0D0*AT*BT*T2/PI
          ANSA=COEFA*SUMA
          ANSB=COEFB*SUMB
          ANSAB=COEFAB*SUMAB
          XA=MIN(A,B)
          XB=MAX(A,B)
          X=XA/XB
          XB2=XB*XB
C         COMPUTE I9C(A,B,0) BY SERIES IF A/B.LT.0.125 
          IF(X.LT.0.125D0) THEN
            REL=0.5D-15
            SUM=0.0D0
            AK=3.0D0
            X2=X*X
            PX=-XA/XB2
            DO 50 K=1,20
              TRM=PX/AK
              SUM=SUM+TRM
              IF(DABS(TRM).LT.REL*DABS(SUM)) GOTO 51
              AK=AK+2.0D0
              PX=-PX*X2
  50        CONTINUE
            IERR=3
  51        CONTINUE
            TEMP=0.5D0*SUM/(XB*PI)+0.25D0/XB2
            ANST0=TEMP-0.5D0*DATAN(X)/(XB2*PI)
          ELSE
            TEMP=0.5D0*(DATAN(X)/X-1.0D0)/(XA*XB*PI)+0.25D0/XB2
            ANST0=TEMP-0.5D0*DATAN(X)/(XB2*PI)
          ENDIF
          ANSI9=ANST0-0.5D0*T2+ANSA+ANSB-ANSAB
          RETURN
        ENDIF
        XB=MAX(A,B)
C       I9C FOR A.LE.B AND XAT.LE.2.4
        XAT=XA*T
        IF(XAT.LE.2.4D0) THEN
          XBT=XB*T
          KKODE=2
          ERFAT=DRERF(XAT,KKODE,NZ)
          KKODE=2
          ERFBT=DRERF(XBT,KKODE,NZ)
          RTAB=DSQRT(A2+B2)
          XAT2=XAT*XAT
          COEF=(XA/XB)*DEXP(-XAT2)/((XB+RTAB)*RTPI*RTAB)
          N0=1
          KKODE=1
          N=60
          REL=0.5D-14
          TOL=REL
          CALL DINERFC(XBT,N0,KKODE,N,TOL,Y,NZ)
          CALL INTEGI5(XB,XA,T,ANSI5,IERR5)
          SUM=(T*ERFAT*Y(1)+ANSI5/RTPI)/XB
          S=0.0D0
          RX=XA*XAT/(XB+RTAB)
          PX=-2.0D0
          AN=2.0D0
          DO 40 N=2,60
            TRM=Y(N)*PX*AN
            S=S+TRM
            IF(DABS(TRM).LT.REL*DABS(S)) GOTO 41
            PX=-2.0D0*PX*RX
            AN=AN+1.0D0
  40      CONTINUE
C          WRITE(7,307)
C  307     FORMAT("NO CONVERGENCE IN INERFC LOOP")
          IERR=3
  41      CONTINUE
          WTRM=COEF*(-Y(1)+XBT*S)
          CALL GSERI9(XA,XB,T,ANSG9,IERRG)
          ANSI9=0.5D0*(SUM+WTRM+ANSG9)
          RETURN
        ENDIF
C       DEFAULT COMPUTATION FOR I9C
        KKODE=1
        CALL DIERFC(BT,KKODE,DIERFB,IERRB)
        CALL DIERFC(AT,KKODE,DIERFA,IERRA)
        SUM=-DIERFA*DIERFB/(A*B)
        CALL INTEGI5(A,B,T,ANSAB,KERR)
        CALL INTEGI5(B,A,T,ANSBA,KERR)
        ANSI9=0.5D0*(SUM+ANSAB/ARTPI+ANSBA/BRTPI)
        RETURN
      ENDIF
      END      
      SUBROUTINE GSERI9(A,B,X,SUM,IERR)
C
C     GSERI9 COMPUTES THE G SUM OF FOLDER 9 = G SUM OF FOLDER 5 LESS
C     FIRST TERM MULTIPLIED BY 1/(A*RTPI).
C
C     IERR=0 NORMAL RETURN
C     IERR=1 UNDERFLOW, SUM=0.0D0
C     IERR=2 NO CONVERGENCE IN 55 TERMS OF THE SERIES
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(55)
      DATA RTPI, PI  /1.772453850905516D0, 3.1415926535897932D0/
      IERR=0
      ARG=A*A+B*B
      DEN=ARG
      ARG=ARG*X*X
      FNH=2.5D0
      KODE=1
      M=55
      TOL=0.5D-15
      NZ=0
      KERR=0
      CALL DHEXINT(ARG, FNH, KODE, M, TOL, EN, NZ, KERR)
      IF(NZ.NE.0) THEN
        IERR=1
        SUM=0.0D0
        RETURN
      ENDIF
      S=0.0D0
      AK=A/DEN
      DH=0.5D0
      Z=A*A/DEN
      FAC=1.0D0
      DO 10 K=1,M-1
        AK=AK*DH/FAC
        TRM=AK*EN(K)
        S=S+TRM
        IF (DABS(TRM).LE.TOL*DABS(S)) GOTO 15        
        DH=DH+1.0D0
        FAC=FAC+1.0D0
        AK=AK*Z
  10  CONTINUE
       IERR=2
C      WRITE (7,300)
C 300  FORMAT('G9-SERIES NOT CONVERGED IN 55 TERMS')
  15  CONTINUE
      SUM=(0.5D0/PI)*(S/DSQRT(DEN))
      RETURN
      END
      SUBROUTINE INTEGI3(A,B,C,T,KODE,ANSI3,IERR,KFORM)
C
C     DONALD E. AMOS   MAY, 2001   DECEMBER, 2005
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     INTEGI3 COMPUTES THE I3 AND THE I3C INTEGRALS OF FOLDER 7,
C
C     ANSI3=INT ON (T,INF)OF EXP(-C*C*W*W)*ERF(A*W)*ERF(B*W)   ON KODE=1
C
C     AND
C
C     ANSI3=INT ON (T,INF)OF EXP(-C*C*W*W)*ERFC(A*W)*ERFC(B*W) ON KODE=2   
C
C     FOR C>0, WITH T, A AND B  NON-NEGATIVE.
C
C     ERROR CONDITIONS
C
C     IERR=0  NORMAL RETURN
C     IERR=1  C IS ZERO OR NEGATIVE AND I3 IS UNDEFINED. ANSI3=0.0D0
C             RETURNED
C     IERR=2  A, B OR T IS NEGATIVE OR KODE IS NOT 1 OR 2. I3 IS DEFINED
C             HERE FOR NON-NEGATIVE A AND B, THOUGH I3 EXITS FOR
C             NEGATIVE VALUES OF A AND B. ANSI3=0.0D0 RETURNED
C     IERR=3  UNDERFLOW, ANSI3=0.0D0 RETURNED
C
C             CONVERGENCE OR COMPUTATIONAL PROBLEMS (NOT LIKELY)
C
C     IERR=4  NO CONVERGENCE IN SSERI3 SERIES IN 85 TERMS. ANSI3=0.0D0
C             RETURNED
C     IERR=5  NO CONVERGENCE IN I5 SERIES IN INTEGI5 IN 55 TERMS
C             ANSI3=0.0D0 RETURNED
C
C     CONDITIONS IERR = 4 OR 5 SHOULD ONLY OCCUR IF THE MACHINE FAILS,
C     SO THERE IS GENERALLY NO REASON TO TEST FOR THEM.
C
C     KFORM IS A RETURN VARIABLE TELLING WHICH FORMULA WAS USED IN THE
C     COMPUTATION OF ANSI3 WHEN IERR=0:
C   
C     KFORM=0 MEANS EITHER A OR B WAS ZERO. ANSI3=0.0D0 RETURNED
C     KFORM=1 MEANS THE ENHANCEMENT (A)- ERF(*)=1, BOTH AT AND BT.GE.6
C     KFORM=2 MEANS THE ENHANCEMENT (B)- POWER SERIES AT,BT,CT.LE.2.0D0
C     KFORM=3 MEANS THE ENHANCEMENT (C)- LARGE C FORMULA
C     KFORM=4,5 MEANS MAIN FORMULA CASE I  FOR C.LE.XA.LE.XB
C     KFORM=6,7 MEANS MAIN FORMULA CASE I  FOR XA.LE.C.LE.XB
C     KFORM=8   MEANS MAIN FORMULA CASE II FOR XA.LE.XB.LE.C
C               WHERE XA=MIN(A,B) AND XB=MAX(A,B)
C
C     CALLS ROUTINES: PHIZ, PKSER, SSERI3, I3LARGEC, I3PWRSER, FHYPER,
C                     INTEGI5, SSERI5, DHEXINT, DRERF
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION ENH(85)
      DATA RTPI ,PI /1.772453850905516D0, 3.14159265358979D0/
      IERR=0
      KFORM=0
      IF (C.LE.0.0D0) THEN
C       IF C=0, THEN I3 DOES NOT EXIST.
        IERR=1
        ANSI3=0.0D0
        RETURN
      ENDIF
      IF((A.LT.0.0D0).OR.(B.LT.0.0D0))THEN
C       I3 HERE IS DEFINED ONLY FOR A AND B NON-NEGATIVE.
        IERR=2 
        ANSI3=0.0D0
        RETURN
      ENDIF
      IF(T.LT.0.0D0) THEN
C       THE INTEGRAL IS DEFINED ONLY FOR T.GE.0.0D0
        IERR=2
        ANSI3=0.0D0
        RETURN
      ENDIF
      IF((KODE.NE.1).AND.(KODE.NE.2)) THEN
        IERR=2
        ANSI3=0.0D0
        RETURN
      ENDIF
      IF(KODE.EQ.2) THEN
        IF((A.EQ.0.0D0).OR.(B.EQ.0.0D0)) THEN
          XB=MAX(A,B)
          IF(XB.EQ.0.0D0) THEN
            ARG=C*T
            ERFA=DRERF(ARG,KODE,NZ) 
            ANSI3=0.5D0*RTPI*ERFA/C
            RETURN
          ELSE
            CALL INTEGI5(C,XB,T,ANSI3,IERR)
            RETURN
          ENDIF
        ENDIF
        CALL J3SER(A,B,C,T,ANSI3)
        RETURN
      ENDIF
      IF((A.EQ.0.0D0).OR.(B.EQ.0.0D0)) THEN
        ANSI3=0.0D0
        RETURN
      ENDIF
      XB=MAX(A,B)
      XA=MIN(A,B)
      AT=XA*T
      BT=XB*T
      CT=C*T
      IF (AT.GE.6.0D0) THEN
C       TEST FOR AT>6 AND BT>6 WHERE BOTH ERF FUNCTIONS ARE 1
C       XA=MIN(A,B), XB=MAX(A,B), AT=XA*T.LE.BT=XB*T
        KKODE=2
        NZ=0
        ERFCT=DRERF(CT,KKODE,NZ)
        IF (NZ.NE.0) THEN
          IERR=3
          ANSI3=0.0D0
          KFORM=1
          RETURN
        ENDIF
        ANSI3=0.5D0*RTPI*ERFCT/C
        KFORM=1
        RETURN
      ENDIF
C     TEST FOR POWER SERIES
      IF((AT.LE.2.0D0).AND.(BT.LE.2.0D0).AND.(CT.LE.2.0D0)) THEN
        CALL I3PWRSER(A,B,C,T,SUM,IERRP)
        IF (IERRP.EQ.1) THEN
C         ON CONVERGENCE FAILURE, LET OTHER CASES TRY TO COMPUTE
          GOTO 100
        ENDIF
        IF (C.LE.XB) THEN
          TT=0.0D0
          CALL PKSER(XA,XB,C,TT,SS,IERR)
          DD=DSQRT(XB*XB+C*C)
          S=DATAN(XA/C)/(C*RTPI)-SS*XA/(PI*DD)
          ANSI3=S-SUM
          KFORM=2
          RETURN
        ELSE
          TT=0.0D0
          CALL PKSER(XB,C,XA,TT,SS,IERR)
          DD=DSQRT(XA*XA+C*C)
          SS=SS*XB/(DD*PI)
          S=XA*SS/C
          CALL PKSER(XA,C,XB,TT,SS,IERR)
          DD=DSQRT(XB*XB+C*C)
          SS=SS*XA/(DD*PI)
          ANSI3=S+XB*SS/C-SUM
          KFORM=2
          RETURN
        ENDIF
      ENDIF
 100  CONTINUE
C     TEST  FOR LARGE C FORMULA
      CT2=CT*CT
      A2B2=A*A+B*B
      IF (CT2.GT.5.0D0) THEN
        FG=CT2-4.0D0
      ELSE
        FG=1.0D0
      ENDIF
      IF ((A2B2.LT.0.5D0*C*C/FG)) THEN
C     JERR=0 MEANS COMPUTATION COMPLETED
C     JERR=1 MEANS NO CONVERGENCE IN LARGEC IN 100 TERMS 
C     JERR=2 MEANS AN UNDER FLOW
C     JERR=3 MEANS SERIES ABORTED BECAUSE TERMS INCREASE IN VALUE IN
C            LARGEC
        CALL I3LARGEC(A,B,C,T,ANSI3,JERR)
        IF (JERR.EQ.0) THEN
          KFORM=3
          RETURN
        ENDIF
        IF (JERR.EQ.2) THEN
          KFORM=3
          ANSI3=0.0D0
          IERR=3
          RETURN
        ENDIF          
C     COMPUTE BY MAIN METHOD FOR CASES I AND II IF JERR=1 OR JERR=3
      ENDIF
      T2=T*T
      X2=XA*XA+XB*XB+C*C
      ARG=T2*X2
      FNH=1.5D0
      KKODE=1
      M=85
      REL=0.50D-15
      KERR=0
      NZ=0
      CALL DHEXINT(ARG, FNH, KKODE, M, REL, ENH, NZ, KERR)
      IF(KERR.EQ.2) THEN
        ANSI3=0.0D0
        IERR=6
        RETURN
      ENDIF
      IF(C.LE.XA) THEN
C       CASE I, C<=A      
        ALFA2=XA*XA+C*C
        ALFA=DSQRT(ALFA2)
        ALT2=ALFA2*T2
        KKODE=1
        NZ=0
        ERFAT=DRERF(AT,KKODE,NZ)
        CALL INTEGI5(C,XB,T,ANSI5,IERR5)
        IF ((IERR5.EQ.5).OR.(IERR5.EQ.6)) THEN
          ANSI3=0.0D0
          IERR=5
          RETURN
        ENDIF
        SS=ERFAT*ANSI5
        CALL SSERI3(XA,XB,C,T,SER,ENH,IERRS)
        IF(IERRS.EQ.2) THEN
          ANSI3=0.0D0
          IERR=4
          RETURN
        ENDIF
        SS=SS+SER
        CRTPI=C*RTPI
        IF(ALT2.LE.3.0D0) THEN
          S=DATAN(XA/C)/CRTPI
          KKODE=1
          NZ=0
          ERFCT=DRERF(CT,KKODE,NZ)
          S=S-0.5D0*RTPI*ERFCT/C+PHIZ(ALT2)*DATAN(C/XA)/CRTPI
          CALL SSERI5(C,XA,T,SS5,IERRS)
          IF(IERRS.EQ.1) THEN
            ANSI3=0.0D0
            IERR=3
            KFORM=4            
            RETURN
          ENDIF
          IF(IERRS.EQ.2) THEN
            ANSI3=0.0D0
            IERR=5
            KFORM=4
          ENDIF
          S=S+0.5D0*ALFA*T2*SS5/RTPI
          ANSI3=S-SS
          KFORM=4          
          RETURN
        ELSE
          KKODE=2
          NZ=0
          ERFCT=DRERF(CT,KKODE,NZ)
          IF (NZ.NE.0) THEN
             ANSI3=0.0D0
             IERR=3
             KFORM=5
             RETURN
          ENDIF
          S=0.5D0*RTPI*ERFCT/C
          CALL INTEGI5(C,XA,T,ANSI5,IERR5)
          IF ((IERR5.EQ.5).OR.(IERR5.EQ.6)) THEN
            ANSI3=0.0D0
            IERR=5
            KFORM=5
            RETURN
          ENDIF
          SD=S-ANSI5
          IF (DABS(SD).GT.1.0D-275) THEN
            ANSI3=SD-SS
          ELSE
            ANSI3=0.0D0
            IERR=3
          ENDIF
          KFORM=5
          RETURN
        ENDIF      
      ENDIF
      IF(C.LE.XB) THEN
C       CASE I, A<C<=B
        BETA2=XB*XB+C*C
        BETA=DSQRT(BETA2)
        BET2=BETA2*T2
        KKODE=1
        NZ=0
        ERFAT=DRERF(AT,KKODE,NZ)
        CALL INTEGI5(XA,C,T,ANSI5,IERR5)
        IF ((IERR5.EQ.5).OR.(IERR5.EQ.6)) THEN
          ANSI3=0.0D0
          IERR=5
          RETURN
        ENDIF
        SS=XA*ANSI5/C
        CALL SSERI3(XA,XB,C,T,SER,ENH,IERRS)
        IF(IERRS.EQ.2) THEN
          ANSI3=00D0
          IERR=4
          RETURN
        ENDIF
        SS=SS-SER
        CRTPI=C*RTPI
        IF(BET2.LE.3.0D0) THEN
          S=DATAN(XB/C)/CRTPI
          KKODE=1
          NZ=0
          ERFCT=DRERF(CT,KKODE,NZ)
          S=S-0.5D0*RTPI*ERFCT/C+PHIZ(BET2)*DATAN(C/XB)/CRTPI
          CALL SSERI5(C,XB,T,SS5,IERRS)
          IF(IERRS.EQ.1) THEN
            ANSI3=0.0D0
            IERR=3
            KFORM=6
            RETURN
          ENDIF
          IF(IERRS.EQ.2) THEN
            ANSI3=0.0D0
            IERR=5
            KFORM=6
            RETURN
          ENDIF
          S=S+0.5D0*BETA*T2*SS5/RTPI
          ANSI3=S*ERFAT+SS
          KFORM=6
          RETURN
        ELSE
          KKODE=2
          NZ=0
          ERFCT=DRERF(CT,KKODE,NZ)
          IF (NZ.NE.0) THEN
             ANSI3=0.0D0
             IERR=3
             KFORM=7
             RETURN
          ENDIF
          S=0.5D0*RTPI*ERFCT/C
          IF(S.LT.1.0D-275) THEN
            ANSI3=0.0D0
            IERR=3
            KFORM=7
            RETURN
          ENDIF
          CALL INTEGI5(C,XB,T,ANSI5,IERR5)
          IF ((IERR5.EQ.5).OR.(IERR5.EQ.6)) THEN
            ANSI3=0.0D0
            IERR=5
            KFORM=7
            RETURN
          ENDIF
          IF(SS.EQ.0.0D0) THEN            
            SD=(S-ANSI5)*ERFAT
            IF (DABS(SD).LT.1.0D-290) THEN
              ANSI3=0.0D0
              IERR=3
            ELSE
              ANSI3=SD            
            ENDIF
          ELSE
            ANSI3=(S-ANSI5)*ERFAT+SS
          ENDIF
          KFORM=7
          RETURN
        ENDIF
      ENDIF
C     CASE II
      KKODE=2
      NZ=0
      ERFCT=DRERF(CT,KKODE,NZ)
      IF (ERFCT.LT.1.0D-275) THEN
        ANSI3=0.0D0
        IERR=3
        KFORM=8
        RETURN
      ENDIF
      KKODE=1
      NZ=0
      ERFAT=DRERF(AT,KKODE,NZ)
      KKODE=1
      NZ=0
      ERFBT=DRERF(BT,KKODE,NZ)
      S=0.5D0*RTPI*ERFBT*ERFCT/C
      IF (S.LT.1.0D-275) THEN
        ANSI3=0.0D0
        IERR=3
        KFORM=8
        RETURN
      ENDIF
      CALL INTEGI5(XB,C,T,ANSI5,IERR5)
      IF ((IERR5.EQ.5).OR.(IERR5.EQ.6)) THEN
        ANSI3=0.0D0
        IERR=5
        KFORM=8
        RETURN
      ENDIF
      S=(S+XB*ANSI5/C)*ERFAT
      IF(S.LT.1.0D-275) THEN
        ANSI3=0.0D0
        IERR=3
        KFORM=8
        RETURN
      ENDIF
      CALL INTEGI5(XA,C,T,ANSI5,IERR5)
      IF ((IERR5.EQ.5).OR.(IERR5.EQ.6)) THEN
        ANSI3=0.0D0
        IERR=5
        KFORM=8
        RETURN
      ENDIF
      S=S+XA*ANSI5*ERFBT/C
      CALL SSERI3(XB,C,XA,T,SS,ENH,IERRS)
      IF(IERRS.EQ.2) THEN
        ANSI3=00D0
        IERR=4
        KFORM=8
        RETURN
      ENDIF
      S=S+XA*SS/C
      CALL SSERI3(XA,C,XB,T,SS,ENH,IERRS)
      IF(IERRS.EQ.2) THEN
        ANSI3=00D0
        IERR=4
        KFORM=8
        RETURN
      ENDIF
      ANSI3=S+XB*SS/C
      KFORM=8
      RETURN
      END
      SUBROUTINE J3SER(A,B,C,T,FJ3)
C     J3SER EVALUATES THE SERIES FORM OF J3=INT ON (T,INF) OF
C     EXP(-C*C*W*W)*ERFC(A*W)*ERFC(B*W), DEVELOPED IN FOLDER7
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION ENH(85)
      DATA RTPI /1.772453850905516D0/
      IF (T.EQ.0.0D0) GOTO 55
      T2=T*T
      X2=A*A+B*B+C*C
      ARG=T2*X2
      FNH=1.5D0
      KODE=1
      M=85
      REL=0.50D-14
      CALL DHEXINT(ARG, FNH, KODE, M, REL, ENH, NZ, IERR)
 55   CONTINUE
      XA=MIN(A,B)
      XB=MAX(A,B)
      IF (C.LE.XB) THEN      
        CALL SSERI3(XA,XB,C,T,SS,ENH,IERR)
        CALL INTEGI5(C,XB,T,ANSI5,IERR)
        ARG=XA*T
        KODE=2
        ERFA=DRERF(ARG,KODE,NZ)
        FJ3=ERFA*ANSI5-SS
      ELSE
        CALL SSERI3(XB,C,XA,T,SS,ENH,IERR)
        CALL INTEGI5(XA,C,T,ANSI5,IERR)
        ARG=XB*T
        KODE=2
        ERFB=DRERF(ARG,KODE,NZ)
        FJ3A=ERFB*ANSI5-SS
        CALL SSERI3(XA,C,XB,T,SS,ENH,IERR)
        CALL INTEGI5(XB,C,T,ANSI5,IERR)
        ARG=XA*T
        KODE=2
        ERFA=DRERF(ARG,KODE,NZ)
        FJ3B=ERFA*ANSI5-SS
        ARG=C*T
        KODE=2
        ERFC=DRERF(ARG,KODE,NZ)
        FJ3=0.5D0*RTPI*ERFA*ERFB*ERFC/C - FJ3A*(XA/C) - FJ3B*(XB/C)
      ENDIF
      RETURN
      END
      DOUBLE PRECISION FUNCTION PHIZ(Z)
C
C     PHIZ EVALUATES THE PHI FUNCTION = 1-EXP(-Z) TO GET SIGNIFICANT
C     DIGITS FOR SMALL Z. REF: FOLDER 5
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      HZ=0.5D0*Z
C     EXP(-40) IS APPROX. 10**(-17.2) AND PHIZ=1.0
      IF(HZ.GT.20.0D0) THEN
        PHIZ=1.0D0
      ELSE
        S=DEXP(-HZ)*DSINH(HZ)
        PHIZ=S+S
      ENDIF
      RETURN
      END
      SUBROUTINE PKSER(A,B,C,T,SUM,IERR)
C
C     EVALUATES THE PK SERIES FOR S(A,B,C,T)
C
C     IERR=0 NORMAL RETURN
C     IERR=1 UNDERFLOW, SUM=0.0D0 RETURNED 
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DATA RTPI /1.772453850905516D0/
      IERR=0
      REL=0.50D-16
      A2=A*A
      B2=B*B
      C2=C*C
      D2=C2+B2
      D=DSQRT(D2)
      X2=D2+A2
      X=DSQRT(X2)
      ARG=T*X
      KKODE=2
      NZ=0
      ERFX=DRERF(ARG,KKODE,NZ)
      IF(NZ.NE.0) THEN
        IERR=1
        SUM=0.0D0
        RETURN
      ENDIF
      XPD=X+D
      S=1.0D0
      R=C2/D2
      CK=1.0D0
      AK=0.5D0
      BK=2.0D0
      Y=A/XPD
      Y=Y*Y
      DO 10 K=1,60
        PK=FHYPER(K,Y)
        CK=CK*AK*R/BK
        TRM=CK*PK
        S=S+TRM
        IF(DABS(CK).LE.REL*DABS(S)) GOTO 20
        AK=AK+1.0D0
        BK=BK+1.0D0
  10  CONTINUE
  20  CONTINUE
      S=S*RTPI/XPD
      SUM=S*ERFX
      RETURN
      END
      DOUBLE PRECISION FUNCTION FHYPER(K,Y)
C
C     FHYPER(K,Y) EVALUATES THE HYPERGEOMETRIC POLYNOMIAL F(-K,1,K+2;Y)
C     FOR THE S SUM IN J3 IN FOLDER 7
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      S=1.0D0
      AK=DBLE(FLOAT(K))
      BK=AK+2.0D0
      AK=-AK
      CK=1.0D0
      REL=0.50D-15
      DO 10 J=1,K
        CK=CK*AK*Y/BK
        S=S+CK
        IF (DABS(CK).LE.REL*DABS(S)) GOTO 20
        AK=AK+1.0D0
        BK=BK+1.0D0
  10  CONTINUE
  20  CONTINUE
      FHYPER=S
      RETURN
      END
      SUBROUTINE I3PWRSER(A,B,C,T,SUM,IERR)
C
C     THIS ROUTINE COMPUTES THE POWER SERIES OF THE INT ON (0,T) OF
C     EXP(-C*C*T*T)*ERF(A*T)*ERF(B*T) (REF: FOLDER 7b)
C
C     IERR=1 MEANS NO CONVERGENCE IN POWER SERIES AFTER 100 TERMS
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION CTRM(100),ATRM(100),BTRM(100),CONVAB(100)
      DATA PI /3.14159265358979D0/
      IERR=0
      REL=0.50D-15
      IF(T.EQ.0.0D0) THEN
        SUM=0.0D0
        RETURN
      ENDIF
      FK=1.0D0
      CT=C*T
      CT2=CT*CT
      CTRM(1)=1.0D0
      AT=A*T
      AT2=AT*AT
      BT=B*T
      BT2=BT*BT
      AK=3.0D0
      SK=5.0D0
      AC=1.0D0
      ATRM(1)=1.0D0
      BC=1.0D0
      BTRM(1)=1.0D0
      CONVAB(1)=1.0D0
      TSUM=1.0D0/3.0D0
      DO 10 K=2,100
        AC=-AC*AT2/FK
        ATRM(K)=AC/AK
        BC=-BC*BT2/FK
        BTRM(K)=BC/AK
        CTRM(K)=-CTRM(K-1)*CT2/FK
C     THE POWERS OF T^2 ARE DISTRIBUTED AMONG THE CONVOLUTIONS TO SCALE
C     A^2, B^2 AND C^2 SINCE A, B OR C COULD BE LARGE IF T IS SMALL
C     CONVOLUTE ERF(AT) AND ERF(BT) COEFFICIENTS FOR PRODUCT COEFFIENTS
C     CONVAB(*)
        S=0.0D0
        DO 15 J=1,K
          S=S+ATRM(J)*BTRM(K-J+1)
  15    CONTINUE
        CONVAB(K)=S
C     CONVOLUTE EXP COEFFICIENTS WITH PRODUCT OF ERF COEFFICIENTS FOR
C     THE COEFFIENTS OF THE PRODUCT OF THE ERF FUNCTIONS AND THE EXP
C     FUNCTION
        SS=0.0D0
        DO 20 J=1,K
          SS=SS+CTRM(J)*CONVAB(K-J+1)
  20    CONTINUE
C     SK IS THE FACTOR PRODUCED BY INTEGRATION OF THE FINAL SERIES IN
C     T^2.  SK=2*K+3=POWER OF T IN THE FINAL
C     SERIES. T^3 AND THE COEFFICIENTS OF THE ERF SERIES ARE FACTORED
C     OUT AND APPLIED TO THE FINAL SUM.
        SS=SS/SK
        TSUM=TSUM+SS
        IF (DABS(SS).LT.TSUM*REL) GOTO 55
        AK=AK+2.0D0
        SK=SK+2.0D0
        FK=FK+1.0D0
  10  CONTINUE
      IERR=1
C      WRITE (7,303)
C  303 FORMAT('NO CONVERGENCE IN I3PWRSER.FOR IN 100 TERMS')
  55  CONTINUE
C     SUM=((4/PI)*A*B*T**3)*TSUM. TSUM=POWER SERIES IN T**2 
      TSUM=TSUM+TSUM
      TSUM=TSUM+TSUM
      SUM=TSUM*AT*BT*T/PI
      RETURN
      END
      SUBROUTINE I3LARGEC(A,B,C,T,SUM,IERR)
C
C     THIS ROUTINE COMPUTES THE LARGE C SERIES OF I3 OF FOLDER 7d.
C
C     IERR=1 MEANS NO CONVERGENCE IN 100 TERMS
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION ATRM(100),BTRM(100),BCOEF(100)
      DATA RTPI, PI /1.772453850905516D0, 3.14159265358979D0/
      REL=0.50D-15
      IERR=0
C     ZERO OUT THE BC ARRAY FOR THE BINOMIAL COEFFIENTS, SET BC(1)=1.0D0 
      BCOEF(1)=1.0D0
      DO 25 J=2,100
        BCOEF(J)=0.0D0
  25  CONTINUE
      FK=1.0D0
      CT=C*T
      CT2=CT*CT
      IF (A.GT.B) THEN
        AT=A/C
        BT=B/C
      ELSE
        AT=B/C
        BT=A/C
      ENDIF
      AT2=AT*AT
      BT2=BT*BT
      AK=3.0D0
      AC=1.0D0
      ATRM(1)=1.0D0
C     FHK=GAM(I+0.5), I=1,2
      AHK=0.5D0
C     FHK=GAM(3/2)
      FHK=AHK*RTPI
      BC=1.0D0
      BTRM(1)=1.0D0
C     INCGAM(1/2,CT2)/GAM(1/2)=ERFC(CT)
      KODE=2
      GHK=DRERF(CT,KODE,NZ)
      IF (NZ.NE.0) THEN
        SUM=0.0D0
        IERR=2
        RETURN
      ENDIF
      EX=DEXP(-CT2)
      GHKTRM=CT*EX/FHK
C     INCGAM(3/2,CT2)/GAM(3/2) FROM INCGAM(1/2,CT2)/GAM(1/2)
      GHK=GHK+GHKTRM
C     FIRST TERM OF SERIES
      TRM=FHK*GHK
      TSUM=TRM
C     UPDATE INCGAM TERMS FOR INDEX=3/2+I (I=1 FOR 2ND TERM OF SERIES)
      AHK=AHK+1.0D0
C     FHK=GAM(3/2+I)/I!, I=INDEX OF SERIES = K-1
      FHK=FHK*AHK/FK
      GHKTRM=GHKTRM*CT2/AHK
      GHK=GHK+GHKTRM
      DO 10 K=2,100
C     GENERATE THE POWERS DIVIDED BY THE FACTOR (2*I+1), I=1,... FOR
C     ERF(AW) AND ERF(BW)
        AC=-AC*AT2
        ATRM(K)=AC/AK
        BC=-BC*BT2
        BTRM(K)=BC/AK
C     GENERATE BINOMIAL COEFFICIENTS FOR PASCAL TRIANGLE FOR EACH K
C     THEY ARE GENERATED FROM RIGHT TO LEFT SO THAT THE NEW VALUES
C     STORED IN THE BCOEF VECTOR WILL NOT OVERWRITE THE OLD VALUES NEEDED
C     FOR THE REMAINING INDICES. 
        DO 35 M=2,K
          J=K-M+2
          BCOEF(J)=BCOEF(J)+BCOEF(J-1)
  35    CONTINUE
C     THE POWERS OF C^2 ARE DISTRIBUTED AMONG THE CONVOLUTIONS TO SCALE
C     A^2, B^2. THAT IS, AT2=(A/C)**2, BT2=(B/C)**2
C     ERF(AW) AND ERF(BW) CONVOLUTION CONTAIN BINOMIAL COEFFICIENTS
C     WHICH ARE GENERATED BY THE PASCAL TRIANGLE. ATERM AND BTERM ARRAYS
C     CONTAIN THE POWERS DIVIDED BY THE FACTOR (2*I+1), I=K-1
        S=0.0D0
        DO 15 J=1,K
          S=S+ATRM(J)*BCOEF(J)*BTRM(K-J+1)
  15    CONTINUE
        TRM1=S*FHK*GHK
        IF (DABS(TRM1).GT.DABS(TRM)) THEN
C          WRITE (7,305) K,TRM,TRM1
C          SS=(AT2+BT2)**(K-1)
C          WRITE (7,306) SS,S,FHK,GHK
C  306     FORMAT(5X,4D14.6)
C  305     FORMAT(I5,2D14.6)
C  307     FORMAT('ABORT IN LARGEC FOR ',4D14.6)
          IF(K.GT.10) THEN
             SUM=0.0D0
             IERR=3
C             WRITE(7,307)A,B,C,T
             RETURN
          ENDIF
        ENDIF
        TRM=TRM1
        TSUM=TSUM+TRM
        IF (DABS(TRM).LE.DABS(TSUM)*REL) GOTO 55
        AK=AK+2.0D0
        FK=FK+1.0D0
        AHK=AHK+1.0D0
        FHK=FHK*AHK/FK
        GHKTRM=GHKTRM*CT2/AHK
        GHK=GHK+GHKTRM
  10  CONTINUE
C      WRITE (7,303) 
C  303 FORMAT('NO CONVERGENCE IN SUBROUTINE I3LARGEC IN 100 TERMS')
      IERR=1
  55  CONTINUE
C      WRITE (7,304) K
C  304 FORMAT(I5)
C     SUM=((2.0/PI)*A*B/C**3)*TSUM.  
      TSUM=(TSUM+TSUM)/PI
      SUM=TSUM*(A/C)*(B/C)/C
      RETURN
      END
      SUBROUTINE SSERI3(A,B,C,T,SUM,ENH,IERR)
C
C     SSERI3 EVALUATES THE SERIES FOR S(A,B,C,T) WHERE
C     A,C <=B IN FOLDER 7 FOR I3 AND J3=I3C
C
C     IERR=0 NORMAL RETURN
C     IERR=1 UNDERFLOW, SUM=0.0D0 RETURNED 
C     IERR=2 NO CONVERGENCE IN THE SSERI3 SERIES IN 85 TERMS
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION ENH(85),BC(86),PAM(86),PCM(86)
      DATA PI /3.14159265358979D0/
      IERR=0
      REL=0.50D-15
      CALL PKSER(A,B,C,T,PKS,IERR)
      IF (IERR.EQ.1) THEN
        SUM=0.0D0
        RETURN
      ENDIF
      A2=A*A
      B2=B*B
      C2=C*C
      D2=C2+B2
      D=DSQRT(D2)
      IF(T.EQ.0.0D0) THEN
        SUM=(PKS/PI)*(A/D)
        RETURN
      ENDIF
      X2=D2+A2
      X=DSQRT(X2)      
      S=1.0D0      
C     ZERO OUT THE BC ARRAY FOR THE BINOMIAL COEFFIENTS, SET BC(1)=1.0D0 
      BC(1)=1.0D0
      DO 25 J=2,81
        BC(J)=0.0D0
  25  CONTINUE        
      SS=ENH(1)
      PA=A2/X2
      PC=C2/X2
      PAM(1)=1.0D0
      PCM(1)=1.0D0
      CK=1.0D0
      AK=1.5D0
      BK=2.0D0
C     M=NUMBER OF THE TERM IN THE FORMULA, M=MINDEX+1
      DO 30 M=2,85
C     GENERATE BINOMIAL COEFFICIENTS FOR PASCAL TRIANGLE FOR EACH M
C     THEY ARE GENERATED FROM RIGHT TO LEFT SO THAT THE NEW VALUES
C     STORED IN THE BC VECTOR WILL NOT OVERWRITE THE OLD VALUES NEEDED
C     FOR THE REMAINING INDICES. 
        DO 35 K=2,M
          J=M-K+2
          BC(J)=BC(J)+BC(J-1)
  35    CONTINUE
        SM=0.0D0
        DK=1.0D0
        PAM(M)=PAM(M-1)*PA
        PCM(M)=PCM(M-1)*PC
        MP1=M+1
        DO 40 K=1,M
          SM=SM+BC(K)*PAM(MP1-K)*PCM(K)/DK
          DK=DK+2.0D0
  40    CONTINUE
        CK=CK*AK/BK
        TRM=CK*SM*ENH(M)
        SS=SS+TRM
        IF (DABS(TRM).LE.REL*DABS(SS)) GOTO 55
        AK=AK+1.0D0
        BK=BK+1.0D0
  30  CONTINUE
      IERR=2
C      WRITE(7,45)
C  45  FORMAT('NO CONVERGENCE IN SSERI3 AFTER 85 TERMS')
  55  CONTINUE
      SS=0.5D0*SS*T*(D/X)
  60  CONTINUE
      S=PKS-SS
      SUM=(S/PI)*(A/D)
      RETURN
      END
      SUBROUTINE INTEGI5(A,B,X,ANSI5,IERR)
C
C     DONALD E. AMOS   DECEMBER, 2000
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     INTEGI5 COMPUTES THE I INTEGRAL OF FOLDER 5,
C
C            ANSI5=INT ON (X,INF) OF EXP(-A*A*W*W)*ERFC(B*W),
C
C     WHERE X.GE.0 AND PARAMETERS A AND B CANNOT BE NEGATIVE OR BOTH
C     ZERO AT THE SAME TIME. THIS INTEGRAL IS DENOTED BY I5 IN OTHER
C     FOLDERS.
C
C     ERROR CONDITIONS:
C
C     IERR=0  NORMAL RETURN, COMPUTATION COMPLETED
C     IERR=1  X IS NEGATIVE, ANSI5=0.0D0 RETURNED
C     IERR=2  BOTH A=0 AND B=0 AND I5 DOES NOT EXIST. ANSI5=0.0D0
C             RETURNED
C     IERR=3  EITHER A OR B OR BOTH ARE NEGATIVE. ANSI5=0.0D0
C             RETURNED
C     IERR=4  UNDERFLOW, ANSI5=0.0D0 RETURNED
C
C             CONVERGENCE OR COMPUTATIONAL PROBLEMS (NOT LIKELY)
C
C     IERR=5  G SERIES NOT CONVERGENT IN 55 TERMS. REDUCED ACCURACY
C             PROBABLE.
C     IERR=6  S SERIES NOT CONVERGENT IN 55 TERMS. REDUCED ACCURACY
C             PROBABLE.
C
C     CONDITIONS IERR = 5 OR 6 SHOULD ONLY OCCUR IF THE MACHINE FAILS,
C     SO THERE IS GENERALLY NO REASON TO TEST FOR THEM.
C 
C     CALLS ROUTINES: GSERI5, SSERI5, DHEXINT, DRERF, DIERFC
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DATA RTPI       /1.772453850905516D0/
      IERR=0
      IF(X.LT.0.0D0) THEN
        ANSI5=0.0D0
        IERR=1
        RETURN
      ENDIF
      IF((A.EQ.0.0D0).AND.(B.EQ.0.0D0)) THEN
C       I5 IS NOT DEFINED FOR BOTH A=0 AND B=0
        ANSI5=0.0D0
        IERR=2
        RETURN
      ENDIF
      IF((A.LT.0.0D0).OR.(B.LT.0.0D0)) THEN
C       I5 IS NOT DEFINED FOR NEGATIVE PARAMETERS A AND B
        ANSI5=0.0D0
        IERR=3
        RETURN
      ENDIF
      IF (A.EQ.0.0D0) THEN
C       FOR A=0, ANSI5=IERFC(B*X)/B
        ARG=B*X
        KODE=1
        CALL DIERFC(ARG,KODE,ANS,KERR)
        IF(KERR.EQ.1) THEN
          ANSI5=0.0D0
          IERR=4
          RETURN
        ENDIF
        ANSI5=ANS/B
      ENDIF
      IF (B.EQ.0.0D0) THEN
C       FOR B=0, ANSI5=RTPI*ERFC(A*X)/(2*A)
        ARG=A*X
        KODE=2
        NZ=0
        ERFA=DRERF(ARG,KODE,NZ)
        IF(NZ.NE.0) THEN
          ANSI5=0.0D0
          IERR=4
          RETURN
        ENDIF
        ANSI5=0.5D0*RTPI*ERFA/A
        RETURN
      ENDIF
      IF (A.LE.B) THEN
        CALL GSERI5(A,B,X,ANSI5,JERR)
        IF(JERR.EQ.1) THEN
          IERR=4
          ANSI5=0.0D0
          RETURN
        ENDIF
        IF(JERR.EQ.2) THEN
          IERR=5
        ENDIF
      ELSE
        ARTPI=A*RTPI
        D2=A*A+B*B
        D=DSQRT(D2)
        X2=X*X
        D2X2=D2*X2
        IF (D2X2.LE.3.0D0) THEN
          CALL SSERI5(B,A,X,ANS,JERR)
          IF(JERR.EQ.1) THEN
            IERR=4
            ANSI5=0.0D0
            RETURN
          ENDIF
          IF(JERR.EQ.2) THEN
            IERR=6
          ENDIF
          ARG=A*X
          KODE=1
          ERFA=DRERF(ARG,KODE,NZ)
          ARG=B*X
          ERFB=DRERF(ARG,KODE,NZ)
          ARG=0.5D0*D2X2
          S=(DATAN(A/B)+2.0D0*DEXP(-ARG)*SINH(ARG)*DATAN(B/A))/ARTPI
          S=S-0.5D0*RTPI*(ERFA+ERFB-ERFA*ERFB)/A
          ANSI5=S+(0.5D0*B*D*X2/ARTPI)*ANS
        ELSE
          CALL GSERI5(B,A,X,ANS,JERR)
          IF(JERR.EQ.1) THEN
            IERR=4
            ANSI5=0.0D0
            RETURN
          ENDIF
          IF(JERR.EQ.2) THEN
            IERR=5            
          ENDIF
          ARG1=A*X
          ARG2=B*X
          XX=ARG1*ARG1+ARG2*ARG2
          IF (XX.GT.670.0D0) THEN
            IERR=4
            ANSI5=0.0D0
            RETURN
          ENDIF
          KODE=2
          ERFCA=DRERF(ARG1,KODE,NZ)
          ERFCB=DRERF(ARG2,KODE,NZ)
          ANSI5=0.5D0*RTPI*ERFCA*ERFCB/A-(B/A)*ANS
        ENDIF
      ENDIF
      RETURN
      END
      SUBROUTINE INTEGJ5(A,B,X,ANSJ5,IERR)
C
C     DONALD E. AMOS    SEPTEMBER, 2001
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     INTEGJ5 COMPUTES THE J FUNCTION OF FOLDER5:
C
C           ANSJ5 = INT ON (X,INF) OF EXP(-A^2)*ERF(B*X)
C
C     WHERE A.GT.0.0D0, B.GE.0.0D0, X.GE.0.0D0
C                 
C     IERR =0  IS A NORMAL RETURN.
C     IERR =1  UNDERFLOW IN GSERI5 OR SSERI5
C     IERR =2  ONE OR MORE OF THE PARAMETERS A, B, OR X IS NEGATIVE
C              OR A IS ZERO.
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DATA RTPI       /1.772453850905516D0/
      IERR=0
      IF ((A.LE.0.0D0).OR.(B.LT.0.0D0).OR.(X.LT.0.0D0)) THEN
        IERR=2
        ANSJ5=0.0D0
        RETURN
      ENDIF
      IF(B.EQ.0.0D0) THEN
        ANSJ5=0.0D0
        RETURN
      ENDIF
      AX=A*X
      BX=B*X
      ARTPI=A*RTPI
      IF ((AX.LE.1.0D0).AND.(BX.LE.1.0D0)) THEN
        CALL V5PSER(A,B,X,SS)
        ANSJ5=DATAN(B/A)/ARTPI-SS
        RETURN
      ENDIF
      IF (A.LE.B) THEN
        D2=A*A+B*B
        D=DSQRT(D2)
        X2=X*X
        D2X2=D2*X2
        IF (D2X2.LE.3.0D0) THEN
          KODE=1
          ARG=A*X
          ERFA=DRERF(ARG,KODE,NZ)
          CALL SSERI5(A,B,X,SS,KERR)
          ARG=0.5D0*D2X2
          S=2.0D0*DEXP(-ARG)*DSINH(ARG)*DATAN(A/B)/ARTPI
         ANSJ5=S+DATAN(B/A)/ARTPI-0.5D0*RTPI*ERFA/A+(0.5D0*X2*D/RTPI)*SS
        ELSE
          CALL GSERI5(A,B,X,SS,KERR)
          KODE=2
          ARG=A*X
          ERFCA=DRERF(ARG,KODE,NZ)
          TRM=0.5D0*RTPI*ERFCA/A
          IF(KERR.EQ.1) THEN
            IF (ARG*ARG.LT.630.0D0) THEN
              ANSJ5=TRM
            ELSE
              IERR=1
              ANSJ5=0.0D0
            ENDIF
          ELSE
            ANSJ5=TRM-SS
          ENDIF
        ENDIF
      ELSE
        CALL GSERI5(B,A,X,SS,KERR)
        NZ=0
        ARG=B*X
        KODE=1
        ERFB=DRERF(ARG,KODE,NZ)
        NZ=0
        ARG=A*X
        KODE=2
        ERFCA=DRERF(ARG,KODE,NZ)
        TRM=0.5D0*RTPI*ERFCA*ERFB/A
        IF(KERR.EQ.1) THEN
          IF(NZ.EQ.0) THEN
            TM=DLOG(TRM)
            IF(DABS(TM).LT.630.0D0) THEN
              ANSJ5=TRM
            ELSE
              IERR=1
              ANSJ5=0.0D0
            ENDIF
          ELSE
            IERR=1
            ANSJ5=0.0D0
          ENDIF
        ELSE
          ANSJ5=TRM+(B/A)*SS
        ENDIF
      ENDIF
      RETURN
      END
      SUBROUTINE INTEGV5(A,B,X,ANSV5,IERR)
C
C     DONALD E. AMOS, JANUARY, 2003
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     INTEGV5 COMPUTES THE V5 INTEGRAL OF FOLDER 5 
C
C         ANSV5=INT ON (0,X) OF EXP(-(A*W)**2)*ERF(B*W)
C
C     WHERE A.GE.0.0D0, B.GE.0.0D0, X.GE.0.0D0.
C
C     IERR=0 IS A  NORMAL RETURN
C     IERR=1 MEANS ONE OR MORE OF THE PARAMETERS A, B, OR X IS NEGATIVE
C            ANSV5=0.0D0 RETURNED
C     IERR=2 IS A NON-CONVERGENCE ERROR FROM GSERI5 OR SSERI5
C
C     UNDERFLOWS DO NOT CONTRIBUTE TO THE ANSWER AND ARE NOT FLAGGED
C-----------------------------------------------------------------------     
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DATA RTPI       /1.772453850905516D0/
      IERR=0
      IF((A.LT.0.0D0).OR.(B.LT.0.0D0).OR.(X.LT.0.0D0)) THEN
        IERR=1
        ANSV5=0.0D0
        RETURN
      ENDIF
      IF(X.EQ.0.0D0) THEN
         ANSV5=0.0D0
         RETURN
      ENDIF
      IF(B.EQ.0.0D0) THEN
        ANSV5=0.0D0
        RETURN
      ENDIF
      IF(A.EQ.0.0D0) THEN
        BX=B*X
        IF(BX.GT.6.0D0) THEN
          ANSV5=(BX-1.0D0/RTPI)/B
          RETURN
        ENDIF
        KKODE=1
        ERFBX=DRERF(BX,KKODE,NZ)
        ARG=0.5D0*BX*BX
        ANSV5=X*ERFBX-2.0D0*DEXP(-ARG)*DSINH(ARG)/(B*RTPI)
        RETURN
      ENDIF
      AX=A*X
      BX=B*X
      IF ((AX.LE.1.0D0).AND.(BX.LE.1.0D0)) THEN
        CALL V5PSER(A,B,X,ANSV5)
        RETURN
      ENDIF
      ARTPI=A*RTPI
      IF (A.LE.B) THEN
        D2=A*A+B*B
        D=DSQRT(D2)
        X2=X*X
        D2X2=D2*X2
        IF (D2X2.LE.3.0D0) THEN
          KKODE=1
          ARG=A*X
          ERFA=DRERF(ARG,KKODE,NZ)
          CALL SSERI5(A,B,X,SS,IERR)
          IF(IERR.EQ.1) IERR=0
          IF(IERR.EQ.2) THEN
            ANSV5=0.0D0
            RETURN
          ENDIF
          ARG=0.5D0*D2X2
          S=0.5D0*RTPI*ERFA/A-2.0D0*DEXP(-ARG)*DSINH(ARG)*DATAN(A/B)/
     &ARTPI
          ANSV5=S-(0.5D0*X2*D/RTPI)*SS
          IPATH=1
        ELSE
          CALL GSERI5(A,B,X,SS,IERR)
          IF(IERR.EQ.1) IERR=0
          IF(IERR.EQ.2) THEN
            ANSV5=0.0D0
            RETURN
          ENDIF
          KKODE=2
          ARG=A*X
          ERFCA=DRERF(ARG,KKODE,NZ)
          ANSV5=-0.5D0*RTPI*ERFCA/A+DATAN(B/A)/ARTPI+SS
          IPATH=2
        ENDIF
      ELSE
        D2=A*A+B*B
        D=DSQRT(D2)
        X2=X*X
        D2X2=D2*X2
        IF (D2X2.LE.3.0D0) THEN
          CALL SSERI5(B,A,X,SS,IERR)
          IF(IERR.EQ.1) IERR=0
          IF(IERR.EQ.2) THEN
            ANSV5=0.0D0
            RETURN
          ENDIF
          ARG=0.5D0*D2X2         
          S=2.0D0*DEXP(-ARG)*DSINH(ARG)*DATAN(B/A)/ARTPI
          ARG=A*X
          KKODE=2
          ERFCA=DRERF(ARG,KKODE,NZ)
          ARG=B*X
          KKODE=1
          ERFB=DRERF(ARG,KKODE,NZ)
          S=S-0.5D0*RTPI*(ERFCA*ERFB)/A
          ANSV5=S+(0.5D0*B*D*X2/ARTPI)*SS
          IPATH=3
        ELSE
          CALL GSERI5(B,A,X,SS,IERR)
          IF(IERR.EQ.1) IERR=0
          IF(IERR.EQ.2) THEN
            ANSV5=0.0D0
            RETURN
          ENDIF
          ARG=A*X
          KKODE=2
          ERFCA=DRERF(ARG,KKODE,NZ)
          ARG=B*X
          KKODE=1
          ERFB=DRERF(ARG,KKODE,NZ)
          S=-0.5D0*RTPI*ERFCA*ERFB/A-(B/A)*SS
          ANSV5=DATAN(B/A)/ARTPI+S
          IPATH=4
        ENDIF
      ENDIF
C      WRITE(7,307)IPATH
C 307  FORMAT('PATH= ',I2)
      RETURN
      END
      SUBROUTINE GSERI5(A,B,X,SUM,IERR)
C
C     GSERI5 COMPUTES THE G SUM OF FOLDER 5.
C
C     IERR=0 NORMAL RETURN
C     IERR=1 UNDERFLOW, SUM=0.0D0
C     IERR=2 NO CONVERGENCE IN 55 TERMS OF THE SERIES
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(55)
      DATA RTPI                  /1.772453850905516D0/
      IERR=0
      ARG=(A*A+B*B)
      DEN=ARG
      ARG=ARG*X*X
      FNH=1.5D0
      KODE=1
      M=55
      TOL=0.5D-15
      NZ=0
      KERR=0
      CALL DHEXINT(ARG, FNH, KODE, M, TOL, EN, NZ, KERR)
      IF(NZ.NE.0) THEN
        IERR=1
        SUM=0.0D0
        RETURN
      ENDIF
      AK=1.0D0
      S=EN(1)
      DH=0.5D0
      Z=A*A/DEN
      FAC=1.0D0
      DO 10 K=1,M-1
        AK=AK*DH/FAC
        AK=AK*Z
        TRM=AK*EN(K+1)
        S=S+TRM
        IF (DABS(TRM).LE.TOL*DABS(S)) GOTO 15        
        DH=DH+1.0D0
        FAC=FAC+1.0D0
  10  CONTINUE
       IERR=2
C      WRITE (7,300)
C 300  FORMAT('G-SERIES NOT CONVERGED IN 55 TERMS')
  15  CONTINUE
      SUM=(0.5D0/RTPI)*(S/DSQRT(DEN))
      RETURN
      END
      SUBROUTINE SSERI5(A,B,X,SUM,IERR)
C
C     SSERI5 COMPUTES THE S SUM OF FOLDER 5.
C
C     IERR=0 NORMAL RETURN
C     IERR=1 UNDERFLOW, SUM=0.0D0
C     IERR=2 NO CONVERGENCE IN 55 TERMS OF THE SERIES
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(55)
      IERR=0
      ARG=(A*A+B*B)
      DEN=ARG
      ARG=ARG*X*X
      FNH=0.5D0
      KODE=1
      M=55
      TOL=0.5D-15
      NZ=0
      KERR=0
      CALL DHEXINT(ARG, FNH, KODE, M, TOL, EN, NZ, KERR)
      IF(NZ.NE.0) THEN
        SUM=0.0D0
        IERR=1
        RETURN
      ENDIF
      AK=1.0D0
      S=2.0D0*EN(1)
      DH=0.5D0
      Z=A*A/DEN
      FAC=1.0D0
      DO 10 K=1,M-1
        AK=AK*DH/FAC
        AK=AK*Z
        TRM=AK*EN(K+1)/(DH+1.0D0)
        S=S+TRM
        IF (DABS(TRM).LE.TOL*DABS(S)) GOTO 15        
        DH=DH+1.0D0
        FAC=FAC+1.0D0
  10  CONTINUE
       IERR=2
C      WRITE (7,300)
C 300  FORMAT('S-SERIES NOT CONVERGED IN 55 TERMS')
  15  CONTINUE
      SUM=S
      RETURN
      END
      SUBROUTINE INTEGI6(A,B,T,KODE,ANSI6,IERR)
C
C     DONALD E. AMOS    DECEMBER,2001, FEBRUARY, 2006
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     ON KODE=1, INTEGI6 COMPUTES THE INTEGRAL
C
C         ANSI6=INT ON (T,INF) OF ERF(A*W)*ERF(B*W)/W**2
C
C     ON KODE=2, INTEGI6 COMPUTES THE INTEGRAL
C
C         ANSI6=INT ON (T,INF) OF ERFC(A*W)*ERFC(B*W)/W**2
C   
C     BY SERIES FROM THE FORMULAE IN FOLDERS 3, 6, AND 15 FOR
C     A, B, AND T NON-NEGATIVE. T=0 IS AN ERROR ON KODE=2.
C
C     IERR=0 IS A NORMAL RETURN
C     IERR=1 IS AN INPUT ERROR, SOME PARAMETER IS NEGATIVE
C     IERR=2 IS AN UNDERFLOW, ANSI6=0.0D0 RETURNED
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DATA RTPI/ 1.772453850905516D0/
      IERR=0
      IF((A.LT.0.0D0) .OR. (B.LT.0.0D0) .OR. (T.LT.0.0D0)) THEN
        ANSI6=0.0D0
        IERR=1
        RETURN
      ENDIF
      AT=A*T
      BT=B*T
      REL=0.50D-14 
      IF(KODE.EQ.1) THEN
        IF ((A.EQ.0.0D0).OR.(B.EQ.0.0D0)) THEN
          ANSI6=0.0D0
          RETURN
        ENDIF
        IF (T.EQ.0.0D0) THEN
          QAB=A/B
          QBA=B/A
          T2=QAB+DSQRT(1.0D0+QAB*QAB)
          T1=QBA+DSQRT(1.0D0+QBA*QBA)
          ANSI6=2.0D0*(A*DLOG(T1)+B*DLOG(T2))/RTPI
          RETURN
        ENDIF
        ICHECK=0
        IF(ICHECK.EQ.0) THEN
C-----------------------------------------------------------------------
C       CASES WHERE ONE OR BOTH OF THE ERF FUNCTIONS HAVE AGUMENTS 6 OR
C       GREATER WHERE THE ERF(*)=1. IN THESE CASES FUNCTION QPI6
C       EVALUATES THE INTEGRALS IN CLOSED FORM. SEE FOLDER 3 FOR THE
C       DERIVATION.
C-----------------------------------------------------------------------
          XAB=MIN(A,B)
          WSM=MIN(6.0D0/A,6.0D0/B)
          WLM=MAX(6.0D0/A,6.0D0/B)
          IF(T.GT.WSM) THEN
            IF (T.LE.WLM)  THEN
              SS=QPI6(XAB,T,WLM,REL)
            ELSE
              SS=1.0D0/T
            ENDIF
            ANSI6=SS
            RETURN
          ENDIF
        ENDIF
        COEF=2.0D0
      ELSE
        IF(T.EQ.0.0D0) THEN
          IERR=1
          ANSI6=0.0D0
          RETURN
        ENDIF
        X=AT*AT+BT*BT
        IF(X.GT.670.0D0) THEN
          IERR=2
          ANSI6=0.0D0
          RETURN
        ENDIF
        COEF=-2.0D0
      ENDIF
      NZ=0
      ERFAT=DRERF(AT,KODE,NZ)
      ERFBT=DRERF(BT,KODE,NZ)
      S=ERFAT*ERFBT/T
      CALL INTEGP(A,B,T,KODE,REL,PANSAB,IERRP)
      CALL INTEGP(B,A,T,KODE,REL,PANSBA,IERRP)
      ANSI6=S+COEF*(A*PANSAB+B*PANSBA)/RTPI
      RETURN
      END
      DOUBLE PRECISION FUNCTION QPI6(XAB,FL,FU,REL)
C-----------------------------------------------------------------------
C     HERE FU=WLM AND THE TERM 1/WLM FROM THE FORMULA IS ADDED IN TO GET
C     THE ERFC(T2)/WLM FUNCTION IN THE SECOND TERM = TRM2 OF THE FORMULA
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(5),FN(5)
      DATA RTPI/1.77245385090552D0/
      T1=XAB*FL
      ARG1=T1*T1
      T2=XAB*FU
      ARG2=T2*T2
      KODE=1
      NZ=0
      TRM1=DRERF(T1,KODE,NZ)/FL
      NZ=0
      KODE=2
      TRM2=DRERF(T2,KODE,NZ)/FU
      S=TRM1+TRM2
      N=1
      KODE=1
      M=1
      TOL=REL
      NZ=0
      IERR=0
      CALL DEXINT(ARG1, N, KODE, M, TOL, EN, NZ, IERR)
      CALL DEXINT(ARG2, N, KODE, M, TOL, FN, NZ, IERR)
      C=XAB/RTPI
      QPI6=S+C*(EN(1)-FN(1))
      RETURN
      END
      SUBROUTINE V5PSER(A,B,X,SUM)
C     POWER SERIES FOR J5
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION COEXP(21),COERF(21),PAX(21),PBX(21)
      DATA RTPI            /1.772453850905516D0/
      DATA (COEXP(J),J=1,21)/
     C0.100000000000000D+01,0.100000000000000D+01,0.500000000000000D+00,
     C0.166666666666667D+00,0.416666666666667D-01,0.833333333333333D-02,
     C0.138888888888889D-02,0.198412698412698D-03,0.248015873015873D-04,
     C0.275573192239859D-05,0.275573192239859D-06,0.250521083854417D-07,
     C0.208767569878681D-08,0.160590438368216D-09,0.114707455977297D-10,
     C0.764716373181982D-12,0.477947733238739D-13,0.281145725434552D-14,
     C0.156192069685862D-15,0.822063524662433D-17,0.411031762331216D-18/
      DATA (COERF(J),J=1,21)/
     C0.100000000000000D+01,0.333333333333333D+00,0.100000000000000D+00,
     C0.238095238095238D-01,0.462962962962963D-02,0.757575757575758D-03,
     C0.106837606837607D-03,0.132275132275132D-04,0.145891690009337D-05,
     C0.145038522231505D-06,0.131225329638028D-07,0.108922210371486D-08,
     C0.835070279514724D-10,0.594779401363764D-11,0.395542951645853D-12,
     C0.246682701026446D-13,0.144832646435981D-14,0.803273501241577D-16,
     C0.422140728880709D-17,0.210785519144214D-18,0.100251649349077D-19/
      TOL=0.5D-17
      AX=A*X
      BX=B*X
      BX2=BX*BX
      AX2=AX*AX
      PAX(1)=1.0D0
      PBX(1)=1.0D0      
      DO 5 J=1,20
        PAX(J+1)=-PAX(J)*AX2
        PBX(J+1)=-PBX(J)*BX2
        IF ((DABS(PAX(J+1)).LT.TOL).AND.(DABS(PBX(J+1)).LT.TOL)) THEN
          KL=J+1
          GOTO 6
        ENDIF
 5    CONTINUE
      KL=21
 6    CONTINUE
      CC=BX*X/RTPI
      S=1.0D0
      DKP1=2.0D0
      KP1=3
      DO 10 K=2,KL
        CK=0.0D0        
        DO 15 M=1,K
          CK=CK+COERF(M)*PBX(M)*COEXP(KP1-M)*PAX(KP1-M)
 15     CONTINUE
        S=S+CK/DKP1
        DKP1=DKP1+1.0D0
        KP1=KP1+1
 10   CONTINUE
      SUM=CC*S
      RETURN
      END
      SUBROUTINE INTEGP(A,B,T,KODE,REL,PANS,IERR)
C
C     DONALD E. AMOS    DECEMBER, 2001;  JANUARY, 2006
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     REFERENCES: FOLDER 11, FOLDER 6
C
C     ON KODE=1, INTEGP COMPUTES THE INTEGRAL
C
C             PANS =  INT ON (T,INF) OF EXP(-(A*W)^2)*ERF(B*W)/W
C
C     ON KODE=2, INTEGP COMPUTES THE COMPLEMENTARY INTEGRAL
C
C             PANS =  INT ON (T,INF) OF EXP(-(A*W)^2)*ERFC(B*W)/W
C
C     BY SERIES FOR A.GT.0, B.GE.0, T.GT.0. A CAN BE ZERO IF KODE=2,
C     AND T CAN BE ZERO IF KODE=1. REL=0.5D-14 FOR HIGHEST POSSIBLE
C     ACCURACY.
C
C     IERR=0 IS A NORMAL RETURN
C     IERR=1 IS AN INPUT ERROR, SOME PARAMETER IS OUT OF ITS RANGE
C            PANS=0.0D0 RETURNED  
C     IERR=2 UNDERFLOW, PANS=0.0D0 RETURNED
C     IERR=3 REQUESTED ERROR NOT MET IN ASYMPTOTIC EXPANSION
C----------------------------------------------------------------------- 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(60),ENH(60)
      DATA RTPI/ 1.772453850905516D0/
      IERR=0
      IF((KODE.NE.1).AND.(KODE.NE.2)) THEN
        PANS=0.0D0
        IERR=1
        RETURN
      ENDIF
      IF((A.LT.0.0D0).OR.(B.LT.0.0D0).OR.(T.LT.0.0D0)) THEN
        IERR=1
        PANS=0.0D0
        RETURN
      ENDIF
      IF((KODE.EQ.1).AND.(A.EQ.0.0D0))THEN
        IERR=1
        PANS=0.0D0
        RETURN
      ENDIF
      IF((KODE.EQ.1).AND.(B.EQ.0.0D0)) THEN
        PANS=0.0D0
        RETURN
      ENDIF    
      IF((KODE.EQ.2).AND.(A.EQ.0.0D0)) THEN
        IF(B.EQ.0.0D0) THEN
          IERR=1
          PANS=0.0D0
          RETURN
        ELSE
          X=B*T
          KKODE=1
          TOL=0.5D-14
          PANS=DGERFC(X,KKODE,TOL,IERR)
          RETURN
        ENDIF
      ENDIF
      A2=A*A
      B2=B*B
      D=DSQRT(A2+B2)
      IF(T.EQ.0.0D0) THEN
        IF(KODE.EQ.2) THEN
          PANS=0.0D0
          IERR=1
          RETURN
        ENDIF
        RX=B/A
        IF(RX.LE.0.1D0) THEN
C         SMALL RATIO B/A COMPUTATION FOR LN[(B+D)/A]
          X2=RX*RX
          ARG=RX+X2/(1.0D0+DSQRT(1.0D0+X2))
          TRMLN=DLNOPX(ARG)
        ELSE
          TRMLN=DLOG((B+D)/A)
        ENDIF
        PANS=TRMLN
        RETURN
      ENDIF
      AT=A*T
      ARGA=AT*AT
      IF(ARGA.GT.670.0D0) THEN 
        IERR=2
        PANS=0.0D0
        RETURN
      ENDIF
      N=1
      KKODE=1
      M=1
      TOL=REL
      CALL DEXINT(ARGA, N, KKODE, M, TOL, EN, NZ, MERR)
      IF(NZ.NE.0) THEN
        IERR=2
        PANS=0.0D0
        RETURN
      ENDIF
      EN1=EN(1)
      IF((KODE.EQ.2).AND.(B.EQ.0.0D0)) THEN
        PANS=0.5D0*EN1
        RETURN
      ENDIF
      BT=B*T
      ARGB=BT*BT
      XT=MAX(AT,BT)
      IF (XT.GT.6.0D0) THEN
        IF(XT.EQ.AT) THEN
          IF(KODE.EQ.1) THEN
            IF(BT.LT.6.0D0) GOTO 25
C           LARGE AT EXPANSION FOR P
C           ERF(BT)=1.0 +O[10**(-16)], FOR BT.GE.6.0, MAKING THE
C           RELATIVE ERROR O[10**(-16)]
            ERFBT=DRERF(BT,KODE,NZ)
            PANS=0.5D0*EN1*ERFBT
            RETURN
          ENDIF
          IF(XT.LT.7.0D0) GOTO 25
C         LARGE AT EXPANSION FOR PC: WANT XT AT LEAST 7.0 TO TERMINATE
C         ASYMPTOTIC EXPANSION
          ERFBT=DRERF(BT,KODE,NZ)
          IF(NZ.NE.0) THEN
            IERR=2
            PANS=0.0D0
            RETURN
          ENDIF
          COE=0.5D0
          AKSTRT=1.0D0
          SUM=EN1*ERFBT
          COES=-(B/A)/(XT*RTPI)
        ELSE
          IF(KODE.EQ.1)THEN
C           LARGE BT EXPANSION FOR P
C           ERF(BT)=1.0 +O[10**(-16)], FOR BT.GE.6.0, MAKING THE
C           RELATIVE ERROR O[10**(-16)]
            PANS=0.5D0*EN1
           RETURN          
          ENDIF
          IF(XT.LT.7.0D0) GOTO 25
C         LARGE BT EXPANSION FOR PC; WANT XT AT LEAST 7.0 TO TERMINATE
C         THE ASYMPTOTIC EXPANSION
          COE=0.5D0/(XT*RTPI)
          AKSTRT=0.5D0
          SUM=0.0D0
          COES=1.0D0
        ENDIF
        ARGAB=ARGA+ARGB
        IF(ARGAB.GT.670.0D0) THEN
          PANS=0.0D0
          IERR=2
          RETURN
        ENDIF
        NZ=0
        FNH=1.5D0
        KKODE=1
        MM=22
        TOL=REL
        CALL DHEXINT(ARGAB, FNH, KKODE, MM, TOL, EN, NZ, IERR)
        IF(NZ.NE.0) THEN
          IERR=2
          QANS=0.0D0
          RETURN
        ENDIF
        S=EN(1)
        TRM1=S
        DTRM1=DABS(TRM1)
        AK=AKSTRT
        X2=XT*XT
        CK=-AK/X2
        DO 20 K=1,MM-1
          TRM2=CK*EN(K+1)
          DTRM2=DABS(TRM2)
          IF (DTRM2.LE.REL*DABS(S)) GOTO 22
          IF (DTRM2.GE.DTRM1) GOTO 21
          S=S+TRM2
          DTRM1=DTRM2
          AK=AK+1.0D0
          CK=-CK*AK/X2
  20    CONTINUE
C        WRITE(7,301)
C  301   FORMAT(" DROP THROUGH THE LOOP IN THE ASYMPTOTIC SERIES" )
  21    CONTINUE
C        WRITE(7,302)
C  302   FORMAT('REQUESTED ERROR NOT MET IN ASYMPTOTIC SERIES')
        IERR=3
  22    CONTINUE
        PANS=COE*(SUM+COES*S)
        RETURN
      ENDIF
 25   CONTINUE
      ARGAB=ARGA+ARGB  
      RTX=T*D
      KKODE=2
      ERFCX=DRERF(RTX,KKODE,NZ)
      IF(NZ.NE.0) THEN
        IERR=2
        PANS=0.0D0
        RETURN
      ENDIF
      IF (A.LE.B) THEN
        RX=A/B
        X2=RX*RX
        IF(X2.LE.0.1D0) THEN
C         COMPUTE LN[2*D/(B+D)] FOR SMALL RATIOS (A/B)**2 TO RETAIN
C         SIGNIFICANT DIGITS
          TRM=0.5D0*DLNOPX(X2)
          ARG=0.5D0*X2/(1.0D0+DSQRT(1.0D0+X2))
          TRMLN=TRM-DLNOPX(ARG)
        ELSE
          ARGLN=(D+D)/(B+D)
          TRMLN=DLOG(ARGLN)
        ENDIF
        SUM=-TRMLN*ERFCX
        FNH=0.5D0
        KKODE=1
        MM=50
        TOL=REL
        CALL DHEXINT(ARGAB, FNH, KKODE, MM, TOL, ENH, NZ, MERR)
        IF(NZ.NE.0) THEN
          IERR=2
          PANS=0.0D0
          RETURN
        ENDIF
        KKODE=1
        TRM=DGERFC(RTX,KKODE,REL,MERR)
        SUM=SUM-TRM
        COE=0.5D0*RTX/RTPI
        CALL S1SER(A,B,ENH,MM,REL,S1,MERR)
        SUM=SUM+COE*S1
        IF(KODE.EQ.1) THEN
          PANS=0.5D0*EN1+SUM
        ELSE
          PANS=-SUM
        ENDIF
        RETURN
      ELSE
        RX=B/A
        IF(RX.LE.0.1D0) THEN
C         SMALL RATIO B/A COMPUTATION FOR LN[(B+D)/A]
          X2=RX*RX
          ARG=RX+X2/(1.0D0+DSQRT(1.0D0+X2))
          TRMLN=DLNOPX(ARG)
        ELSE
          TRMLN=DLOG((B+D)/A)
        ENDIF
        SUM=TRMLN*ERFCX
        N=1
        KKODE=1
        MM=50
        TOL=REL
        CALL DEXINT(ARGAB, N, KKODE, MM, TOL, EN, NZ, MERR)
        IF(NZ.NE.0) THEN
          IERR=2
          PANS=0.0D0
          RETURN
        ENDIF
        COE=B*T/RTPI
        CALL S2SER(B,A,EN,MM,REL,S2,IERR)
        SUM=SUM-COE*S2
        ERFBT=DRERF(BT,KODE,NZ)
        IF(KODE.EQ.1) THEN
          PANS=SUM+0.5D0*EN1*ERFBT
        ELSE
          PANS=-SUM+0.5D0*EN1*ERFBT 
        ENDIF
      ENDIF
      RETURN
      END
      SUBROUTINE INTEGQ(A,B,T,REL,QANS,IERR)
C
C     DONALD E. AMOS    DECEMBER, 2001;  JANUARY, 2006
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     REFERENCES: FOLDER 11, FOLDER 6
C
C     INTEGQ COMPUTES THE INTEGRAL
C
C       QANS =  INT ON (T,INF) OF EXP(-(A*W)**2)*E(1,(B*W)**2)
C
C     BY SERIES FOR A.GE.0, B.GT.0, T.GE.0. REL=0.5D-14 FOR HIGHEST
C     POSSIBLE ACCURACY.
C
C     IERR=0 IS A NORMAL RETURN
C     IERR=1 IS AN INPUT ERROR, SOME PARAMETER IS OUT OF ITS RANGE
C            QANS=0.0D0 RETURNED  
C     IERR=2 UNDERFLOW, QANS=0.0D0 RETURNED
C     IERR=3 REQUESTED ERROR NOT MET IN ASYMPTOTIC EXPANSION
C----------------------------------------------------------------------- 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(100),ENH(100)
      DATA RTPI/ 1.772453850905516D0/
      IERR=0
      IF((A.LT.0.0D0).OR.(B.LE.0.0D0).OR.(T.LT.0.0D0)) THEN
        IERR=1
        QANS=0.0D0
        RETURN
      ENDIF
      IF (A.EQ.0.0D0) THEN
        BT=B*T
        ARGB=BT*BT
        IF(ARGB.GT.670.0D0)THEN
          IERR=2
          QANS=0.0D0
          RETURN
        ENDIF
        KKODE=2
        ERFB=DRERF(BT,KKODE,NZ)
        IF(NZ.NE.0) THEN
          IERR=2
          QANS=0.0D0
          RETURN
        ENDIF
        TRM=ERFB*RTPI/B
        IF(T.EQ.0.0D0) THEN
          QANS=TRM
          RETURN
        ENDIF
        N=1
        KKODE=1
        M=1
        TOL=REL
        CALL DEXINT(ARGB, N, KKODE, M, TOL, EN, NZ, IERR)
        IF(NZ.NE.0) THEN
          IERR=2
          QANS=0.0D0
          RETURN
        ENDIF
        QANS=TRM-T*EN(1)
        RETURN
      ENDIF
      A2=A*A
      B2=B*B
      D=DSQRT(A2+B2)
      IF(T.EQ.0.0D0) THEN
        RX=A/B
        IF(RX.LE.0.1D0) THEN
C         SMALL RATIO A/B COMPUTATION FOR LN[(A+D)/B]
          X2=RX*RX
          ARG=RX+X2/(1.0D0+DSQRT(1.0D0+X2))
          TRMLN=DLNOPX(ARG)
        ELSE
          TRMLN=DLOG((A+D)/B)
        ENDIF
        QANS=RTPI*TRMLN/A
        RETURN
      ENDIF
      AT=A*T
      BT=B*T
      ARGA=AT*AT
      ARGB=BT*BT
      ARGAB=ARGA+ARGB
      IF(ARGAB.GT.670.0D0) THEN 
        IERR=2
        QANS=0.0D0
        RETURN
      ENDIF
      XT=MAX(AT,BT)
      IF (XT.GE.7.0D0) THEN
C-----------------------------------------------------------------------
C       LARGE AT EXPANSION:
C
C       Q(A,B,T)=RTPI*(P(B,A,T)-0.5*E(1,(BT)**2)*ERF(AT))/A USING THE
C       ASYMPTOTIC EXPANSION FOR P(B,A,T) FOR AT.GE.7.0D0
C
C       LARGE BT EXPANSION:
C       INTEGRATION OF Q USING THE ASYMPTOTIC EXPANSION OF E(1,(BT)**2)
C
C       BOTH HAVE SIMILAR SUMS WITH ONLY MINOR CHANGES IN START VALUES
C       AND COEFFICIENTS     
C-----------------------------------------------------------------------
        IF(XT.EQ.AT) THEN
C         LARGE AT EXPANSION
          COE=0.5D0*RTPI/A
          AKSTRT=0.5D0
          COES=-1.0D0/(AT*RTPI)          
          KKODE=2
          ERFAT=DRERF(AT,KKODE,NZ)
          IF(NZ.NE.0) THEN
            IERR=2
            QANS=0.0D0
            RETURN
          ENDIF
          NZ=0
          N=1
          KKODE=1
          MM=1
          TOL=REL
          CALL DEXINT(ARGB, N, KKODE, MM, TOL, EN, NZ, IERR)
          IF(NZ.NE.0) THEN
            IERR=2
            QANS=0.0D0
            RETURN
          ENDIF
          SUM=EN(1)*ERFAT
        ELSE
C         LARGE BT EXPANSION
          COE=0.5D0/(BT*B)
          AKSTRT=1.0D0
          SUM=0.0D0
          COES=1.0D0
        ENDIF      
        NZ=0
        FNH=1.5D0
        KKODE=1
        MM=22
        TOL=REL
        CALL DHEXINT(ARGAB, FNH, KKODE, MM, TOL, EN, NZ, IERR)
        IF(NZ.NE.0) THEN
          IERR=2
          QANS=0.0D0
          RETURN
        ENDIF
        S=EN(1)
        TRM1=S
        DTRM1=DABS(TRM1)
        AK=AKSTRT
        X2=XT*XT
        CK=-AK/X2
        DO 20 K=1,MM-1
          TRM2=CK*EN(K+1)
          DTRM2=DABS(TRM2)
          IF (DTRM2.LE.REL*DABS(S)) GOTO 22
          IF (DTRM2.GE.DTRM1) GOTO 21
          S=S+TRM2
          DTRM1=DTRM2
          AK=AK+1.0D0
          CK=-CK*AK/X2
  20    CONTINUE
C        WRITE(7,301)
C  301   FORMAT(" DROP THROUGH THE LOOP IN THE ASYMPTOTIC SERIES" )
  21    CONTINUE
C        WRITE(7,302)
C  302   FORMAT('REQUESTED ERROR NOT MET IN ASYMPTOTIC SERIES')
        IERR=3
  22    CONTINUE
        QANS=COE*(SUM+COES*S)
        RETURN
      ENDIF
C     START OF CONVERGENT SERIES FOR A.LE.B AND A.GT.B
      RTX=T*D
      KKODE=2
      ERFCX=DRERF(RTX,KKODE,NZ)
      IF(NZ.NE.0) THEN
        IERR=2
        QANS=0.0D0
        RETURN
      ENDIF
      IF (A.LE.B) THEN
        RX=A/B
        IF(RX.LE.0.1D0) THEN
C       SMALL RATIO A/B COMPUTATION FOR LN[(A+D)/B]
          X2=RX*RX
          ARG=RX+X2/(1.0D0+DSQRT(1.0D0+X2))
          TRMLN=DLNOPX(ARG)
        ELSE
          TRMLN=DLOG((A+D)/B)
        ENDIF
        SUM=TRMLN*ERFCX*RTPI/A
        N=1
        KKODE=1
        MM=50
        TOL=REL
        CALL DEXINT(ARGAB, N, KKODE, MM, TOL, EN, NZ, IERR)
        IF(NZ.NE.0) THEN
          IERR=2
          QANS=0.0D0
          RETURN
        ENDIF
        CALL S2SER(A,B,EN,MM,REL,S2,IERR)
        QANS=SUM-T*S2
      ELSE
        N=1
        KKODE=1
        M=1
        TOL=REL
        CALL DEXINT(ARGB, N, KKODE, M, TOL, EN, NZ, IERR)
        IF(NZ.NE.0) THEN
          IERR=2
          QANS=0.0D0
          RETURN
        ENDIF
        SUM=0.5D0*EN(1)
        KKODE=2
        ERFCT=DRERF(AT,KKODE,NZ)
        IF(NZ.NE.0) THEN
          IERR=2
          QANS=0.0D0
          RETURN
        ENDIF
        SUM=SUM*ERFCT
        RX=B/A
        X2=RX*RX
        IF(X2.LE.0.1D0) THEN
C         COMPUTE LN[2*D/(A+D)] FOR SMALL RATIOS (B/A)**2 TO RETAIN
C         SIGNIFICANT DIGITS 
          TRM=0.5D0*DLNOPX(X2)
          ARG=0.5D0*X2/(1.0D0+DSQRT(1.0D0+X2))
          TRMLN=TRM-DLNOPX(ARG)
        ELSE
          ARGLN=(D+D)/(A+D)
          TRMLN=DLOG(ARGLN)
        ENDIF
        SUM=SUM-TRMLN*ERFCX
        FNH=0.5D0
        KKODE=1
        MM=50
        TOL=REL
        CALL DHEXINT(ARGAB, FNH, KKODE, MM, TOL, ENH, NZ, IERR)
        IF(NZ.NE.0) THEN
          IERR=2
          QANS=0.0D0
          RETURN
        ENDIF
        KKODE=1
        TRM=DGERFC(RTX,KKODE,REL,IERR)
        SUM=(SUM-TRM)*RTPI
        COE=0.5D0*RTX
        CALL S1SER(B,A,ENH,MM,REL,S1,IERR)
        QANS=(SUM+COE*S1)/A
      ENDIF
      RETURN
      END
      SUBROUTINE S1SER(A,B,ENHX,MM,REL,S1,IERR)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION ENHX(MM)
      A2=A*A
      B2=B*B
      D2=A2+B2
      RAB=A2/D2     
      SNH=0.0D0
      AKH=0.5D0
      BKH=1.0D0
      CKH=0.5D0*RAB
      DO 10 K=1,MM-1
        TRM=CKH*ENHX(K+1)/BKH
        SNH=SNH+TRM      
        IF(DABS(TRM).LE.REL*DABS(SNH)) GOTO 11
        BKH=BKH+1.0D0
        AKH=AKH+1.0D0
        CKH=CKH*(AKH/BKH)*RAB
  10  CONTINUE
      IERR=5 
C      WRITE(7,200) 
C 200  FORMAT('NO CONVERGENCE IN S1SER IN MM TERMS')
  11  CONTINUE 
      S1=SNH
      RETURN
      END
      SUBROUTINE S2SER(A,B,ENX,MM,REL,S2,IERR)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION ENX(MM)
      A2=A*A
      B2=B*B
      D2=A2+B2
      RAB=A2/D2
      SN=ENX(1)
      AK=3.0D0
      CK=RAB
      DO 10 K=1,MM-1
        TRM=CK*ENX(K+1)/AK
        SN=SN+TRM        
        IF(DABS(TRM).LE.REL*DABS(SN)) GOTO 11
        CK=CK*RAB
        AK=AK+2.0D0
  10  CONTINUE
      IERR=5 
C      WRITE(7,200) 
C 200  FORMAT('NO CONVERGENCE IN S2SER IN MM TERMS')
  11  CONTINUE 
      S2=SN
      RETURN
      END
      DOUBLE PRECISION FUNCTION DLNOPX(X)
C
C     DLNOPX COMPUTES THE LN(1+X) FOR SMALL X.LE.0.1D0 TO RETAIN
C     SIGNIFICANT DIGITS WHEN X IS KNOWN AND SMALL 
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      REL=0.5D-16
      S=0.0D0
      AK=1.0D0
      PX=1.0D0
      DO 10 K=1,50
        TRM=PX/AK
        S=S+TRM
        IF (DABS(TRM).LT.REL*DABS(S)) GOTO 11
        AK=AK+1.0D0
        PX=-PX*X
 10   CONTINUE
 11   CONTINUE
      DLNOPX=X*S
      RETURN
      END
      SUBROUTINE INTEGW3(A,B,T,KODE,REL,ANSW3,IERR)
C
C     DONALD E. AMOS, JANUARY, 2003
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C           
C     INTEGW3 COMPUTES THE W3 INTEGRAL OF FOLDER 10 USING THE CLOSED
C     FORM DERIVED IN FOLDER 10A ON KODE=1:
C
C         ANSW3=INT ON (T,INF) OF ERF(A*W)*ERF(B*W)/(W**3)
C
C     ON KODE=2, THE COMPLEMENTARY FUNCTION W3C IS COMPUTED,
C
C         ANSW3=INT ON (T,INF) OF ERFC(A*W)*ERFC(B*W)/(W**3)
C
C     WHERE A.GE.0.0D0, B.GE.0.0D0, T.GT.0.0D0.
C
C     IERR=0 IS A  NORMAL RETURN
C     IERR=1 MEANS T.LE.0.0D0, ANSI1=0.0D0 RETURNED FROM INTEGI1
C     IERR=2 MEANS ONE OR BOTH OF THE PARAMETERS A OR B IS NEGATIVE
C            OR KODE IS NOT 1 OR 2 ANSW3=0.0D0 RETURND
C     IERR=4 IS AN UNDERFLOW AND ANSI1=0.0D0 IS RETURNED FROM INTEGI1
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DATA RTPI       /1.772453850905516D0/
      IF((KODE.NE.1).AND.(KODE.NE.2)) THEN
        IERR=2
        ANSW3=0.0D0
        RETURN
      ENDIF
      IF(KODE.EQ.1) THEN
        COEF=1.0D0
      ELSE
        COEF=-1.0D0
      ENDIF
      ARG=A*T
      ERFAT=DRERF(ARG,KODE,NZ)
      ARG=B*T
      ERFBT=DRERF(ARG,KODE,NZ)
      CALL INTEGI1(A,B,T,KODE,REL,ANSAB,IERR)
      CALL INTEGI1(B,A,T,KODE,REL,ANSBA,IERR)
      ANSW3=0.5D0*(ERFAT/T)*(ERFBT/T)+COEF*(A*ANSAB+B*ANSBA)/RTPI      
      RETURN
      END
      SUBROUTINE INTEGI21(A,B,C,T,KODE,ANSI21,I21ERR,KFORM)
C
C     DONALD E. AMOS, OCTOBER, 2002
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     REF: FOLDER 21
C
C     ON KODE=1, INTEGI21 COMPUTES
C
C        ANSI21=INT ON (0,T) OF U(A,B,W)*ERF(C/RW);
C
C     ON KODE=2, INTEGI21 COMPUTES THE COMPLEMENTARY FUNCTION
C
C        ANSI21=INT ON (0,T) OF U(A,B,W)*ERFC(C/RW)
C
C     WHERE U(A,B,W)=EXP(A*A*W+2*A*B)*ERFC(A*RW+B/RW), RW=SQRT(W)
C     AND  A , B AND C ARE NON-NEGATIVE.
C
C     ERROR FLAG:
C          I21ERR= 0 NORMAL RETURN
C                =-1 CONVERGENCE NOT SEEN IS SOME SERIES
C                = 4 INPUT VARIABLE OUT OF RANGE
C                = 5 EXPONENTIAL UNDERFLOW, (B^2+C^2)/T TOO LARGE
C                =   ANY OTHER VALUE IS AN ERROR FROM DGAUS8
C
C     LEGENDS FOR KFORM:
C
C     KFORM DIGIT=0 AND I21ERR=0 MEANS A SPECIAL FORMULA WAS USED
C     KFORM DIGIT=1 MEANS T.LE.(C*C/36) WHERE ERF(C/DSQRT(T))=1.0D0
C     KFORM DIGIT=2 MEANS THE CLOSED FORM IS USED FOR DVOFT
C     KFORM DIGIT=3 MEANS THE LARGE L.GE.2 SERIES IS USED FOR W3
C     KFORM DIGIT=4 MEANS THE A*SQRT(T).LT.1 SERIES IS USED FOR I21
C     KFORM DIGIT=5 MEANS THE THE QUADRATURE IS USED FOR W3
C
C     EXAMPLE:
C     KFORM=12 MEANS A COMBINATION OF 1 AND 2 WAS USED
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(1),YN(100)
      DATA RTPI /1.772453850905516D0/
      I21ERR=0
      KFORM=0
      IF(A.LT.0.0D0)THEN
         I21ERR=4
         ANSI21=0.0D0
         RETURN
      ENDIF
      IF(B.LT.0.0D0)THEN
         I21ERR=4
         ANSI21=0.0D0
         RETURN
      ENDIF
      IF(C.LT.0.0D0)THEN
         I21ERR=4
         ANSI21=0.0D0
         RETURN
      ENDIF
      IF(T.LT.0.0D0)THEN
         I21ERR=4
         ANSI21=0.0D0
         RETURN
      ENDIF
      IF((KODE.NE.1).AND.(KODE.NE.2))THEN
         I21ERR=4
         ANSI21=0.0D0
         RETURN
      ENDIF
      IF(T.EQ.0.0D0)THEN
         ANSI21=0.0D0
         RETURN
      ENDIF
      IF(KODE.EQ.1) THEN
        IF(C.EQ.0.0D0)THEN
           ANSI21=0.0D0
           RETURN
        ENDIF
      ENDIF
      REL=0.5D-14
      RT=DSQRT(T)
      CAPT=1.0D0/RT
      IF((A.EQ.0.0D0).AND.(B.EQ.0.0D0)) THEN
        X=C*CAPT
        NZ=0
        ERFC=DRERF(X,KODE,NZ)
        KKODE=1
        CALL DIERFC(X,KKODE,ANS,IERR)
        IF(KODE.EQ.1) THEN
          ANSI21=T*ERFC+(C+C)*RT*ANS
        ELSE
          IF((NZ.NE.0).OR.(IERR.EQ.2)) THEN
            ANSI21=0.0D0
            I21ERR=5
          ELSE
            ANSI21=T*ERFC-(C+C)*RT*ANS
          ENDIF
        ENDIF
        RETURN
      ENDIF
      CS=C*C
      BS=B*B
      CAPX=(BS+CS)/T
      IF(CAPX.GT.667.0D0) THEN
        ANSI21=0.0D0
        I21ERR=5
        RETURN
      ENDIF
      VT=DVOFT(A,B,T,REL,IVERR,IFORM)
      IF(KODE.EQ.1) THEN
        TLIM=CS/36.0D0        
        IF(T.LE.TLIM) THEN
C         ERF(C/RT)=1 + O(10**(-16)) HERE
          ANSI21=VT
          KFORM=10+IFORM
          RETURN
        ENDIF
      ELSE
        IF(C.EQ.0.0D0) THEN
          ANSI21=VT
          KFORM=IFORM
          RETURN
        ENDIF
      ENDIF
      IF(IFORM.EQ.4) THEN
C       SERIES FOR A*RT.LE.1 COMPUTED IN DVOFT WHEN IFORM=4
C       RESOLUTION OF INDETERMINANT FORM FOR SMALL A AND SERIES FOR
C       A*RT.LE.1
        KFORM=4        
        CRT=A*RT
        CT=CRT+CRT
        IF(A.EQ.0.0D0) THEN
          M=4
        ELSE
          M=40
        ENDIF
        TOL=REL
        CALL GNSEQ(C,B,CAPT,M,TOL,YN)
        S=0.0D0
        PW=1.0D0
        DO 20 K=3,M
          TRM=PW*YN(K)
          S=S+TRM
          IF(DABS(TRM).LE.DABS(S)*REL) GOTO 65
          PW=-PW*CT
 20     CONTINUE
        I21ERR=-1
C        PRINT*,'END OF G LOOP'
 65     CONTINUE
        TERM2=4.0D0*RT*C*S/RTPI
        ARG=C/RT
        ERFA=DRERF(ARG,KODE,NZ)
        IF(KODE.EQ.1) THEN
          ANSI21=VT*ERFA+TERM2*DEXP(-CAPX)
        ELSE
          ANSI21=VT*ERFA-TERM2*DEXP(-CAPX)
        ENDIF
        RETURN
      ENDIF     
      ARG=C/RT
      NZ=0
      ERFA=DRERF(ARG,KODE,NZ)
      TRM0=VT*ERFA
      COE=2.0D0/(A*RTPI)
      N=1
      KKODE=1
      M=1
      TOL=REL
      NZ=0
      IERR=0
      CALL DEXINT(CAPX, N, KKODE, M, TOL, EN, NZ, IERR)
      CW1=COE*EN(1)
      CALL INTEGS1(A,B,C,T,TOL,CW3,IERR,JFORM)
      AS=A*A
      COE=(RT+RT)/(AS*RTPI)
      CW3=CW3*COE
      KFORM=10*IFORM+JFORM
      IF (IERR.EQ.1) THEN 
        I21ERR=0
      ELSE
        I21ERR=IERR
      ENDIF
      CALL INTEGI5(C,B,CAPT,ANSI5,IERR)
      CW2=2.0D0*((B+B)/A+1.0D0/AS)*ANSI5
      IF(KODE.EQ.1) THEN
        ANSI21=TRM0+C*(CW1-CW2+CW3)/RTPI
      ELSE
        ANSI21=TRM0-C*(CW1-CW2+CW3)/RTPI
      ENDIF
      RETURN
      END
      SUBROUTINE INTEGJ21(A,B,C,T,ANSJ21,J21ERR,KFORM)
C
C     DONALD E. AMOS, OCTOBER, 2002
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     REF: FOLDER 21
C
C     INTEGJ21 COMPUTES
C
C     ANSJ21= INT ON (0,T) OF U(A,B,W)*DEXP(-C^2/W)/(W*RW)
C
C     WHERE U(A,B,W)=EXP(A*A*W+2*A*B)*ERFC(A*RW+B/RW), RW=SQRT(W)
C
C     AND C IS POSITIVE, A NON-NEGATIVE, B NON-NEGATIVE   OR
C         C IS ZERO,     A NON-NEGATIVE, B POSITIVE
C
C     ERROR FLAG:
C          J21ERR= 0 NORMAL RETURN
C                =-1 CONVERGENCE NOT SEEN IS SOME SERIES
C                = 4 INPUT VARIABLE OUT OF RANGE
C                = 5 EXPONENTIAL UNDERFLOW, (B^2+C^2)/T TOO LARGE
C                =   ANY OTHER VALUE IS AN ERROR FROM DGAUS8 OR INTEGI5
C
C     LEGENDS FOR KFORM:
C     KFORM DIGIT=0 AND J21ERR=0 MEANS A SPECIAL CASE FORMULA IS USED 
C     KFORM DIGIT=3 MEANS THE LARGE L.GE.2 SERIES IS USED FOR S1
C     KFORM DIGIT=4 MEANS THE SERIES FOR SMALL A*DSQRT(T).LE.1 WAS USED
C     KFORM DIGIT=5 MEANS THE THE QUADRATURE IS USED FOR S1
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION YN(60)
      DATA RTPI /1.772453850905516D0/
      J21ERR=0
      KFORM=0
      IF(A.LT.0.0D0)THEN
         J21ERR=4
         ANSJ21=0.0D0
         RETURN
      ENDIF
      IF(B.LT.0.0D0)THEN
         J21ERR=4
         ANSJ21=0.0D0
         RETURN
      ENDIF
      IF(C.LT.0.0D0)THEN
         J21ERR=4
         ANSJ21=0.0D0
         RETURN
      ENDIF
      IF(T.LT.0.0D0)THEN
         J21ERR=4
         ANSJ21=0.0D0
         RETURN
      ENDIF
C     SPECIAL FORMULAE FOR ONE OR MORE PARAMETERS EQUAL TO ZERO
C     IF THE INTEGRAL EXISTS, THEN THE CASE T=0 IS TESTED
      IF(C.EQ.0.0D0) THEN
        IF(B.EQ.0.0D0) THEN              
          J21ERR=4
          ANSJ21=0.0D0
          RETURN
        ELSE
          IF(A.EQ.0.0D0) THEN
            IF(T.EQ.0.0D0) THEN
              ANSJ21=0.0D0
              RETURN
            ENDIF
            X=B/DSQRT(T)
            KODE=1
            CALL DIERFC(X,KODE,ANS,IERR)
            IF(IERR.EQ.2) THEN
              J21ERR=5
              ANSJ21=0.0D0
              RETURN
            ENDIF
            ANSJ21=2.0D0*ANS/B
            RETURN
          ENDIF
        ENDIF
      ELSE
        IF((A.EQ.0.0D0).AND.(B.EQ.0.0D0)) THEN
          IF(T.EQ.0.0D0) THEN
            ANSJ21=0.0D0
            RETURN
          ENDIF
          ARG=C/DSQRT(T)
          KODE=2
          NZ=0
          ERFC=DRERF(ARG,KODE,NZ)
          IF(NZ.NE.0) THEN
             ANSJ21=0.0D0
             J21ERR=5
             RETURN
          ENDIF
          ANSJ21=RTPI*ERFC/C
          RETURN
        ENDIF
        IF(A.EQ.0.0D0) THEN
          IF(T.EQ.0.0D0) THEN
            ANSJ21=0.0D0
            RETURN
          ENDIF          
          CAPT=1.0D0/DSQRT(T)
          CALL INTEGI5(C,B,CAPT,ANSI5,IERR)
          IF(IERR.EQ.4) THEN
            ANSJ21=0.0D0
            J21ERR=5
            RETURN
          ENDIF
          ANSJ21=ANSI5+ANSI5
          RETURN
        ENDIF
      ENDIF
      IF(T.EQ.0.0D0)THEN
         ANSJ21=0.0D0
         RETURN
      ENDIF
      REL=0.5D-14
      RT=DSQRT(T)
      ART=A*RT
      IF(ART.LE.1.0D0) THEN
        KFORM=4
        CS=C*C
        BS=B*B
        CAPX=(CS+BS)/T
        IF(CAPX.GT.667.0D0) THEN
          ANSJ21=0.0D0
          J21ERR=5
          RETURN
        ENDIF
        AT=ART+ART
        CAPT=1.0D0/RT
        M=40
        TOL=REL
        CALL GNSEQ(C,B,CAPT,M,TOL,YN)
        S=YN(1)
        PW=-AT
        DO 10 K=1,M-1
          TRM=PW*YN(K+1)
          S=S+TRM
          IF(DABS(TRM).LT.DABS(S)*TOL) GOTO 20
          PW=-PW*AT
 10     CONTINUE
 20     CONTINUE
        ANSJ21=S*DEXP(-CAPX)/RT
        RETURN
      ENDIF
      BS=B*B
      CS=C*C
      CAPX=(CS+BS)/T
      IF(CAPX.GT.667.0D0) THEN
        ANSJ21=0.0D0
        J21ERR=5
        RETURN
      ENDIF
      TOL=REL
      CALL INTEGS1(A,B,C,T,TOL,CS1,IERR,KFORM)
      IF (IERR.EQ.1) THEN 
        J21ERR=0
      ELSE
        J21ERR=IERR
      ENDIF
      ANSJ21=2.0D0*RT*CS1/RTPI
      RETURN
      END
      SUBROUTINE INTEGI22(A,B,C,T,ANSI22,I22ERR,KFORM)
C
C     DONALD E. AMOS, OCTOBER, 2002
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C     REF: FOLDER 22
C
C     INTEGI22 COMPUTES
C
C     ANSI22=INT ON (0,T) OF U(A,B,W)*EXP(-C*C/W)/RW
C
C     WHERE U(A,B,W)=EXP(A*A*W+2*A*B)*ERFC(A*RW+B/RW), RW=SQRT(W)
C
C     AND  A , B  AND C ARE NON-NEGATIVE.
C
C     ERROR FLAG:
C          I22ERR= 0 NORMAL RETURN
C                =-1 CONVERGENCE NOT SEEN IS SOME SERIES
C                = 4 INPUT VARIABLE OUT OF RANGE
C                = 5 EXPONENTIAL UNDERFLOW, (B*B+C*C)/T TOO LARGE
C                =   ANY OTHER VALUE IS AN ERROR FROM DGAUS8
C
C     LEGENDS FOR KFORM:
C     KFORM DIGIT=0 MEANS A SPECIAL FORMULA WAS USED
C     KFORM DIGIT=1 MEANS A=0 
C     KFORM DIGIT=2 MEANS (A=0 AND B=0) OR (B=0 AND C=0) 
C     KFORM DIGIT=3 MEANS THE LARGE L.GE.2 SERIES IS USED FOR S2
C     KFORM DIGIT=4 MEANS THE SERIES FOR A*DSQRT(T).LE.1 IS USED
C     KFORM DIGIT=5 MEANS THE THE QUADRATURE IS USED FOR S2
C
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(1),YN(60)
      DATA RTPI /1.772453850905516D0/
      I22ERR=0
      KFORM=0
      IF(A.LT.0.0D0)THEN
         I22ERR=4
         ANSI22=0.0D0
         RETURN
      ENDIF
      IF(B.LT.0.0D0)THEN
         I22ERR=4
         ANSI22=0.0D0
         RETURN
      ENDIF
      IF(C.LT.0.0D0)THEN
         I22ERR=4
         ANSI22=0.0D0
         RETURN
      ENDIF
      IF(T.LT.0.0D0)THEN
         I22ERR=4
         ANSI22=0.0D0
         RETURN
      ENDIF
      IF(T.EQ.0.0D0)THEN
         ANSI22=0.0D0
         RETURN
      ENDIF
      RT=DSQRT(T)
      IF((A.EQ.0.0D0).AND.(B.EQ.0.0D0).AND.(C.EQ.0.0D0)) THEN
        ANSI22=RT+RT
        RETURN
      ENDIF
      IF((A.EQ.0.0D0).AND.(B.EQ.0.0D0)) THEN
        X=C/RT
        KODE=1
        CALL DIERFC(X,KODE,DI1ERFC,IERR)
        IF(IERR.EQ.2) THEN
          I22ERR=5
          ANSI22=0.0D0
          RETURN
        ENDIF
        ANSI22=2.0D0*RTPI*RT*DI1ERFC
        KFORM=2
        RETURN
      ENDIF
      REL=0.5D-14
      IF((B.EQ.0.0D0).AND.(C.EQ.0.0D0)) THEN
        KFORM=2
        X=A*RT
        ANSI22=2.0D0*DHERFC(X)/A
        RETURN
      ENDIF
      ART=A*RT
      IF(ART.LE.1.0D0) THEN
C       SMALL A*DSQRT(T).LE.1.0D0 SERIES FOR INTEGI22
        KFORM=4
        CAPT=1.0D0/RT
        ARG=(B*B+C*C)/T
        IF(ARG.GT.667.0D0) THEN
          ANSI22=0.0D0
          I22ERR=5
          RETURN
        ENDIF
        IF(A.EQ.0.0D0) THEN
          M=4
        ELSE
          M=50
        ENDIF
        REL=0.5D-14
        CALL GNSEQ(C,B,CAPT,M,REL,YN)
        S1=YN(2)
        S2=2.0D0*YN(3)
        FK=ART+ART
        PK=-FK
        AK=3.0D0
        IFLAG1=0
        IFLAG2=0
        JFLAG=0
        TOL=REL/10.0D0
        DO 10 K=1,M-3
          IF(IFLAG1.EQ.0) THEN
            TRM1=PK*YN(K+2)
            S1=S1+TRM1
            IF(DABS(TRM1).LT.TOL*DABS(S1)) THEN
              IFLAG1=1
              JFLAG=JFLAG+1
            ENDIF
          ENDIF
          IF(IFLAG2.EQ.0) THEN
            TRM2=PK*YN(K+3)*AK
            S2=S2+TRM2
            IF(DABS(TRM2).LT.TOL*DABS(S2)) THEN
              IFLAG2=1
              JFLAG=JFLAG+1
            ENDIF
          ENDIF
          IF(JFLAG.EQ.2) GOTO 20
          PK=-PK*FK
          AK=AK+1.0D0
 10     CONTINUE
        I22ERR=-1
 20     CONTINUE
        COEF=2.0D0*DEXP(-ARG)
        ANSI22=COEF*(B*S1+RT*S2)
        RETURN
      ENDIF
      BS=B*B
      CS=C*C
      ARG=(BS+CS)/T
      N=1
      KODE=1
      M=1
      NZ=0
      TOL=REL
      CALL DEXINT(ARG, N, KODE, M, TOL, EN, NZ, IERR)
      IF(NZ.NE.0) THEN
        ANSI22=0.0D0
        I22ERR=5
        KFORM=35
        RETURN
      ENDIF
      TOL=REL
      CALL INTEGS2(A,B,C,T,TOL,S2,IERR,KFORM)
      I22ERR=IERR
      ANSI22=(EN(1)-(RT+RT)*S2)/(A*RTPI)
      RETURN
      END
      SUBROUTINE INTEGJ22(A,B,C,T,ANSJ22,J22ERR,KFORM)
C
C     DONALD E. AMOS, OCTOBER, 2002
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C     REF: FOLDER 22
C
C     INTEGJ22 COMPUTES
C
C     ANSJ22=INT ON (0,T) OF U(A,B,W)*EXP(-C*C/W)*RW
C
C     WHERE U(A,B,W)=EXP(A*A*W+2*A*B)*ERFC(A*RW+B/RW), RW=SQRT(W)
C
C     AND  A , B  and C ARE NON-NEGATIVE.
C
C     ERROR FLAG:
C          J22ERR= 0 NORMAL RETURN
C                =-1 CONVERGENCE NOT SEEN IS SOME SERIES
C                = 4 INPUT VARIABLE OUT OF RANGE
C                = 5 EXPONENTIAL UNDERFLOW, (B^2+C^2)/T TOO LARGE
C                =   ANY OTHER VALUE IS AN ERROR FROM DGAUS8
C
C     LEGENDS FOR KFORM:
C
C     KFORM DIGIT=0 MEANS A SPECIAL FORMULA WAS USED
C     KFORM DIGIT=1 MEANS A=0
C     KFORM DIGIT=2 MEANS (A=0 & B=0) OR (A=0 & C=0) OR (B=0 & C=0)
C     KFORM DIGIT=3 MEANS THE LARGE L.GE.2 SERIES IS USED FOR S1 OR S2
C     KFORM DIGIT=4 MEANS THE SERIES FOR A*DSQRT(T).LE.1 IS USED
C     KFORM DIGIT=5 MEANS THE THE QUADRATURE IS USED FOR S1 OR S2
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(2),YN(100)
      DATA RTPI /1.772453850905516D0/
      J22ERR=0
      KFORM=0
      IF(A.LT.0.0D0)THEN
         J22ERR=4
         ANSJ22=0.0D0
         RETURN
      ENDIF
      IF(B.LT.0.0D0)THEN
         J22ERR=4
         ANSJ22=0.0D0
         RETURN
      ENDIF
      IF(C.LT.0.0D0)THEN
         J22ERR=4
         ANSJ22=0.0D0
         RETURN
      ENDIF
      IF(T.LT.0.0D0)THEN
         J22ERR=4
         ANSJ22=0.0D0
         RETURN
      ENDIF
      IF(T.EQ.0.0D0)THEN
         ANSJ22=0.0D0
         RETURN
      ENDIF
      RT=DSQRT(T)
      CAPT=1.0D0/RT
      IF((A.EQ.0.0D0).AND.(B.EQ.0.0D0).AND.(C.EQ.0.0D0)) THEN
        ANSJ22=2.0D0*RT*T/3.0D0
        RETURN
      ENDIF
      AS=A*A
      BS=B*B
      CS=C*C
      REL=0.5D-14
      IF((A.EQ.0.0D0).AND.(B.EQ.0.0D0)) THEN
        KFORM=2
        X=CS/T
        FNH=2.5D0
        KODE=1
        M=1
        TOL=REL
        NZ=0
        CALL DHEXINT(X, FNH, KODE, M, TOL, EN, NZ, IERR) 
        IF(NZ.NE.0) THEN
          J22ERR=5
          ANSJ22=0.0D0
          RETURN
        ENDIF
        ANSJ22=T*RT*EN(1)
        RETURN
      ENDIF
      IF((A.EQ.0.0D0).AND.(C.EQ.0.0D0)) THEN
        KFORM=2
        X2=B*CAPT
        KODE=2
        NZ=0
        ERFX=DRERF(X2,KODE,NZ)
        ARG=BS/T
        N=2
        KODE=1
        M=1
        NZ=0
        TOL=REL
        CALL DEXINT(ARG, N, KODE, M, TOL, EN, NZ, IERR)
        IF(NZ.NE.0) THEN
          ANSJ22=0.0D0
          J22ERR=5
          RETURN
        ENDIF
        ANSJ22=(T+T)*(RT*ERFX-B*EN(1)/RTPI)/3.0D0
        RETURN
      ENDIF
      IF((B.EQ.0.0D0).AND.(C.EQ.0.0D0)) THEN
C       CASE FOR J23 OF FOLDER 23
        KFORM=2
        X=A*RT
        TOL=REL
        IF (X.LE.1.0D0) THEN
C         PE=RECURRENCE ON EVEN TERMS; PO=RECURRENCE ON ODD TERMS
          PO=2.0D0*X/RTPI
          S=1.0D0/3.0D0-PO/4.0D0
          IF(DABS(PO).LE.TOL*DABS(S)) THEN
            ANSJ22=(S+S)*T*RT
            RETURN
          ENDIF
          PE=1.0D0
          AK=2.0D0
          BK=1.0D0
          XS=X*X
          DO 15 K=2,41,2
            PE=PE*XS/BK
            S=S+PE/(AK+3.0D0)
            AK=AK+1.0D0
            PO=2.0D0*XS*PO/AK
            TRMO=-PO/(AK+3.0D0)
            S=S+TRMO
            IF(DABS(TRMO).LE.TOL*DABS(S)) GOTO 20
            AK=AK+1.0D0
            BK=BK+1.0D0
  15      CONTINUE
  20      CONTINUE
          ANSJ22=(S+S)*T*RT
          RETURN
        ELSE
          KODE=3
          NZ=0
          ERFA=DRERF(X,KODE,NZ)
          S=X*ERFA+X*X/RTPI-DHERFC(X)
          ANSJ22=S/(A*AS)
          RETURN
        ENDIF        
      ENDIF
      ART=A*RT
      IF(ART.LE.1.0D0) THEN
C       SMALL A*DSQRT(T).LE.1.0D0 SERIES FOR INTEGJ22
        KFORM=4
        ARG=(BS+CS)/T
        IF(ARG.GT.667.0D0) THEN
          ANSJ22=0.0D0
          J22ERR=5
          RETURN
        ENDIF
        IF(A.EQ.0.0D0) THEN
          M=6
        ELSE
          M=60
        ENDIF
        TOL=REL
        CALL GNSEQ(C,B,CAPT,M,TOL,YN)
        S1=YN(3)
        S2=5.0D0*YN(4)
        S3=8.0D0*YN(5)
        FK=ART+ART
        PK=-FK
        AK=7.0D0
        BK=15.0D0
        CK=7.0D0
        IFLAG1=0
        IFLAG2=0
        IFLAG3=0
        JFLAG=0
        TOL=REL/10.0D0
        DO 10 K=1,M-5
          IF(IFLAG1.EQ.0) THEN
            TRM1=PK*YN(K+3)
            S1=S1+TRM1
            IF(DABS(TRM1).LT.TOL*DABS(S1)) THEN
              IFLAG1=1
              JFLAG=JFLAG+1
            ENDIF
          ENDIF
          IF(IFLAG2.EQ.0) THEN
            TRM2=PK*YN(K+4)*AK
            S2=S2+TRM2
            IF(DABS(TRM2).LT.TOL*DABS(S2)) THEN
              IFLAG2=1
              JFLAG=JFLAG+1
            ENDIF
          ENDIF
          IF(IFLAG3.EQ.0) THEN
            TRM3=PK*YN(K+5)*BK
            S3=S3+TRM3
            IF(DABS(TRM3).LT.TOL*DABS(S3)) THEN
              IFLAG3=1
              JFLAG=JFLAG+1
            ENDIF
          ENDIF
          IF(JFLAG.EQ.3) GOTO 30
          PK=-PK*FK
          AK=AK+2.0D0
          CK=CK+2.0D0
C         BK=(K+2)*(K+4), B(K+1)=B(K)+2K+7=B(K)+CK
          BK=BK+CK
 10     CONTINUE
        J22ERR=-1
 30     CONTINUE
        COEF=4.0D0*DEXP(-ARG)
        ANSJ22=COEF*(BS*S1+B*RT*S2+T*S3)*RT
        RETURN
      ENDIF
      ARG=(BS+CS)/T
      N=1
      KODE=1
      M=2
      TOL=REL
      NZ=0
      CALL DEXINT(ARG, N, KODE, M, TOL, EN, NZ, IERR)
      IF(NZ.NE.0) THEN
        J22ERR=5
        ANSJ22=0.0D0
        KFORM=35
        RETURN
      ENDIF
      CAPB=A*RT+B/RT
      KODE=3
      NZ=0
      ERFB=DRERF(CAPB,KODE,NZ)
      COE=(B+0.5D0/A)/A
      S=(T*EN(2)-COE*EN(1))/(A*RTPI)+(RT/AS)*ERFB*DEXP(-ARG)
      CALL INTEGS1(A,B,C,T,TOL,S1,IERR,KFORM1)
      CALL INTEGS2(A,B,C,T,TOL,S2,IERR,KFORM2)
      KFORM=10*KFORM1+KFORM2
      ANSJ22=S+(-2.0D0*CS*RT*S1+RT*S2/A)/(AS*RTPI)
      RETURN
      END
      SUBROUTINE INTEGI29(A,B,T,N0,NN,YN,IERR)
C
C     DONALD E. AMOS   AUGUST, 2006
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C     REF: FOLDER 29, FOLDER 28 
C
C     INTEGI29 GENERATES NN MEMBER SEQUENCES FOR THE INTEGRALS  
C
C     INT ON (T,INF) OF EXP(-A^2*W^2)*I(N0+2*K-2)ERFC(B*W), K=1,NN
C
C     FOR ORDERS N0 TO N0+2*NN-2 IN INCREMENTS OF 2 WHICH ARE STORED IN
C     YN(*) IN LOCATIONS 1 THRU NN. IF N0 IS EVEN, WE GET A SEQUENCE OF
C     EVEN ORDERS FOR YN(K), K=1,NN. IF N0 IS ODD,  WE GET A SEQUENCE OF
C     ODD  ORDERS FOR YN(K), K=1,NN WITH N0.GE.0, NN.GE.1, A.GE.0.0D0,
C     B.GE.0.0D0, T.GE.0.0D0 BUT A AND B NOT BOTH ZERO AT THE SAME TIME.
C     I(M)ERFC(X) IS THE ITERATED COERROR FUNCTION OF ORDER M.
C 
C     ERROR CONDITIONS
C      IERR=0 NORMAL RETURN, COMPUTATION COMPLETED
C      IERR=1 INPUT PARAMETER OUT OF RANGE, YN(K)=0.0D0, K=1,NN RETURNED
C      IERR=2 UNDERFLOW DURING COMPUTATION, COMPUTATION NOT COMPLETED,
C             YN(K)=0.0D0, K=1,NN  RETURNED
C      IERR=3 A LEGITIMATE UNDERFLOW. YN(K)=0.0D0, K=1,NN IS THE LOGICAL
C             RETURN FOR AN ANSWER WHICH IS OFF SCALE
C
C     INTEGI29 CALLS DINERFC   
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION YA(300),YB(300),YN(NN)
      DATA RTPI /1.772453850905516D0/
      DATA REL  /0.5D-14/
      IERR=0
      IF ((A.LT.0.0D0).OR.(B.LT.0.0D0).OR.(T.LT.0.0D0)) THEN
        IERR=1
        GOTO 10
      ENDIF
      IF((A.EQ.0.0D0).AND.(B.EQ.0.D0)) THEN
        IERR=1
        GOTO 10
      ENDIF
      IF((N0.LT.0).OR.(NN.LT.1)) THEN
        IERR=1
        GOTO 10
      ENDIF
 10   CONTINUE
      IF(IERR.NE.0) THEN
        DO 15 K=1,NN
          YN(K)=0.0D0
 15     CONTINUE
        RETURN
      ENDIF
      IF(A.LE.B) THEN
        AA=A
        BB=B
        ICASE=1
      ELSE
        AA=B
        BB=A
        ICASE=2
      ENDIF
      A2=AA*AA
      B2=BB*BB
      D2=A2+B2
      T2=T*T
      ELIM=T2*D2
      IF(ELIM.GT.680.0D0) THEN
        IERR=3
        GOTO 10
      ENDIF
      EX=DEXP(-A2*T2)
C     GENERATE LAST MEMBER OF THE SEQUENCE
      NL=N0+2*NN-2
      NP1=NL+1
      IF(ICASE.EQ.1) THEN
        NI=NP1
        ND=0
      ELSE
        NI=0
        ND=NP1
      ENDIF
      X=BB*T
      KODE=1
      KLAST=75
      KN=2*KLAST+1+ND
      TOL=REL
      CALL DINERFC(X,NI,KODE,KN,TOL,YB,NZ)
      NGOOD=KN-NZ
      IF(ICASE.EQ.1) THEN
        SS=YB(1)
        MM=3
      ELSE
        SS=YB(NL+2)
        MM=NL+4
      ENDIF
      AN=0.5D0*DBLE(FLOAT(NL))+1.0D0
      FACN=AN
      C=4.0D0*A2/D2
      P=C
      DO 20 K=2,KLAST
        IF(YB(MM).EQ.0.0D0) THEN
          IERR=2
          GOTO 10
        ENDIF
        TRM=P*FACN*YB(MM)
        SS=SS+TRM
        IF(DABS(TRM).LT.DABS(SS)*REL) GOTO 21
        P=P*C
        AN=AN+1.0D0
        FACN=FACN*AN
        MM=MM+2
 20   CONTINUE
 21   CONTINUE
      COE=BB/D2
      TANS=SS*COE*EX
      IF(ICASE.EQ.1) THEN
        YN(NN)=TANS
      ELSE
        X=AA*T
        NI=0
        KODE=1
        KN=NP1
        TOL=REL
        CALL DINERFC(X,NI,KODE,KN,TOL,YA,NZ)
        IF(NZ.NE.0) THEN
          IERR=2
          GOTO 10
        ENDIF
        IF(NGOOD.LT.NP1) THEN
          IERR=2
          GOTO 10
        ENDIF
        SS=0.0D0
        RAB=AA/BB
        P=1.0D0
        IK=NP1
        DO 30 K=1,NP1
          SS=SS+P*YB(K)*YA(IK)
          P=-P*RAB
          IK=IK-1
  30    CONTINUE
        YN(NN)=0.5D0*RTPI*SS/BB+P*TANS
      ENDIF
      IF(NN.EQ.1) RETURN
C     RECUR BACKWARD FROM INDEX NL=N0+2*NN-2 TO INDEX N0 WITH THE
C     FORMULA Y(N-2)=RAB*(2*N*Y(N)+CC*I(N)ERFC(BT)), RAB=A^2/(A^2+B^2),
C     CC=B*EXP(-A^2*T^2)/A^2, WITH Y VALUES STORED CONSECUTIVELY IN YN.
      IF(ICASE.EQ.1) THEN
        X=B*T
        NI=N0
        KODE=1
        KN=2*NN-2
C       FIRST INDEX IS N0 AND THE LAST IS N0+2*NN-3=NL-1 IN LOCATIONS
C       1 THRU KN
        TOL=REL
        CALL DINERFC(X,NI,KODE,KN,TOL,YB,NZ)
        IF(NZ.NE.0) THEN
          IERR=2
          GOTO 10
        ENDIF
        LNB=KN
      ELSE
        DO 45 K=1,NL
          YB(K)=YA(K)
 45     CONTINUE
        EX=DEXP(-A*A*T2)
        LNB=NL
      ENDIF
      DN=DBLE(FLOAT(NL))
      TDN=DN+DN
      LN=NN
      CC=B*EX/(A*A)
      RAB=(A*A)/D2
      KLAST=NN-1
      DO 40 K=1,KLAST
        YN(LN-1)=RAB*(TDN*YN(LN)+CC*YB(LNB))
        TDN=TDN-4.0D0
        LN=LN-1
        LNB=LNB-2
 40   CONTINUE
      RETURN
      END
      DOUBLE PRECISION FUNCTION DVOFT(A,B,T,REL,IERR,KFORM)
C-----------------------------------------------------------------------
C     REF: FOLDER 21
C  
C     DVOFT COMPUTES THE INTEGRAL 
C
C     DVOFT=INT ON (0,T) OF EXP(A*A*W+2*A*B)*ERFC(A*RW+B/RW), RW=SQRT(W)
C
C     WHERE A AND B ARE NON-NEGATIVE AND REL IS THE PRECISION DESIRED.
C     IERR IS AN ERROR FLAG:
C
C     IERR= 0 IS A NORMAL RETURN
C         = 4 AN INPUT VARIABLE IS OUT OF RANGE. DVOFT=0.0D0 RETURNED
C         = 5 AN EXPONENTIAL UNDERFLOW OCCURRED. DVOFT=0.0D0 RETURNED
C         =-1 CONVERGENCE CRITERION NOT MET IN THE SERIES
C           
C     KFORM INDICATES WHICH FORMULA OF FOLDER 21 WAS USED:
C
C     KFORM =0  AND IERR=0 MEANS A SPECIAL CASE: T=0 OR (A=0 AND B=0)
C           =2  MEANS THE CLOSED FORM WAS USED
C           =4  MEANS THE SERIES FOR SMALL A*SQRT(T).LE.1.0D0 WAS USED
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION SY(60)
      DATA RTPI /1.772453850905516D0/
      IERR=0
      KFORM=0
      IF(A.LT.0.0D0) THEN
        IERR=4
        DVOFT=0.0D0
        RETURN
      ENDIF
      IF(B.LT.0.0D0) THEN
        IERR=4
        DVOFT=0.0D0
        RETURN
      ENDIF
      IF(T.LT.0.0D0) THEN
        IERR=4
        DVOFT=0.0D0
        RETURN
      ENDIF
      IF((A.EQ.0.0D0).AND.(B.EQ.0.0D0)) THEN
        DVOFT=T
        RETURN
      ENDIF
      IF(T.EQ.0.0D0) THEN
        DVOFT=0.0D0
        RETURN
      ENDIF
      RT=DSQRT(T)
      CRT=A*RT
      IF (CRT.LE.1.00D0) THEN
C       SERIES FOR SMALL A*RT.LE.1.0D0
        KFORM=4
        CT=CRT+CRT
        ARG=B/RT
        N0=2
        KODE=2
        IF(A.EQ.0.0D0) THEN
          N=2
        ELSE
          N=40
        ENDIF
        CALL DINERFC(ARG,N0,KODE,N,REL,SY,NZ)
        S=0.0D0
        PW=1.0D0
        DO 10 K=1,N
          TRM=PW*SY(K)
          S=S+TRM
          IF(DABS(TRM).LE.DABS(S)*REL) GOTO 55
          PW=-PW*CT
 10     CONTINUE
        IERR=-1
C       PRINT *, 'END OF INERFC LOOP'
 55     CONTINUE
        ARG=B*B/T
        IF(ARG.GT.670.0D0) THEN
          IERR=5
          DVOFT=0.0D0
          RETURN
        ENDIF
        DVOFT=4.0D0*T*S*DEXP(-ARG)
        RETURN
      ENDIF
      KFORM=2
      BS=B*B
      AS=A*A
      ARG=A*RT+B/RT
      KODE=3
      NZ=0
      ERFX=DRERF(ARG,KODE,NZ)
      ARGB=BS/T
      IF(ARGB.GT.670.0D0) THEN
        IERR=5
        DVOFT=0.0D0
        RETURN
      ENDIF      
      EX=DEXP(-ARGB)
      ARG=B/RT
      KODE=3
      NZ=0
      ERFB=DRERF(ARG,KODE,NZ)
      COE=2.0D0*RT/A
      KODE=2
      CALL DIERFC(ARG,KODE,ANS,IERR)
      DVOFT=(COE*ANS+(ERFX-ERFB)/AS)*EX
      RETURN
      END
      SUBROUTINE INTEGS1(A,B,C,T,TOL,S1,IERR,KFORM)
C
C     DONALD E. AMOS, OCTOBER, 2002
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C     INTEGS1 COMPUTES THE INTEGRAL
C
C     S1=INT ON (CAPB,INF) OF EXP(A*A*T+2.0D0*A*B-C*C/T)*EXP(-W*W)/
C                             (C*C+T*(W-A*RT)**2)
C
C     WHERE  CAPB=A*RT+B/RT, RT=DSQRT(T)
C            A IS NON-NEGATIVE, B IS NON-NEGATIVE AND C IS POSITIVE   OR 
C            C IS ZERO WITH B POSITIVE AND A NON-NEGATIVE
C
C            TOL IS THE ERROR TOLERANCE FOR THE COMPUTATION
C
C            KFORM=0 A SPECIAL FORMULA IS EVALUATED FOR A SPECIAL CASE
C                 =3 FORMULA FOR CAPL.GE.2.0D0 IS EVALUATED
C                 =5 QUADRATURE BY DQUAD8 IS USED
C 
C            IERR= 0 IS A NORMAL RETURN
C            IERR= 5 IS AN UNDERFLOW: (B*B+C*C)/T IS TOO LARGE
C            IERR=-1 A CONVERGENCE CONDITION NOT MET
C                = ANY OTHER VALUE IS AN ERROR FROM DGAUS8
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(55),ENH(55)
      COMMON /CQFS1/CCS,CRT,CB,CTCAPB
      EXTERNAL QFS1
      DATA HPI /1.5707963267948966D0/
      DATA RTPI /1.772453850905516D0/
      IERR=0
      KFORM=0
      IF(T.EQ.0.0D0) THEN
        S1=0.0D0
        RETURN
      ENDIF
      RT=DSQRT(T)
      AS=A*A
      BS=B*B
      CS=C*C
      IF((A.EQ.0.0D0).AND.(B.EQ.0.0D0)) THEN
C       C IS POSITIVE HERE IF INPUT IS OBSERVED
        X=C/RT
        KODE=2
        ERFX=DRERF(X,KODE,NZ)
        IF(NZ.NE.0) THEN
           IERR=5
           S1=0.0D0
           RETURN
        ENDIF
        S1=HPI*ERFX/(C*RT)
        RETURN
      ENDIF
      IF(A.EQ.0.0D0) THEN
        CAPT=1.0D0/RT
        CALL INTEGI5(C,B,CAPT,ANSI5,JERR)
        IF(JERR.EQ.4) THEN
          S1=0.0D0
          IERR=5
          RETURN
        ENDIF
        S1=RTPI*ANSI5/RT
        RETURN
      ENDIF
      CAPB=A*RT+B/RT
      CAPX=(BS+CS)/T
      IF(CAPX.GT.667.0D0) THEN
        S1=0.0D0
        IERR=5
        RETURN
      ENDIF
      ARG=CS/T+AS*T
      DEN=DSQRT(ARG)
      CAPL=CAPB/DEN
      IF(CAPL.LT.2.0D0) THEN
C       S1 BY QUADRATURE
        KFORM=5
        CB=B
        CRT=RT
        CCS=CS
        TCAPB=CAPB+CAPB
        CTCAPB=TCAPB
        X1=0.0D0
        SIG1=0.5D0/CAPB
        SIG2=0.707106781181865475D0
        SIG=4.0D0*MIN(SIG1,SIG2)
        INIT=0
        DO 5 JR=1,5
          ETOL=TOL
          CALL DQUAD8(QFS1,INIT,X1,SIG,ETOL,X2,QANS,IERR)
C         TRUNCATION ERROR ESTIMATE; CONTINUE INTEGRATION IF NOT MET
          BR=DEXP(-(TCAPB+X2)*X2)/(CS+(B+X2*RT)**2)          
          BR=0.5D0*BR/X2
          IF(DABS(BR).LE.DABS(TOL*QANS)) GO TO 6
 5      CONTINUE
        IERR=-1
 6      CONTINUE
        IF(IERR.EQ.1) THEN
          IERR=0
        ENDIF
        S1=QANS*DEXP(-CAPX)
        RETURN
      ELSE
C       S1 BY SERIES IN 1/CAPL**K
        KFORM=3
        ARG=CAPB*CAPB
        N=2
        KODE=2
        M=40
        NZ=0
        IERR=0
        CALL DEXINT(ARG, N, KODE, M, TOL, EN, NZ, IERR)
        KODE=2
        FNH=1.5D0
        MP=M+1
        CALL DHEXINT(ARG,FNH,KODE,MP,TOL,ENH,NZ,IERR)
        U1=1.0D0
        X=A*RT/DEN      
        T1=X+X
        U2=T1
        U3=T1*U2-U1
        S=ENH(1)
        TRMS=1.0D0/CAPL
        REL=TOL/10.0D0
        AK=1.0D0
        DO 10 K=1,59,2
          I=(K+1)/2
          S=S+U2*EN(I)*TRMS        
          U1=U2
          U2=U3
          U3=T1*U2-U1
          TRMS=TRMS/CAPL
          S=S+U2*ENH(I+1)*TRMS
          AK=AK+2.0D0
          IF(DABS(TRMS*AK).LT.REL) GO TO 55
          U1=U2
          U2=U3
          U3=T1*U2-U1
          TRMS=TRMS/CAPL
 10     CONTINUE
        IERR=-1
 55     CONTINUE
        S=S/CAPB
        S1=0.5D0*S*DEXP(-CAPX)/T
        RETURN
      ENDIF
      END
      DOUBLE PRECISION FUNCTION QFS1(W)
C     INTEGRAND FOR DQUAD8 IN INTEGS1
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CQFS1/CCS,CRT,CB,CTCAPB
      DEN=CCS+(CB+CRT*W)**2
      QFS1=DEXP(-(CTCAPB+W)*W)/DEN
      RETURN
      END
      SUBROUTINE INTEGS2(A,B,C,T,TOL,S2,IERR,KFORM)
C
C     DONALD E. AMOS, OCTOBER, 2002
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C     INTEGS2 COMPUTES THE INTEGRAL
C
C     S2=INT ON (CAPB,INF) OF RT*EXP(A*A*T+2.0D0*A*B-C*C/T)*EXP(-W*W)*
C                             (W-A*RT)/(C*C+T*(W-A*RT)**2)
C
C     WHERE  CAPB=A*RT+B/RT, RT=DSQRT(T)
C            A IS NON-NEGATIVE, B IS NON-NEGATIVE AND C IS POSITIVE   OR 
C            C IS ZERO WITH B POSITIVE AND A NON-NEGATIVE
C
C            TOL IS THE ERROR TOLERANCE FOR THE COMPUTATION
C            KFORM=0 A SPECIAL FORMULA IS EVALUATED FOR A SPECIAL CASE
C                 =3 FORMULA FOR CAPL.GE.2.0D0 IS EVALUATED
C                 =5 QUADRATURE BY DQUAD8 IS USED
C 
C            IERR= 0 IS A NORMAL RETURN
C                = 5 IS AN UNDERFLOW: (B*B+C*C)/T IS TOO LARGE
C                =-1 A CONVERGENCE CONDITION NOT MET
C                = ANY OTHER VALUE IS AN ERROR FROM DGAUS8
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION EN(55),ENH(55)
      COMMON /CQFS2/CCS,CRT,CB,CTCAPB
      EXTERNAL QFS2
      IERR=0
      KFORM=0
      IF(T.EQ.0.0D0) THEN
        S2=0.0D0
        RETURN
      ENDIF
      RT=DSQRT(T)
      IF(A.EQ.0.0D0) THEN
        ARG=(CS+BS)/T
        N=1
        KODE=1
        M=1
        NZ=0
        CALL DEXINT(ARG, N, KODE, M, TOL, EN, NZ, J22ERR)
        IF(NZ.NE.0) THEN
          S2=0.0D0
          IERR=5
          RETURN
        ENDIF
        S2=0.5D0*EN(1)/RT
        RETURN
      ENDIF
C     A IS POSITIVE HERE
      AS=A*A
      BS=B*B
      CS=C*C      
      ARG=CS/T+AS*T
      DEN=DSQRT(ARG)
      CAPB=A*RT+B/RT
      CAPX=(BS+CS)/T
      IF(CAPX.GT.667.0D0) THEN
        S2=0.0D0
        IERR=5
        RETURN
      ENDIF
      CAPL=CAPB/DEN
      IF(CAPL.LT.2.0D0) THEN
C       S2 BY QUADRATURE
        KFORM=5
        CB=B
        CRT=RT
        CCS=CS
        TCAPB=CAPB+CAPB
        CTCAPB=TCAPB
        X1=0.0D0
        SIG=3.0D0
        INIT=0
        DO 5 JR=1,5
          REL=TOL
          CALL DQUAD8(QFS2,INIT,X1,SIG,REL,X2,QANS,IERR)
C         TRUNCATION ERROR ESTIMATE; CONTINUE INTEGRATION IF NOT MET
          BR=DEXP(-(TCAPB+X2)*X2)/DSQRT(CS+(B+X2*RT)**2)
          BR=0.5D0*BR/X2
          IF(DABS(BR).LE.DABS(TOL*QANS)) GO TO 6
 5      CONTINUE
        IERR=-1
 6      CONTINUE
        IF(IERR.EQ.1) THEN
          IERR=0
        ENDIF
        S2=QANS*DEXP(-CAPX)
        RETURN
      ELSE
C       S2 BY SERIES IN 1/CAPL**K
        KFORM=3
        ARGB=CAPB*CAPB
        N=1
        KODE=2
        M=40
        NZ=0
        IERR=0
        CALL DEXINT(ARGB, N, KODE, M, TOL, EN, NZ, IERR)
        KODE=2
        FNH=1.5D0
        MP=M+1
        CALL DHEXINT(ARGB,FNH,KODE,MP,TOL,ENH,NZ,IERR)
        X=A*RT/DEN      
        TX=X+X
        TM=1.0D0
        TK=X
        TP=TX*TK-TM
        S=EN(1)
        TRMS=1.0D0/CAPL
        REL=TOL/10.0D0
        AK=1.0D0
        DO 10 K=1,59,2
          I=(K+1)/2
          S=S+TK*ENH(I)*TRMS        
          TM=TK
          TK=TP
          TP=TX*TK-TM
          TRMS=TRMS/CAPL
          S=S+TK*EN(I+1)*TRMS
          AK=AK+2.0D0
          IF(DABS(TRMS*AK).LT.REL) GO TO 55
          TM=TK
          TK=TP
          TP=TX*TK-TM
          TRMS=TRMS/CAPL
 10     CONTINUE
        IERR=-1
 55     CONTINUE
        S2=0.5D0*S*DEXP(-CAPX)/RT
        RETURN
      ENDIF
      END
      DOUBLE PRECISION FUNCTION QFS2(W)
C     INTEGRAND FOR DQUAD8 IN INTEGS2
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /CQFS2/CCS,CRT,CB,CTCAPB
      DIFF=CB+CRT*W
      DEN=CCS+DIFF*DIFF
      QFS2=DIFF*DEXP(-(CTCAPB+W)*W)/DEN
      RETURN
      END
      SUBROUTINE GNSEQ(A,B,CAPT,M,REL,YN)
C
C     YN(N+1) CONTAINS 2*G(A,B,N)*(CAPT**(N-1))*DEXP(CAPX), N=0,...,M-1
C     WHERE CAPX=(A*A+B*B)*CAPT*CAPT, CAPT=1/DSQRT(T)
C
C     DIMENSION YN BY AT LEAST M
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION SY(100),EN(1),YN(M)
      DATA RTPI       /1.772453850905516D0/
      COMMON/CGSEQ/ KN,CREL,CCAPX,CBT
      EXTERNAL DGNFUN
      A2=A*A
      B2=B*B
      CAPX=(A2+B2)*CAPT*CAPT
      BT=B*CAPT
C     NFB IS THE INDEX AT WHICH FORWARD AND BACKWARD RECURRENCE STARTS
C     IF NFB.LT.3 ALL INDICES ARE COMPUTED BY FORWARD RECURRENCE
      ARG=CAPX+CAPX
      NFB=INT(SNGL(ARG))
      IF (NFB.LT.3) THEN
        CALL INTEGI5(A,B,CAPT,ANSI5,IERR)
        TRM=DEXP(CAPX)*ANSI5
        S=TRM+TRM
        YN(1)=S/CAPT
        IF(M.EQ.1) RETURN
        N=1
        KODE=2
        MM=1
        NZ=0
        IERR=0
        CALL DEXINT(CAPX, N, KODE, MM, REL, EN, NZ, IERR)
        YN(2)=-B*S+EN(1)/RTPI
        IF(M.EQ.2) RETURN
        N0=0
        KODE=2
        MM=M
        CALL DINERFC(BT,N0,KODE,MM,REL,SY,NZ)
        NN=1
        N=M-2
        CALL GFORW(CAPX,BT,NN,N,M,SY,YN)
        RETURN
      ELSE
C       NFB.GE.3
C       SET COMMON PARAMETERS FOR DGNFUN
        KS=MIN(NFB,M-1)
        KN=KS
        CREL=REL
        CCAPX=CAPX
        CBT=BT
        INIT=0
        X1=0.0D0
        SIG1=1.0D0/DSQRT(CAPX)
        SIG2=0.5/CAPX
        DSIG=3.0D0*MIN(SIG1,SIG2)
        DREL=REL
        CALL DQUAD8(DGNFUN,INIT,X1,DSIG,DREL,X2,DQANS,IERR)
        YN(KS+1)=DQANS
        IF(M.EQ.1) RETURN
        KN=KS-1
        CREL=REL
        INIT=0
        X1=0.0D0
        DREL=REL
        CALL DQUAD8(DGNFUN,INIT,X1,DSIG,DREL,X2,DQANS,IERR)
        YN(KS)=DQANS
        IF(M.EQ.2) RETURN
        N=M-KS-1
        IF(N.GT.0)THEN
          MM=M
          N0=0
          KODE=2          
          CALL DINERFC(BT,N0,KODE,MM,REL,SY,NZ)
          CALL GFORW(CAPX,BT,KS,N,M,SY,YN)
          CALL GBACK(CAPX,BT,KS,M,SY,YN)
          RETURN
        ELSE
          MM=KS
          N0=0
          KODE=2
          CALL DINERFC(BT,N0,KODE,MM,REL,SY,NZ)
          CALL GBACK(CAPX,BT,KS,M,SY,YN)
          RETURN
        ENDIF
      ENDIF
      END
      SUBROUTINE GFORW(CAPX,BT,NSTART,NF,M,SY,YN)
C
C     YN FOR INDICES NSTART AND NSTART+1 ARE KNOWN. THE FIRST
C     RECURRENCE GENERATES NSTART+2. NF IS THE NUMBER OF FORWARD
C     RECURRENCES FOR INDICES NSTART THROUGH NSTART+NF+1. M= NUMBER
C     OF MEMBERS IN THE SEQUENCE.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION SY(100),YN(M)
      J1=NSTART+2
      AJ=DBLE(FLOAT(J1-2))
      TAJ=AJ+AJ
      AJP=AJ+1.0D0
      PROD=AJ*AJP
      BJ=2.0D0*AJP
      DO 10 J=1,NF
        YN(J1)=(SY(J1-2)-TAJ*BT*YN(J1-1)-CAPX*YN(J1-2))/PROD
        J1=J1+1
        TAJ=TAJ+2.0D0
        PROD=PROD+BJ
        BJ=BJ+2.0D0
  10  CONTINUE
      RETURN
      END
      SUBROUTINE GBACK(CAPX,BT,NSTART,M,SY,YN)
C
C     YN MEMBERS FOR INDICES NSTART AND NSTART+1 ARE KNOWN. YN MEMBERS
C     FOR INDICES NSTART-1 DOWN TO 1 ARE GENERATED. M=NUMBER OF MEMBERS
C     IN THE SEQUENCE.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION SY(100),YN(M)
      IF(NSTART.EQ.1) RETURN
      MM=NSTART-1
      AJ=DBLE(FLOAT(MM))
      TAJ=AJ+AJ
      PROD=AJ*(AJ+1.0D0)
      BJ=TAJ
      KJ=MM
      DO 10 J=1,MM
        YN(KJ)=(SY(KJ)-TAJ*BT*YN(KJ+1)-PROD*YN(KJ+2))/CAPX
        KJ=KJ-1
        TAJ=TAJ-2.0D0
        PROD=PROD-BJ
        BJ=BJ-2.0D0
  10  CONTINUE
      RETURN
      END
      DOUBLE PRECISION FUNCTION DGNFUN(W)
C
C     DGNFUN COMPUTES THE INTEGRAND OF G FOR THE QUADRATURES
C              YN(KN+1)=2.0D0*G(A,B,KN)*(RT**(KN-1))*DEXP(CCAPX)
C              YN(KN)=2.0D0*G(A,B,NK-1)*(RT**(KN-2))*DEXP(CCAPX)
C     TO START THE RECURRENCE FOR THE YN SEQUENCE. HERE KN=MIN(NFB,M-1),
C     NFB=INT(SNGL(2.0D0*CCAPX)), CCAPX=(A*A+B*B)/T, RT=DSQRT(T),
C     M=NUMBER IN THE SEQUENCE. CBT=B*CAPT=B/RT    
C
      IMPLICIT DOUBLE PRECISION (A-H, O-Z)
      DIMENSION Y(1)
      COMMON/CGSEQ/ KN,CREL,CCAPX,CBT      
      ARG=-CCAPX*(2.0D0+W)*W
      TM=DEXP(ARG)
      U=1.0D0+W
      ARG=CBT*U
      KODE=2
      N=1
      NZ=0
      CALL DINERFC(ARG,KN,KODE,N,CREL,Y,NZ) 
      S=TM*Y(1)/U**KN
      DGNFUN=S+S
      RETURN
      END
