      SUBROUTINE AMOSINFO
C-----------------------------------------------------------------------
C     DONALD E. AMOS, MAY 1, 2002
C
C     EACH CALLABLE ROUTINE HAS A REFERENCE IN THE PROLOGUE TO 
C     APPROPRIATE FOLDERS WHICH DESCRIBE THE ANALYTICAL BASIS OF THE
C     CODE.
C
C     USAGE:
C
C     COMPILE THIS FILE AND LINK IT TO ANY PROGRAM NEEDING A SUBROUTINE
C     OR FUNCTION FROM THE FOLLOWING LIST:
C
C     CALLABLE SUBROUTINES:                                    FOLDER
C           DGAUS8(FUN, A, B, ERR, ANS, IERR)                  SLATEC
C           DQUAD8(DQFUN,INIT,X1,SIG,REL,X2,QANS,IERR)           21
C           DEXINT(X, N, KODE, M, TOL, EN, NZ, IERR)           SLATEC
C           DHEXINT(X, FNH, KODE, M, TOL, EN, NZ, IERR)          18
C           DGEXINT(X, N, KODE, M, TOL, GN, EN, NZ, IERR)        18
C           DGHEXINT(X, FNH, KODE, M, TOL, GN, EN, NZ, IERR)     18
C           DIERFC(X,KODE,ANS,IERR)                            10,16
C           DINERFC(X,N0,KODE,N,REL,Y,NZ)                      SLATEC
C           DPSIFN(X, N, KODE, M, ANS, NZ, IERR)               SLATEC
C           XERROR(MESS,NMESS,NERR,LEVEL)                      SLATEC
C           XERRWV(MESSG,NMESSG,NERR,LEVEL,NI,I1,I2,NR,R1,R2)  SLATEC
C           FDUMP                                              SLATEC
C
C      CALLABLE FUNCTIONS:                                     
C           DOUBLE PRECISION FUNCTION DRERF(X,KODE,NZ)         AMOSLIB
C           DOUBLE PRECISION FUNCTION DFERF(X,REL,IERR)          16
C           DOUBLE PRECISION FUNCTION DGERFC(X,KODE,REL,IERR)    16
C           DOUBLE PRECISION FUNCTION DHERFC(X)                  23
C           DOUBLE PRECISION FUNCTION DPSIXN(N)                SLATEC
C           DOUBLE PRECISION FUNCTION DGAMLN(Z,IERR)           SLATEC
C           DOUBLE PRECISION FUNCTION D1MACH(I)                SLATEC
C           INTEGER FUNCTION I1MACH(I)                         SLATEC
C           REAL FUNCTION R1MACH(I)                            SLATEC
C-----------------------------------------------------------------------
      END
      DOUBLE PRECISION FUNCTION DRERF(X,KODE,NZ)
C 
C     WRITTEN BY D.E.AMOS AND S.L.DANIEL, JANUARY, 1975.
C                MODIFIED MAY, 1990; MARCH, 2002: JANUARY, 2003
C 
C     REFERENCE: SANDIA LABORATORY REPORT SC-DR-72-0918 
C 
C     ABSTRACT
C         DRERF COMPUTES THE ERROR FUNCTION ERF(X) AND THE COERROR
C         FUNCTION ERFC(X)=1.-ERF(X) FOR REAL, UNRESTRICTED X. TO RETAIN
C         SIGNIFICANT DIGITS THE FORMULAE 
C 
C                -1                             2 
C                -1+EXP(-X*X)*R/ABS(X)          2-EXP(-X*X)*R/ABS(X)
C                -1+EXP(-X*X)*S/ABS(X)          2-EXP(-X*X)*S/ABS(X)
C         ERF(X)= X*T                   ERFC(X)=1-X*T 
C                 1-EXP(-X*X)*U/X               EXP(-X*X)*U/X 
C                 1-EXP(-X*X)*V/X               EXP(-X*X)*V/X 
C                 1                             0 
C 
C         ARE USED WHERE THE BREAK POINTS ARE 
C 
C             X= -6, -4, -2, +2, +4, +6   FOR ERF(X)
C 
C         AND 
C             X= -6, -4, -2, +2, +4       FOR ERFC(X)
C 
C         HERE R, S, T, U AND V ARE CHEBYSHEV EXPANSIONS
C         ON APPROPRIATE INTERVALS. A SCALING OPTION, EXP(X**2)*ERFC(X) 
C         FOR X.GE.0.0, IS ALSO PROVIDED. 
C 
C     DESCRIPTION OF ARGUMENTS
C 
C         INPUT 
C           X      - X,UNRESTRICTED 
C           KODE   - A SELECTION PARAMETER
C                    KODE=1 RETURNS DRERF=ERF(X)
C                    KODE=2 RETURNS DRERF=ERFC(X) 
C                    KODE=3 RETURNS DRERF=         ERFC(X),  X.LT.0.0 
C                                   DRERF=EXP(X*X)*ERFC(X),  X.GE.0.0 
C 
C         OUTPUT
C           DRERF  - VALUE FOR ERF(X),ERFC(X), OR EXP(X**2)*ERFC(X) 
C                    DEPENDING ON KODE
C           NZ     - UNDERFLOW FLAG 
C                    NZ=0  NORMAL RETURN
C                    NZ=1  UNDERFLOW FOR KODE=2,
C                          DRERF=0.0 RETURNED 
C 
C     ERROR CONDITIONS
C         KODE NOT 1, 2, OR 3 IS A FATAL ERROR
C         UNDERFLOW IS A NON-FATAL ERROR, DRERF=0.0 AND NZ=1 RETURNED 
C
C     RERF   USES SUBROUTINES I1MACH,D1MACH,XERROR
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z) 
      DIMENSION A1(24),A2(23),A3(17)
C     CHEBYSHEV COEFFICIENTS FOR [0,2]
      DATA   (A1(K),K=1,24)  / 
     1      0.832316137886179500D+00,     -0.341657034193776500D+00,
     2     -0.152213336353308800D-01,      0.271881389879122100D-01,
     3     -0.443860662153016500D-02,     -0.886371605188296700D-03,
     4      0.380536428195736700D-03,     -0.571020720810514000D-05,
     5     -0.170532778625881700D-04,      0.207469149563607100D-05,
     6      0.475742503160444100D-06,     -0.119217251859814700D-06,
     7     -0.662105198579041900D-08,      0.436703352221523300D-08,
     8     -0.108586273948044500D-09,     -0.118452196128678300D-09,
     9      0.103592366410598800D-10,      0.245412015623528400D-11,
     A     -0.391710955513175500D-12,     -0.369418614881312400D-13,
     B      0.106467918695034400D-13,      0.284514287425412300D-15,
     C     -0.232515791318906500D-15,      0.453152396988929400D-17/
C     CHEBYSHEV COEFFICIENTS FOR [4,6]
      DATA (A2(K),K=1,23)  / 5.55925693425176D-01, 0.00000000000000D+00,
     1-8.09033209211571D-03, 0.00000000000000D+00, 1.67768369286852D-04,
     2 0.00000000000000D+00,-5.53104163380836D-06, 0.00000000000000D+00,
     3 2.44369364046347D-07, 0.00000000000000D+00,-1.33270043958223D-08,
     4 0.00000000000000D+00, 8.55027489170315D-10, 0.00000000000000D+00,
     5-6.25400760100857D-11, 0.00000000000000D+00, 5.10173516527965D-12,
     6 0.00000000000000D+00,-4.56694846856777D-13, 0.00000000000000D+00,
     7 4.43106374588405D-14, 0.00000000000000D+00,-4.61460815796214D-15/
C     CHEBYSHEV COEFFICIENTS FOR [2,4]
      DATA (A3(K),K=1,17)  / 5.33315442660327D-01, 1.78876062098494D-02,
     1-3.80175293816804D-03, 6.97111435056370D-04,-1.16368846095251D-04,
     2 1.81367676091642D-05,-2.67719941001743D-06, 3.77701336341082D-07,
     3-5.12491193181002D-08, 6.71870624801104D-09,-8.54020784885325D-10,
     4 1.05545351331091D-10,-1.27109174471917D-11, 1.49449942042063D-12,
     5-1.71824290388960D-13, 1.93435061097394D-14,-2.13482679031942D-15/
      DATA N1,N2,N3/24,23,17/ 
C-----------------------------------------------------------------------
      IF(KODE.LT.1 .OR. KODE.GT.3) GO TO 900
      NZ=0
      XX=X
      IF(KODE.GT.1) GOTO 200
C     COMPUTATION OF ERF FUNCTION   (KODE=1)
  100 CONTINUE
      IF(XX.GT.-6.0D0) GO TO 10 
      DRERF=-1.0D0
      RETURN
   10 IF(XX.LT.6.0D0) GO TO 12
      DRERF=1.0D0 
      RETURN
   12 CONTINUE
      IF(XX.LT.4.0D0) GO TO 33
C     INTERVAL (4,6)
      APB=0.0D0
      BMA=XX
      XD=2.0D0
      CALL DCHBYS(APB,BMA,XD,N2,A2,SUM)
      DRERF=1.0D0-SUM*DEXP(-XX*XX)/XX
      RETURN 
   33 CONTINUE
      IF(XX.LE.2.0D0) GO TO 40
C     INTERVAL (2,4)
      APB=6.0D0
      BMA=2.0D0
      CALL DCHBYS(APB,BMA,XX,N3,A3,SUM)
      DRERF=1.0D0-SUM*DEXP(-XX*XX)/XX
      RETURN
   40 CONTINUE
      IF(XX.LT.-2.0D0) GO TO 50
C     INTERVAL (-2,2)
      APB=2.0D0
      BMA=2.0D0
      AXX=DABS(XX)
      CALL DCHBYS(APB,BMA,AXX,N1,A1,SUM)
      DRERF=XX*SUM
      RETURN
   50 CONTINUE
      IF(XX.LE.-4.0D0) GO TO 60
C     INTERVAL (-4,-2)
      XX=-XX
      APB=6.0D0
      BMA=2.0D0
      CALL DCHBYS(APB,BMA,XX,N3,A3,SUM)
      DRERF=-1.0D0+SUM*DEXP(-XX*XX)/XX 
      RETURN
   60 CONTINUE
C     INTERVAL (-6,-4)
      XX=-XX
      APB=0.0D0
      BMA=XX
      XD=2.0D0
      CALL DCHBYS(APB,BMA,XD,N2,A2,SUM)
      DRERF=-1.0D0+SUM*DEXP(-XX*XX)/XX
      RETURN
C     COMPUTATION OF ERFC FUNCTION (KODE=2 AND KODE=3)
  200 CONTINUE
      IF(XX.GT.-6.0D0) GO TO 205
      DRERF=2.0D0 
      RETURN
  205 CONTINUE
      KK=MIN(IABS(I1MACH(15)),IABS(I1MACH(16)))
      ELIM=2.303D0*(DBLE(FLOAT(KK))*D1MACH(5)-3.0D0)
      XLIM=DSQRT(ELIM)
      IF(XX.LT.XLIM) GO TO 210
C     INTERVAL (XLIM,INF)
      IF(KODE.EQ.3) THEN
        APB=0.0D0
        BMA=XX
        XD=2.0D0
        CALL DCHBYS(APB,BMA,XD,N2,A2,SUM)
        DRERF=SUM/XX
        RETURN
      ELSE
        NZ=1
        DRERF=0.0D0 
        RETURN
      ENDIF
  210 CONTINUE
      IF(XX.LT.4.0D0) GO TO 215
C     INTERVAL (4,XLIM)
      APB=0.0D0
      BMA=XX
      XD=2.0D0
      CALL DCHBYS(APB,BMA,XD,N2,A2,SUM)
      IF(KODE.EQ.3) THEN
        DRERF=SUM/XX
        RETURN
      ELSE
        DRERF=SUM*DEXP(-XX*XX)/XX
        RETURN
      ENDIF
  215 CONTINUE
      IF(XX.GT.2.0D0) GO TO 220 
      IF(XX.LT.-2.0D0) GO TO 225
C     INTERVAL (-2,2)
      APB=2.0D0
      BMA=2.0D0
      AXX=DABS(XX)
      CALL DCHBYS(APB,BMA,AXX,N1,A1,SUM)
      SUM=XX*SUM
      IF(X.GT.0.0D0 .AND. KODE.EQ.3) THEN
        DRERF=(1.0D0-SUM)*DEXP(XX*XX)
        RETURN
      ELSE
        DRERF=1.0D0-SUM
        RETURN
      ENDIF
  220 CONTINUE
C     INTERVAL (2,4)
      APB=6.0D0
      BMA=2.0D0
      CALL DCHBYS(APB,BMA,XX,N3,A3,SUM)
      IF(KODE.EQ.3) THEN
        DRERF=SUM/XX
        RETURN
      ELSE
        DRERF=SUM*DEXP(-XX*XX)/XX
        RETURN
      ENDIF
  225 CONTINUE
      IF(XX.GT.-4.0D0) GO TO 230
C     INTERVAL (-INF,-4)
      IF(XX.LT.-6.0D0) THEN
        DRERF=2.0D0
        RETURN
      ENDIF
      XX=-XX
      APB=0.0D0
      BMA=XX
      XD=2.0D0
      CALL DCHBYS(APB,BMA,XD,N2,A2,SUM)
      DRERF=2.0D0-SUM*DEXP(-XX*XX)/XX
      RETURN
  230 CONTINUE
C     INTERVAL (-4,-2)
      XX=-XX
      APB=6.0D0
      BMA=2.0D0
      CALL DCHBYS(APB,BMA,XX,N3,A3,SUM)
      DRERF=2.0D0-SUM*DEXP(-XX*XX)/XX
      RETURN
  900 CONTINUE
      CALL XERROR(' IN DRERF, KODE NOT 1, 2, OR 3',30,1,2)
      RETURN
      END 
      SUBROUTINE DGAUS8(FUN, A, B, ERR, ANS, IERR)
C 
C     WRITTEN BY R.E. JONES 
C 
C     ABSTRACT  *** A DOUBLE PRECISION ROUTINE ***
C        DGAUS8 INTEGRATES REAL FUNCTIONS OF ONE VARIABLE OVER FINITE 
C        INTERVALS USING AN ADAPTIVE 8-POINT LEGENDRE-GAUSS ALGORITHM.
C        DGAUS8 IS INTENDED PRIMARILY FOR HIGH ACCURACY INTEGRATION 
C        OR INTEGRATION OF SMOOTH FUNCTIONS.
C 
C        THE MAXIMUM NUMBER OF SIGNIFICANT DIGITS OBTAINABLE IN ANS 
C        IS THE SMALLER OF 18 AND THE NUMBER OF DIGITS CARRIED IN 
C        DOUBLE PRECISION ARITHMETIC. 
C 
C        DGAUS8 CALLS I1MACH, D1MACH, XERROR
C 
C     DESCRIPTION OF ARGUMENTS
C 
C        INPUT--* FUN,A,B,ERR ARE DOUBLE PRECISION *
C        FUN - NAME OF EXTERNAL FUNCTION TO BE INTEGRATED. THIS NAME
C              MUST BE IN AN EXTERNAL STATEMENT IN THE CALLING PROGRAM. 
C              FUN MUST BE A DOUBLE PRECISION FUNCTION OF ONE DOUBLE
C              PRECISION ARGUMENT. THE VALUE OF THE ARGUMENT TO FUN 
C              IS THE VARIABLE OF INTEGRATION WHICH RANGES FROM A TO
C              B. 
C        A   - LOWER LIMIT OF INTEGRAL
C        B   - UPPER LIMIT OF INTEGRAL (MAY BE LESS THAN A) 
C        ERR - IS A REQUESTED PSEUDORELATIVE ERROR TOLERANCE.  NORMALLY 
C              PICK A VALUE OF DABS(ERR) SO THAT DTOL.LT.DABS(ERR).LE.
C              1.0D-3 WHERE DTOL IS THE LARGER OF 1.0D-18 AND THE 
C              DOUBLE PRECISION UNIT ROUNDOFF = D1MACH(4). ANS WILL 
C              NORMALLY HAVE NO MORE ERROR THAN DABS(ERR) TIMES THE 
C              INTEGRAL OF THE ABSOLUTE VALUE OF FUN(X). USUALLY, 
C              SMALLER VALUES OF ERR YIELD MORE ACCURACY AND REQUIRE
C              MORE FUNCTION EVALUATIONS. 
C 
C              A NEGATIVE VALUE FOR ERR CAUSES AN ESTIMATE OF THE 
C              ABSOLUTE ERROR IN ANS TO BE RETURNED IN ERR. NOTE THAT 
C              ERR MUST BE A VARIABLE (NOT A CONSTANT) IN THIS CASE.
C              NOTE ALSO THAT THE USER MUST RESET THE VALUE OF ERR
C              BEFORE MAKING ANY MORE CALLS THAT USE THE VARIABLE ERR.
C 
C        OUTPUT--* ERR,ANS ARE DOUBLE PRECISION * 
C        ERR - WILL BE AN ESTIMATE OF THE ABSOLUTE ERROR IN ANS IF THE
C              INPUT VALUE OF ERR WAS NEGATIVE.  (ERR IS UNCHANGED IF 
C              THE INPUT VALUE OF ERR WAS NONNEGATIVE.)  THE ESTIMATED
C              ERROR IS SOLELY FOR INFORMATION TO THE USER AND SHOULD 
C              NOT BE USED AS A CORRECTION TO THE COMPUTED INTEGRAL.
C        ANS - COMPUTED VALUE OF INTEGRAL 
C        IERR- A STATUS CODE
C            --NORMAL CODES 
C               1 ANS MOST LIKELY MEETS REQUESTED ERROR TOLERANCE,
C                 OR A=B. 
C              -1 A AND B ARE TOO NEARLY EQUAL TO ALLOW NORMAL
C                 INTEGRATION.  ANS IS SET TO ZERO. 
C            --ABNORMAL CODE
C               2 ANS PROBABLY DOES NOT MEET REQUESTED ERROR TOLERANCE. 
C***END PROLOGUE
      EXTERNAL FUN
      INTEGER ICALL, IERR, K, KML, KMX, L, LMN, LMX, LR, MXL, NBITS,
     * NIB, NLMN, NLMX
      INTEGER I1MACH
      DOUBLE PRECISION A,AA,AE,ANIB,ANS,AREA,B,C,CE,DTOL,EE,EF, 
     * EPS, ERR, EST, GL, GLR, GR, HH, SQ2, TOL, VL, VR, W1, W2, W3,
     * W4, X1, X2, X3, X4, X, H 
      DOUBLE PRECISION D1MACH, G8, FUN
      DIMENSION AA(30), HH(30), LR(30), VL(30), GR(30)
      DATA X1, X2, X3, X4/
     1     1.83434642495649805D-01,     5.25532409916328986D-01,
     2     7.96666477413626740D-01,     9.60289856497536232D-01/
      DATA W1, W2, W3, W4/
     1     3.62683783378361983D-01,     3.13706645877887287D-01,
     2     2.22381034453374471D-01,     1.01228536290376259D-01/
      DATA ICALL  /  0  / 
      DATA SQ2/1.41421356D0/
      DATA NLMN/1/,KMX/5000/,KML/6/ 
      G8(X,H)=H*((W1*(FUN(X-X1*H) + FUN(X+X1*H))
     1           +W2*(FUN(X-X2*H) + FUN(X+X2*H))) 
     2          +(W3*(FUN(X-X3*H) + FUN(X+X3*H))
     3           +W4*(FUN(X-X4*H) + FUN(X+X4*H))))
C 
C     INITIALIZE
C 
      IF (ICALL.NE.0) CALL XERROR('DGAUS8- DGAUS8 CALLED RECURSIVELY. RE
     *CURSIVE CALLS ARE ILLEGAL IN FORTRAN.',74, 7, 2)
      ICALL = 1 
      DTOL=D1MACH(4)
      DTOL=DMAX1(DTOL,1.0D-18)
      IF(DABS(ERR).LT.DTOL) GO TO 150 
      K = I1MACH(14)
      ANIB = D1MACH(5)*DBLE(FLOAT(K))/0.30102000D0
      NBITS = INT(SNGL(ANIB)) 
      NLMX = (NBITS*5)/8
      ANS = 0.0D0 
      IERR = 1
      CE = 0.0D0
      IF (A.EQ.B) GO TO 140 
      LMX = NLMX
      LMN = NLMN
      IF (B.EQ.0.0D0) GO TO 10
      IF (DSIGN(1.0D0,B)*A.LE.0.0D0) GO TO 10 
      C = DABS(1.0D0-A/B) 
      IF (C.GT.0.1D0) GO TO 10
      IF (C.LE.0.0D0) GO TO 140 
      ANIB = 0.5D0 - DLOG(C)/0.69314718D0 
      NIB = INT(SNGL(ANIB)) 
      LMX = MIN0(NLMX,NBITS-NIB-7)
      IF (LMX.LT.1) GO TO 130 
      LMN = MIN0(LMN,LMX) 
   10 TOL = DMAX1(DABS(ERR),2.0D0**(5-NBITS))/2.0D0 
      IF (ERR.EQ.0.0D0) TOL = DSQRT(D1MACH(4))
      EPS = TOL 
      HH(1) = (B-A)/4.0D0 
      AA(1) = A 
      LR(1) = 1 
      L = 1 
      EST = G8(AA(L)+2.0D0*HH(L),2.0D0*HH(L)) 
      K = 8 
      AREA = DABS(EST)
      EF = 0.5D0
      MXL = 0 
C 
C     COMPUTE REFINED ESTIMATES, ESTIMATE THE ERROR, ETC. 
C 
   20 GL = G8(AA(L)+HH(L),HH(L))
      GR(L) = G8(AA(L)+3.0D0*HH(L),HH(L)) 
      K = K + 16
      AREA = AREA + (DABS(GL)+DABS(GR(L))-DABS(EST))
C     IF (L.LT.LMN) GO TO 11
      GLR = GL + GR(L)
      EE = DABS(EST-GLR)*EF 
      AE = DMAX1(EPS*AREA,TOL*DABS(GLR))
      IF (EE-AE) 40, 40, 50 
   30 MXL = 1 
   40 CE = CE + (EST-GLR) 
      IF (LR(L)) 60, 60, 80 
C 
C     CONSIDER THE LEFT HALF OF THIS LEVEL
C 
   50 IF (K.GT.KMX) LMX = KML 
      IF (L.GE.LMX) GO TO 30
      L = L + 1 
      EPS = EPS*0.5D0 
      EF = EF/SQ2 
      HH(L) = HH(L-1)*0.5D0 
      LR(L) = -1
      AA(L) = AA(L-1) 
      EST = GL
      GO TO 20
C 
C     PROCEED TO RIGHT HALF AT THIS LEVEL 
C 
   60 VL(L) = GLR 
   70 EST = GR(L-1) 
      LR(L) = 1 
      AA(L) = AA(L) + 4.0D0*HH(L) 
      GO TO 20
C 
C     RETURN ONE LEVEL
C 
   80 VR = GLR
   90 IF (L.LE.1) GO TO 120 
      L = L - 1 
      EPS = EPS*2.0D0 
      EF = EF*SQ2 
      IF (LR(L)) 100, 100, 110
  100 VL(L) = VL(L+1) + VR
      GO TO 70
  110 VR = VL(L+1) + VR 
      GO TO 90
C 
C      EXIT 
C 
  120 ANS = VR
      IF ((MXL.EQ.0) .OR. (DABS(CE).LE.2.0D0*TOL*AREA)) GO TO 140 
      IERR = 2
      CALL XERROR('DGAUS8- ANS IS PROBABLY INSUFFICIENTLY ACCURATE.',
     * 48, 3, 1)
      GO TO 140 
  130 IERR = -1 
      CALL XERROR('DGAUS8- THE FOLLOWING TEMPORARY DIAGNOSTIC WILL APPEA
     *R ONLY ONCE.  A AND B ARE TOO NEARLY EQUAL TO ALLOW NORMAL INTEGRA
     *TION.  ANS IS SET TO ZERO, AND IERR=-1.', 158, 1, -1)
  140 ICALL = 0 
      IF (ERR.LT.0.0D0) ERR = CE
      RETURN
  150 CONTINUE
      CALL XERROR('DGAUS8- ABS(ERR) IS TOO SMALL.', 30, 2, 1)
      RETURN
      END 
*DECK I1MACH
      INTEGER FUNCTION I1MACH(I)
C***BEGIN PROLOGUE  I1MACH
C***DATE WRITTEN   750101   (YYMMDD)
C***REVISION DATE  890213   (YYMMDD)
C***CATEGORY NO.  R1
C***KEYWORDS  LIBRARY=SLATEC,TYPE=INTEGER(I1MACH-I),MACHINE CONSTANTS
C***AUTHOR  FOX, P. A., (BELL LABS)
C           HALL, A. D., (BELL LABS)
C           SCHRYER, N. L., (BELL LABS)
C***PURPOSE  Returns integer machine dependent constants
C***DESCRIPTION
C
C     I1MACH can be used to obtain machine-dependent parameters
C     for the local machine environment.  It is a function
C     subroutine with one (input) argument, and can be called
C     as follows, for example
C
C          K = I1MACH(I)
C
C     where I=1,...,16.  The (output) value of K above is
C     determined by the (input) value of I.  The results for
C     various values of I are discussed below.
C
C  I/O unit numbers.
C    I1MACH( 1) = the standard input unit.
C    I1MACH( 2) = the standard output unit.
C    I1MACH( 3) = the standard punch unit.
C    I1MACH( 4) = the standard error message unit.
C
C  Words.
C    I1MACH( 5) = the number of bits per integer storage unit.
C    I1MACH( 6) = the number of characters per integer storage unit.
C
C  Integers.
C    assume integers are represented in the S-digit, base-A form
C
C               sign ( X(S-1)*A**(S-1) + ... + X(1)*A + X(0) )
C
C               where 0 .LE. X(I) .LT. A for I=0,...,S-1.
C    I1MACH( 7) = A, the base.
C    I1MACH( 8) = S, the number of base-A digits.
C    I1MACH( 9) = A**S - 1, the largest magnitude.
C
C  Floating-Point Numbers.
C    Assume floating-point numbers are represented in the T-digit,
C    base-B form
C               sign (B**E)*( (X(1)/B) + ... + (X(T)/B**T) )
C
C               where 0 .LE. X(I) .LT. B for I=1,...,T,
C               0 .LT. X(1), and EMIN .LE. E .LE. EMAX.
C    I1MACH(10) = B, the base.
C
C  Single-Precision
C    I1MACH(11) = T, the number of base-B digits.
C    I1MACH(12) = EMIN, the smallest exponent E.
C    I1MACH(13) = EMAX, the largest exponent E.
C
C  Double-Precision
C    I1MACH(14) = T, the number of base-B digits.
C    I1MACH(15) = EMIN, the smallest exponent E.
C    I1MACH(16) = EMAX, the largest exponent E.
C
C  To alter this function for a particular environment,
C  the desired set of DATA statements should be activated by
C  removing the C from column 1.  Also, the values of
C  I1MACH(1) - I1MACH(4) should be checked for consistency
C  with the local operating system.
C
C***REFERENCES  FOX P.A., HALL A.D., SCHRYER N.L.,*FRAMEWORK FOR A
C                 PORTABLE LIBRARY*, ACM TRANSACTIONS ON MATHEMATICAL
C                 SOFTWARE, VOL. 4, NO. 2, JUNE 1978, PP. 177-188.
C***ROUTINES CALLED  (NONE)
C***END PROLOGUE  I1MACH
C
      INTEGER IMACH(16),OUTPUT
      SAVE IMACH
      EQUIVALENCE (IMACH(4),OUTPUT)
C
C     MACHINE CONSTANTS FOR THE AMIGA
C     ABSOFT COMPILER
C
C     DATA IMACH(1) /    5 /
C     DATA IMACH(2) /    6 /
C     DATA IMACH(3) /    5 /
C     DATA IMACH(4) /    6 /
C     DATA IMACH(5) /   32 /
C     DATA IMACH(6) /    4 /
C     DATA IMACH(7) /    2 /
C     DATA IMACH(8) /   31 /
C     DATA IMACH(9) / 2147483647 /
C     DATA IMACH(10)/    2 /
C     DATA IMACH(11)/   24 /
C     DATA IMACH(12)/ -126 /
C     DATA IMACH(13)/  127 /
C     DATA IMACH(14)/   53 /
C     DATA IMACH(15)/ -1022 /
C     DATA IMACH(16)/  1023 /
C
C     MACHINE CONSTANTS FOR THE APOLLO
C
C     DATA IMACH(1) /    5 /
C     DATA IMACH(2) /    6 /
C     DATA IMACH(3) /    6 /
C     DATA IMACH(4) /    6 /
C     DATA IMACH(5) /   32 /
C     DATA IMACH(6) /    4 /
C     DATA IMACH(7) /    2 /
C     DATA IMACH(8) /   31 /
C     DATA IMACH(9) / 2147483647 /
C     DATA IMACH(10)/    2 /
C     DATA IMACH(11)/   24 /
C     DATA IMACH(12)/ -125 /
C     DATA IMACH(13)/  129 /
C     DATA IMACH(14)/   53 /
C     DATA IMACH(15)/ -1021 /
C     DATA IMACH(16)/  1025 /
C
C     MACHINE CONSTANTS FOR THE BURROUGHS 1700 SYSTEM
C
C     DATA IMACH( 1) /    7 /
C     DATA IMACH( 2) /    2 /
C     DATA IMACH( 3) /    2 /
C     DATA IMACH( 4) /    2 /
C     DATA IMACH( 5) /   36 /
C     DATA IMACH( 6) /    4 /
C     DATA IMACH( 7) /    2 /
C     DATA IMACH( 8) /   33 /
C     DATA IMACH( 9) / Z1FFFFFFFF /
C     DATA IMACH(10) /    2 /
C     DATA IMACH(11) /   24 /
C     DATA IMACH(12) / -256 /
C     DATA IMACH(13) /  255 /
C     DATA IMACH(14) /   60 /
C     DATA IMACH(15) / -256 /
C     DATA IMACH(16) /  255 /
C
C     MACHINE CONSTANTS FOR THE BURROUGHS 5700 SYSTEM
C
C     DATA IMACH( 1) /   5 /
C     DATA IMACH( 2) /   6 /
C     DATA IMACH( 3) /   7 /
C     DATA IMACH( 4) /   6 /
C     DATA IMACH( 5) /  48 /
C     DATA IMACH( 6) /   6 /
C     DATA IMACH( 7) /   2 /
C     DATA IMACH( 8) /  39 /
C     DATA IMACH( 9) / O0007777777777777 /
C     DATA IMACH(10) /   8 /
C     DATA IMACH(11) /  13 /
C     DATA IMACH(12) / -50 /
C     DATA IMACH(13) /  76 /
C     DATA IMACH(14) /  26 /
C     DATA IMACH(15) / -50 /
C     DATA IMACH(16) /  76 /
C
C     MACHINE CONSTANTS FOR THE BURROUGHS 6700/7700 SYSTEMS
C
C     DATA IMACH( 1) /   5 /
C     DATA IMACH( 2) /   6 /
C     DATA IMACH( 3) /   7 /
C     DATA IMACH( 4) /   6 /
C     DATA IMACH( 5) /  48 /
C     DATA IMACH( 6) /   6 /
C     DATA IMACH( 7) /   2 /
C     DATA IMACH( 8) /  39 /
C     DATA IMACH( 9) / O0007777777777777 /
C     DATA IMACH(10) /   8 /
C     DATA IMACH(11) /  13 /
C     DATA IMACH(12) / -50 /
C     DATA IMACH(13) /  76 /
C     DATA IMACH(14) /  26 /
C     DATA IMACH(15) / -32754 /
C     DATA IMACH(16) /  32780 /
C
C     MACHINE CONSTANTS FOR THE CDC 170/180 SERIES USING NOS/VE
C
C     DATA IMACH( 1) /     5 /
C     DATA IMACH( 2) /     6 /
C     DATA IMACH( 3) /     7 /
C     DATA IMACH( 4) /     6 /
C     DATA IMACH( 5) /    64 /
C     DATA IMACH( 6) /     8 /
C     DATA IMACH( 7) /     2 /
C     DATA IMACH( 8) /    63 /
C     DATA IMACH( 9) / 9223372036854775807 /
C     DATA IMACH(10) /     2 /
C     DATA IMACH(11) /    47 /
C     DATA IMACH(12) / -4095 /
C     DATA IMACH(13) /  4094 /
C     DATA IMACH(14) /    94 /
C     DATA IMACH(15) / -4095 /
C     DATA IMACH(16) /  4094 /
C
C     MACHINE CONSTANTS FOR THE CDC 6000/7000 SERIES
C
C     DATA IMACH( 1) /    5 /
C     DATA IMACH( 2) /    6 /
C     DATA IMACH( 3) /    7 /
C     DATA IMACH( 4) /6LOUTPUT/
C     DATA IMACH( 5) /   60 /
C     DATA IMACH( 6) /   10 /
C     DATA IMACH( 7) /    2 /
C     DATA IMACH( 8) /   48 /
C     DATA IMACH( 9) / 00007777777777777777B /
C     DATA IMACH(10) /    2 /
C     DATA IMACH(11) /   47 /
C     DATA IMACH(12) / -929 /
C     DATA IMACH(13) / 1070 /
C     DATA IMACH(14) /   94 /
C     DATA IMACH(15) / -929 /
C     DATA IMACH(16) / 1069 /
C
C     MACHINE CONSTANTS FOR THE CELERITY C1260
C
C     DATA IMACH(1) /    5 /
C     DATA IMACH(2) /    6 /
C     DATA IMACH(3) /    6 /
C     DATA IMACH(4) /    0 /
C     DATA IMACH(5) /   32 /
C     DATA IMACH(6) /    4 /
C     DATA IMACH(7) /    2 /
C     DATA IMACH(8) /   31 /
C     DATA IMACH(9) / Z'7FFFFFFF' /
C     DATA IMACH(10)/    2 /
C     DATA IMACH(11)/   24 /
C     DATA IMACH(12)/ -126 /
C     DATA IMACH(13)/  127 /
C     DATA IMACH(14)/   53 /
C     DATA IMACH(15)/ -1022 /
C     DATA IMACH(16)/  1023 /
C
C     MACHINE CONSTANTS FOR THE CONVEX C-1
C
C     DATA IMACH( 1) /     5/
C     DATA IMACH( 2) /     6/
C     DATA IMACH( 3) /     7/
C     DATA IMACH( 4) /     6/
C     DATA IMACH( 5) /    32/
C     DATA IMACH( 6) /     4/
C     DATA IMACH( 7) /     2/
C     DATA IMACH( 8) /    31/
C     DATA IMACH( 9) /2147483647/
C     DATA IMACH(10) /     2/
C     DATA IMACH(11) /    24/
C     DATA IMACH(12) /  -128/
C     DATA IMACH(13) /   127/
C     DATA IMACH(14) /    53/
C     DATA IMACH(15) / -1024/
C     DATA IMACH(16) /  1023/
C
C     MACHINE CONSTANTS FOR THE CRAY-1
C
C     DATA IMACH( 1) /   100 /
C     DATA IMACH( 2) /   101 /
C     DATA IMACH( 3) /   102 /
C     DATA IMACH( 4) /   101 /
C     DATA IMACH( 5) /    64 /
C     DATA IMACH( 6) /     8 /
C     DATA IMACH( 7) /     2 /
C     DATA IMACH( 8) /    63 /
C     DATA IMACH( 9) /  777777777777777777777B /
C     DATA IMACH(10) /     2 /
C     DATA IMACH(11) /    47 /
C     DATA IMACH(12) / -8189 /
C     DATA IMACH(13) /  8190 /
C     DATA IMACH(14) /    94 /
C     DATA IMACH(15) / -8099 /
C     DATA IMACH(16) /  8190 /
C
C     MACHINE CONSTANTS FOR THE DATA GENERAL ECLIPSE S/200
C
C     DATA IMACH( 1) /   11 /
C     DATA IMACH( 2) /   12 /
C     DATA IMACH( 3) /    8 /
C     DATA IMACH( 4) /   10 /
C     DATA IMACH( 5) /   16 /
C     DATA IMACH( 6) /    2 /
C     DATA IMACH( 7) /    2 /
C     DATA IMACH( 8) /   15 /
C     DATA IMACH( 9) /32767 /
C     DATA IMACH(10) /   16 /
C     DATA IMACH(11) /    6 /
C     DATA IMACH(12) /  -64 /
C     DATA IMACH(13) /   63 /
C     DATA IMACH(14) /   14 /
C     DATA IMACH(15) /  -64 /
C     DATA IMACH(16) /   63 /
C
C     MACHINE CONSTANTS FOR THE ELXSI 6400
C
C     DATA IMACH( 1) /     5/
C     DATA IMACH( 2) /     6/
C     DATA IMACH( 3) /     6/
C     DATA IMACH( 4) /     6/
C     DATA IMACH( 5) /    32/
C     DATA IMACH( 6) /     4/
C     DATA IMACH( 7) /     2/
C     DATA IMACH( 8) /    32/
C     DATA IMACH( 9) /2147483647/
C     DATA IMACH(10) /     2/
C     DATA IMACH(11) /    24/
C     DATA IMACH(12) /  -126/
C     DATA IMACH(13) /   127/
C     DATA IMACH(14) /    53/
C     DATA IMACH(15) / -1022/
C     DATA IMACH(16) /  1023/
C
C     MACHINE CONSTANTS FOR THE HARRIS 220
C
C     DATA IMACH( 1) /       5 /
C     DATA IMACH( 2) /       6 /
C     DATA IMACH( 3) /       0 /
C     DATA IMACH( 4) /       6 /
C     DATA IMACH( 5) /      24 /
C     DATA IMACH( 6) /       3 /
C     DATA IMACH( 7) /       2 /
C     DATA IMACH( 8) /      23 /
C     DATA IMACH( 9) / 8388607 /
C     DATA IMACH(10) /       2 /
C     DATA IMACH(11) /      23 /
C     DATA IMACH(12) /    -127 /
C     DATA IMACH(13) /     127 /
C     DATA IMACH(14) /      38 /
C     DATA IMACH(15) /    -127 /
C     DATA IMACH(16) /     127 /
C
C     MACHINE CONSTANTS FOR THE HONEYWELL 600/6000 SERIES
C
C     DATA IMACH( 1) /    5 /
C     DATA IMACH( 2) /    6 /
C     DATA IMACH( 3) /   43 /
C     DATA IMACH( 4) /    6 /
C     DATA IMACH( 5) /   36 /
C     DATA IMACH( 6) /    6 /
C     DATA IMACH( 7) /    2 /
C     DATA IMACH( 8) /   35 /
C     DATA IMACH( 9) / O377777777777 /
C     DATA IMACH(10) /    2 /
C     DATA IMACH(11) /   27 /
C     DATA IMACH(12) / -127 /
C     DATA IMACH(13) /  127 /
C     DATA IMACH(14) /   63 /
C     DATA IMACH(15) / -127 /
C     DATA IMACH(16) /  127 /
C
C     MACHINE CONSTANTS FOR THE HP 2100
C     3 WORD DOUBLE PRECISION OPTION WITH FTN4
C
C     DATA IMACH(1) /      5/
C     DATA IMACH(2) /      6 /
C     DATA IMACH(3) /      4 /
C     DATA IMACH(4) /      1 /
C     DATA IMACH(5) /     16 /
C     DATA IMACH(6) /      2 /
C     DATA IMACH(7) /      2 /
C     DATA IMACH(8) /     15 /
C     DATA IMACH(9) /  32767 /
C     DATA IMACH(10)/      2 /
C     DATA IMACH(11)/     23 /
C     DATA IMACH(12)/   -128 /
C     DATA IMACH(13)/    127 /
C     DATA IMACH(14)/     39 /
C     DATA IMACH(15)/   -128 /
C     DATA IMACH(16)/    127 /
C
C     MACHINE CONSTANTS FOR THE HP 2100
C     4 WORD DOUBLE PRECISION OPTION WITH FTN4
C
C     DATA IMACH(1) /      5 /
C     DATA IMACH(2) /      6 /
C     DATA IMACH(3) /      4 /
C     DATA IMACH(4) /      1 /
C     DATA IMACH(5) /     16 /
C     DATA IMACH(6) /      2 /
C     DATA IMACH(7) /      2 /
C     DATA IMACH(8) /     15 /
C     DATA IMACH(9) /  32767 /
C     DATA IMACH(10)/      2 /
C     DATA IMACH(11)/     23 /
C     DATA IMACH(12)/   -128 /
C     DATA IMACH(13)/    127 /
C     DATA IMACH(14)/     55 /
C     DATA IMACH(15)/   -128 /
C     DATA IMACH(16)/    127 /
C
C     MACHINE CONSTANTS FOR THE HP 9000
C
C     DATA IMACH(1)  /    5 /
C     DATA IMACH(2)  /    6 /
C     DATA IMACH(3)  /    6 /
C     DATA IMACH(3)  /    7 /
C     DATA IMACH(5)  /   32 /
C     DATA IMACH(6)  /    4 /
C     DATA IMACH(7)  /    2 /
C     DATA IMACH(8)  /   32 /
C     DATA IMACH(9)  /2147483647 /
C     DATA IMACH(10) /    2 /
C     DATA IMACH(11) /   24 /
C     DATA IMACH(12) / -126 /
C     DATA IMACH(13) /  127 /
C     DATA IMACH(14) /   53 /
C     DATA IMACH(15) /-1015 /
C     DATA IMACH(16) / 1017 /
C
C     MACHINE CONSTANTS FOR THE IBM 360/370 SERIES,
C     THE XEROX SIGMA 5/7/9, THE SEL SYSTEMS 85/86, AND
C     THE PERKIN ELMER (INTERDATA) 7/32.
C
C     DATA IMACH( 1) /   5 /
C     DATA IMACH( 2) /   6 /
C     DATA IMACH( 3) /   7 /
C     DATA IMACH( 4) /   6 /
C     DATA IMACH( 5) /  32 /
C     DATA IMACH( 6) /   4 /
C     DATA IMACH( 7) /  16 /
C     DATA IMACH( 8) /  31 /
C     DATA IMACH( 9) / Z7FFFFFFF /
C     DATA IMACH(10) /  16 /
C     DATA IMACH(11) /   6 /
C     DATA IMACH(12) / -64 /
C     DATA IMACH(13) /  63 /
C     DATA IMACH(14) /  14 /
C     DATA IMACH(15) / -64 /
C     DATA IMACH(16) /  63 /
C
C     MACHINE CONSTANTS FOR THE IBM PC
C
      DATA IMACH( 1) /     5 /
      DATA IMACH( 2) /     6 /
      DATA IMACH( 3) /     0 /
      DATA IMACH( 4) /     0 /
      DATA IMACH( 5) /    32 /
      DATA IMACH( 6) /     4 /
      DATA IMACH( 7) /     2 /
      DATA IMACH( 8) /    31 /
      DATA IMACH( 9) / 2147483647 /
      DATA IMACH(10) /     2 /
      DATA IMACH(11) /    24 /
      DATA IMACH(12) /  -125 /
      DATA IMACH(13) /   127 /
      DATA IMACH(14) /    53 /
      DATA IMACH(15) / -1021 /
      DATA IMACH(16) /  1023 /
C
C     MACHINE CONSTANTS FOR THE IBM RS 6000
C
C     DATA IMACH( 1) /          5 /
C     DATA IMACH( 2) /          6 /
C     DATA IMACH( 3) /          6 /
C     DATA IMACH( 4) /          0 /
C     DATA IMACH( 5) /         32 /
C     DATA IMACH( 6) /          4 /
C     DATA IMACH( 7) /          2 /
C     DATA IMACH( 8) /         31 /
C     DATA IMACH( 9) / 2147483647 /
C     DATA IMACH(10) /          2 /
C     DATA IMACH(11) /         24 /
C     DATA IMACH(12) /       -125 /
C     DATA IMACH(13) /        128 /
C     DATA IMACH(14) /         53 /
C     DATA IMACH(15) /      -1021 /
C     DATA IMACH(16) /       1024 /
C
C     MACHINE CONSTANTS FOR THE PDP-10 (KA PROCESSOR)
C
C     DATA IMACH( 1) /    5 /
C     DATA IMACH( 2) /    6 /
C     DATA IMACH( 3) /    5 /
C     DATA IMACH( 4) /    6 /
C     DATA IMACH( 5) /   36 /
C     DATA IMACH( 6) /    5 /
C     DATA IMACH( 7) /    2 /
C     DATA IMACH( 8) /   35 /
C     DATA IMACH( 9) / "377777777777 /
C     DATA IMACH(10) /    2 /
C     DATA IMACH(11) /   27 /
C     DATA IMACH(12) / -128 /
C     DATA IMACH(13) /  127 /
C     DATA IMACH(14) /   54 /
C     DATA IMACH(15) / -101 /
C     DATA IMACH(16) /  127 /
C
C     MACHINE CONSTANTS FOR THE PDP-10 (KI PROCESSOR)
C
C     DATA IMACH( 1) /    5 /
C     DATA IMACH( 2) /    6 /
C     DATA IMACH( 3) /    5 /
C     DATA IMACH( 4) /    6 /
C     DATA IMACH( 5) /   36 /
C     DATA IMACH( 6) /    5 /
C     DATA IMACH( 7) /    2 /
C     DATA IMACH( 8) /   35 /
C     DATA IMACH( 9) / "377777777777 /
C     DATA IMACH(10) /    2 /
C     DATA IMACH(11) /   27 /
C     DATA IMACH(12) / -128 /
C     DATA IMACH(13) /  127 /
C     DATA IMACH(14) /   62 /
C     DATA IMACH(15) / -128 /
C     DATA IMACH(16) /  127 /
C
C     MACHINE CONSTANTS FOR PDP-11 FORTRAN SUPPORTING
C     32-BIT INTEGER ARITHMETIC.
C
C     DATA IMACH( 1) /    5 /
C     DATA IMACH( 2) /    6 /
C     DATA IMACH( 3) /    5 /
C     DATA IMACH( 4) /    6 /
C     DATA IMACH( 5) /   32 /
C     DATA IMACH( 6) /    4 /
C     DATA IMACH( 7) /    2 /
C     DATA IMACH( 8) /   31 /
C     DATA IMACH( 9) / 2147483647 /
C     DATA IMACH(10) /    2 /
C     DATA IMACH(11) /   24 /
C     DATA IMACH(12) / -127 /
C     DATA IMACH(13) /  127 /
C     DATA IMACH(14) /   56 /
C     DATA IMACH(15) / -127 /
C     DATA IMACH(16) /  127 /
C
C     MACHINE CONSTANTS FOR PDP-11 FORTRAN SUPPORTING
C     16-BIT INTEGER ARITHMETIC.
C
C     DATA IMACH( 1) /    5 /
C     DATA IMACH( 2) /    6 /
C     DATA IMACH( 3) /    5 /
C     DATA IMACH( 4) /    6 /
C     DATA IMACH( 5) /   16 /
C     DATA IMACH( 6) /    2 /
C     DATA IMACH( 7) /    2 /
C     DATA IMACH( 8) /   15 /
C     DATA IMACH( 9) / 32767 /
C     DATA IMACH(10) /    2 /
C     DATA IMACH(11) /   24 /
C     DATA IMACH(12) / -127 /
C     DATA IMACH(13) /  127 /
C     DATA IMACH(14) /   56 /
C     DATA IMACH(15) / -127 /
C     DATA IMACH(16) /  127 /
C
C     MACHINE CONSTANTS FOR THE SILICON GRAPHICS IRIS
C
C     DATA IMACH( 1) /     5 /
C     DATA IMACH( 2) /     6 /
C     DATA IMACH( 3) /     6 /
C     DATA IMACH( 4) /     0 /
C     DATA IMACH( 5) /    32 /
C     DATA IMACH( 6) /     4 /
C     DATA IMACH( 7) /     2 /
C     DATA IMACH( 8) /    31 /
C     DATA IMACH( 9) / 2147483647 /
C     DATA IMACH(10) /     2 /
C     DATA IMACH(11) /    23 /
C     DATA IMACH(12) /  -126 /
C     DATA IMACH(13) /   127 /
C     DATA IMACH(14) /    52 /
C     DATA IMACH(15) / -1022 /
C     DATA IMACH(16) /  1023 /
C
C     MACHINE CONSTANTS FOR THE SUN
C
C     DATA IMACH(1) /    5 /
C     DATA IMACH(2) /    6 /
C     DATA IMACH(3) /    6 /
C     DATA IMACH(4) /    6 /
C     DATA IMACH(5) /   32 /
C     DATA IMACH(6) /    4 /
C     DATA IMACH(7) /    2 /
C     DATA IMACH(8) /   31 /
C     DATA IMACH(9) /2147483647 /
C     DATA IMACH(10)/    2 /
C     DATA IMACH(11)/   24 /
C     DATA IMACH(12)/ -125 /
C     DATA IMACH(13)/  128 /
C     DATA IMACH(14)/   53 /
C     DATA IMACH(15)/ -1021 /
C     DATA IMACH(16)/  1024 /
C
C     MACHINE CONSTANTS FOR THE UNIVAC 1100 SERIES FTN COMPILER
C
C
C     DATA IMACH( 1) /    5 /
C     DATA IMACH( 2) /    6 /
C     DATA IMACH( 3) /    1 /
C     DATA IMACH( 4) /    6 /
C     DATA IMACH( 5) /   36 /
C     DATA IMACH( 6) /    4 /
C     DATA IMACH( 7) /    2 /
C     DATA IMACH( 8) /   35 /
C     DATA IMACH( 9) / O377777777777 /
C     DATA IMACH(10) /    2 /
C     DATA IMACH(11) /   27 /
C     DATA IMACH(12) / -128 /
C     DATA IMACH(13) /  127 /
C     DATA IMACH(14) /   60 /
C     DATA IMACH(15) /-1024 /
C     DATA IMACH(16) / 1023 /
C
C     MACHINE CONSTANTS FOR THE VAX 11/780
C
C     DATA IMACH(1) /    5 /
C     DATA IMACH(2) /    6 /
C     DATA IMACH(3) /    5 /
C     DATA IMACH(4) /    6 /
C     DATA IMACH(5) /   32 /
C     DATA IMACH(6) /    4 /
C     DATA IMACH(7) /    2 /
C     DATA IMACH(8) /   31 /
C     DATA IMACH(9) /2147483647 /
C     DATA IMACH(10)/    2 /
C     DATA IMACH(11)/   24 /
C     DATA IMACH(12)/ -127 /
C     DATA IMACH(13)/  127 /
C     DATA IMACH(14)/   56 /
C     DATA IMACH(15)/ -127 /
C     DATA IMACH(16)/  127 /
C
C     MACHINE CONSTANTS FOR THE VAX 11/780, G-FLOAT OPTION
C
C     DATA IMACH(1) /    5 /
C     DATA IMACH(2) /    6 /
C     DATA IMACH(3) /    5 /
C     DATA IMACH(4) /    6 /
C     DATA IMACH(5) /   32 /
C     DATA IMACH(6) /    4 /
C     DATA IMACH(7) /    2 /
C     DATA IMACH(8) /   31 /
C     DATA IMACH(9) /2147483647 /
C     DATA IMACH(10)/    2 /
C     DATA IMACH(11)/   24 /
C     DATA IMACH(12)/ -127 /
C     DATA IMACH(13)/  127 /
C     DATA IMACH(14)/   53 /
C     DATA IMACH(15)/ -1022 /
C     DATA IMACH(16)/  1023 /
C
C     MACHINE CONSTANTS FOR THE Z80 MICROPROCESSOR
C
C     DATA IMACH( 1) /     1/
C     DATA IMACH( 2) /     1/
C     DATA IMACH( 3) /     0/
C     DATA IMACH( 4) /     1/
C     DATA IMACH( 5) /    16/
C     DATA IMACH( 6) /     2/
C     DATA IMACH( 7) /     2/
C     DATA IMACH( 8) /    15/
C     DATA IMACH( 9) / 32767/
C     DATA IMACH(10) /     2/
C     DATA IMACH(11) /    24/
C     DATA IMACH(12) /  -127/
C     DATA IMACH(13) /   127/
C     DATA IMACH(14) /    56/
C     DATA IMACH(15) /  -127/
C     DATA IMACH(16) /   127/
C
C
C***FIRST EXECUTABLE STATEMENT  I1MACH
      IF (I .LT. 1  .OR.  I .GT. 16) GO TO 10
C
      I1MACH = IMACH(I)
      RETURN
C
   10 CONTINUE
      WRITE (UNIT = OUTPUT, FMT = 9000)
 9000 FORMAT ('1ERROR    1 IN I1MACH - I OUT OF BOUNDS')
C
C     CALL FDUMP
C
C
      STOP
      END
*DECK R1MACH
      REAL FUNCTION R1MACH(I)
C***BEGIN PROLOGUE  R1MACH
C***DATE WRITTEN   790101   (YYMMDD)
C***REVISION DATE  890213   (YYMMDD)
C***CATEGORY NO.  R1
C***KEYWORDS  LIBRARY=SLATEC,TYPE=SINGLE PRECISION(R1MACH-S D1MACH-D),
C             MACHINE CONSTANTS
C***AUTHOR  FOX, P. A., (BELL LABS)
C           HALL, A. D., (BELL LABS)
C           SCHRYER, N. L., (BELL LABS)
C***PURPOSE  Returns single precision machine dependent constants
C***DESCRIPTION
C
C   R1MACH can be used to obtain machine-dependent parameters
C   for the local machine environment.  It is a function
C   subroutine with one (input) argument, and can be called
C   as follows, for example
C
C        A = R1MACH(I)
C
C   where I=1,...,5.  The (output) value of A above is
C   determined by the (input) value of I.  The results for
C   various values of I are discussed below.
C
C   Single-Precision Machine Constants
C   R1MACH(1) = B**(EMIN-1), the smallest positive magnitude.
C   R1MACH(2) = B**EMAX*(1 - B**(-T)), the largest magnitude.
C   R1MACH(3) = B**(-T), the smallest relative spacing.
C   R1MACH(4) = B**(1-T), the largest relative spacing.
C   R1MACH(5) = LOG10(B)
C
C   Assume single precision numbers are represented in the T-digit,
C   base-B form
C
C              sign (B**E)*( (X(1)/B) + ... + (X(T)/B**T) )
C
C   where 0 .LE. X(I) .LT. B for I=1,...,T, 0 .LT. X(1), and
C   EMIN .LE. E .LE. EMAX.
C
C   The values of B, T, EMIN and EMAX are provided in I1MACH as
C   follows:
C   I1MACH(10) = B, the base.
C   I1MACH(11) = T, the number of base-B digits.
C   I1MACH(12) = EMIN, the smallest exponent E.
C   I1MACH(13) = EMAX, the largest exponent E.
C
C   To alter this function for a particular environment,
C   the desired set of DATA statements should be activated by
C   removing the C from column 1.  Also, the values of
C   R1MACH(1) - R1MACH(4) should be checked for consistency
C   with the local operating system.
C
C***REFERENCES  FOX, P.A., HALL, A.D., SCHRYER, N.L, *FRAMEWORK FOR
C                 A PORTABLE LIBRARY*, ACM TRANSACTIONS ON MATHE-
C                 MATICAL SOFTWARE, VOL. 4, NO. 2, JUNE 1978,
C                 PP. 177-188.
C***ROUTINES CALLED  XERROR
C***END PROLOGUE  R1MACH
C
      INTEGER SMALL(2)
      INTEGER LARGE(2)
      INTEGER RIGHT(2)
      INTEGER DIVER(2)
      INTEGER LOG10(2)
C
      REAL RMACH(5)
      SAVE RMACH
C
      EQUIVALENCE (RMACH(1),SMALL(1))
      EQUIVALENCE (RMACH(2),LARGE(1))
      EQUIVALENCE (RMACH(3),RIGHT(1))
      EQUIVALENCE (RMACH(4),DIVER(1))
      EQUIVALENCE (RMACH(5),LOG10(1))
C
C     MACHINE CONSTANTS FOR THE AMIGA
C     ABSOFT FORTRAN COMPILER USING THE 68020/68881 COMPILER OPTION
C
C     DATA SMALL(1) / Z'00800000' /
C     DATA LARGE(1) / Z'7F7FFFFF' /
C     DATA RIGHT(1) / Z'33800000' /
C     DATA DIVER(1) / Z'34000000' /
C     DATA LOG10(1) / Z'3E9A209B' /
C
C     MACHINE CONSTANTS FOR THE AMIGA
C     ABSOFT FORTRAN COMPILER USING SOFTWARE FLOATING POINT
C
C     DATA SMALL(1) / Z'00800000' /
C     DATA LARGE(1) / Z'7EFFFFFF' /
C     DATA RIGHT(1) / Z'33800000' /
C     DATA DIVER(1) / Z'34000000' /
C     DATA LOG10(1) / Z'3E9A209B' /
C
C     MACHINE CONSTANTS FOR THE APOLLO
C
C     DATA SMALL(1) / 16#00800000 /
C     DATA LARGE(1) / 16#7FFFFFFF /
C     DATA RIGHT(1) / 16#33800000 /
C     DATA DIVER(1) / 16#34000000 /
C     DATA LOG10(1) / 16#3E9A209B /
C
C     MACHINE CONSTANTS FOR THE BURROUGHS 1700 SYSTEM
C
C     DATA RMACH(1) / Z400800000 /
C     DATA RMACH(2) / Z5FFFFFFFF /
C     DATA RMACH(3) / Z4E9800000 /
C     DATA RMACH(4) / Z4EA800000 /
C     DATA RMACH(5) / Z500E730E8 /
C
C     MACHINE CONSTANTS FOR THE BURROUGHS 5700/6700/7700 SYSTEMS
C
C     DATA RMACH(1) / O1771000000000000 /
C     DATA RMACH(2) / O0777777777777777 /
C     DATA RMACH(3) / O1311000000000000 /
C     DATA RMACH(4) / O1301000000000000 /
C     DATA RMACH(5) / O1157163034761675 /
C
C     MACHINE CONSTANTS FOR THE CDC 170/180 SERIES USING NOS/VE
C
C     DATA RMACH(1) / Z"3001800000000000" /
C     DATA RMACH(2) / Z"4FFEFFFFFFFFFFFE" /
C     DATA RMACH(3) / Z"3FD2800000000000" /
C     DATA RMACH(4) / Z"3FD3800000000000" /
C     DATA RMACH(5) / Z"3FFF9A209A84FBCF" /
C
C     MACHINE CONSTANTS FOR THE CDC 6000/7000 SERIES
C
C     DATA RMACH(1) / 00564000000000000000B /
C     DATA RMACH(2) / 37767777777777777776B /
C     DATA RMACH(3) / 16414000000000000000B /
C     DATA RMACH(4) / 16424000000000000000B /
C     DATA RMACH(5) / 17164642023241175720B /
C
C     MACHINE CONSTANTS FOR THE CELERITY C1260
C
C     DATA SMALL(1) / Z'00800000' /
C     DATA LARGE(1) / Z'7F7FFFFF' /
C     DATA RIGHT(1) / Z'33800000' /
C     DATA DIVER(1) / Z'34000000' /
C     DATA LOG10(1) / Z'3E9A209B' /
C
C     MACHINE CONSTANTS FOR THE CONVEX C-1
C
C     DATA SMALL(1) / '00800000'X /
C     DATA LARGE(1) / '7FFFFFFF'X /
C     DATA RIGHT(1) / '34800000'X /
C     DATA DIVER(1) / '35000000'X /
C     DATA LOG10(1) / '3F9A209B'X /
C
C     MACHINE CONSTANTS FOR THE CRAY-1
C
C     DATA RMACH(1) / 200034000000000000000B /
C     DATA RMACH(2) / 577767777777777777776B /
C     DATA RMACH(3) / 377224000000000000000B /
C     DATA RMACH(4) / 377234000000000000000B /
C     DATA RMACH(5) / 377774642023241175720B /
C
C     MACHINE CONSTANTS FOR THE DATA GENERAL ECLIPSE S/200
C
C     NOTE - IT MAY BE APPROPRIATE TO INCLUDE THE FOLLOWING CARD -
C     STATIC RMACH(5)
C
C     DATA SMALL /    20K,       0 /
C     DATA LARGE / 77777K, 177777K /
C     DATA RIGHT / 35420K,       0 /
C     DATA DIVER / 36020K,       0 /
C     DATA LOG10 / 40423K,  42023K /
C
C     MACHINE CONSTANTS FOR THE HARRIS 220
C
C     DATA SMALL(1), SMALL(2) / '20000000, '00000201 /
C     DATA LARGE(1), LARGE(2) / '37777777, '00000177 /
C     DATA RIGHT(1), RIGHT(2) / '20000000, '00000352 /
C     DATA DIVER(1), DIVER(2) / '20000000, '00000353 /
C     DATA LOG10(1), LOG10(2) / '23210115, '00000377 /
C
C     MACHINE CONSTANTS FOR THE HONEYWELL 600/6000 SERIES
C
C     DATA RMACH(1) / O402400000000 /
C     DATA RMACH(2) / O376777777777 /
C     DATA RMACH(3) / O714400000000 /
C     DATA RMACH(4) / O716400000000 /
C     DATA RMACH(5) / O776464202324 /
C
C     MACHINE CONSTANTS FOR THE HP 2100
C     3 WORD DOUBLE PRECISION WITH FTN4
C
C     DATA SMALL(1), SMALL(2) / 40000B,       1 /
C     DATA LARGE(1), LARGE(2) / 77777B, 177776B /
C     DATA RIGHT(1), RIGHT(2) / 40000B,    325B /
C     DATA DIVER(1), DIVER(2) / 40000B,    327B /
C     DATA LOG10(1), LOG10(2) / 46420B,  46777B /
C
C     MACHINE CONSTANTS FOR THE HP 2100
C     4 WORD DOUBLE PRECISION WITH FTN4
C
C     DATA SMALL(1), SMALL(2) / 40000B,       1 /
C     DATA LARGE91), LARGE(2) / 77777B, 177776B /
C     DATA RIGHT(1), RIGHT(2) / 40000B,    325B /
C     DATA DIVER(1), DIVER(2) / 40000B,    327B /
C     DATA LOG10(1), LOG10(2) / 46420B,  46777B /
C
C     MACHINE CONSTANTS FOR THE HP 9000
C
C     DATA SMALL(1) / 00004000000B /
C     DATA LARGE(1) / 17677777777B /
C     DATA RIGHT(1) / 06340000000B /
C     DATA DIVER(1) / 06400000000B /
C     DATA LOG10(1) / 07646420233B /
C
C     MACHINE CONSTANTS FOR THE ELXSI 6400
C       ASSUMING REAL*4 IS THE DEFAULT REAL
C
C     DATA SMALL(1) / '00800000'X /
C     DATA LARGE(1) / '7F7FFFFF'X /
C     DATA RIGHT(1) / '33800000'X /
C     DATA DIVER(1) / '34000000'X /
C     DATA LOG10(1) / '3E9A209B'X /
C
C     MACHINE CONSTANTS FOR THE IBM 360/370 SERIES,
C     THE XEROX SIGMA 5/7/9, THE SEL SYSTEMS 85/86  AND
C     THE PERKIN ELMER (INTERDATA) 7/32.
C
C     DATA RMACH(1) / Z00100000 /
C     DATA RMACH(2) / Z7FFFFFFF /
C     DATA RMACH(3) / Z3B100000 /
C     DATA RMACH(4) / Z3C100000 /
C     DATA RMACH(5) / Z41134413 /
C
C     MACHINE CONSTANTS FOR THE IBM PC
C
      DATA SMALL(1) /     8420761 /
      DATA LARGE(1) /  2139081118 /
      DATA RIGHT(1) /   863997169 /
      DATA DIVER(1) /   872385777 /
      DATA LOG10(1) /  1050288283 /
C
C     MACHINE CONSTANTS FOR THE IBM RS 6000
C
C     DATA SMALL(1) / Z'00800000' /
C     DATA LARGE(1) / Z'7F7FFFFF' /
C     DATA RIGHT(1) / Z'33800000' /
C     DATA DIVER(1) / Z'34000000' /
C     DATA LOG10(1) / Z'3E9A209B' /
C
C     MACHINE CONSTANTS FOR THE PDP-10 (KA OR KI PROCESSOR)
C
C     DATA RMACH(1) / "000400000000 /
C     DATA RMACH(2) / "377777777777 /
C     DATA RMACH(3) / "146400000000 /
C     DATA RMACH(4) / "147400000000 /
C     DATA RMACH(5) / "177464202324 /
C
C     MACHINE CONSTANTS FOR PDP-11 FORTRAN SUPPORTING
C     32-BIT INTEGERS (EXPRESSED IN INTEGER AND OCTAL).
C
C     DATA SMALL(1) /    8388608 /
C     DATA LARGE(1) / 2147483647 /
C     DATA RIGHT(1) /  880803840 /
C     DATA DIVER(1) /  889192448 /
C     DATA LOG10(1) / 1067065499 /
C
C     DATA RMACH(1) / O00040000000 /
C     DATA RMACH(2) / O17777777777 /
C     DATA RMACH(3) / O06440000000 /
C     DATA RMACH(4) / O06500000000 /
C     DATA RMACH(5) / O07746420233 /
C
C     MACHINE CONSTANTS FOR PDP-11 FORTRAN SUPPORTING
C     16-BIT INTEGERS  (EXPRESSED IN INTEGER AND OCTAL).
C
C     DATA SMALL(1), SMALL(2) /   128,     0 /
C     DATA LARGE(1), LARGE(2) / 32767,    -1 /
C     DATA RIGHT(1), RIGHT(2) / 13440,     0 /
C     DATA DIVER(1), DIVER(2) / 13568,     0 /
C     DATA LOG10(1), LOG10(2) / 16282,  8347 /
C
C     DATA SMALL(1), SMALL(2) / O000200, O000000 /
C     DATA LARGE(1), LARGE(2) / O077777, O177777 /
C     DATA RIGHT(1), RIGHT(2) / O032200, O000000 /
C     DATA DIVER(1), DIVER(2) / O032400, O000000 /
C     DATA LOG10(1), LOG10(2) / O037632, O020233 /
C
C     MACHINE CONSTANTS FOR THE SILICON GRAPHICS IRIS
C
c     data rmach(1) / 1.17549 424 e-38 /
c     data rmach(2) / 3.40282 356 e+38 /
c     data rmach(3) / 1.19209 290 e-07 /
c     data rmach(4) / 2.38418 579 e-07 /
c     data rmach(5) / 0.30103 001 /
C
C     DATA SMALL(1) / Z'00800000' /
C     DATA LARGE(1) / Z'7F7FFFFF' /
C     DATA RIGHT(1) / Z'34000000' /
C     DATA DIVER(1) / Z'34800000' /
C     DATA LOG10(1) / Z'3E9A209B' /
C
C     MACHINE CONSTANTS FOR THE SUN
C
C     DATA SMALL(1) / Z'00800000' /
C     DATA LARGE(1) / Z'7F7FFFFF' /
C     DATA RIGHT(1) / Z'33800000' /
C     DATA DIVER(1) / Z'34000000' /
C     DATA LOG10(1) / Z'3E9A209B' /
C
C     MACHINE CONSTANTS FOR THE UNIVAC 1100 SERIES
C
C     DATA RMACH(1) / O000400000000 /
C     DATA RMACH(2) / O377777777777 /
C     DATA RMACH(3) / O146400000000 /
C     DATA RMACH(4) / O147400000000 /
C     DATA RMACH(5) / O177464202324 /
C
C     MACHINE CONSTANTS FOR THE VAX 11/780
C     (EXPRESSED IN INTEGER AND HEXADECIMAL)
C     THE HEX FORMAT BELOW MAY NOT BE SUITABLE FOR UNIX SYSTEMS
C     THE INTEGER FORMAT SHOULD BE OK FOR UNIX SYSTEMS
C
C     DATA SMALL(1) /       128 /
C     DATA LARGE(1) /    -32769 /
C     DATA RIGHT(1) /     13440 /
C     DATA DIVER(1) /     13568 /
C     DATA LOG10(1) / 547045274 /
C
C     DATA SMALL(1) / Z00000080 /
C     DATA LARGE(1) / ZFFFF7FFF /
C     DATA RIGHT(1) / Z00003480 /
C     DATA DIVER(1) / Z00003500 /
C     DATA LOG10(1) / Z209B3F9A /
C
C     MACHINE CONSTANTS FOR THE Z80 MICROPROCESSOR
C
C     DATA SMALL(1), SMALL(2) /     0,    256/
C     DATA LARGE(1), LARGE(2) /    -1,   -129/
C     DATA RIGHT(1), RIGHT(2) /     0,  26880/
C     DATA DIVER(1), DIVER(2) /     0,  27136/
C     DATA LOG10(1), LOG10(2) /  8347,  32538/
C
C
C***FIRST EXECUTABLE STATEMENT  R1MACH
      IF (I .LT. 1  .OR.  I .GT. 5)
     1   CALL XERROR ('R1MACH -- I OUT OF BOUNDS', 25, 1, 2)
C
      R1MACH = RMACH(I)
      RETURN
C
      END
*DECK D1MACH
      DOUBLE PRECISION FUNCTION D1MACH(I)
C***BEGIN PROLOGUE  D1MACH
C***DATE WRITTEN   750101   (YYMMDD)
C***REVISION DATE  890213   (YYMMDD)
C***CATEGORY NO.  R1
C***KEYWORDS  LIBRARY=SLATEC,TYPE=DOUBLE PRECISION(R1MACH-S D1MACH-D),
C             MACHINE CONSTANTS
C***AUTHOR  FOX, P. A., (BELL LABS)
C           HALL, A. D., (BELL LABS)
C           SCHRYER, N. L., (BELL LABS)
C***PURPOSE  Returns double precision machine dependent constants
C***DESCRIPTION
C
C   D1MACH can be used to obtain machine-dependent parameters
C   for the local machine environment.  It is a function
C   subprogram with one (input) argument, and can be called
C   as follows, for example
C
C        D = D1MACH(I)
C
C   where I=1,...,5.  The (output) value of D above is
C   determined by the (input) value of I.  The results for
C   various values of I are discussed below.
C
C   D1MACH( 1) = B**(EMIN-1), the smallest positive magnitude.
C   D1MACH( 2) = B**EMAX*(1 - B**(-T)), the largest magnitude.
C   D1MACH( 3) = B**(-T), the smallest relative spacing.
C   D1MACH( 4) = B**(1-T), the largest relative spacing.
C   D1MACH( 5) = LOG10(B)
C
C   Assume double precision numbers are represented in the T-digit,
C   base-B form
C
C              sign (B**E)*( (X(1)/B) + ... + (X(T)/B**T) )
C
C   where 0 .LE. X(I) .LT. B for I=1,...,T, 0 .LT. X(1), and
C   EMIN .LE. E .LE. EMAX.
C
C   The values of B, T, EMIN and EMAX are provided in I1MACH as
C   follows:
C   I1MACH(10) = B, the base.
C   I1MACH(14) = T, the number of base-B digits.
C   I1MACH(15) = EMIN, the smallest exponent E.
C   I1MACH(16) = EMAX, the largest exponent E.
C
C   To alter this function for a particular environment,
C   the desired set of DATA statements should be activated by
C   removing the C from column 1.  Also, the values of
C   D1MACH(1) - D1MACH(4) should be checked for consistency
C   with the local operating system.
C
C***REFERENCES  FOX P.A., HALL A.D., SCHRYER N.L.,*FRAMEWORK FOR A
C                 PORTABLE LIBRARY*, ACM TRANSACTIONS ON MATHEMATICAL
C                 SOFTWARE, VOL. 4, NO. 2, JUNE 1978, PP. 177-188.
C***ROUTINES CALLED  XERROR
C***END PROLOGUE  D1MACH
C
      INTEGER SMALL(4)
      INTEGER LARGE(4)
      INTEGER RIGHT(4)
      INTEGER DIVER(4)
      INTEGER LOG10(4)
C
      DOUBLE PRECISION DMACH(5)
      SAVE DMACH
C
      EQUIVALENCE (DMACH(1),SMALL(1))
      EQUIVALENCE (DMACH(2),LARGE(1))
      EQUIVALENCE (DMACH(3),RIGHT(1))
      EQUIVALENCE (DMACH(4),DIVER(1))
      EQUIVALENCE (DMACH(5),LOG10(1))
C
C     MACHINE CONSTANTS FOR THE AMIGA
C     ABSOFT FORTRAN COMPILER USING THE 68020/68881 COMPILER OPTION
C
C     DATA SMALL(1), SMALL(2) / Z'00100000', Z'00000000' /
C     DATA LARGE(1), LARGE(2) / Z'7FEFFFFF', Z'FFFFFFFF' /
C     DATA RIGHT(1), RIGHT(2) / Z'3CA00000', Z'00000000' /
C     DATA DIVER(1), DIVER(2) / Z'3CB00000', Z'00000000' /
C     DATA LOG10(1), LOG10(2) / Z'3FD34413', Z'509F79FF' /
C
C     MACHINE CONSTANTS FOR THE AMIGA
C     ABSOFT FORTRAN COMPILER USING SOFTWARE FLOATING POINT
C
C     DATA SMALL(1), SMALL(2) / Z'00100000', Z'00000000' /
C     DATA LARGE(1), LARGE(2) / Z'7FDFFFFF', Z'FFFFFFFF' /
C     DATA RIGHT(1), RIGHT(2) / Z'3CA00000', Z'00000000' /
C     DATA DIVER(1), DIVER(2) / Z'3CB00000', Z'00000000' /
C     DATA LOG10(1), LOG10(2) / Z'3FD34413', Z'509F79FF' /
C
C     MACHINE CONSTANTS FOR THE APOLLO
C
C     DATA SMALL(1), SMALL(2) / 16#00100000, 16#00000000 /
C     DATA LARGE(1), LARGE(2) / 16#7FFFFFFF, 16#FFFFFFFF /
C     DATA RIGHT(1), RIGHT(2) / 16#3CA00000, 16#00000000 /
C     DATA DIVER(1), DIVER(2) / 16#3CB00000, 16#00000000 /
C     DATA LOG10(1), LOG10(2) / 16#3FD34413, 16#509F79FF /
C
C     MACHINE CONSTANTS FOR THE BURROUGHS 1700 SYSTEM
C
C     DATA SMALL(1) / ZC00800000 /
C     DATA SMALL(2) / Z000000000 /
C     DATA LARGE(1) / ZDFFFFFFFF /
C     DATA LARGE(2) / ZFFFFFFFFF /
C     DATA RIGHT(1) / ZCC5800000 /
C     DATA RIGHT(2) / Z000000000 /
C     DATA DIVER(1) / ZCC6800000 /
C     DATA DIVER(2) / Z000000000 /
C     DATA LOG10(1) / ZD00E730E7 /
C     DATA LOG10(2) / ZC77800DC0 /
C
C     MACHINE CONSTANTS FOR THE BURROUGHS 5700 SYSTEM
C
C     DATA SMALL(1) / O1771000000000000 /
C     DATA SMALL(2) / O0000000000000000 /
C     DATA LARGE(1) / O0777777777777777 /
C     DATA LARGE(2) / O0007777777777777 /
C     DATA RIGHT(1) / O1461000000000000 /
C     DATA RIGHT(2) / O0000000000000000 /
C     DATA DIVER(1) / O1451000000000000 /
C     DATA DIVER(2) / O0000000000000000 /
C     DATA LOG10(1) / O1157163034761674 /
C     DATA LOG10(2) / O0006677466732724 /
C
C     MACHINE CONSTANTS FOR THE BURROUGHS 6700/7700 SYSTEMS
C
C     DATA SMALL(1) / O1771000000000000 /
C     DATA SMALL(2) / O7770000000000000 /
C     DATA LARGE(1) / O0777777777777777 /
C     DATA LARGE(2) / O7777777777777777 /
C     DATA RIGHT(1) / O1461000000000000 /
C     DATA RIGHT(2) / O0000000000000000 /
C     DATA DIVER(1) / O1451000000000000 /
C     DATA DIVER(2) / O0000000000000000 /
C     DATA LOG10(1) / O1157163034761674 /
C     DATA LOG10(2) / O0006677466732724 /
C
C     MACHINE CONSTANTS FOR THE CDC 170/180 SERIES USING NOS/VE
C
C     DATA SMALL(1) / Z"3001800000000000" /
C     DATA SMALL(2) / Z"3001000000000000" /
C     DATA LARGE(1) / Z"4FFEFFFFFFFFFFFE" /
C     DATA LARGE(2) / Z"4FFE000000000000" /
C     DATA RIGHT(1) / Z"3FD2800000000000" /
C     DATA RIGHT(2) / Z"3FD2000000000000" /
C     DATA DIVER(1) / Z"3FD3800000000000" /
C     DATA DIVER(2) / Z"3FD3000000000000" /
C     DATA LOG10(1) / Z"3FFF9A209A84FBCF" /
C     DATA LOG10(2) / Z"3FFFF7988F8959AC" /
C
C     MACHINE CONSTANTS FOR THE CDC 6000/7000 SERIES
C
C     DATA SMALL(1) / 00564000000000000000B /
C     DATA SMALL(2) / 00000000000000000000B /
C     DATA LARGE(1) / 37757777777777777777B /
C     DATA LARGE(2) / 37157777777777777777B /
C     DATA RIGHT(1) / 15624000000000000000B /
C     DATA RIGHT(2) / 00000000000000000000B /
C     DATA DIVER(1) / 15634000000000000000B /
C     DATA DIVER(2) / 00000000000000000000B /
C     DATA LOG10(1) / 17164642023241175717B /
C     DATA LOG10(2) / 16367571421742254654B /
C
C     MACHINE CONSTANTS FOR THE CELERITY C1260
C
C     DATA SMALL(1), SMALL(2) / Z'00100000', Z'00000000' /
C     DATA LARGE(1), LARGE(2) / Z'7FEFFFFF', Z'FFFFFFFF' /
C     DATA RIGHT(1), RIGHT(2) / Z'3CA00000', Z'00000000' /
C     DATA DIVER(1), DIVER(2) / Z'3CB00000', Z'00000000' /
C     DATA LOG10(1), LOG10(2) / Z'3FD34413', Z'509F79FF' /
C
C     MACHINE CONSTANTS FOR THE CONVEX C-1
C
C     DATA SMALL(1), SMALL(2) / '00100000'X,'00000000'X /
C     DATA LARGE(1), LARGE(2) / '7FFFFFFF'X,'FFFFFFFF'X /
C     DATA RIGHT(1), RIGHT(2) / '3CC00000'X,'00000000'X /
C     DATA DIVER(1), DIVER(2) / '3CD00000'X,'00000000'X /
C     DATA LOG10(1), LOG10(2) / '3FF34413'X,'509F79FF'X /
C
C     MACHINE CONSTANTS FOR THE CRAY-1
C
C     DATA SMALL(1) / 201354000000000000000B /
C     DATA SMALL(2) / 000000000000000000000B /
C     DATA LARGE(1) / 577767777777777777777B /
C     DATA LARGE(2) / 000007777777777777774B /
C     DATA RIGHT(1) / 376434000000000000000B /
C     DATA RIGHT(2) / 000000000000000000000B /
C     DATA DIVER(1) / 376444000000000000000B /
C     DATA DIVER(2) / 000000000000000000000B /
C     DATA LOG10(1) / 377774642023241175717B /
C     DATA LOG10(2) / 000007571421742254654B /
C
C     MACHINE CONSTANTS FOR THE DATA GENERAL ECLIPSE S/200
C
C     NOTE - IT MAY BE APPROPRIATE TO INCLUDE THE FOLLOWING CARD -
C     STATIC DMACH(5)
C
C     DATA SMALL /    20K, 3*0 /
C     DATA LARGE / 77777K, 3*177777K /
C     DATA RIGHT / 31420K, 3*0 /
C     DATA DIVER / 32020K, 3*0 /
C     DATA LOG10 / 40423K, 42023K, 50237K, 74776K /
C
C     MACHINE CONSTANTS FOR THE ELXSI 6400
C     (ASSUMING REAL*8 IS THE DEFAULT DOUBLE PRECISION)
C
C     DATA SMALL(1), SMALL(2) / '00100000'X,'00000000'X /
C     DATA LARGE(1), LARGE(2) / '7FEFFFFF'X,'FFFFFFFF'X /
C     DATA RIGHT(1), RIGHT(2) / '3CB00000'X,'00000000'X /
C     DATA DIVER(1), DIVER(2) / '3CC00000'X,'00000000'X /
C     DATA LOG10(1), LOG10(2) / '3FD34413'X,'509F79FF'X /
C
C     MACHINE CONSTANTS FOR THE HARRIS 220
C
C     DATA SMALL(1), SMALL(2) / '20000000, '00000201 /
C     DATA LARGE(1), LARGE(2) / '37777777, '37777577 /
C     DATA RIGHT(1), RIGHT(2) / '20000000, '00000333 /
C     DATA DIVER(1), DIVER(2) / '20000000, '00000334 /
C     DATA LOG10(1), LOG10(2) / '23210115, '10237777 /
C
C     MACHINE CONSTANTS FOR THE HONEYWELL 600/6000 SERIES
C
C     DATA SMALL(1), SMALL(2) / O402400000000, O000000000000 /
C     DATA LARGE(1), LARGE(2) / O376777777777, O777777777777 /
C     DATA RIGHT(1), RIGHT(2) / O604400000000, O000000000000 /
C     DATA DIVER(1), DIVER(2) / O606400000000, O000000000000 /
C     DATA LOG10(1), LOG10(2) / O776464202324, O117571775714 /
C
C     MACHINE CONSTANTS FOR THE HP 2100
C     THREE WORD DOUBLE PRECISION OPTION WITH FTN4
C
C     DATA SMALL(1), SMALL(2), SMALL(3) / 40000B,       0,       1 /
C     DATA LARGE(1), LARGE(2), LARGE(3) / 77777B, 177777B, 177776B /
C     DATA RIGHT(1), RIGHT(2), RIGHT(3) / 40000B,       0,    265B /
C     DATA DIVER(1), DIVER(2), DIVER(3) / 40000B,       0,    276B /
C     DATA LOG10(1), LOG10(2), LOG10(3) / 46420B,  46502B,  77777B /
C
C     MACHINE CONSTANTS FOR THE HP 2100
C     FOUR WORD DOUBLE PRECISION OPTION WITH FTN4
C
C     DATA SMALL(1), SMALL(2) /  40000B,       0 /
C     DATA SMALL(3), SMALL(4) /       0,       1 /
C     DATA LARGE(1), LARGE(2) /  77777B, 177777B /
C     DATA LARGE(3), LARGE(4) / 177777B, 177776B /
C     DATA RIGHT(1), RIGHT(2) /  40000B,       0 /
C     DATA RIGHT(3), RIGHT(4) /       0,    225B /
C     DATA DIVER(1), DIVER(2) /  40000B,       0 /
C     DATA DIVER(3), DIVER(4) /       0,    227B /
C     DATA LOG10(1), LOG10(2) /  46420B,  46502B /
C     DATA LOG10(3), LOG10(4) /  76747B, 176377B /
C
C     MACHINE CONSTANTS FOR THE HP 9000
C
C     DATA SMALL(1), SMALL(2) / 00040000000B, 00000000000B /
C     DATA LARGE(1), LARGE(2) / 17737777777B, 37777777777B /
C     DATA RIGHT(1), RIGHT(2) / 07454000000B, 00000000000B /
C     DATA DIVER(1), DIVER(2) / 07460000000B, 00000000000B /
C     DATA LOG10(1), LOG10(2) / 07764642023B, 12047674777B /
C
C     MACHINE CONSTANTS FOR THE IBM 360/370 SERIES,
C     THE XEROX SIGMA 5/7/9, THE SEL SYSTEMS 85/86, AND
C     THE PERKIN ELMER (INTERDATA) 7/32.
C
C     DATA SMALL(1), SMALL(2) / Z00100000, Z00000000 /
C     DATA LARGE(1), LARGE(2) / Z7FFFFFFF, ZFFFFFFFF /
C     DATA RIGHT(1), RIGHT(2) / Z33100000, Z00000000 /
C     DATA DIVER(1), DIVER(2) / Z34100000, Z00000000 /
C     DATA LOG10(1), LOG10(2) / Z41134413, Z509F79FF /
C
C     MACHINE CONSTANTS FOR THE IBM PC
C     ASSUMES THAT ALL ARITHMETIC IS DONE IN DOUBLE PRECISION
C     ON 8088, I.E., NOT IN 80 BIT FORM FOR THE 8087.
C
      DATA SMALL(1),SMALL(2) /  2002288515,    1050897 /
      DATA LARGE(1),LARGE(2) /  1487780761, 2146426097 /
      DATA RIGHT(1),RIGHT(2) / -1209488034, 1017118298 /
      DATA DIVER(1),DIVER(2) / -1209488034, 1018166874 /
      DATA LOG10(1),LOG10(2) /  1352628735, 1070810131 /
C
C     MACHINE CONSTANTS FOR THE IBM RS 6000
C
C     DATA SMALL(1), SMALL(2) / Z'00100000', Z'00000000' /
C     DATA LARGE(1), LARGE(2) / Z'7FEFFFFF', Z'FFFFFFFF' /
C     DATA RIGHT(1), RIGHT(2) / Z'3CA00000', Z'00000000' /
C     DATA DIVER(1), DIVER(2) / Z'3CB00000', Z'00000000' /
C     DATA LOG10(1), LOG10(2) / Z'3FD34413', Z'509F79FF' /
C
C     MACHINE CONSTANTS FOR THE PDP-10 (KA PROCESSOR)
C
C     DATA SMALL(1), SMALL(2) / "033400000000, "000000000000 /
C     DATA LARGE(1), LARGE(2) / "377777777777, "344777777777 /
C     DATA RIGHT(1), RIGHT(2) / "113400000000, "000000000000 /
C     DATA DIVER(1), DIVER(2) / "114400000000, "000000000000 /
C     DATA LOG10(1), LOG10(2) / "177464202324, "144117571776 /
C
C     MACHINE CONSTANTS FOR THE PDP-10 (KI PROCESSOR)
C
C     DATA SMALL(1), SMALL(2) / "000400000000, "000000000000 /
C     DATA LARGE(1), LARGE(2) / "377777777777, "377777777777 /
C     DATA RIGHT(1), RIGHT(2) / "103400000000, "000000000000 /
C     DATA DIVER(1), DIVER(2) / "104400000000, "000000000000 /
C     DATA LOG10(1), LOG10(2) / "177464202324, "476747767461 /
C
C     MACHINE CONSTANTS FOR PDP-11 FORTRAN SUPPORTING
C     32-BIT INTEGERS (EXPRESSED IN INTEGER AND OCTAL).
C
C     DATA SMALL(1), SMALL(2) /    8388608,           0 /
C     DATA LARGE(1), LARGE(2) / 2147483647,          -1 /
C     DATA RIGHT(1), RIGHT(2) /  612368384,           0 /
C     DATA DIVER(1), DIVER(2) /  620756992,           0 /
C     DATA LOG10(1), LOG10(2) / 1067065498, -2063872008 /
C
C     DATA SMALL(1), SMALL(2) / O00040000000, O00000000000 /
C     DATA LARGE(1), LARGE(2) / O17777777777, O37777777777 /
C     DATA RIGHT(1), RIGHT(2) / O04440000000, O00000000000 /
C     DATA DIVER(1), DIVER(2) / O04500000000, O00000000000 /
C     DATA LOG10(1), LOG10(2) / O07746420232, O20476747770 /
C
C     MACHINE CONSTANTS FOR PDP-11 FORTRAN SUPPORTING
C     16-BIT INTEGERS (EXPRESSED IN INTEGER AND OCTAL).
C
C     DATA SMALL(1), SMALL(2) /    128,      0 /
C     DATA SMALL(3), SMALL(4) /      0,      0 /
C     DATA LARGE(1), LARGE(2) /  32767,     -1 /
C     DATA LARGE(3), LARGE(4) /     -1,     -1 /
C     DATA RIGHT(1), RIGHT(2) /   9344,      0 /
C     DATA RIGHT(3), RIGHT(4) /      0,      0 /
C     DATA DIVER(1), DIVER(2) /   9472,      0 /
C     DATA DIVER(3), DIVER(4) /      0,      0 /
C     DATA LOG10(1), LOG10(2) /  16282,   8346 /
C     DATA LOG10(3), LOG10(4) / -31493, -12296 /
C
C     DATA SMALL(1), SMALL(2) / O000200, O000000 /
C     DATA SMALL(3), SMALL(4) / O000000, O000000 /
C     DATA LARGE(1), LARGE(2) / O077777, O177777 /
C     DATA LARGE(3), LARGE(4) / O177777, O177777 /
C     DATA RIGHT(1), RIGHT(2) / O022200, O000000 /
C     DATA RIGHT(3), RIGHT(4) / O000000, O000000 /
C     DATA DIVER(1), DIVER(2) / O022400, O000000 /
C     DATA DIVER(3), DIVER(4) / O000000, O000000 /
C     DATA LOG10(1), LOG10(2) / O037632, O020232 /
C     DATA LOG10(3), LOG10(4) / O102373, O147770 /
C
C     MACHINE CONSTANTS FOR THE SILICON GRAPHICS IRIS
C
c     data dmach(1) / 2.22507 38585 072012 d-308 /
c     data dmach(2) / 1.79769 31348 623158 d+308 /
c     data dmach(3) / 2.22044 60492 503131 d-16  /
c     data dmach(4) / 4.44089 20985 006262 d-16  /
c     data dmach(5) / 0.30102 99956 639812       /
C
C     DATA SMALL(1), SMALL(2) / Z'00100000',Z'00000000' /
C     DATA LARGE(1), LARGE(2) / Z'7FEFFFFF',Z'FFFFFFFF' /
C     DATA RIGHT(1), RIGHT(2) / Z'3CB00000',Z'00000000' /
C     DATA DIVER(1), DIVER(2) / Z'3CC00000',Z'00000000' /
C     DATA LOG10(1), LOG10(2) / Z'3FD34413',Z'509F79FF' /
C
C     MACHINE CONSTANTS FOR THE SUN
C
C     from SLATEC CML committee - work for Sun3, Sun4, and Sparc
C
C     DATA SMALL(1), SMALL(2) / Z'00100000', Z'00000000' /
C     DATA LARGE(1), LARGE(2) / Z'7FEFFFFF', Z'FFFFFFFF' /
C     DATA RIGHT(1), RIGHT(2) / Z'3CA00000', Z'00000000' /
C     DATA DIVER(1), DIVER(2) / Z'3CB00000', Z'00000000' /
C     DATA LOG10(1), LOG10(2) / Z'3FD34413', Z'509F79FF' /
C
C     from Sun Microsystems - work for Sun 386i
C
C     DATA SMALL(1), SMALL(2) / Z'00000000', Z'00100000' /
C     DATA LARGE(1), LARGE(2) / Z'FFFFFFFF', Z'7FEFFFFF' /
C     DATA RIGHT(1), RIGHT(2) / Z'00000000', Z'3CA00000' /
C     DATA DIVER(1), DIVER(2) / Z'00000000', Z'3CB00000' /
C     DATA LOG10(1), LOG10(2) / Z'509F79FF', Z'3FD34413' /
C
C     MACHINE CONSTANTS FOR THE UNIVAC 1100 SERIES FTN COMPILER
C
C     DATA SMALL(1), SMALL(2) / O000040000000, O000000000000 /
C     DATA LARGE(1), LARGE(2) / O377777777777, O777777777777 /
C     DATA RIGHT(1), RIGHT(2) / O170540000000, O000000000000 /
C     DATA DIVER(1), DIVER(2) / O170640000000, O000000000000 /
C     DATA LOG10(1), LOG10(2) / O177746420232, O411757177572 /
C
C     MACHINE CONSTANTS FOR VAX 11/780
C     (EXPRESSED IN INTEGER AND HEXADECIMAL)
C     THE HEX FORMAT BELOW MAY NOT BE SUITABLE FOR UNIX SYSYEMS
C     THE INTEGER FORMAT SHOULD BE OK FOR UNIX SYSTEMS
C
C     DATA SMALL(1), SMALL(2) /        128,           0 /
C     DATA LARGE(1), LARGE(2) /     -32769,          -1 /
C     DATA RIGHT(1), RIGHT(2) /       9344,           0 /
C     DATA DIVER(1), DIVER(2) /       9472,           0 /
C     DATA LOG10(1), LOG10(2) /  546979738,  -805796613 /
C
C     DATA SMALL(1), SMALL(2) / Z00000080, Z00000000 /
C     DATA LARGE(1), LARGE(2) / ZFFFF7FFF, ZFFFFFFFF /
C     DATA RIGHT(1), RIGHT(2) / Z00002480, Z00000000 /
C     DATA DIVER(1), DIVER(2) / Z00002500, Z00000000 /
C     DATA LOG10(1), LOG10(2) / Z209A3F9A, ZCFF884FB /
C
C     MACHINE CONSTANTS FOR VAX 11/780 (G-FLOATING)
C     (EXPRESSED IN INTEGER AND HEXADECIMAL)
C     THE HEX FORMAT BELOW MAY NOT BE SUITABLE FOR UNIX SYSYEMS
C     THE INTEGER FORMAT SHOULD BE OK FOR UNIX SYSTEMS
C
C     DATA SMALL(1), SMALL(2) /         16,           0 /
C     DATA LARGE(1), LARGE(2) /     -32769,          -1 /
C     DATA RIGHT(1), RIGHT(2) /      15552,           0 /
C     DATA DIVER(1), DIVER(2) /      15568,           0 /
C     DATA LOG10(1), LOG10(2) /  1142112243, 2046775455 /
C
C     DATA SMALL(1), SMALL(2) / Z00000010, Z00000000 /
C     DATA LARGE(1), LARGE(2) / ZFFFF7FFF, ZFFFFFFFF /
C     DATA RIGHT(1), RIGHT(2) / Z00003CC0, Z00000000 /
C     DATA DIVER(1), DIVER(2) / Z00003CD0, Z00000000 /
C     DATA LOG10(1), LOG10(2) / Z44133FF3, Z79FF509F /
C
C
C***FIRST EXECUTABLE STATEMENT  D1MACH
      IF (I .LT. 1  .OR.  I .GT. 5)
     1   CALL XERROR ('D1MACH -- I OUT OF BOUNDS', 25, 1, 2)
C
      D1MACH = DMACH(I)
      RETURN
C
      END
      SUBROUTINE FDUMP
C***BEGIN PROLOGUE  FDUMP
C***DATE WRITTEN   790801   (YYMMDD)
C***REVISION DATE  861211   (YYMMDD)
C***CATEGORY NO.  R3
C***KEYWORDS  LIBRARY=SLATEC(XERROR),TYPE=ALL(FDUMP-A),ERROR
C***AUTHOR  JONES, R. E., (SNLA)
C***PURPOSE  Symbolic dump (should be locally written).
C***DESCRIPTION
C
C        ***Note*** Machine Dependent Routine
C        FDUMP is intended to be replaced by a locally written
C        version which produces a symbolic dump.  Failing this,
C        it should be replaced by a version which prints the
C        subprogram nesting list.  Note that this dump must be
C        printed on each of up to five files, as indicated by the
C        XGETUA routine.  See XSETUA and XGETUA for details.
C
C     Written by Ron Jones, with SLATEC Common Math Library Subcommittee
C***REFERENCES  (NONE)
C***ROUTINES CALLED  (NONE)
C***END PROLOGUE  FDUMP
C***FIRST EXECUTABLE STATEMENT  FDUMP
      RETURN
      END
      SUBROUTINE XERROR(MESS,NMESS,NERR,LEVEL)
C***BEGIN PROLOGUE  XERROR
C***DATE WRITTEN   880401   (YYMMDD)
C***REVISION DATE  880401   (YYMMDD)
C***CATEGORY NO.  R3C 
C***KEYWORDS  ERROR,XERROR PACKAGE
C***AUTHOR  AMOS, D. E., (SNLA) 
C***PURPOSE  TO PRINT AN ERROR MESSAGE
C***DESCRIPTION 
C 
C     ABSTRACT
C        XERROR IS A DUMMY SLATEC LIBRARY ROUTINE TO PRINT AN ERROR 
C        MESSAGE.  THE CALL LIST IS CONSISTENT WITH THE CORRESPONDING 
C        SLATEC LIBRARY SUBROUTINE. 
C 
C     DESCRIPTION OF PARAMETERS 
C      --INPUT--
C        MESSG - THE HOLLERITH MESSAGE TO BE PROCESSED. 
C        NMESSG- THE ACTUAL NUMBER OF CHARACTERS IN MESSG.
C        NERR  - THE ERROR NUMBER ASSOCIATED WITH THIS MESSAGE. 
C                (A DUMMY VARIABLE NOT USED, BUT NEEDED TO BE COMPATIBLE
C                 WITH THE CORRESPONDING SLATEC ROUTINE)
C        LEVEL - ERROR CATEGORY.
C                (A DUMMY VARIABLE NOT USED, BUT NEEDED TO BE COMPATIBLE
C                 WITH THE CORRESPONDING SLATEC ROUTINE)
C 
C***REFERENCES  JONES R.E., KAHANER D.K., 'XERROR, THE SLATEC ERROR-
C                 HANDLING PACKAGE', SAND82-0800, SANDIA LABORATORIES,
C                 1982. 
C***ROUTINES CALLED  NONE 
C***END PROLOGUE  XERROR
      CHARACTER*(*) MESS
      INTEGER NMESS,NERR,LEVEL,NN,NR,K,I,KMIN,MIN 
      IF (NMESS.LE.0) THEN
        PRINT *,' IN XERROR, NMESS IS OUT OF RANGE' 
      ELSE
        NN=NMESS/70 
        NR=NMESS-70*NN
        IF(NR.NE.0) NN=NN+1 
        K=1 
        PRINT 900 
  900   FORMAT(/) 
        DO 10 I=1,NN
          KMIN=MIN(K+69,NMESS)
          PRINT *, MESS(K:KMIN) 
          K=K+70
   10   CONTINUE
        RETURN
      END IF
      END 
      SUBROUTINE XERRWV(MESSG,NMESSG,NERR,LEVEL,NI,I1,I2,NR,R1,R2)
C***BEGIN PROLOGUE  XERRWV
C***DATE WRITTEN   880401   (YYMMDD)
C***REVISION DATE  880401   (YYMMDD)
C***CATEGORY NO.  R3C 
C***KEYWORDS  ERROR,XERROR PACKAGE
C***AUTHOR  AMOS, D. E., (SNLA) 
C***PURPOSE  PROCESS AN ERROR MESSAGE ALLOWING 2 INTEGER AND 2 REAL 
C            VALUES TO BE INCLUDED IN THE MESSAGE.
C***DESCRIPTION 
C 
C     ABSTRACT
C        XERRWV IS A DUMMY SLATEC LIBRARY ROUTINE TO PRINT AN ERROR 
C        MESSAGE AND PRINT UP TO 2 INTEGER VARIABLES AND 2 REAL 
C        VARIABLES. THE CALL LIST IS CONSISTENT WITH THE CORRESPONDING
C        SLATEC LIBRARY SUBROUTINE. 
C 
C     DESCRIPTION OF PARAMETERS 
C      --INPUT--
C        MESSG - THE HOLLERITH MESSAGE TO BE PROCESSED. 
C        NMESSG- THE ACTUAL NUMBER OF CHARACTERS IN MESSG.
C        NERR  - THE ERROR NUMBER ASSOCIATED WITH THIS MESSAGE. 
C                (A DUMMY VARIABLE NOT USED, BUT NEEDED TO BE COMPATIBLE
C                 WITH THE CORRESPONDING SLATEC ROUTINE)
C        LEVEL - ERROR CATEGORY.
C                (A DUMMY VARIABLE NOT USED, BUT NEEDED TO BE COMPATIBLE
C                 WITH THE CORRESPONDING SLATEC ROUTINE)
C        NI    - NUMBER OF INTEGER VALUES TO BE PRINTED. (0 TO 2) 
C        I1    - FIRST INTEGER VALUE. 
C        I2    - SECOND INTEGER VALUE.
C        NR    - NUMBER OF REAL VALUES TO BE PRINTED. (0 TO 2)
C        R1    - FIRST REAL VALUE.
C        R2    - SECOND REAL VALUE. 
C 
C***REFERENCES  JONES R.E., KAHANER D.K., 'XERROR, THE SLATEC ERROR-
C                 HANDLING PACKAGE', SAND82-0800, SANDIA LABORATORIES,
C                 1982. 
C***ROUTINES CALLED  XERROR 
C***END PROLOGUE  XERRWV
      CHARACTER*(*) MESSG 
      INTEGER I1,I2,NI,NR,NMESSG,NERR,LEVEL 
      REAL R1,R2
      CALL XERROR(MESSG,NMESSG,NERR,LEVEL)
      IF (NI.LT.0 .OR. NI.GT.2) THEN
        PRINT *,' IN XERRWV, NI IS OUT OF RANGE'
      ELSE
        IF (NI.NE.0) THEN 
          IF (NI.EQ.1) THEN 
            PRINT *,' I1 = ',I1 
          ELSE
            PRINT *,' I1 , I2 =',I1,I2
          END IF
        END IF
      END IF
      IF (NR.LT.0 .OR. NR.GT.2) THEN
        PRINT *,' IN XERRWV, NR IS OUT OF RANGE'
      ELSE
        IF (NR.NE.0) THEN 
          IF (NR.EQ.1) THEN 
            PRINT *,' R1 =',R1
          ELSE
            PRINT *,' R1 , R2 =',R1,R2
          END IF
        END IF
      END IF
      RETURN
      END 
      SUBROUTINE DEXINT(X, N, KODE, M, TOL, EN, NZ, IERR) 
C***BEGIN PROLOGUE  DEXINT
C***DATE WRITTEN   820601   (YYMMDD)
C***REVISION DATE  820601   (YYMMDD)
C***CATEGORY NO.  B5E 
C***KEYWORDS  EXPONENTIAL INTEGRAL
C***AUTHOR  AMOS, DONALD E., SANDIA NATIONAL LABORATORIES 
C***PURPOSE  DEXINT COMPUTES THE EXPONENTIAL INTEGRAL 
C***DESCRIPTION 
C 
C                   * A DOUBLE PRECISION ROUTINE *
C         DEXINT COMPUTES M MEMBER SEQUENCES OF EXPONENTIAL INTEGRALS 
C         E(N+K,X), K=0,1,...,M-1 FOR N.GE.1 AND X.GE.0.0D0.  THE 
C         EXPONENTIAL INTEGRAL IS DEFINED BY
C 
C         E(N,X)=INTEGRAL ON (1,INFINITY) OF EXP(-XT)/T**N
C 
C         WHERE X=0.0 AND N=1 CANNOT OCCUR SIMULTANEOUSLY. FORMULAS 
C         AND NOTATION ARE FOUND IN THE NBS HANDBOOK OF MATHEMATICAL
C         FUNCTIONS (REF. 1). 
C 
C         EXINT IS THE SINGLE PRECISION VERSION OF DEXINT 
C 
C         INPUT     X AND TOL ARE DOUBLE PRECISION
C           X       X.GT.0.0D0 FOR N=1 AND  X.GE.0.0D0 FOR N.GE.2 
C           N       ORDER OF THE FIRST MEMBER OF THE SEQUENCE, N.GE.1 
C                   (X=0.0D0 AND N=1 IS AN ERROR) 
C           KODE    A SELECTION PARAMETER FOR SCALED VALUES 
C                   KODE=1   RETURNS        E(N+K,X), K=0,1,...,M-1.
C                       =2   RETURNS EXP(X)*E(N+K,X), K=0,1,...,M-1.
C           M       NUMBER OF EXPONENTIAL INTEGRALS IN THE SEQUENCE,
C                   M.GE.1
C           TOL     RELATIVE ACCURACY WANTED, ETOL.LE.TOL.LE.0.1D0
C                   ETOL = MAXIMUM OF DOUBLE PRECISION UNIT ROUNDOFF
C                   (D1MACH(4)) AND 1.0D-18 
C 
C         OUTPUT    EN IS A DOUBLE PRECISION VECTOR 
C           EN      A VECTOR OF DIMENSION AT LEAST M CONTAINING VALUES
C                   EN(K) = E(N+K-1,X) OR EXP(X)*E(N+K-1,X), K=1,M
C                   DEPENDING ON KODE 
C           NZ      UNDERFLOW INDICATOR 
C                   NZ=0   A NORMAL RETURN
C                   NZ=M   X EXCEEDS XLIM AND AN UNDERFLOW OCCURS.
C                          EN(K)=0.0D0 , K=1,M RETURNED ON KODE=1 
C           IERR    ERROR FLAG
C                   IERR=0, NORMAL RETURN, COMPUTATION COMPLETED
C                   IERR=1, INPUT ERROR,   NO COMPUTATION 
C                   IERR=2, ERROR,         NO COMPUTATION 
C                           ALGORITHM TERMINATION CONDITION NOT MET 
C 
C***REFERENCES  HANDBOOK OF MATHEMATICAL FUNCTIONS BY M. ABRAMOWITZ 
C                 AND I. A. STEGUN, NBS AMS SERIES 55, U.S. DEPT. OF
C                 COMMERCE, 1955. 
C 
C               COMPUTATION OF EXPONENTIAL INTEGRALS BY D. E. AMOS, 
C                 ACM TRANS. MATH SOFTWARE, 6, 1980,PP. 365-377, 420-428
C***ROUTINES CALLED  I1MACH,D1MACH,DPSIXN 
C***END PROLOGUE  DEXINT
C 
      DOUBLE PRECISION A, AA, AAMS, AH, AK, AT, B, BK, BT, CC, CNORM, 
     * CT, EM, EMX, EN, ETOL, FNM, FX, PT, P1, P2, S, TOL, TX, X, XCUT, 
     * XLIM, XTOL, Y, YT, Y1, Y2, FN, FKN, XOVR, RIX
      DOUBLE PRECISION D1MACH, DPSIXN 
      INTEGER I, IC, ICASE, ICT, IERR, IK, IND, IX, I1M, JSET, K, KK, 
     * KN, KODE, KS, M, ML, MU, N, ND, NM, NZ, I1P
      INTEGER I1MACH
      DIMENSION EN(M), A(99), B(99), Y(2)
C 
      DATA XCUT                  /     2.0D0                 /
C 
C***FIRST EXECUTABLE STATEMENT  DEXINT
      IERR=0
      NZ=0
      ETOL = DMAX1(D1MACH(4),0.5D-18) 
      IF (X.LT.0.0D0) IERR=1
      IF (N.LT.1) IERR=1
      IF (KODE.LT.1 .OR. KODE.GT.2) IERR=1
      IF (M.LT.1) IERR=1
      IF (TOL.LT.ETOL .OR. TOL.GT.0.1D0) IERR=1 
      IF (X.EQ.0.0D0 .AND. N.EQ.1) IERR=1 
      IF(IERR.NE.0) RETURN
      I1M = -I1MACH(15) 
      PT = 2.3026D0*DBLE(FLOAT(I1M))*D1MACH(5)
      XLIM = PT - 6.907755D0
      BT = PT + DBLE(FLOAT(N+M-1))
      IF (BT.GT.1000.0D0) XLIM = PT - DLOG(BT)
C-----------------------------------------------------------------------
C     SET OVERFLOW LIMIT ON X. THE UNDERFLOW LIMIT IS 1.0D0/XOVR.
C     TEST EXPONENTS ON SMALLEST AND LARGEST NUMBERS TO BE SURE THE
C     RECIPROCALS EXIST. EXPONENTS ARE I1MACH(15) AND I1MACH(16).
C-----------------------------------------------------------------------
      I1P = I1MACH(16)
      IF(I1M.LT.I1P) THEN
        XOVR = 1.0D-3/D1MACH(1)
      ELSE
        XOVR = 1.0D-3*D1MACH(2)
      ENDIF
      IF (X.GT.XOVR) GO TO 105
      IF (X.GT.XCUT) GO TO 100
C     IF (X.EQ.0.0D0 .AND. N.GT.1) GO TO 80
      IF (X.LT.1.0D0/XOVR) THEN
        IF ( N.GT.1 ) GO TO 80
        IERR=1
        RETURN
      ENDIF
C-----------------------------------------------------------------------
C     SERIES FOR E(N,X) FOR X.LE.XCUT 
C-----------------------------------------------------------------------
C      TX = X + 0.5D0
C      IX = INT(SNGL(TX))
       RIX = ANINT(X)
C-----------------------------------------------------------------------
C     ICASE=1 MEANS INTEGER CLOSEST TO X IS 2 AND N=1 
C     ICASE=2 MEANS INTEGER CLOSEST TO X IS 0,1, OR 2 AND N.GE.2
C-----------------------------------------------------------------------
      ICASE = 2 
      IF (RIX.GT.DBLE(FLOAT(N))) ICASE = 1
      NM = N - ICASE + 1
      ND = NM + 1 
      IND = 3 - ICASE 
      MU = M - IND
      ML = 1
      KS = ND 
      FNM = DBLE(FLOAT(NM)) 
      S = 0.0D0 
      XTOL = 3.0D0*TOL
      IF (ND.EQ.1) GO TO 10 
      XTOL = 0.3333D0*TOL 
      S = 1.0D0/FNM 
   10 CONTINUE
      AA = 1.0D0
      AK = 1.0D0
      IC = 35 
      IF (X.LT.ETOL) IC = 1 
      DO 50 I=1,IC
        AA = -AA*X/AK 
        IF (I.EQ.NM) GO TO 30 
        S = S - AA/(AK-FNM) 
        IF (DABS(AA).LE.XTOL*DABS(S)) GO TO 20
        AK = AK + 1.0D0 
        GO TO 50
   20   CONTINUE
        IF (I.LT.2) GO TO 40
        IF (ND-2.GT.I .OR. I.GT.ND-1) GO TO 60
        AK = AK + 1.0D0 
        GO TO 50
   30   S = S + AA*(-DLOG(X)+DPSIXN(ND))
        XTOL = 3.0D0*TOL
   40   AK = AK + 1.0D0 
   50 CONTINUE
      IF (IC.NE.1) GO TO 340
   60 IF (ND.EQ.1) S = S + (-DLOG(X)+DPSIXN(1)) 
      IF (KODE.EQ.2) S = S*DEXP(X)
      EN(1) = S 
      EMX = 1.0D0 
      IF (M.EQ.1) GO TO 70
      EN(IND) = S 
      AA = DBLE(FLOAT(KS))
      IF (KODE.EQ.1) EMX = DEXP(-X) 
      GO TO (220, 240), ICASE 
   70 IF (ICASE.EQ.2) RETURN
      IF (KODE.EQ.1) EMX = DEXP(-X) 
      EN(1) = (EMX-S)/X 
      RETURN
   80 CONTINUE
      DO 90 I=1,M 
        EN(I) = 1.0D0/DBLE(FLOAT(N+I-2))
   90 CONTINUE
      RETURN
C-----------------------------------------------------------------------
C     BACKWARD RECURSIVE MILLER ALGORITHM FOR 
C              E(N,X)=DEXP(-X)*(X**(N-1))*U(N,N,X)
C     WITH RECURSION AWAY FROM N=INTEGER CLOSEST TO X.
C     U(A,B,X) IS THE SECOND CONFLUENT HYPERGEOMETRIC FUNCTION
C-----------------------------------------------------------------------
  100 CONTINUE
      EMX = 1.0D0 
      IF (KODE.EQ.2) GO TO 130
      IF (X.LE.XLIM) GO TO 120
  105 CONTINUE
      NZ = M
      DO 110 I=1,M
        EN(I) = 0.0D0 
  110 CONTINUE
      RETURN
  120 EMX = DEXP(-X)
  130 CONTINUE
C     TX = X + 0.5D0
C     IX = INT(SNGL(TX))
      RIX = ANINT(X)
      KN = N + M - 1
      FKN = DBLE(FLOAT(KN))
      IF (FKN.LE.RIX) GO TO 140
      FN = DBLE(FLOAT(N))
      IF (FN.LT.RIX .AND. RIX.LT.FKN) GO TO 170
      IF (FN.GE.RIX) GO TO 160
      GO TO 340 
  140 ICASE = 1 
      KS = KN 
      ML = M - 1
      MU = -1 
      IND = M 
      IF (KN.GT.1) GO TO 180
  150 KS = 2
      ICASE = 3 
      GO TO 180 
  160 ICASE = 2 
      IND = 1 
      KS = N
      MU = M - 1
      IF (N.GT.1) GO TO 180 
      IF (KN.EQ.1) GO TO 150
      RIX = 2.0D0
  170 CONTINUE
      IX=INT(SNGL(RIX))
      ICASE = 1
      KS = IX 
      ML = IX - N 
      IND = ML + 1
      MU = KN - IX
  180 CONTINUE
      IK = KS/2 
      AH = DBLE(FLOAT(IK))
      JSET = 1 + KS - (IK+IK) 
C-----------------------------------------------------------------------
C     START COMPUTATION FOR 
C              EN(IND) = C*U( A , A ,X)    JSET=1 
C              EN(IND) = C*U(A+1,A+1,X)    JSET=2 
C     FOR AN EVEN INTEGER A.
C-----------------------------------------------------------------------
      IC = 0
      AA = AH + AH
      AAMS = AA - 1.0D0 
      AAMS = AAMS*AAMS
      TX = X + X
      FX = TX + TX
      AK = AH 
      XTOL = TOL
      IF (TOL.LE.1.0D-3) XTOL = 20.0D0*TOL
      CT = AAMS + FX*AH 
C-----------------------------------------------------------------------
C     PREVENT OVERFLOW IN DEN OF EM = (AH+1.0D0)/((X+AA)*XTOL*SQRT(CT))
C-----------------------------------------------------------------------
      P1 = (AH+1.0D0)/((X+AA)*XTOL)
      P2 = SQRT(CT)
      IF( P1 .LT. P2/XOVR ) THEN
        Y(1) = 1.0D0/X
        Y(2) = 1.0D0/X
        GO TO 205
      ENDIF
      EM = P1/P2
      BK = AA 
      CC = AH*AH
C-----------------------------------------------------------------------
C     FORWARD RECURSION FOR P(IC),P(IC+1) AND INDEX IC FOR BACKWARD 
C     RECURSION 
C-----------------------------------------------------------------------
      P1 = 0.0D0
      P2 = 1.0D0
  190 CONTINUE
      IF (IC.EQ.99) GO TO 340 
      IC = IC + 1 
      AK = AK + 1.0D0 
      AT = BK/(BK+AK+CC+DBLE(FLOAT(IC)))
      BK = BK + AK + AK 
      A(IC) = AT
      BT = (AK+AK+X)/(AK+1.0D0) 
      B(IC) = BT
      PT = P2 
      P2 = BT*P2 - AT*P1
      P1 = PT 
      CT = CT + FX
      EM = EM*AT*(1.0D0-TX/CT)
      IF (EM*(AK+1.0D0).GT.P1*P1) GO TO 190 
      ICT = IC
      KK = IC + 1 
      BT = TX/(CT+FX) 
      Y2 = (BK/(BK+CC+DBLE(FLOAT(KK))))*(P1/P2)*(1.0D0-BT+0.375D0*BT*BT)
      Y1 = 1.0D0
C-----------------------------------------------------------------------
C     BACKWARD RECURRENCE FOR 
C              Y1=             C*U( A ,A,X) 
C              Y2= C*(A/(1+A/2))*U(A+1,A,X) 
C-----------------------------------------------------------------------
      DO 200 K=1,ICT
        KK = KK - 1 
        YT = Y1 
        Y1 = (B(KK)*Y1-Y2)/A(KK)
        Y2 = YT 
  200 CONTINUE
C-----------------------------------------------------------------------
C     THE CONTIGUOUS RELATION 
C              X*U(B,C+1,X)=(C-B)*U(B,C,X)+U(B-1,C,X) 
C     WITH  B=A+1 , C=A IS USED FOR 
C              Y(2) = C * U(A+1,A+1,X)
C     X IS INCORPORATED INTO THE NORMALIZING RELATION 
C-----------------------------------------------------------------------
      PT = Y2/Y1
      CNORM = 1.0E0 - PT*(AH+1.0E0)/AA
      Y(1) = 1.0E0/(CNORM*AA+X) 
      Y(2) = CNORM*Y(1) 
  205 CONTINUE
      IF (ICASE.EQ.3) GO TO 210 
      EN(IND) = EMX*Y(JSET) 
      IF (M.EQ.1) RETURN
      AA = DBLE(FLOAT(KS))
      GO TO (220, 240), ICASE 
C-----------------------------------------------------------------------
C     RECURSION SECTION  N*E(N+1,X) + X*E(N,X)=EMX
C-----------------------------------------------------------------------
  210 EN(1) = EMX*(1.0E0-Y(1))/X
      RETURN
  220 K = IND - 1 
      DO 230 I=1,ML 
        AA = AA - 1.0D0 
        EN(K) = (EMX-AA*EN(K+1))/X
        K = K - 1 
  230 CONTINUE
      IF (MU.LE.0) RETURN 
      AA = DBLE(FLOAT(KS))
  240 K = IND 
      DO 250 I=1,MU 
        EN(K+1) = (EMX-X*EN(K))/AA
        AA = AA + 1.0D0 
        K = K + 1 
  250 CONTINUE
      RETURN
  340 CONTINUE
      IERR=2
      RETURN
      END 
      DOUBLE PRECISION FUNCTION DPSIXN(N) 
C***BEGIN PROLOGUE  DPSIXN
C***REFER TO  DEXINT,DBSKIN 
C 
C     THIS SUBROUTINE RETURNS VALUES OF PSI(X)=DERIVATIVE OF LOG
C     GAMMA(X), X.GT.0.0 AT INTEGER ARGUMENTS. A TABLE LOOK-UP IS 
C     PERFORMED FOR N.LE.100, AND THE ASYMPTOTIC EXPANSION IS 
C     EVALUATED FOR N.GT.100. 
C 
C***ROUTINES CALLED  D1MACH 
C***END PROLOGUE  DPSIXN
C 
      INTEGER N, K
      DOUBLE PRECISION AX, B, C, FN, RFN2, TRM, S, WDTOL
      DOUBLE PRECISION D1MACH 
      DIMENSION B(6), C(100)
C 
C             DPSIXN(N), N = 1,100
      DATA C(1), C(2), C(3), C(4), C(5), C(6), C(7), C(8), C(9), C(10), 
     1     C(11), C(12), C(13), C(14), C(15), C(16), C(17), C(18),
     2     C(19), C(20), C(21), C(22), C(23), C(24)/
     3    -5.77215664901532861D-01,     4.22784335098467139D-01,
     4     9.22784335098467139D-01,     1.25611766843180047D+00,
     5     1.50611766843180047D+00,     1.70611766843180047D+00,
     6     1.87278433509846714D+00,     2.01564147795561000D+00,
     7     2.14064147795561000D+00,     2.25175258906672111D+00,
     8     2.35175258906672111D+00,     2.44266167997581202D+00,
     9     2.52599501330914535D+00,     2.60291809023222227D+00,
     A     2.67434666166079370D+00,     2.74101332832746037D+00,
     B     2.80351332832746037D+00,     2.86233685773922507D+00,
     C     2.91789241329478063D+00,     2.97052399224214905D+00,
     D     3.02052399224214905D+00,     3.06814303986119667D+00,
     E     3.11359758531574212D+00,     3.15707584618530734D+00/
      DATA C(25), C(26), C(27), C(28), C(29), C(30), C(31), C(32),
     1     C(33), C(34), C(35), C(36), C(37), C(38), C(39), C(40),
     2     C(41), C(42), C(43), C(44), C(45), C(46), C(47), C(48)/
     3     3.19874251285197401D+00,     3.23874251285197401D+00,
     4     3.27720405131351247D+00,     3.31424108835054951D+00,
     5     3.34995537406483522D+00,     3.38443813268552488D+00,
     6     3.41777146601885821D+00,     3.45002953053498724D+00,
     7     3.48127953053498724D+00,     3.51158256083801755D+00,
     8     3.54099432554389990D+00,     3.56956575411532847D+00,
     9     3.59734353189310625D+00,     3.62437055892013327D+00,
     A     3.65068634839381748D+00,     3.67632737403484313D+00,
     B     3.70132737403484313D+00,     3.72571761793728215D+00,
     C     3.74952714174680596D+00,     3.77278295570029433D+00,
     D     3.79551022842756706D+00,     3.81773245064978928D+00,
     E     3.83947158108457189D+00,     3.86074817682925274D+00/
      DATA C(49), C(50), C(51), C(52), C(53), C(54), C(55), C(56),
     1     C(57), C(58), C(59), C(60), C(61), C(62), C(63), C(64),
     2     C(65), C(66), C(67), C(68), C(69), C(70), C(71), C(72)/
     3     3.88158151016258607D+00,     3.90198967342789220D+00,
     4     3.92198967342789220D+00,     3.94159751656514710D+00,
     5     3.96082828579591633D+00,     3.97969621032421822D+00,
     6     3.99821472884273674D+00,     4.01639654702455492D+00,
     7     4.03425368988169777D+00,     4.05179754953082058D+00,
     8     4.06903892884116541D+00,     4.08598808138353829D+00,
     9     4.10265474805020496D+00,     4.11904819067315578D+00,
     A     4.13517722293122029D+00,     4.15105023880423617D+00,
     B     4.16667523880423617D+00,     4.18205985418885155D+00,
     C     4.19721136934036670D+00,     4.21213674247469506D+00,
     D     4.22684262482763624D+00,     4.24133537845082464D+00,
     E     4.25562109273653893D+00,     4.26970559977879245D+00/
      DATA C(73), C(74), C(75), C(76), C(77), C(78), C(79), C(80),
     1     C(81), C(82), C(83), C(84), C(85), C(86), C(87), C(88),
     2     C(89), C(90), C(91), C(92), C(93), C(94), C(95), C(96)/
     3     4.28359448866768134D+00,     4.29729311880466764D+00,
     4     4.31080663231818115D+00,     4.32413996565151449D+00,
     5     4.33729786038835659D+00,     4.35028487337536958D+00,
     6     4.36310538619588240D+00,     4.37576361404398366D+00,
     7     4.38826361404398366D+00,     4.40060929305632934D+00,
     8     4.41280441500754886D+00,     4.42485260777863319D+00,
     9     4.43675736968339510D+00,     4.44852207556574804D+00,
     A     4.46014998254249223D+00,     4.47164423541605544D+00,
     B     4.48300787177969181D+00,     4.49424382683587158D+00,
     C     4.50535493794698269D+00,     4.51634394893599368D+00,
     D     4.52721351415338499D+00,     4.53796620232542800D+00,
     E     4.54860450019776842D+00,     4.55913081598724211D+00/
      DATA C(97), C(98), C(99), C(100)/ 
     1     4.56954748265390877D+00,     4.57985676100442424D+00,
     2     4.59006084263707730D+00,     4.60016185273808740D+00/
C             COEFFICIENTS OF ASYMPTOTIC EXPANSION
      DATA B(1), B(2), B(3), B(4), B(5), B(6)/
     1     8.33333333333333333D-02,    -8.33333333333333333D-03,
     2     3.96825396825396825D-03,    -4.16666666666666666D-03,
     3     7.57575757575757576D-03,    -2.10927960927960928D-02/
C 
      IF (N.GT.100) GO TO 10
      DPSIXN = C(N) 
      RETURN
   10 CONTINUE
      WDTOL = DMAX1(D1MACH(4),1.0D-18)
      FN = DBLE(FLOAT(N)) 
      AX = 1.0D0
      S = -0.5D0/FN 
      IF (DABS(S).LE.WDTOL) GO TO 30
      RFN2 = 1.0D0/(FN*FN)
      DO 20 K=1,6 
        AX = AX*RFN2
        TRM = -B(K)*AX
        IF (DABS(TRM).LT.WDTOL) GO TO 30
        S = S + TRM 
   20 CONTINUE
   30 CONTINUE
      DPSIXN = S + DLOG(FN) 
      RETURN
      END
      SUBROUTINE DHEXINT(X, FNH, KODE, M, TOL, EN, NZ, IERR)
C***BEGIN PROLOGUE  DEXINT
C***DATE WRITTEN   000704   (YYMMDD)
C***REVISION DATE  020425   (YYMMDD)
C***CATEGORY NO.  B5E 
C***KEYWORDS  EXPONENTIAL INTEGRAL
C***AUTHOR  AMOS, DONALD E. 
C***PURPOSE  DHEXINT COMPUTES THE EXPONENTIAL INTEGRAL OF HALF ODD ORDER 
C***DESCRIPTION 
C 
C                   * A DOUBLE PRECISION ROUTINE *
C         DHEXINT COMPUTES M MEMBER SEQUENCES OF EXPONENTIAL INTEGRALS 
C         E(FNH+K,X), K=0,1,...,M-1 FOR HALF ODD ORDERS FNH.GE.0.5 AND
C         X.GE.0.0D0.  THE EXPONENTIAL INTEGRAL IS DEFINED BY
C 
C         E(FNH,X)=INTEGRAL ON (1,INFINITY) OF EXP(-XT)/T**FNH
C 
C         WHERE X=0.0 AND FHN=0.5 CANNOT OCCUR SIMULTANEOUSLY. FORMULAS 
C         AND NOTATION ARE FOUND IN THE NBS HANDBOOK OF MATHEMATICAL
C         FUNCTIONS (REF. 1). THIS IS AN ADAPTATION OF REF. 2 TO HALF
C         ODD ORDERS. E(0.5,X)=SQRT(PI)*ERFC(Z)/Z,  Z=SQRT(X), WITH
C         FORWARD RECURRENCE IS USED IN PLACE OF THE SERIES FOR INTEGER
C         ORDERS.
C 
C         INPUT     X, FNH AND TOL ARE DOUBLE PRECISION
C           X       X.GT.0.0D0 FOR FNH=0.5 AND  X.GE.0.0D0 FOR FNH.GE.1.5
C           FNH     ORDER OF THE FIRST MEMBER OF THE SEQUENCE, FNH.GE.0.5 
C           KODE    A SELECTION PARAMETER FOR SCALED VALUES 
C                   KODE=1   RETURNS        E(FNH+K,X), K=0,1,...,M-1.
C                       =2   RETURNS EXP(X)*E(FNH+K,X), K=0,1,...,M-1.
C           M       NUMBER OF EXPONENTIAL INTEGRALS IN THE SEQUENCE,
C                   M.GE.1
C           TOL     RELATIVE ACCURACY WANTED, ETOL.LE.TOL.LE.0.1D0
C                   ETOL = MAXIMUM OF DOUBLE PRECISION UNIT ROUNDOFF
C                   (D1MACH(4)) AND 1.0D-18 
C 
C         OUTPUT    EN IS A DOUBLE PRECISION VECTOR 
C           EN      A VECTOR OF DIMENSION AT LEAST M CONTAINING VALUES
C                   EN(K) = E(FNH+K-1,X) OR EXP(X)*E(FNH+K-1,X), K=1,M
C                   DEPENDING ON KODE 
C           NZ      UNDERFLOW INDICATOR 
C                   NZ=0   A NORMAL RETURN
C                   NZ=M   X EXCEEDS XLIM AND AN UNDERFLOW OCCURS.
C                          EN(K)=0.0D0 , K=1,M RETURNED ON KODE=1 
C           IERR    ERROR FLAG
C                   IERR=0, NORMAL RETURN, COMPUTATION COMPLETED
C                   IERR=1, INPUT ERROR,   NO COMPUTATION 
C                   IERR=2, ERROR,         NO COMPUTATION 
C                           ALGORITHM TERMINATION CONDITION NOT MET 
C 
C***REFERENCES  HANDBOOK OF MATHEMATICAL FUNCTIONS BY M. ABRAMOWITZ 
C                 AND I. A. STEGUN, NBS AMS SERIES 55, U.S. DEPT. OF
C                 COMMERCE, 1955.
C 
C               COMPUTATION OF EXPONENTIAL INTEGRALS BY D. E. AMOS, 
C                 ACM TRANS. MATH SOFTWARE, 6, 1980,PP. 365-377, 420-428
C
C               FOLDER 18 BY D. E. AMOS EVALUATION OF E(NU,X), G(NU,X)=
C                 -DE(NU,X)/DNU FOR INTEGER AND HALF ODD INTEGER ORDERS
C                 WITH APPLICATION TO I(N,B,T)= INTEGRAL ON (T,INFINITY)
C                 OF EXP(-B^2*X^2)*LN(X)/X**N, N=1,2,3,... 
C
C***ROUTINES CALLED  I1MACH,D1MACH,DRERF 
C***END PROLOGUE  DHEXINT
C      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C 
      DOUBLE PRECISION A, AA, AAMS, AH, AK, AT, B, BK, BT, CC, CNORM, 
     * CT, EM, EMX, EN, ERF2, ETOL, FX, FN, FNH, FKN, PT, P1, P2, RIX,
     * RTZ, RTPI, TOL, TX, X, XCUT, XLIM, XTOL, YT, Y1, Y2, XOVR
      DOUBLE PRECISION D1MACH, DRERF 
      INTEGER I, IC, ICASE, ICT, IERR, IND, IX, I1M, K, KK, 
     * KODE, KODEP, M, ML, MU, N, NH, NZ, I1P
      INTEGER I1MACH
      DIMENSION EN(M), A(99), B(99)
C 
      DATA XCUT                  /     2.0D0                 /
      DATA RTPI                  /1.772453850905516D0/
C 
C***FIRST EXECUTABLE STATEMENT  DHEXINT
      IERR=0
      NZ=0
      ETOL = DMAX1(D1MACH(4),0.5D-18) 
      IF (X.LT.0.0D0) IERR=1
      IF (FNH.LT.0.5D0) IERR=1
      FN=FNH-0.5D0
      IF (FN-DBLE(FLOAT(INT(FN))).NE.0.0D0) IERR=1 
      IF (KODE.LT.1 .OR. KODE.GT.2) IERR=1
      IF (M.LT.1) IERR=1
      IF (TOL.LT.ETOL .OR. TOL.GT.0.1D0) IERR=1 
      IF (X.EQ.0.0D0 .AND. FNH.EQ.0.5D0) IERR=1 
      IF (IERR.NE.0) RETURN
      I1M = -I1MACH(15) 
      PT = 2.3026D0*DBLE(FLOAT(I1M))*D1MACH(5)
      XLIM = PT - 6.907755D0
      BT = PT + DBLE(FLOAT(N+M-1))
      IF (BT.GT.1000.0D0) XLIM = PT - DLOG(BT)
C-----------------------------------------------------------------------
C     SET OVERFLOW LIMIT ON X. THE UNDERFLOW LIMIT IS 1.0D0/XOVR.
C     TEST EXPONENTS ON SMALLEST AND LARGEST NUMBERS TO BE SURE THE
C     RECIPROCALS EXIST. EXPONENTS ARE I1MACH(15) AND I1MACH(16).
C-----------------------------------------------------------------------
      I1P = I1MACH(16)
      IF(I1M.LT.I1P) THEN
        XOVR = 1.0D-3/D1MACH(1)
      ELSE
        XOVR = 1.0D-3*D1MACH(2)
      ENDIF
      IF (X.GT.XOVR) GO TO 105
      IF (X.LT.1.0D0/XOVR) THEN
        IF (FNH.GT.0.5D0) GOTO 80
        IERR=1
        RETURN
      ENDIF
C-----------------------------------------------------------------------
C     START SEQUENCE AT E(0.5,X) FOR X.LE.XCUT 
C-----------------------------------------------------------------------
      IF (X.GT.XCUT) GOTO 100
      IF (FNH.GT.20.0D0) GOTO 100
      RTZ=DSQRT(X)
      IF (KODE.EQ.1) THEN
        EMX=DEXP(-X)
      ELSE
        EMX=1.0D0
      ENDIF
      IF (FNH.EQ.0.5D0) THEN
        NZERO=0
        KODEP=KODE+1
        ERF2=DRERF(RTZ,KODEP,NZERO)
        Y1=RTPI*ERF2/RTZ
        EN(1)=Y1
        IF (M.EQ.1) RETURN
C        FNH=1.5 HERE E(3/2,X)=2*RTPI*IERFC(SQRT(X))
        CALL DIERFC(RTZ,KODE,YT,IERR)
        Y1=(YT+YT)*RTPI
        EN(2)=Y1
        IF(M.EQ.2) RETURN
        IND = 2
        MU = M-2
        AA = 1.5D0
      ELSE
        CALL DIERFC(RTZ,KODE,YT,IERR)
        Y1=(YT+YT)*RTPI
        AA=1.5D0
        NH=INT(FN)-1
        IF (NH.LE.0) GOTO 11
        DO 10 K=1,NH
          Y2=(EMX-X*Y1)/AA
          Y1=Y2
          YI1=(Y2-X*YI1)/AA
          AA=AA+1.0D0
   10   CONTINUE
   11   CONTINUE
        EN(1)=Y1
        IF(M.EQ.1) RETURN
        IND = 1
        MU = M-1
      ENDIF
      GOTO 240
   80 CONTINUE
      DO 90 I=1,M 
        EN(I) = 1.0D0/(FNH-1.0D0+DBLE(FLOAT(I-1)))
   90 CONTINUE
      RETURN
C-----------------------------------------------------------------------
C     BACKWARD RECURSIVE MILLER ALGORITHM FOR 
C              E(N,X)=DEXP(-X)*(X**(N-1))*U(N,N,X)
C     WITH RECURSION AWAY FROM N=INTEGER CLOSEST TO X.
C     U(A,B,X) IS THE SECOND CONFLUENT HYPERGEOMETRIC FUNCTION
C-----------------------------------------------------------------------
  100 CONTINUE
      EMX = 1.0D0 
      IF (KODE.EQ.2) GO TO 130
      IF (X.LE.XLIM) GO TO 120
  105 CONTINUE
      IF (FNH.EQ.0.5D0) THEN
        IERR=1
        RETURN
      ENDIF
      NZ = M
      DO 110 I=1,M
        EN(I) = 0.0D0 
  110 CONTINUE
      RETURN
  120 EMX = DEXP(-X)
  130 CONTINUE
      MF=0
      RIX = ANINT(X)+0.5D0
      FKN = FNH+DBLE(FLOAT(M-1))
      IF (FKN.LE.RIX) GO TO 140
      IF (FNH.LT.RIX .AND. RIX.LT.FKN) GO TO 170
      IF (FNH.GE.RIX) GO TO 160
      GO TO 340 
  140 ICASE = 1
C     WHOLE SEQUENCE LIES TO THE LEFT OF RIX. START AT FKS, RECUR DOWN
C     ADJUST INDEX UP TO GET BETTER CONVERGENCE AND MORE ACCURACY
C     RECUR DOWN MF TIMES (UNINDEXED) TO GET BACK TO ORIGINAL INDEX
      FKS=FKN
      IF(FKN.LT.2.5D0) THEN
        FKS=2.5D0
        MF=INT(SNGL(2.5D0-FKN))
      ENDIF
      ML = M - 1
      MU = -1 
      IND = M
      GOTO 180
  160 ICASE = 2
C     WHOLE SEQUENCE LIES TO THE RIGHT OF RIX. START AT FNH, RECUR UP
      IND = 1 
      FKS=FNH
      MU = M - 1
      ML = -1
      GOTO 180
  170 CONTINUE
C     RIX LIES IN THE RANGE OF THE SEQUENCE. START AT RIX, RECUR UP &
C     DOWN
      IX=INT(SNGL(RIX))
      ICASE = 1
      N = INT(FN)
      ML = IX - N      
      IND = ML + 1
      FKS = FNH+DBLE(FLOAT(IND-1))
      MU = M - IND 
  180 CONTINUE
       AH = FKS/2.0D0
C-----------------------------------------------------------------------
C     START COMPUTATION FOR 
C              EN(IND) = C*U( A , A ,X)     
C-----------------------------------------------------------------------
      IC = 0
      AA = AH + AH
      AAMS = AA - 1.0D0 
      AAMS = AAMS*AAMS
      TX = X + X
      FX = TX + TX
      AK = AH 
      XTOL = TOL
      IF (TOL.LE.1.0D-3) XTOL = 20.0D0*TOL
      CT = AAMS + FX*AH 
C-----------------------------------------------------------------------
C     PREVENT OVERFLOW IN DEN OF EM = (AH+1.0D0)/((X+AA)*XTOL*SQRT(CT))
C-----------------------------------------------------------------------
      P1 = (AH+1.0D0)/((X+AA)*XTOL)
      P2 = SQRT(CT)
      IF( P1 .LT. P2/XOVR ) THEN
        Y1 = 1.0D0/X
        GO TO 205
      ENDIF
      EM = P1/P2
      BK = AA 
      CC = AH*AH
C-----------------------------------------------------------------------
C     FORWARD RECURSION FOR P(IC),P(IC+1) AND INDEX IC FOR BACKWARD 
C     RECURSION 
C-----------------------------------------------------------------------
      P1 = 0.0D0
      P2 = 1.0D0
  190 CONTINUE
      IF (IC.EQ.99) GO TO 340 
      IC = IC + 1 
      AK = AK + 1.0D0 
      AT = BK/(BK+AK+CC+DBLE(FLOAT(IC)))
      BK = BK + AK + AK 
      A(IC) = AT
      BT = (AK+AK+X)/(AK+1.0D0) 
      B(IC) = BT
      PT = P2 
      P2 = BT*P2 - AT*P1
      P1 = PT 
      CT = CT + FX
      EM = EM*AT*(1.0D0-TX/CT)
      IF (EM*(AK+1.0D0).GT.P1*P1) GO TO 190
      ICT = IC
      KK = IC + 1 
      BT = TX/(CT+FX) 
      Y2 = (BK/(BK+CC+DBLE(FLOAT(KK))))*(P1/P2)*(1.0D0-BT+0.375D0*BT*BT)
      Y1 = 1.0D0
C-----------------------------------------------------------------------
C     BACKWARD RECURRENCE FOR 
C              Y1=             C*U( A ,A,X) 
C              Y2= C*(A/(1+A/2))*U(A+1,A,X) 
C-----------------------------------------------------------------------
      DO 200 K=1,ICT
        KK = KK - 1 
        YT = Y1 
        Y1 = (B(KK)*Y1-Y2)/A(KK)
        Y2 = YT 
  200 CONTINUE
C-----------------------------------------------------------------------
C     THE CONTIGUOUS RELATION 
C              X*U(B,C+1,X)=(C-B)*U(B,C,X)+U(B-1,C,X) 
C     WITH  B=A+1 , C=A IS USED FOR 
C              Y(2) = C * U(A+1,A+1,X)
C     X IS INCORPORATED INTO THE NORMALIZING RELATION 
C-----------------------------------------------------------------------
      PT = Y2/Y1
      CNORM = 1.0D0 - PT*(AH+1.0D0)/AA
      Y1 = 1.0D0/(CNORM*AA+X) 
  205 CONTINUE
      IF (MF.NE.0) THEN
        AK=FKS
        DO 206 J=1,MF
          AK=AK-1.0D0
          Y1=(1.0D0-AK*Y1)/X
  206   CONTINUE
        FKS=AK
      ENDIF
      EN(IND) = EMX*Y1
      IF (M.EQ.1) RETURN
      AA = FKS
      GO TO (220, 240), ICASE 
C-----------------------------------------------------------------------
C     RECURSION SECTION  FKH*E(FKH+1,X) + X*E(FKH,X)=EMX
C-----------------------------------------------------------------------
  220 K = IND-1       
      IF (ML.LE.0) GOTO 235        
      DO 230 I=1,ML 
        AA = AA - 1.0D0
        EN(K) = (EMX-AA*EN(K+1))/X
        K = K - 1 
  230 CONTINUE
  235 IF (MU.LE.0) RETURN 
      AA = FKS
  240 K = IND 
      DO 250 I=1,MU
        EN(K+1) = (EMX-X*EN(K))/AA
        AA = AA + 1.0D0 
        K = K + 1
  250 CONTINUE
      RETURN
  340 CONTINUE
      IERR=2
      RETURN
      END
      SUBROUTINE DGEXINT(X, N, KODE, M, TOL, GN, EN, NZ, IERR)
C***BEGIN PROLOGUE  DGEXINT
C***DATE WRITTEN   020304   (YYMMDD)
C***REVISION DATE  020425   (YYMMDD)
C***CATEGORY NO.  B5E 
C***KEYWORDS  EXPONENTIAL INTEGRAL
C***AUTHOR  AMOS, DONALD E. 
C***PURPOSE  DGEXINT COMPUTES THE DERIVATIVE WRT ORDER OF THE
C***         EXPONENTIAL INTEGRAL AT INTEGER ORDERS AS WELL AS THE
C***         EXPONENTIAL INTEGRAL ITSELF AT INTEGER ORDERS
C***DESCRIPTION 
C 
C                   * A DOUBLE PRECISION ROUTINE *
C         DGEXINT COMPUTES M MEMBER SEQUENCES OF EXPONENTIAL INTEGRALS 
C         E(N+K,X) AND G(N+K,X),  K=0,1,...,M-1 FOR INTEGER ORDERS
C         N.GE.1 AND X.GE.0.0D0. THE EXPONENTIAL INTEGRAL IS DEFINED BY
C 
C         E(N,X)=INTEGRAL ON (1,INFINITY) OF EXP(-XT)/T**N
C
C         AND THE G INTEGRAL (=-DERIVATIVE WRT ORDER OF E AT N) IS
C         DEFINED BY
C
C         G(N,X)=INTEGRAL ON (1,INFINITY) OF E(N,XT)/T**N
C 
C         WHERE X=0.0 AND N=1 CANNOT OCCUR SIMULTANEOUSLY. FORMULAS 
C         AND NOTATION ARE FOUND IN THE NBS HANDBOOK OF MATHEMATICAL
C         FUNCTIONS (REF. 1). THIS IS AN ADAPTATION OF REF. 2 TO INTEGER
C         ORDERS. SERIES FOR E(1,X) OR E(2,X) ALONG WITH G(1,X) AND
C         G(2,X) ARE COMPUTED FOR X.LE.2 AND FORWARD RECURRENCE IS USED
C         FOR HIGHER INTEGER ORDERS. THE MILLER ALGORITHM IS RETAINED
C         FOR THE E FUNCTION AND THE DOCUMENTATION IN FOLDER 18 EXPLAINS
C         THE G COMPUTATION FOR X.GT.2. 
C 
C         INPUT     X , AND TOL ARE DOUBLE PRECISION
C           X       X.GT.0.0D0 FOR N=1 AND  X.GE.0.0D0 FOR N.GE.2
C           N       ORDER OF THE FIRST MEMBER OF THE SEQUENCE, N.GE.1 
C           KODE    A SELECTION PARAMETER FOR SCALED VALUES 
C                   KODE=1   RETURNS E(N+K,X), G(N+K,X) K=0,1,...,M-1.
C                       =2   RETURNS EXP(X)*E(N+K,X), EXP(X)*G(N,X),
C                                    K=0,1,...,M-1.
C           M       NUMBER OF INTEGRALS IN THE E AND G SEQUENCES, M.GE.1
C           TOL     RELATIVE ACCURACY WANTED, ETOL.LE.TOL.LE.0.1D0
C                   ETOL = MAXIMUM OF DOUBLE PRECISION UNIT ROUNDOFF
C                   (D1MACH(4)) AND 1.0D-18 
C 
C         OUTPUT    EN IS A DOUBLE PRECISION VECTOR 
C           EN      A VECTOR OF DIMENSION AT LEAST M CONTAINING VALUES
C                   EN(K) = E(N+K-1,X) OR EXP(X)*E(N+K-1,X), K=1,M
C                   DEPENDING ON KODE
C           GN      A VECTOR OF DIMENSION AT LEAST M CONTAINING VALUES
C                   GN(K) = G(N+K-1,X) OR EXP(X)*G(N+K-1,X), K=1,M
C                   DEPENDING ON KODE 
C           NZ      UNDERFLOW INDICATOR 
C                   NZ=0   A NORMAL RETURN
C                   NZ=M   X EXCEEDS XLIM AND AN UNDERFLOW OCCURS.
C                          EN(K)=GN(K)=0.0D0 , K=1,M RETURNED ON KODE=1 
C           IERR    ERROR FLAG
C                   IERR=0, NORMAL RETURN, COMPUTATION COMPLETED
C                   IERR=1, INPUT ERROR,   NO COMPUTATION 
C                   IERR=2, ERROR,         NO COMPUTATION 
C                           ALGORITHM TERMINATION CONDITION NOT MET 
C 
C***REFERENCES  HANDBOOK OF MATHEMATICAL FUNCTIONS BY M. ABRAMOWITZ 
C                 AND I. A. STEGUN, NBS AMS SERIES 55, U.S. DEPT. OF
C                 COMMERCE, 1955. 
C 
C               COMPUTATION OF EXPONENTIAL INTEGRALS BY D. E. AMOS, 
C                 ACM TRANS. MATH SOFTWARE, 6, 1980,PP. 365-377, 420-428
C
C               FOLDER 18 EVALUATION OF E(NU,X), G(NU,X)=-DE(NU,X)/DNU
C                 FOR INTEGER AND HALF ODD INTEGER ORDERS WITH
C                 APPLICATION TO I(N,B,T)= INTEGRAL ON (T,INFINITY) OF
C                 EXP(-B^2*X^2)*LN(X)/X**N, N.GE.0 
C***ROUTINES CALLED  I1MACH,D1MACH 
C***END PROLOGUE  DGEXINT
C      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C 
      DOUBLE PRECISION A, AA, AAMS, AH, AK, AT, B, BK, BT, CC, CNORM, 
     * CT, EM, EMX, EN, ETOL, FX, FN, FKN, PT, P1, P2, RIX,
     * TOL, TX, X, XCUT, XLIM, XTOL, YT, Y1, Y2, XOVR
      DOUBLE PRECISION D1MACH,  GN, YI1, DICT, DKCT, ANUM, ADEN
      DOUBLE PRECISION G1FUN, G2FUN, REL, PTOL
      INTEGER I, IC, ICASE, ICT, IERR, IND, IX, I1M, K, KK, MF, 
     * KODE,  M, ML, MU, N, NZ, I1P, MND, ND
      INTEGER I1MACH
      DIMENSION EN(M), GN(M), A(150), B(150)
C 
      DATA XCUT, PTOL /  2.0D0, 1.0D+10   /
C 
C***FIRST EXECUTABLE STATEMENT  DGEXINT
      IERR=0
      NZ=0
      ETOL = DMAX1(D1MACH(4),0.5D-18) 
      IF (X.LT.0.0D0) IERR=1
      IF (N.LT.1) IERR=1
      IF (KODE.LT.1 .OR. KODE.GT.2) IERR=1
      IF (M.LT.1) IERR=1
      IF (TOL.LT.ETOL .OR. TOL.GT.0.1D0) IERR=1 
      IF (X.EQ.0.0D0 .AND. N.EQ.1) IERR=1 
      IF (IERR.NE.0) RETURN
      I1M = -I1MACH(15) 
      PT = 2.3026D0*DBLE(FLOAT(I1M))*D1MACH(5)
      XLIM = PT - 6.907755D0
      BT = PT + DBLE(FLOAT(N+M-1))
      IF (BT.GT.1000.0D0) XLIM = PT - DLOG(BT)
C-----------------------------------------------------------------------
C     SET OVERFLOW LIMIT ON X. THE UNDERFLOW LIMIT IS 1.0D0/XOVR.
C     TEST EXPONENTS ON SMALLEST AND LARGEST NUMBERS TO BE SURE THE
C     RECIPROCALS EXIST. EXPONENTS ARE I1MACH(15) AND I1MACH(16).
C-----------------------------------------------------------------------
      I1P = I1MACH(16)
      IF(I1M.LT.I1P) THEN
        XOVR = 1.0D-3/D1MACH(1)
      ELSE
        XOVR = 1.0D-3*D1MACH(2)
      ENDIF
      FN=DBLE(FLOAT(N))
      IF (X.GT.XOVR) GO TO 105
      IF (X.LT.1.0D0/XOVR) THEN
        IF (N.GT.1) GOTO 80
        IERR=1
        RETURN
      ENDIF
C-----------------------------------------------------------------------
C     START SEQUENCE AT E(1,X),G(1,X) OR E(2,X),G(2,X)  FOR X.LE.XCUT 
C-----------------------------------------------------------------------
      IF (X.GT.XCUT) GOTO 100
      IF (N.GT.20) GOTO 100
      IF(KODE.EQ.1) THEN
        EMX=DEXP(-X)
      ELSE
        EMX=1.0D0
      ENDIF
      MF=0
      MND=0
      IF (N.GT.2) THEN
        ND=2
        MF=N-2
      ELSE
        ND=N 
      ENDIF
      XTOL=TOL/100.0D0
      REL=DMAX1(ETOL,XTOL)
      IF(ND.EQ.1) THEN
C       MF=0 HERE
C       SERIES FOR YI1=G1 AND Y1=E1 FOR N=1 SCALED BY KODE
        YI1=G1FUN(X,KODE,REL,ETOL,Y1)
        EN(1)=Y1
        GN(1)=YI1
        IF(M.EQ.1) THEN
          RETURN
        ENDIF
        ND=2
        MND=1
      ENDIF
      IF(ND.EQ.2) THEN
C       SERIES FOR YI1=G2 AND Y1=E2 FOR N=2 SCALED BY KODE
        YI1=G2FUN(X,KODE,REL,ETOL,Y1)
        IF(MND.EQ.1) THEN
C         MF=0 HERE
          EN(2)=Y1
          GN(2)=YI1
          IF(M.EQ.2) THEN
            RETURN
          ENDIF
          AA=2.0D0
          IND=2
          MU=M-2
        ELSE
          IF(MF.EQ.0) THEN
            EN(1)=Y1
            GN(1)=YI1
            IF(M.EQ.1) THEN
              RETURN
            ENDIF
            IND=1
            MU=M-1
          ENDIF
          AA=2.0D0
        ENDIF
      ENDIF
      IF(MF.NE.0) THEN
        DO 10 K=1,MF
          Y2=(EMX-X*Y1)/AA
          Y1=Y2
          YI1=(Y2-X*YI1)/AA
          AA=AA+1.0D0
 10     CONTINUE
        EN(1)=Y1
        GN(1)=YI1
        IF(M.EQ.1) RETURN          
        IND = 1
        MU = M-1
      ENDIF
      GOTO 240
   80 CONTINUE
      DO 90 I=1,M 
        CC = 1.0D0/(FN-1.0D0+DBLE(FLOAT(I-1)))
        EN(I)=CC
        GN(I)=CC*CC
   90 CONTINUE
      RETURN
C-----------------------------------------------------------------------
C     BACKWARD RECURSIVE MILLER ALGORITHM FOR 
C              E(N,X)=DEXP(-X)*(X**(N-1))*U(N,N,X)
C     WITH RECURSION AWAY FROM N=INTEGER CLOSEST TO X.
C     U(A,B,X) IS THE SECOND CONFLUENT HYPERGEOMETRIC FUNCTION
C-----------------------------------------------------------------------
  100 CONTINUE
      EMX = 1.0D0 
      IF (KODE.EQ.2) GO TO 130
      IF (X.LE.XLIM) GO TO 120
  105 CONTINUE
      IF (N.EQ.1) THEN
        IERR=1
        RETURN
      ENDIF
      NZ = M
      DO 110 I=1,M
        EN(I) = 0.0D0
        GN(I) = 0.0D0
  110 CONTINUE
      RETURN
  120 EMX = DEXP(-X)
  130 CONTINUE
      MF=0
      RIX = ANINT(X+0.5D0)
      FKN = FN+DBLE(FLOAT(M-1))
      IF (FKN.LE.RIX) GO TO 140
      IF (FN.LT.RIX .AND. RIX.LT.FKN) GO TO 170
      IF (FN.GE.RIX) GO TO 160
      GO TO 340 
  140 ICASE = 1
C     WHOLE SEQUENCE LIES TO THE LEFT OF RIX. START AT FKN, RECUR DOWN
C     ADJUST INDEX UP TO GET BETTER CONVERGENCE AND MORE ACCURACY.
C     RECUR DOWN MF TIMES (UNINDEXED) TO GET BACK TO ORIGINAL INDEX.
      FKS=FKN
      IF(FKN.LT.2.0D0) THEN
        FKS=2.0D0
        MF=INT(SNGL(2.0D0-FKN))
      ENDIF
      ML = M - 1
      MU = -1 
      IND = M
      GOTO 180
  160 ICASE = 2
C     WHOLE SEQUENCE LIES TO THE RIGHT OF RIX. START AT FN, RECUR UP
      IND = 1 
      FKS=FN
      MU = M - 1
      ML = -1
      GOTO 180
  170 CONTINUE
C     RIX LIES IN THE RANGE OF THE SEQUENCE. START AT RIX, RECUR UP &
C     DOWN
      IX=INT(SNGL(RIX))
      ICASE = 1
C      N = INT(FN)
      ML = IX - N      
      IND = ML + 1
      FKS = FN+DBLE(FLOAT(IND-1))
      MU = M - IND 
  180 CONTINUE
      AH = FKS/2.0D0
C-----------------------------------------------------------------------
C     START COMPUTATION FOR 
C              EN(IND) = C*U( A , A ,X)     
C-----------------------------------------------------------------------
      IC = 0
      AA = AH + AH
      AAMS = AA - 1.0D0 
      AAMS = AAMS*AAMS
      TX = X + X
      FX = TX + TX
      AK = AH 
      XTOL = TOL
      IF (TOL.LE.1.0D-3) XTOL = 20.0D0*TOL
      CT = AAMS + FX*AH 
C-----------------------------------------------------------------------
C     PREVENT OVERFLOW IN DEN OF EM = (AH+1.0D0)/((X+AA)*XTOL*SQRT(CT))
C-----------------------------------------------------------------------
      P1 = (AH+1.0D0)/((X+AA)*XTOL)
      P2 = SQRT(CT)
      IF( P1 .LT. P2/XOVR ) THEN
        Y1 = 1.0D0/X
        GO TO 205
      ENDIF
      EM = P1/P2
      BK = AA 
      CC = AH*AH
C-----------------------------------------------------------------------
C     FORWARD RECURSION FOR P(IC),P(IC+1) AND INDEX IC FOR BACKWARD 
C     RECURSION 
C-----------------------------------------------------------------------
      P1 = 0.0D0
      P2 = 1.0D0
  190 CONTINUE
      IF (IC.EQ.150) GO TO 340 
      IC = IC + 1 
      AK = AK + 1.0D0 
      AT = BK/(BK+AK+CC+DBLE(FLOAT(IC)))
      BK = BK + AK + AK 
      A(IC) = AT
      BT = (AK+AK+X)/(AK+1.0D0) 
      B(IC) = BT
      PT = P2 
      P2 = BT*P2 - AT*P1
      P1 = PT 
      CT = CT + FX
      EM = EM*AT*(1.0D0-TX/CT)
      IF (EM*(AK+1.0D0).GT.P1*P1/PTOL) GOTO 190
      ICT = IC
      KK = IC + 1
      BT = TX/(CT+FX) 
      Y2 = (BK/(BK+CC+DBLE(FLOAT(KK))))*(P1/P2)*(1.0D0-BT+0.375D0*BT*BT)
      Y1 = 1.0D0
      DICT=DBLE(FLOAT(KK))
      ANUM=AH+DICT
      ADEN=AA+DICT-1.0D0      
      YT=Y2/(DICT)
      YI1=ANUM*YT/ADEN
      DKCT=DICT
C-----------------------------------------------------------------------
C     BACKWARD RECURRENCE FOR 
C              Y1=             C*U( A ,A,X) 
C              Y2= C*(A/(1+A/2))*U(A+1,A,X) 
C-----------------------------------------------------------------------
      DO 200 K=1,ICT
        KK = KK - 1 
        YT = Y1 
        Y1 = (B(KK)*Y1-Y2)/A(KK)
        Y2 = YT
        DKCT=DKCT-1.0D0
        ANUM=AH+DKCT
        ADEN=AA+DKCT-1.0D0
        YI1=(ANUM/ADEN)*(Y2/DKCT+YI1)
  200 CONTINUE
C-----------------------------------------------------------------------
C     THE CONTIGUOUS RELATION 
C              X*U(B,C+1,X)=(C-B)*U(B,C,X)+U(B-1,C,X) 
C     WITH  B=A+1 , C=A IS USED FOR 
C              Y(2) = C * U(A+1,A+1,X)
C     X IS INCORPORATED INTO THE NORMALIZING RELATION 
C-----------------------------------------------------------------------
C      WRITE(7,203) RIX,DKCT,ICT
C 203  FORMAT(2D15.5,I5)
      PT = Y2/Y1
      ADEN = AA - PT*(AH+1.0D0)+X
      CNORM=1.0D0/ADEN
      YI1=YI1*CNORM/Y1
      Y1 = CNORM
  205 CONTINUE
      IF (MF.NE.0) THEN
        AK=FKS
        DO 206 J=1,MF
          AK=AK-1.0D0
          Y2=(1.0D0-AK*Y1)/X
          YI1=(Y1-AK*YI1)/X
          Y1=Y2
  206   CONTINUE
        FKS=AK
      ENDIF
      EN(IND) = EMX*Y1
      GN(IND) = EMX*YI1
      IF (M.EQ.1) RETURN
      AA = FKS
      GO TO (220, 240), ICASE 
C-----------------------------------------------------------------------
C     RECURSION SECTION  FKH*E(FKH+1,X) + X*E(FKH,X)=EMX
C-----------------------------------------------------------------------
  220 K = IND-1       
      IF (ML.LE.0) GOTO 235        
      DO 230 I=1,ML 
        AA = AA - 1.0D0
        EN(K) = (EMX-AA*EN(K+1))/X
        GN(K)= (EN(K+1)-AA*GN(K+1))/X
        K = K - 1 
  230 CONTINUE
  235 IF (MU.LE.0) RETURN 
      AA = FKS
  240 K = IND 
      DO 250 I=1,MU
        EN(K+1) = (EMX-X*EN(K))/AA
        GN(K+1)= (EN(K+1)-X*GN(K))/AA
        AA = AA + 1.0D0 
        K = K + 1
  250 CONTINUE
      RETURN
  340 CONTINUE
      IERR=2
      RETURN
      END
      DOUBLE PRECISION FUNCTION G1FUN(X,KODE,REL,ETOL,E1)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION B(100)
      DATA EULER /0.5772156649015328606D0/
      DATA HPI   /1.5707963267948966D0/
      IFLAG=0
      TOL=REL/10.0D0
      IF(X.LT.ETOL) THEN
C       ELIMINATING UNDERFLOW WHEN X IS SMALL
        SUM0=-X
        SUM1=-X
        ASUM=0.0D0
        IFLAG=1
      ELSE
        AK=1.0D0
        TRM=1.0D0
        SUM0=0.0D0
        SUM1=0.0D0
        DO 10 K=1,100
          TRM=-TRM*X/AK
          TRM0=TRM/AK
          SUM0=SUM0+TRM0
          SUM1=SUM1+TRM0/AK
          IF(DABS(TRM).LT.TOL*DABS(SUM0)) GOTO 20
          AK=AK+1.0D0
  10    CONTINUE
        PRINT *,'IN G1FUN, DROP THRU LOOP 1'
        PAUSE
  20  CONTINUE
      ENDIF
      EPLNX=EULER+DLOG(X)
      E1=-EPLNX-SUM0
      S=HPI*HPI/3.0D0+0.5D0*E1*E1
      S=S-EPLNX*SUM0+SUM1
      IF(IFLAG.EQ.1) GOTO 45
      B(1)=1.0D0
      B(2)=1.0D0
      DO 25 J=3,100
        B(J)=0.0D0
  25  CONTINUE
C     HERE B VECTOR CONTAINS THE BINOMIAL COEFFICIENTS FOR (X+Y)**1
      FACK=2.0D0
      TRM=-X
      AK=2.0D0
      ASUM=0.0D0
      DO 30 K=2,99
        TRM=-TRM*X
C       GENERATE ROW K OF PASCAL TRIANGLE( BACKWARD RECURRENCE DOES NOT
C       OVERWRITE K-1 ENTRIES UNTIL THEY ARE NO LONGER NEEDED FOR THE
C       ROW K COMPUTATION)
        M=K
        DO 35 J=1,K
          B(M+1)=B(M)+B(M+1)
          M=M-1
  35    CONTINUE
C       B VECTOR = BINOMIAL COEFFICIENTS FOR (X+Y)**K        
        SUM1=0.0D0
        AM=AK-1.0D0
        DO 40 J=2,K
          SUM1=SUM1+B(J)/AM
          AM=AM-1.0D0
  40    CONTINUE
        CS=SUM1/FACK
        TRM1=TRM*CS/AK
        ASUM=ASUM+TRM1
        IF(DABS(TRM1).LT.TOL*DABS(ASUM)) GOTO 45
        AK=AK+1.0D0
        FACK=FACK*AK
  30  CONTINUE
      PRINT *,'IN G1FUN, DROP THRU LOOP 2'
      PAUSE
  45  CONTINUE
      G1=S-ASUM
      IF(KODE.EQ.1) THEN
       G1FUN=G1
      ELSE
       TRM=DEXP(X)
       G1FUN=G1*TRM
       E1=E1*TRM
      ENDIF
      RETURN
      END
      DOUBLE PRECISION FUNCTION G2FUN(X,KODE,REL,ETOL,E2)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION B(100)
      DATA EULER /0.5772156649015328606D0/
      DATA HPI /1.5707963267948966D0/
      TOL=REL/10.0D0
      IFLAG=0
      IF(X.EQ.0.0D0) THEN
        G2FUN=1.0D0
        E2=1.0D0
        RETURN
      ENDIF
      IF(X.LT.ETOL) THEN
        SUM0=-0.5D0*X
        SUM2=0.0D0
        ASUM=0.0D0
        DSUM=0.0D0
        IFLAG=1
      ELSE
        AK=2.0D0
        AKM=1.0D0
        TRM=-X
        SUM0=0.5D0*TRM
        SUM2=0.0D0
        DO 10 K=2,100
          TRM=-TRM*X/AK
          CK=AK+1.0D0
          TRM0=TRM/(AK*AK)
          TRM0=TRM0/CK
          SUM0=SUM0+TRM0
          TRM2=TRM/AKM
          SUM2=SUM2+TRM2
          IF(DABS(TRM0).LT.TOL*DABS(SUM0)) GOTO 20
          AKM=AK
          AK=AK+1.0D0
  10    CONTINUE
        PRINT *,'IN G2FUN, DROP THRU SUM0 LOOP'
        PAUSE
  20    CONTINUE
      ENDIF
      EPLNX=EULER+DLOG(X)
      E2=X*(EPLNX-1.0D0)-SUM2+1.0D0
      S=-0.5D0*EPLNX*EPLNX+EPLNX-SUM0-HPI*HPI/3.0D0-1.0D0
      IF(IFLAG.EQ.1) GOTO 145
C     START SUMS WITH A(N) AND D(N) IN THEM
      B(1)=1.0D0
      B(2)=1.0D0
      DO 25 J=3,100
        B(J)=0.0D0
  25  CONTINUE
C     HERE B VECTOR CONTAINS THE BINOMIAL COEFFICIENTS FOR (X+Y)**1
      FACK=2.0D0
      TRM=X
      AK=2.0D0
      ASUM=0.0D0
      DSUM=0.0D0
      DO 130 K=2,99
        TRM=-TRM*X
C       GENERATE ROW K OF PASCAL TRIANGLE( BACKWARD RECURRENCE DOES NOT
C       OVERWRITE K-1 ENTRIES UNTIL THEY ARE NO LONGER NEEDED FOR THE
C       ROW K COMPUTATION)
        M=K
        DO 135 J=1,K
          B(M+1)=B(M)+B(M+1)
          M=M-1
  135   CONTINUE
C       B VECTOR (K+1 COMPONENTS) = BINOMIAL COEFFICIENTS FOR (X+Y)**K
        SUMA=0.0D0
        SUMD=0.0D0
        AM=AK-1.0D0
        BM=1.0D0
        DO 140 J=2,K
          TRM=B(J)/AM
          SUMA=SUMA+TRM
          SUMD=SUMD+TRM/BM
          AM=AM-1.0D0
          BM=BM+1.0D0
  140   CONTINUE
        CA=SUMA/FACK
        CD=SUMD/FACK
        CK=AK+1.0D0
        TRMA=TRM*CA/(AK*CK)
        TRMD=TRM*CD/CK
        ASUM=ASUM+TRMA
        DSUM=DSUM+TRMD
        IF(DABS(TRMD).LT.TOL*DABS(DSUM)) GOTO 145
        AK=AK+1.0D0
        FACK=FACK*AK
  130 CONTINUE
      PRINT *,'IN G2FUN, DROP THRU A(N),D(N) SUM LOOP'
      PAUSE
  145 CONTINUE
      S=S+ASUM-0.5D0*DSUM
      G2=1.0D0+X*S
      IF(KODE.EQ.1) THEN
        G2FUN=G2
      ELSE
        TRM=DEXP(X)
        G2FUN=G2*TRM
        E2=E2*TRM
      ENDIF
      RETURN
      END
      SUBROUTINE DGHEXINT(X, FNH, KODE, M, TOL, GN, EN, NZ, IERR)
C***BEGIN PROLOGUE  DGHEXINT
C***DATE WRITTEN   020304   (YYMMDD)
C***REVISION DATE  020425   (YYMMDD)
C***CATEGORY NO.  B5E 
C***KEYWORDS  EXPONENTIAL INTEGRAL
C***AUTHOR  AMOS, DONALD E. 
C***PURPOSE  DGHEXINT COMPUTES THE DERIVATIVE WRT ORDER OF THE
C***         EXPONENTIAL INTEGRAL AT HALF ODD ORDERS AS WELL AS THE
C***         EXPONENTIAL INTEGRAL ITSELF AT HALF ODD INTEGERS
C***DESCRIPTION
C 
C                   * A DOUBLE PRECISION ROUTINE *
C         DGHEXINT COMPUTES M MEMBER SEQUENCES OF EXPONENTIAL INTEGRALS 
C         E(FNH+K,X) AND G(FNH+K,X),  K=0,1,...,M-1 FOR HALF ODD INTEGER
C         ORDERS FNH.GE.0.5 AND X.GT.0.0D0. THE EXPONENTIAL INTEGRAL IS
C         DEFINED BY (IN THIS CONTEXT NU=0.5,1.5,2.5,...)
C 
C         E(NU,X)=INTEGRAL ON (1,INFINITY) OF EXP(-XT)/T**NU, NU.GT.0
C
C         AND THE G INTEGRAL (=-DERIVATIVE WRT ORDER OF E AT NU) IS
C         DEFINED BY
C
C         G(NU,X)=INTEGRAL ON (1,INFINITY) OF E(N,XT)/T**NU
C 
C         WHERE X=0.0 AND FNH=0.5 CANNOT OCCUR SIMULTANEOUSLY. FORMULAS 
C         AND NOTATION ARE FOUND IN THE NBS HANDBOOK OF MATHEMATICAL
C         FUNCTIONS (REF. 1). THIS IS AN ADAPTATION OF REF. 2 TO HALF
C         ODD INTEGER ORDERS. SERIES FOR E(0.5,X) OR E(1.5,X) ALONG
C         WITH G(0.5,X) ARE COMPUTED FOR X.LE.2 AND FORWARD RECURRENCE
C         IS USED FOR HIGHER ORDERS. THE MILLER ALGORITHM IS RETAINED
C         FOR THE E FUNCTION AND THE DOCUMENTATION IN FOLDER 18 EXPLAINS
C         THE G COMPUTATION FOR X.GT.2.
C 
C         INPUT     X , FNH, AND TOL ARE DOUBLE PRECISION
C           X       X.GT.0.0D0 FOR FNH=0.5 AND X.GE.0.0D0 FOR FNH.GE.1.5
C           FNH     ORDER OF FIRST MEMBER OF THE SEQUENCE, FNH.GE.0.5 
C           KODE    A SELECTION PARAMETER FOR SCALED VALUES 
C                   KODE=1   RETURNS E(FNH+K,X), G(FNH+K,X) K=0,1,...,M-1.
C                       =2   RETURNS EXP(X)*E(FNH+K,X), EXP(X)*G(FNH,X),
C                                    K=0,1,...,M-1.
C           M       NUMBER OF INTEGRALS IN THE E AND G SEQUENCES, M.GE.1
C           TOL     RELATIVE ACCURACY WANTED, ETOL.LE.TOL.LE.0.1D0
C                   ETOL = MAXIMUM OF DOUBLE PRECISION UNIT ROUNDOFF
C                   (D1MACH(4)) AND 1.0D-18 
C 
C         OUTPUT    EN IS A DOUBLE PRECISION VECTOR 
C           EN      A VECTOR OF DIMENSION AT LEAST M CONTAINING VALUES
C                   EN(K) = E(FNH+K-1,X) OR EXP(X)*E(FNH+K-1,X), K=1,M
C                   DEPENDING ON KODE
C           GN      A VECTOR OF DIMENSION AT LEAST M CONTAINING VALUES
C                   GN(K) = G(FNH+K-1,X) OR EXP(X)*G(FNH+K-1,X), K=1,M
C                   DEPENDING ON KODE 
C           NZ      UNDERFLOW INDICATOR 
C                   NZ=0   A NORMAL RETURN
C                   NZ=M   X EXCEEDS XLIM AND AN UNDERFLOW OCCURS.
C                          EN(K)=GN(K)=0.0D0 , K=1,M RETURNED ON KODE=1 
C           IERR    ERROR FLAG
C                   IERR=0, NORMAL RETURN, COMPUTATION COMPLETED
C                   IERR=1, INPUT ERROR,   NO COMPUTATION 
C                   IERR=2, ERROR,         NO COMPUTATION 
C                           ALGORITHM TERMINATION CONDITION NOT MET 
C 
C***REFERENCES  HANDBOOK OF MATHEMATICAL FUNCTIONS BY M. ABRAMOWITZ 
C                 AND I. A. STEGUN, NBS AMS SERIES 55, U.S. DEPT. OF
C                 COMMERCE, 1955. 
C 
C               COMPUTATION OF EXPONENTIAL INTEGRALS BY D. E. AMOS, 
C                 ACM TRANS. MATH SOFTWARE, 6, 1980,PP. 365-377, 420-428
C
C               FOLDER 18 BY D. E. AMOS EVALUATION OF E(NU,X), G(NU,X)=
C                 -DE(NU,X)/DNU FOR INTEGER AND HALF ODD INTEGER ORDERS
C                 WITH APPLICATION TO I(N,B,T)= INTEGRAL ON (T,INFINITY)
C                 OF EXP(-B^2*X^2)*LN(X)/X**N, N=1,2,3,... 
C 
C***ROUTINES CALLED  I1MACH,D1MACH,DRERF 
C***END PROLOGUE  DGHEXINT
C      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C 
      DOUBLE PRECISION A, AA, AAMS, AH, AK, AT, B, BK, BT, CC, CNORM, 
     * CT, EM, EMX, EN, ERF2, ETOL, FX, FN, FNH, FKN, PT, P1, P2, RIX,
     * RTZ, RTPI, TOL, TX, X, XCUT, XLIM, XTOL, YT, Y1, Y2, XOVR, PTOL
      DOUBLE PRECISION D1MACH, DRERF, GN, YI1, DICT, DKCT, ANUM, ADEN
      DOUBLE PRECISION DGERFC
      INTEGER I, IC, ICASE, ICT, IERR, IND, IX, I1M, K, KK, MF, 
     * KODE, KODEP, M, ML, MU, N, NH, NZ, I1P
      INTEGER I1MACH
      DIMENSION EN(M), GN(M), A(150), B(150)
C 
      DATA XCUT,PTOL  /     2.0D0,    1.0D+10     /
      DATA RTPI                  /1.772453850905516D0/
C 
C***FIRST EXECUTABLE STATEMENT  DGHEXINT
      IERR=0
      NZ=0
      ETOL = DMAX1(D1MACH(4),0.5D-18) 
      IF (X.LT.0.0D0) IERR=1
      IF (FNH.LT.0.5D0) IERR=1
      FN=FNH-0.5D0
      IF (FN-DBLE(FLOAT(INT(FN))).NE.0.0D0) IERR=1 
      IF (KODE.LT.1 .OR. KODE.GT.2) IERR=1
      IF (M.LT.1) IERR=1
      IF (TOL.LT.ETOL .OR. TOL.GT.0.1D0) IERR=1 
      IF (X.EQ.0.0D0 .AND. FNH.EQ.0.5D0) IERR=1 
      IF (IERR.NE.0) RETURN
      I1M = -I1MACH(15) 
      PT = 2.3026D0*DBLE(FLOAT(I1M))*D1MACH(5)
      XLIM = PT - 6.907755D0
      BT = PT + DBLE(FLOAT(N+M-1))
      IF (BT.GT.1000.0D0) XLIM = PT - DLOG(BT)
C-----------------------------------------------------------------------
C     SET OVERFLOW LIMIT ON X. THE UNDERFLOW LIMIT IS 1.0D0/XOVR.
C     TEST EXPONENTS ON SMALLEST AND LARGEST NUMBERS TO BE SURE THE
C     RECIPROCALS EXIST. EXPONENTS ARE I1MACH(15) AND I1MACH(16).
C-----------------------------------------------------------------------
      I1P = I1MACH(16)
      IF(I1M.LT.I1P) THEN
        XOVR = 1.0D-3/D1MACH(1)
      ELSE
        XOVR = 1.0D-3*D1MACH(2)
      ENDIF
      IF (X.GT.XOVR) GO TO 105
      IF (X.LT.1.0D0/XOVR) THEN
        IF (FNH.GT.0.5D0) GOTO 80
        IERR=1
        RETURN
      ENDIF
C-----------------------------------------------------------------------
C     START SEQUENCE AT G(X) (A=0.5D0) FOR X.LE.XCUT 
C-----------------------------------------------------------------------
      IF (X.GT.XCUT) GOTO 100
      IF (FNH.GT.20.0D0) GOTO 100
      RTZ=DSQRT(X)
      IF (KODE.EQ.1) THEN
        EMX=DEXP(-X)
      ELSE
        EMX=1.0D0
      ENDIF
C     E(0.5,X)=RTPI*ERFC(RTX)/RTX  , RTX=SQRT(X)
      IF (FNH.EQ.0.5D0) THEN
        NZERO=0
        KODEP=KODE+1
        ERF2=DRERF(RTZ,KODEP,NZERO)
        CC=RTPI/RTZ
        Y1=CC*ERF2
        YT=DGERFC(RTZ,KODE,TOL,IERRG)
        YT=YT+YT
        YI1=YT*CC
        EN(1)=Y1
        GN(1)=YI1
        IF (M.EQ.1) RETURN
C        FNH=1.5 HERE E(3/2,X)=2*RTPI*IERFC(SQRT(X))
        CALL DIERFC(RTZ,KODE,YT,IERR)
        Y1=(YT+YT)*RTPI
        YI1=Y1-X*YI1
        YI1=YI1+YI1
        EN(2)=Y1
        GN(2)=YI1
        IF(M.EQ.2) RETURN
        IND = 2
        MU = M-2
        AA = 1.5D0
      ELSE
        CALL DIERFC(RTZ,KODE,YT,IERR)
        Y1=(YT+YT)*RTPI
        YT=DGERFC(RTZ,KODE,TOL,IERRG)
        YT=YT+YT
        CC=RTPI/RTZ
        YI1=YT*CC
        YI1=Y1-X*YI1
        YI1=YI1+YI1
        AA=1.5D0
        NH=INT(FN)-1
        IF (NH.LE.0) GOTO 11
        DO 10 K=1,NH
          Y2=(EMX-X*Y1)/AA
          Y1=Y2
          YI1=(Y2-X*YI1)/AA
          AA=AA+1.0D0
   10   CONTINUE
   11   CONTINUE
        EN(1)=Y1
        GN(1)=YI1
        IF(M.EQ.1) RETURN
        IND = 1
        MU = M-1
      ENDIF
      GOTO 240
   80 CONTINUE
      DO 90 I=1,M 
        EN(I) = 1.0D0/(FNH-1.0D0+DBLE(FLOAT(I-1)))
        GN(I)=EN(I)*EN(I)
   90 CONTINUE
      RETURN
C-----------------------------------------------------------------------
C     BACKWARD RECURSIVE MILLER ALGORITHM FOR 
C              E(N,X)=DEXP(-X)*(X**(N-1))*U(N,N,X)
C     WITH RECURSION AWAY FROM N=INTEGER CLOSEST TO X.
C     U(A,B,X) IS THE SECOND CONFLUENT HYPERGEOMETRIC FUNCTION
C-----------------------------------------------------------------------
  100 CONTINUE
      EMX = 1.0D0 
      IF (KODE.EQ.2) GO TO 130
      IF (X.LE.XLIM) GO TO 120
  105 CONTINUE
      IF (FNH.EQ.0.5D0) THEN
        IERR=1
        RETURN
      ENDIF
      NZ = M
      DO 110 I=1,M
        EN(I) = 0.0D0
        GN(I) = 0.0D0
  110 CONTINUE
      RETURN
  120 EMX = DEXP(-X)
  130 CONTINUE
      MF=0
      RIX = ANINT(X)+0.5D0
      FKN = FNH+DBLE(FLOAT(M-1))
      IF (FKN.LE.RIX) GO TO 140
      IF (FNH.LT.RIX .AND. RIX.LT.FKN) GO TO 170
      IF (FNH.GE.RIX) GO TO 160
      GO TO 340 
  140 ICASE = 1
C     WHOLE SEQUENCE LIES TO THE LEFT OF RIX. START AT FKN, RECUR DOWN
C     ADJUST INDEX UP TO GET BETTER CONVERGENCE AND MORE ACCURACY.
C     RECUR DOWN MF TIMES (UNINDEXED) TO GET BACK TO ORIGINAL INDEX.
      FKS=FKN
      IF(FKN.LT.2.5D0) THEN
        FKS=2.5D0
        MF=INT(SNGL(2.5D0-FKN))
      ENDIF
      ML = M - 1
      MU = -1 
      IND = M
      GOTO 180
  160 ICASE = 2
C     WHOLE SEQUENCE LIES TO THE RIGHT OF RIX. START AT FNH, RECUR UP
      IND = 1 
      FKS=FNH
      MU = M - 1
      ML = -1
      GOTO 180
  170 CONTINUE
C     RIX LIES IN THE RANGE OF THE SEQUENCE. START AT RIX, RECUR UP &
C     DOWN
      IX=INT(SNGL(RIX))
      ICASE = 1
      N = INT(FN)
      ML = IX - N      
      IND = ML + 1
      FKS = FNH+DBLE(FLOAT(IND-1))
      MU = M - IND 
  180 CONTINUE
      AH = FKS/2.0D0
C-----------------------------------------------------------------------
C     START COMPUTATION FOR 
C              EN(IND) = C*U( A , A ,X)     
C-----------------------------------------------------------------------
      IC = 0
      AA = AH + AH
      AAMS = AA - 1.0D0 
      AAMS = AAMS*AAMS
      TX = X + X
      FX = TX + TX
      AK = AH 
      XTOL = TOL
      IF (TOL.LE.1.0D-3) XTOL = 20.0D0*TOL
      CT = AAMS + FX*AH 
C-----------------------------------------------------------------------
C     PREVENT OVERFLOW IN DEN OF EM = (AH+1.0D0)/((X+AA)*XTOL*SQRT(CT))
C-----------------------------------------------------------------------
      P1 = (AH+1.0D0)/((X+AA)*XTOL)
      P2 = SQRT(CT)
      IF( P1 .LT. P2/XOVR ) THEN
        Y1 = 1.0D0/X
        GO TO 205
      ENDIF
      EM = P1/P2
      BK = AA 
      CC = AH*AH
C-----------------------------------------------------------------------
C     FORWARD RECURSION FOR P(IC),P(IC+1) AND INDEX IC FOR BACKWARD 
C     RECURSION 
C-----------------------------------------------------------------------
      P1 = 0.0D0
      P2 = 1.0D0
  190 CONTINUE
      IF (IC.EQ.150) GO TO 340 
      IC = IC + 1 
      AK = AK + 1.0D0 
      AT = BK/(BK+AK+CC+DBLE(FLOAT(IC)))
      BK = BK + AK + AK 
      A(IC) = AT
      BT = (AK+AK+X)/(AK+1.0D0) 
      B(IC) = BT
      PT = P2 
      P2 = BT*P2 - AT*P1
      P1 = PT 
      CT = CT + FX
      EM = EM*AT*(1.0D0-TX/CT)
      IF (EM*(AK+1.0D0).GT.P1*P1/PTOL) GOTO 190
      ICT = IC
      KK = IC + 1
      BT = TX/(CT+FX) 
      Y2 = (BK/(BK+CC+DBLE(FLOAT(KK))))*(P1/P2)*(1.0D0-BT+0.375D0*BT*BT)
      Y1 = 1.0D0
      DICT=DBLE(FLOAT(KK))
      ANUM=AH+DICT
      ADEN=AA+DICT-1.0D0      
      YT=Y2/(DICT)
      YI1=ANUM*YT/ADEN
      DKCT=DICT
C-----------------------------------------------------------------------
C     BACKWARD RECURRENCE FOR 
C              Y1=             C*U( A ,A,X) 
C              Y2= C*(A/(1+A/2))*U(A+1,A,X) 
C-----------------------------------------------------------------------
      DO 200 K=1,ICT
        KK = KK - 1 
        YT = Y1 
        Y1 = (B(KK)*Y1-Y2)/A(KK)
        Y2 = YT
        DKCT=DKCT-1.0D0
        ANUM=AH+DKCT
        ADEN=AA+DKCT-1.0D0
        YI1=(ANUM/ADEN)*(Y2/DKCT+YI1)
  200 CONTINUE
C-----------------------------------------------------------------------
C     THE CONTIGUOUS RELATION 
C              X*U(B,C+1,X)=(C-B)*U(B,C,X)+U(B-1,C,X) 
C     WITH  B=A+1 , C=A IS USED FOR 
C              Y(2) = C * U(A+1,A+1,X)
C     X IS INCORPORATED INTO THE NORMALIZING RELATION 
C-----------------------------------------------------------------------
C      WRITE(7,203) RIX,DKCT,ICT
C 203  FORMAT(2D15.5,I5)
      PT = Y2/Y1
      ADEN = AA - PT*(AH+1.0D0)+X
      CNORM=1.0D0/ADEN
      YI1=YI1*CNORM/Y1
      Y1 = CNORM
  205 CONTINUE
      IF (MF.NE.0) THEN
        AK=FKS
        DO 206 J=1,MF
          AK=AK-1.0D0
          Y2=(1.0D0-AK*Y1)/X
          YI1=(Y1-AK*YI1)/X
          Y1=Y2
  206   CONTINUE
        FKS=AK
      ENDIF
      EN(IND) = EMX*Y1
      GN(IND)= EMX*YI1
      IF (M.EQ.1) RETURN
      AA = FKS
      GO TO (220, 240), ICASE 
C-----------------------------------------------------------------------
C     RECURSION SECTION  FKH*E(FKH+1,X) + X*E(FKH,X)=EMX
C-----------------------------------------------------------------------
  220 K = IND-1       
      IF (ML.LE.0) GOTO 235        
      DO 230 I=1,ML 
        AA = AA - 1.0D0
        EN(K) = (EMX-AA*EN(K+1))/X
        GN(K)= (EN(K+1)-AA*GN(K+1))/X
        K = K - 1 
  230 CONTINUE
  235 IF (MU.LE.0) RETURN 
      AA = FKS
  240 K = IND 
      DO 250 I=1,MU
        EN(K+1) = (EMX-X*EN(K))/AA
        GN(K+1)= (EN(K+1)-X*GN(K))/AA
        AA = AA + 1.0D0 
        K = K + 1
  250 CONTINUE
      RETURN
  340 CONTINUE
      IERR=2
      RETURN
      END 
      DOUBLE PRECISION FUNCTION DGAMLN(Z,IERR)
C***BEGIN PROLOGUE  DGAMLN
C***DATE WRITTEN   830501   (YYMMDD)
C***REVISION DATE  830501   (YYMMDD)
C***CATEGORY NO.  B5F
C***KEYWORDS  GAMMA FUNCTION,LOGARITHM OF GAMMA FUNCTION
C***AUTHOR  AMOS, DONALD E., SANDIA NATIONAL LABORATORIES
C***PURPOSE  TO COMPUTE THE LOGARITHM OF THE GAMMA FUNCTION
C***DESCRIPTION
C
C               **** A DOUBLE PRECISION ROUTINE ****
C         DGAMLN COMPUTES THE NATURAL LOG OF THE GAMMA FUNCTION FOR
C         Z.GT.0.  THE ASYMPTOTIC EXPANSION IS USED TO GENERATE VALUES
C         GREATER THAN ZMIN WHICH ARE ADJUSTED BY THE RECURSION
C         G(Z+1)=Z*G(Z) FOR Z.LE.ZMIN.  THE FUNCTION WAS MADE AS
C         PORTABLE AS POSSIBLE BY COMPUTIMG ZMIN FROM THE NUMBER OF BASE
C         10 DIGITS IN A WORD, RLN=AMAX1(-ALOG10(R1MACH(4)),0.5E-18)
C         LIMITED TO 18 DIGITS OF (RELATIVE) ACCURACY.
C
C         SINCE INTEGER ARGUMENTS ARE COMMON, A TABLE LOOK UP ON 100
C         VALUES IS USED FOR SPEED OF EXECUTION.
C
C     DESCRIPTION OF ARGUMENTS
C
C         INPUT      Z IS D0UBLE PRECISION
C           Z      - ARGUMENT, Z.GT.0.0D0
C
C         OUTPUT      DGAMLN IS DOUBLE PRECISION
C           DGAMLN  - NATURAL LOG OF THE GAMMA FUNCTION AT Z.NE.0.0D0
C           IERR    - ERROR FLAG
C                     IERR=0, NORMAL RETURN, COMPUTATION COMPLETED
C                     IERR=1, Z.LE.0.0D0,    NO COMPUTATION
C
C
C***REFERENCES  COMPUTATION OF BESSEL FUNCTIONS OF COMPLEX ARGUMENT
C                 BY D. E. AMOS, SAND83-0083, MAY, 1983.
C***ROUTINES CALLED  I1MACH,D1MACH
C***END PROLOGUE  DGAMLN
      DOUBLE PRECISION CF, CON, FLN, FZ, GLN, RLN, S, TLG, TRM, TST,
     * T1, WDTOL, Z, ZDMY, ZINC, ZM, ZMIN, ZP, ZSQ, D1MACH
      INTEGER I, IERR, I1M, K, MZ, NZ, I1MACH
      DIMENSION CF(22), GLN(100)
C           LNGAMMA(N), N=1,100
      DATA GLN(1), GLN(2), GLN(3), GLN(4), GLN(5), GLN(6), GLN(7),
     1     GLN(8), GLN(9), GLN(10), GLN(11), GLN(12), GLN(13), GLN(14),
     2     GLN(15), GLN(16), GLN(17), GLN(18), GLN(19), GLN(20),
     3     GLN(21), GLN(22)/
     4     0.00000000000000000D+00,     0.00000000000000000D+00,
     5     6.93147180559945309D-01,     1.79175946922805500D+00,
     6     3.17805383034794562D+00,     4.78749174278204599D+00,
     7     6.57925121201010100D+00,     8.52516136106541430D+00,
     8     1.06046029027452502D+01,     1.28018274800814696D+01,
     9     1.51044125730755153D+01,     1.75023078458738858D+01,
     A     1.99872144956618861D+01,     2.25521638531234229D+01,
     B     2.51912211827386815D+01,     2.78992713838408916D+01,
     C     3.06718601060806728D+01,     3.35050734501368889D+01,
     D     3.63954452080330536D+01,     3.93398841871994940D+01,
     E     4.23356164607534850D+01,     4.53801388984769080D+01/
      DATA GLN(23), GLN(24), GLN(25), GLN(26), GLN(27), GLN(28),
     1     GLN(29), GLN(30), GLN(31), GLN(32), GLN(33), GLN(34),
     2     GLN(35), GLN(36), GLN(37), GLN(38), GLN(39), GLN(40),
     3     GLN(41), GLN(42), GLN(43), GLN(44)/
     4     4.84711813518352239D+01,     5.16066755677643736D+01,
     5     5.47847293981123192D+01,     5.80036052229805199D+01,
     6     6.12617017610020020D+01,     6.45575386270063311D+01,
     7     6.78897431371815350D+01,     7.12570389671680090D+01,
     8     7.46582363488301644D+01,     7.80922235533153106D+01,
     9     8.15579594561150372D+01,     8.50544670175815174D+01,
     A     8.85808275421976788D+01,     9.21361756036870925D+01,
     B     9.57196945421432025D+01,     9.93306124547874269D+01,
     C     1.02968198614513813D+02,     1.06631760260643459D+02,
     D     1.10320639714757395D+02,     1.14034211781461703D+02,
     E     1.17771881399745072D+02,     1.21533081515438634D+02/
      DATA GLN(45), GLN(46), GLN(47), GLN(48), GLN(49), GLN(50),
     1     GLN(51), GLN(52), GLN(53), GLN(54), GLN(55), GLN(56),
     2     GLN(57), GLN(58), GLN(59), GLN(60), GLN(61), GLN(62),
     3     GLN(63), GLN(64), GLN(65), GLN(66)/
     4     1.25317271149356895D+02,     1.29123933639127215D+02,
     5     1.32952575035616310D+02,     1.36802722637326368D+02,
     6     1.40673923648234259D+02,     1.44565743946344886D+02,
     7     1.48477766951773032D+02,     1.52409592584497358D+02,
     8     1.56360836303078785D+02,     1.60331128216630907D+02,
     9     1.64320112263195181D+02,     1.68327445448427652D+02,
     A     1.72352797139162802D+02,     1.76395848406997352D+02,
     B     1.80456291417543771D+02,     1.84533828861449491D+02,
     C     1.88628173423671591D+02,     1.92739047287844902D+02,
     D     1.96866181672889994D+02,     2.01009316399281527D+02,
     E     2.05168199482641199D+02,     2.09342586752536836D+02/
      DATA GLN(67), GLN(68), GLN(69), GLN(70), GLN(71), GLN(72),
     1     GLN(73), GLN(74), GLN(75), GLN(76), GLN(77), GLN(78),
     2     GLN(79), GLN(80), GLN(81), GLN(82), GLN(83), GLN(84),
     3     GLN(85), GLN(86), GLN(87), GLN(88)/
     4     2.13532241494563261D+02,     2.17736934113954227D+02,
     5     2.21956441819130334D+02,     2.26190548323727593D+02,
     6     2.30439043565776952D+02,     2.34701723442818268D+02,
     7     2.38978389561834323D+02,     2.43268849002982714D+02,
     8     2.47572914096186884D+02,     2.51890402209723194D+02,
     9     2.56221135550009525D+02,     2.60564940971863209D+02,
     A     2.64921649798552801D+02,     2.69291097651019823D+02,
     B     2.73673124285693704D+02,     2.78067573440366143D+02,
     C     2.82474292687630396D+02,     2.86893133295426994D+02,
     D     2.91323950094270308D+02,     2.95766601350760624D+02,
     E     3.00220948647014132D+02,     3.04686856765668715D+02/
      DATA GLN(89), GLN(90), GLN(91), GLN(92), GLN(93), GLN(94),
     1     GLN(95), GLN(96), GLN(97), GLN(98), GLN(99), GLN(100)/
     2     3.09164193580146922D+02,     3.13652829949879062D+02,
     3     3.18152639620209327D+02,     3.22663499126726177D+02,
     4     3.27185287703775217D+02,     3.31717887196928473D+02,
     5     3.36261181979198477D+02,     3.40815058870799018D+02,
     6     3.45379407062266854D+02,     3.49954118040770237D+02,
     7     3.54539085519440809D+02,     3.59134205369575399D+02/
C             COEFFICIENTS OF ASYMPTOTIC EXPANSION
      DATA CF(1), CF(2), CF(3), CF(4), CF(5), CF(6), CF(7), CF(8),
     1     CF(9), CF(10), CF(11), CF(12), CF(13), CF(14), CF(15),
     2     CF(16), CF(17), CF(18), CF(19), CF(20), CF(21), CF(22)/
     3     8.33333333333333333D-02,    -2.77777777777777778D-03,
     4     7.93650793650793651D-04,    -5.95238095238095238D-04,
     5     8.41750841750841751D-04,    -1.91752691752691753D-03,
     6     6.41025641025641026D-03,    -2.95506535947712418D-02,
     7     1.79644372368830573D-01,    -1.39243221690590112D+00,
     8     1.34028640441683920D+01,    -1.56848284626002017D+02,
     9     2.19310333333333333D+03,    -3.61087712537249894D+04,
     A     6.91472268851313067D+05,    -1.52382215394074162D+07,
     B     3.82900751391414141D+08,    -1.08822660357843911D+10,
     C     3.47320283765002252D+11,    -1.23696021422692745D+13,
     D     4.88788064793079335D+14,    -2.13203339609193739D+16/
C
C             LN(2*PI)
      DATA CON                    /     1.83787706640934548D+00/
C
C***FIRST EXECUTABLE STATEMENT  DGAMLN
      IERR=0
      IF (Z.LE.0.0D0) GO TO 70
      IF (Z.GT.101.0D0) GO TO 10
      NZ = INT(SNGL(Z))
      FZ = Z - FLOAT(NZ)
      IF (FZ.GT.0.0D0) GO TO 10
      IF (NZ.GT.100) GO TO 10
      DGAMLN = GLN(NZ)
      RETURN
   10 CONTINUE
      WDTOL = D1MACH(4)
      WDTOL = DMAX1(WDTOL,0.5D-18)
      I1M = I1MACH(14)
      RLN = D1MACH(5)*FLOAT(I1M)
      FLN = DMIN1(RLN,20.0D0)
      FLN = DMAX1(FLN,3.0D0)
      FLN = FLN - 3.0D0
      ZM = 1.8000D0 + 0.3875D0*FLN
      MZ = INT(SNGL(ZM)) + 1
      ZMIN = FLOAT(MZ)
      ZDMY = Z
      ZINC = 0.0D0
      IF (Z.GE.ZMIN) GO TO 20
      ZINC = ZMIN - FLOAT(NZ)
      ZDMY = Z + ZINC
   20 CONTINUE
      ZP = 1.0D0/ZDMY
      T1 = CF(1)*ZP
      S = T1
      IF (ZP.LT.WDTOL) GO TO 40
      ZSQ = ZP*ZP
      TST = T1*WDTOL
      DO 30 K=2,22
        ZP = ZP*ZSQ
        TRM = CF(K)*ZP
        IF (DABS(TRM).LT.TST) GO TO 40
        S = S + TRM
   30 CONTINUE
   40 CONTINUE
      IF (ZINC.NE.0.0D0) GO TO 50
      TLG = DLOG(Z)
      DGAMLN = Z*(TLG-1.0D0) + 0.5D0*(CON-TLG) + S
      RETURN
   50 CONTINUE
      ZP = 1.0D0
      NZ = INT(SNGL(ZINC))
      DO 60 I=1,NZ
        ZP = ZP*(Z+FLOAT(I-1))
   60 CONTINUE
      TLG = DLOG(ZDMY)
      DGAMLN = ZDMY*(TLG-1.0D0) - DLOG(ZP) + 0.5D0*(CON-TLG) + S
      RETURN
C
C
   70 CONTINUE
      IERR=1
      RETURN
      END
      SUBROUTINE DPSIFN(X, N, KODE, M, ANS, NZ, IERR) 
C***BEGIN PROLOGUE  DPSIFN
C***DATE WRITTEN   820601   (YYMMDD)
C***REVISION DATE  820601   (YYMMDD)
C***CATEGORY NO.  B5F 
C***KEYWORDS  PSI FUNCTION, GAMMA FUNCTION, 
C             DERIVATIVES OF THE GAMMA FUNCTION 
C***AUTHOR  AMOS, DONALD E., SANDIA NATIONAL LABORATORIES 
C***PURPOSE  DPSIFN COMPUTES DERIVATIVES OF THE PSI FUNCTION
C***DESCRIPTION 
C 
C                  * A DOUBLE PRECISION ROUTINE * 
C         DPSIFN COMPUTES M MEMBER SEQUENCES OF SCALED DERIVATIVES OF 
C         THE PSI FUNCTION
C 
C              W(K,X)=(-1)**(K+1)*PSI(K,X)/GAMMA(K+1) 
C 
C         K=N,...,N+M-1 WHERE PSI(K,X) IS THE K-TH DERIVATIVE OF THE PSI
C         FUNCTION.  ON KODE=1, DPSIFN RETURNS THE SCALED DERIVATIVES 
C         AS DESCRIBED.  KODE=2 IS OPERATIVE ONLY WHEN K=0 AND DPSIFN 
C         RETURNS -PSI(X) + LN(X).  THAT IS, THE LOGARITHMIC BEHAVIOR 
C         FOR LARGE X IS REMOVED WHEN KODE=2 AND K=0. WHEN SUMS OR
C         DIFFERENCES OF PSI FUNCTIONS ARE COMPUTED, THE LOGARITHMIC
C         TERMS CAN BE COMBINED ANALYTICALLY AND COMPUTED SEFARATELY
C         TO HELP RETAIN SIGNIFICANT DIGITS. DEFINTIONS AND NOTAION ARE 
C         FOUND IN THE NBS HANDBOOK OF MATHEMATICAL FUNCTIONS (REF. 1). 
C 
C         THE BASIC METHOD OF EVALUATION IS THE ASYMPTOTIC EXPANSION
C         FOR LARGE X.GE.XMIN FOLLOWED BY BACKWARD RECURSION ON A TWO 
C         TERM RECURSION RELATION 
C 
C                  W(X+1) + X**(-N-1) = W(X). 
C 
C         THIS IS SUPPLEMENTED BY A SERIES
C 
C                  SUM( (X+K)**(-N-1) , K=0,1,2,... ) 
C 
C         WHICH CONVERGES RAPIDLY FOR LARGE N. BOTH XMIN AND THE
C         NUMBER OF TERMS OF THE SERIES ARE CALCULATED FROM THE UNIT
C         ROUND OFF OF THE MACHINE ENVIRONMENT. 
C 
C         THE NOMINAL COMPUTATIONAL ACCURACY IS THE MAXIMUM OF UNIT 
C         ROUNDOFF (=D1MACH(4)) AND 1.0D-18 SINCE CRITICAL CONSTANTS
C         ARE GIVEN TO ONLY 18 DIGITS.
C 
C         PSIFN IS THE SINGLE PRECISION VERSION OF DPSIFN.
C 
C 
C         INPUT      X IS DOUBLE PRECISION
C           X      - ARGUMENT, X.GT.0.0D0 
C           N      - FIRST MEMBER OF THE SEQUENCE, 0.LE.N.LE.100
C                    N=0 GIVES ANS(1) = -PSI(X)       ON KODE=1 
C                                       -PSI(X)+LN(X) ON KODE=2 
C           KODE   - SELECTION PARAMETER
C                    KODE=1 RETURNS SCALED DERIVATIVES OF THE PSI 
C                    FUNCTION 
C                    KODE=2 RETURNS SCALED DERIVATIVES OF THE PSI 
C                    FUNCTION EXCEPT WHEN N=0. IN THIS CASE,
C                    ANS(1) = -PSI(X) + LN(X) IS RETURNED.
C           M      - NUMBER OF MEMBERS OF THE SEQUENCE, M.GE.1
C 
C         OUTPUT     ANS IS DOUBLE PRECISION
C           ANS    - A VECTOR OF LENGTH AT LEAST M WHOSE FIRST M
C                    COMPONENTS CONTAIN THE SEQUENCE OF  DERIVATIVES
C                    SCALED ACCORDING TO KODE 
C           NZ     - UNDERFLOW FLAG 
C                    NZ.EQ.0, A NORMAL RETURN 
C                    NZ.NE.0, UNDERFLOW, LAST NZ COMPONENTS OF ANS SET
C                             TO ZERO, ANS(M-K+1)=0.0, K=1,...,NZ 
C           IERR   - ERROR FLAG 
C                    IERR=0, A NORMAL RETURN, COMPUTATION COMPLETED 
C                    IERR=1, INPUT ERROR,     NO COMPUTATION
C                    IERR=2, OVERFLOW,        X TOO SMALL OR N+M-1 TOO
C                            LARGE OR BOTH
C                    IERR=3, ERROR,           N TOO LARGE. DIMENSIONED
C                            ARRAY TRMR(NMAX) IS NOT LARGE ENOUGH FOR N 
C 
C***REFERENCES  HANDBOOK OF MATHEMATICAL FUNCTIONS, AMS 55, NATIONAL
C                 BUREAU OF STANDARDS BY M. ABRAMOWITZ AND I. A.
C                 STEGUN, 1964, PP.258-260, EQTNS. 6.3.5, 6.3.18, 6.4.6,
C                 6.4.9, 6.4.10 
C 
C               A PORTABLE FORTRAN SUBROUTINE FOR DERIVATIVES OF THE
C                 PSI FUNCTION BY D. E. AMOS, ACM TRANS. MATH SOFTWARE, 
C                 1983
C***ROUTINES CALLED  I1MACH,D1MACH
C***END PROLOGUE  DPSIFN
      INTEGER I, IERR, J, K, KODE, M, MM, MX, N, NMAX, NN, NP, NX, NZ 
      INTEGER I1MACH
      DOUBLE PRECISION ANS, ARG, B, DEN, ELIM, EPS, FLN, FN, FNP, FNS,
     * FX, RLN, RXSQ, R1M4, R1M5, S, SLOPE, T, TA, TK, TOL, TOLS, TRM,
     * TRMR, TSS, TST, TT, T1, T2, WDTOL, X, XDMLN, XDMY, XINC, XLN,
     * XM, XMIN, XQ, YINT 
      DOUBLE PRECISION D1MACH 
      DIMENSION B(22), TRM(22), TRMR(100), ANS(M) 
      DATA NMAX /100/ 
C-----------------------------------------------------------------------
C             BERNOULLI NUMBERS 
C-----------------------------------------------------------------------
      DATA B(1), B(2), B(3), B(4), B(5), B(6), B(7), B(8), B(9), B(10), 
     * B(11), B(12), B(13), B(14), B(15), B(16), B(17), B(18), B(19), 
     * B(20), B(21), B(22) /1.00000000000000000D+00,
     * -5.00000000000000000D-01,1.66666666666666667D-01,
     * -3.33333333333333333D-02,2.38095238095238095D-02,
     * -3.33333333333333333D-02,7.57575757575757576D-02,
     * -2.53113553113553114D-01,1.16666666666666667D+00,
     * -7.09215686274509804D+00,5.49711779448621554D+01,
     * -5.29124242424242424D+02,6.19212318840579710D+03,
     * -8.65802531135531136D+04,1.42551716666666667D+06,
     * -2.72982310678160920D+07,6.01580873900642368D+08,
     * -1.51163157670921569D+10,4.29614643061166667D+11,
     * -1.37116552050883328D+13,4.88332318973593167D+14,
     * -1.92965793419400681D+16/
C 
C 
C***FIRST EXECUTABLE STATEMENT  DPSIFN
      IERR = 0
      NZ=0
      IF (X.LE.0.0D0) IERR=1
      IF (N.LT.0) IERR=1
      IF (KODE.LT.1 .OR. KODE.GT.2) IERR=1
      IF (M.LT.1) IERR=1
      IF (IERR.NE.0) RETURN 
      MM=M
      NX = MIN0(-I1MACH(15),I1MACH(16)) 
      R1M5 = D1MACH(5)
      R1M4 = D1MACH(4)*0.5D0
      WDTOL = DMAX1(R1M4,0.5D-18) 
C-----------------------------------------------------------------------
C     ELIM = APPROXIMATE EXPONENTIAL OVER AND UNDERFLOW LIMIT 
C-----------------------------------------------------------------------
      ELIM = 2.302D0*(DBLE(FLOAT(NX))*R1M5-3.0D0) 
      XLN = DLOG(X) 
   41 CONTINUE
      NN = N + MM - 1 
      FN = DBLE(FLOAT(NN))
      FNP = FN + 1.0D0
      T = FNP*XLN 
C-----------------------------------------------------------------------
C     OVERFLOW AND UNDERFLOW TEST FOR SMALL AND LARGE X 
C-----------------------------------------------------------------------
      IF (DABS(T).GT.ELIM) GO TO 290
      IF (X.LT.WDTOL) GO TO 260 
C-----------------------------------------------------------------------
C     COMPUTE XMIN AND THE NUMBER OF TERMS OF THE SERIES, FLN+1 
C-----------------------------------------------------------------------
      RLN = R1M5*DBLE(FLOAT(I1MACH(14)))
      RLN = DMIN1(RLN,18.06D0)
      FLN = DMAX1(RLN,3.0D0) - 3.0D0
      YINT = 3.50D0 + 0.40D0*FLN
      SLOPE = 0.21D0 + FLN*(0.0006038D0*FLN+0.008677D0) 
      XM = YINT + SLOPE*FN
      MX = INT(SNGL(XM)) + 1
      XMIN = DBLE(FLOAT(MX))
      IF (N.EQ.0) GO TO 50
      XM = -2.302D0*RLN - DMIN1(0.0D0,XLN)
      FNS = DBLE(FLOAT(N))
      ARG = XM/FNS
      ARG = DMIN1(0.0D0,ARG)
      EPS = DEXP(ARG) 
      XM = 1.0D0 - EPS
      IF (DABS(ARG).LT.1.0D-3) XM = -ARG
      FLN = X*XM/EPS
      XM = XMIN - X 
      IF (XM.GT.7.0D0 .AND. FLN.LT.15.0D0) GO TO 200
   50 CONTINUE
      XDMY = X
      XDMLN = XLN 
      XINC = 0.0D0
      IF (X.GE.XMIN) GO TO 60 
      NX = INT(SNGL(X)) 
      XINC = XMIN - DBLE(FLOAT(NX)) 
      XDMY = X + XINC 
      XDMLN = DLOG(XDMY)
   60 CONTINUE
C-----------------------------------------------------------------------
C     GENERATE W(N+MM-1,X) BY THE ASYMPTOTIC EXPANSION
C-----------------------------------------------------------------------
      T = FN*XDMLN
      T1 = XDMLN + XDMLN
      T2 = T + XDMLN
      TK = DMAX1(DABS(T),DABS(T1),DABS(T2)) 
      IF (TK.GT.ELIM) GO TO 380 
      TSS = DEXP(-T)
      TT = 0.5D0/XDMY 
      T1 = TT 
      TST = WDTOL*TT
      IF (NN.NE.0) T1 = TT + 1.0D0/FN 
      RXSQ = 1.0D0/(XDMY*XDMY)
      TA = 0.5D0*RXSQ 
      T = FNP*TA
      S = T*B(3)
      IF (DABS(S).LT.TST) GO TO 80
      TK = 2.0D0
      DO 70 K=4,22
        T = T*((TK+FN+1.0D0)/(TK+1.0D0))*((TK+FN)/(TK+2.0D0))*RXSQ
        TRM(K) = T*B(K) 
        IF (DABS(TRM(K)).LT.TST) GO TO 80 
        S = S + TRM(K)
        TK = TK + 2.0D0 
   70 CONTINUE
   80 CONTINUE
      S = (S+T1)*TSS
      IF (XINC.EQ.0.0D0) GO TO 100
C-----------------------------------------------------------------------
C     BACKWARD RECUR FROM XDMY TO X 
C-----------------------------------------------------------------------
      NX = INT(SNGL(XINC))
      NP = NN + 1 
      IF (NX.GT.NMAX) GO TO 390 
      IF (NN.EQ.0) GO TO 160
      XM = XINC - 1.0D0 
      FX = X + XM 
C-----------------------------------------------------------------------
C     THIS LOOP SHOULD NOT BE CHANGED. FX IS ACCURATE WHEN X IS SMALL 
C-----------------------------------------------------------------------
      DO 90 I=1,NX
        TRMR(I) = FX**(-NP) 
        S = S + TRMR(I) 
        XM = XM - 1.0D0 
        FX = X + XM 
   90 CONTINUE
  100 CONTINUE
      ANS(MM) = S 
      IF (FN.EQ.0.0D0) GO TO 180
C-----------------------------------------------------------------------
C     GENERATE LOWER DERIVATIVES, J.LT.N+MM-1 
C-----------------------------------------------------------------------
      IF (MM.EQ.1) RETURN 
      DO 150 J=2,MM 
        FNP = FN
        FN = FN - 1.0D0 
        TSS = TSS*XDMY
        T1 = TT 
        IF (FN.NE.0.0D0) T1 = TT + 1.0D0/FN 
        T = FNP*TA
        S = T*B(3)
        IF (DABS(S).LT.TST) GO TO 120 
        TK = 3.0D0 + FNP
        DO 110 K=4,22 
          TRM(K) = TRM(K)*FNP/TK
          IF (DABS(TRM(K)).LT.TST) GO TO 120
          S = S + TRM(K)
          TK = TK + 2.0D0 
  110   CONTINUE
  120   CONTINUE
        S = (S+T1)*TSS
        IF (XINC.EQ.0.0D0) GO TO 140
        IF (FN.EQ.0.0D0) GO TO 160
        XM = XINC - 1.0D0 
        FX = X + XM 
        DO 130 I=1,NX 
          TRMR(I) = TRMR(I)*FX
          S = S + TRMR(I) 
          XM = XM - 1.0D0 
          FX = X + XM 
  130   CONTINUE
  140   CONTINUE
        MX = MM - J + 1 
        ANS(MX) = S 
        IF (FN.EQ.0.0D0) GO TO 180
  150 CONTINUE
      RETURN
C-----------------------------------------------------------------------
C     RECURSION FOR N = 0 
C-----------------------------------------------------------------------
  160 CONTINUE
      DO 170 I=1,NX 
        S = S + 1.0D0/(X+DBLE(FLOAT(NX-I))) 
  170 CONTINUE
  180 CONTINUE
      IF (KODE.EQ.2) GO TO 190
      ANS(1) = S - XDMLN
      RETURN
  190 CONTINUE
      IF (XDMY.EQ.X) RETURN 
      XQ = XDMY/X 
      ANS(1) = S - DLOG(XQ) 
      RETURN
C-----------------------------------------------------------------------
C     COMPUTE BY SERIES (X+K)**(-(N+1)) , K=0,1,2,... 
C-----------------------------------------------------------------------
  200 CONTINUE
      NN = INT(SNGL(FLN)) + 1 
      NP = N + 1
      T1 = (FNS+1.0D0)*XLN
      T = DEXP(-T1) 
      S = T 
      DEN = X 
      DO 210 I=1,NN 
        DEN = DEN + 1.0D0 
        TRM(I) = DEN**(-NP) 
        S = S + TRM(I)
  210 CONTINUE
      ANS(1) = S
      IF (N.NE.0) GO TO 220 
      IF (KODE.EQ.2) ANS(1) = S + XLN 
  220 CONTINUE
      IF (MM.EQ.1) RETURN 
C-----------------------------------------------------------------------
C     GENERATE HIGHER DERIVATIVES, J.GT.N 
C-----------------------------------------------------------------------
      TOL = WDTOL/5.0D0 
      DO 250 J=2,MM 
        T = T/X 
        S = T 
        TOLS = T*TOL
        DEN = X 
        DO 230 I=1,NN 
          DEN = DEN + 1.0D0 
          TRM(I) = TRM(I)/DEN 
          S = S + TRM(I)
          IF (TRM(I).LT.TOLS) GO TO 240 
  230   CONTINUE
  240   CONTINUE
        ANS(J) = S
  250 CONTINUE
      RETURN
C-----------------------------------------------------------------------
C     SMALL X.LT.UNIT ROUND OFF 
C-----------------------------------------------------------------------
  260 CONTINUE
      ANS(1) = X**(-N-1)
      IF (MM.EQ.1) GO TO 280
      K = 1 
      DO 270 I=2,MM 
        ANS(K+1) = ANS(K)/X 
        K = K + 1 
  270 CONTINUE
  280 CONTINUE
      IF (N.NE.0) RETURN
      IF (KODE.EQ.2) ANS(1) = ANS(1) + XLN
      RETURN
  290 CONTINUE
      IF (T.GT.0.0D0) GO TO 380 
      NZ=0
      IERR=2
      RETURN
  380 CONTINUE
      NZ=NZ+1 
      ANS(MM)=0.0D0 
      MM=MM-1 
      IF (MM.EQ.0) RETURN 
      GO TO 41
  390 CONTINUE
      NZ=0
      IERR=3
      RETURN
      END
      SUBROUTINE DIERFC(X,KODE,ANS,IERR)
C
C     DONALD E. AMOS  DECEMBER,2000; APRIL, 2002
C
C     DIERFC COMPUTES THE ITERATED COERROR FUNCTION ANS=IERFC(X) FOR
C     X.GE.0.0D0 FOR KODE=1 AND ANS=DEXP(X^2)*IERFC(X) FOR KODE=2
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     ERROR CONDITIONS:
C
C     IERR=0 NORMAL RETURN, COMPUTATION COMPLETED
C     IERR=1 X IS NEGATIVE, OR KODE IS NOT 1 OR 2. ANS=0.0D0 RETURNED
C     IERR=2 UNDERFLOW, ANS=0.0D0 RETURNED
C
C     CALLS ROUTINES: DRERF
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION COE(22,3),COEF(24)
      DATA RTPI             /1.772453850905516027D0/
C     CHEBYSHEV COEFFIENTS FOR [1,3]
      DATA (COE(I,1),I=1,22)/
     1   .20234490536480757018D+0  ,   .52104244263797924458D-1  ,      
     2  -.11481896726185159156D-1  ,   .19250980931000234771D-2  ,      
     3  -.22874339469432405757D-3  ,   .76413350418226798653D-5  ,      
     4   .57038155131006421955D-5  ,  -.22679132495328632457D-5  ,      
     5   .59345956500181181188D-6  ,  -.12973982700380805317D-6  ,      
     6   .25393839164658043100D-7  ,  -.45900654450527613629D-8  ,      
     7   .77945560472983517757D-9  ,  -.12569752051278888622D-9  ,      
     8   .19392371763887008531D-10 ,  -.28776546478664314330D-11 ,      
     9   .41241005463317949716D-12 ,  -.57267334816262484620D-13 ,      
     A   .77252252761786501244D-14 ,  -.10145811447810374360D-14 ,      
     B   .12996661027298386093D-15 ,  -.16264168799487232138D-16 /      
C     CHEBYSHEV COEFFIENTS FOR [3.0,5.5]
      DATA (COE(I,2), I=1,20)/
     1   .25921033022179837406D+0  ,   .11863530222703435905D-1  ,      
     2  -.22456106528565330611D-2  ,   .36452384007591723187D-3  ,      
     3  -.53326825393439572246D-4  ,   .71668700726266341359D-5  ,      
     4  -.88930993054872490740D-6  ,   .10122521979082776356D-6  ,      
     5  -.10319937364889814600D-7  ,   .88033499475257759951D-9  ,      
     6  -.47579661094375305761D-10 ,  -.27339349067429062305D-11 ,      
     7   .14647408524171149142D-11 ,  -.31955443360429115183D-12 ,      
     8   .55559386991032515372D-13 ,  -.86214247220764385389D-14 ,      
     9   .12445509652263484721D-14 ,  -.17053165852882875168D-15 ,      
     A   .22435960748663718688D-16 ,  -.28550649356569489785D-17 /
C     CHEBYSHEV COEFFIENTS FOR [5.5,8.0]
      DATA (COE(I,3),I=1,16)/
     1   .27286438146173752156D+00 ,   .32425882603098985126D-02 ,      
     2  -.42224100912591457365D-03 ,   .48123603005869450314D-04 ,      
     3  -.50583642058403311588D-05 ,   .50182354871013814033D-06 ,      
     4  -.47550893368040893857D-07 ,   .43318144700139924622D-08 ,      
     5  -.38068269625179404945D-09 ,   .32311570263089790410D-10 ,      
     6  -.26467861486267136191D-11 ,   .20860606373577721774D-12 ,      
     7  -.15719568201565543872D-13 ,   .11189571252221398860D-14 ,      
     8  -.73431174026315067217D-16 ,   .41944332960396830269D-17 /
C     COEFFICIENTS FOR ASYMPTOTIC EXPANSION, X > 8.0D0
      DATA (COEF(I),I=1,24)/
     1   .50000000000000000000D+0  ,    -.75000000000000000000D+0  ,
     2   .18750000000000000000D+1  ,    -.65625000000000000000D+1  ,                                            
     3   .29531250000000000000D+2  ,    -.16242187500000000000D+3  ,                                            
     4   .10557421875000000000D+4  ,    -.79180664062500000000D+4  ,                                            
     5   .67303564453125000000D+5  ,    -.63938386230468750000D+6  ,       
     6   .67135305541992187500D+7  ,    -.77205601373291015625D+8  ,        
     7   .96507001716613769531D+9  ,    -.13028445231742858886D+11 ,       
     8   .18891245586027145385D+12 ,    -.29281430658342075347D+13 ,       
     9   .48314360586264424324D+14 ,    -.84550131025962742567D+15 ,          
     A   .15641774239803107374D+17 ,    -.30501459767616059381D+18 ,         
     B   .62527992523612921731D+19 ,    -.13443518392576778172D+21 ,         
     C   .30247916383297750887D+22 ,    -.71082603500749714585D+23 /
      IERR=0
      IF(X.LT.0.0D0) THEN
        ANS=0.0D0
        IERR=1
        RETURN
      ENDIF
      IF ((KODE.LT.1).OR.(KODE.GT.2)) THEN
        ANS=0.0D0
        IERR=1
        RETURN
      ENDIF
      X2=X*X
      IF(KODE.EQ.1) THEN
        IF(X2.GT.675.0D0) THEN
          IERR=2
          ANS=0.0D0
          RETURN
        ENDIF
        DX=DEXP(-X2)
      ELSE
        DX=1.0D0
      ENDIF
      IF(X.LE.1.0D0) THEN
C       RECURRENCE FOR IERFC: IERFC(X)=EXP(-X*X)/SQRT(PI)-X*ERFC(X)
        KODEA=KODE+1
        NZ=0
        ERFA=DRERF(X,KODEA,NZ)
        ANS=DX/RTPI-X*ERFA
        RETURN
      ENDIF
      IF (X.LE.3.0D0) THEN
        APB=4.0D0
        BMA=2.0D0
        N=22
        CALL DCHBYS(APB,BMA,X,N,COE(1,1),SUM)
        ANS=DX*SUM/X2
        RETURN
      ENDIF
      IF (X.LE.5.5D0) THEN
        APB=8.50D0
        BMA=2.5D0
        N=20
        CALL DCHBYS(APB,BMA,X,N,COE(1,2),SUM)
        ANS=DX*SUM/X2
        RETURN
      ENDIF
      IF (X.LE.8.0D0) THEN
        APB=13.50D0
        BMA=2.5D0
        N=16
        CALL DCHBYS(APB,BMA,X,N,COE(1,3),SUM)
        ANS=DX*SUM/X2
        RETURN
      ENDIF
C     ASYMPTOTIC EXPANSION FOR IERFC(X) FOR   X > 8.0D0
      SS=COEF(1)
      P=1.0D0
      REL=0.50D-15
      M=25
      ATRM1=SS
      DO 30 I=2,M
        P=P/X2
        TRM2=COEF(I)*P
        ATRM2=DABS(TRM2)
        IF(ATRM2.LT.REL*DABS(SS)) GOTO 40
        IF(ATRM2.GE.ATRM1)  GOTO 40
        SS=SS+TRM2
        ATRM1=ATRM2
 30   CONTINUE
 40   CONTINUE
      SS=DX*SS/RTPI
      ANS=SS/X2
      RETURN
      END
      SUBROUTINE DCHBYS(APB,BMA,X,N,COEF,SUM)
C-----------------------------------------------------------------------
C     CHBYS SUMS A CHEBYSHEV SERIES WITH COEFFICIENTS COEF(I), I=1,N
C     ON [A,B] AT X. APB=A+B , BMA=B-A
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)       
      DIMENSION COEF(N)
      T=(X+X-APB)/BMA
      TT=T+T
      F1=0.0D0
      F2=0.0D0
      NM=N-1
      DO 160 I=1,NM
        J=N-I+1
        TEMP=F1
        F1=TT*F1-F2+COEF(J)
        F2=TEMP
  160 CONTINUE
      SUM=T*F1-F2+COEF(1)
      RETURN
      END
      SUBROUTINE DINERFC(X,N0,KODE,N,REL,Y,NZ) 
C-----------------------------------------------------------------------
C     WRITTEN BY DONALD E. AMOS 
C     DATE: JULY, 2001
C 
C     REFERENCES
C         AMOS, D.E. AND DANIEL, S.L., CDC 6600 CODES FOR THE ERROR
C         FUNCTION, CUMULATIVE NORMAL AND RELATED FUNCTIONS, SANDIA
C         NATIONAL LABORATORIES REPORT SC-DR-72 0918, DECEMBER, 1972
C 
C         AMOS, D.E., BOUNDS ON ITERATED COERROR FUNCTIONS AND THEIR
C         RATIOS, MATH. COMP.,27, 1973, PP.413-427
C 
C     ABSTRACT       * A DOUBLE PRECISION ROUTINE*
C         DINERFC COMPUTES N-ITERATED COERROR FUNCTIONS(REPEATED 
C         INTEGRALS OF THE COERROR FUNCTION ERFC(X))
C 
C           Y(K)= I(SUPER N0+K-1)ERFC(X)  K=1,2,...,N 
C 
C                     OR  SCALED  FUNCTIONS 
C 
C           Y(K)= I(SUPER N0+K-1)ERFC(X)*EXP(X**2), K=1,2,...,N 
C 
C         FOR X.GT.0. FOR X.GT.0, ACCURATE RATIOS ARE COMPUTED
C         WITH THE SEQUENCE NORMALIZED BY Y(-1)=2.*EXP(-X**2)/SQRT(PI)
C         FOR X.LT.0., STARTING VALUES USING 2.*EXP(-X**2)/SQRT(PI) 
C         AND ERFC(X) ARE USED WITH FORWARD RECURSION ON
C 
C                   2*(K+1)*Y(K+1)=-2*X*Y(K)+Y(K-1).
C 
C         ERFC(X) FOR X.LT.0 IS COMPUTED FROM 
C 
C                      ERFC(X) = 2.0 - ERFC(-X)
C 
C         WHERE ERFC(-X) IS COMPUTED FROM THE CODE FOR X.GT.0.
C 
C     DESCRIPTION OF ARGUMENTS
C                     
C         INPUT            * A DOUBLE PRECISION ROUTINE*
C           X      - X, ARGUMENT, UNRESTRICTED
C           N0     - FIRST ITERATED COERROR FUNCTION DESIRED, N0.GE.0 
C           KODE   - A SELECTION PARAMETER
C                    KODE=1  RETURNS Y(K)=I(SUPER N0+K-1)ERFC(X) 
C                            FOR K=1,2,...N 
C                    KODE=2  RETURNS
C                          Y(K)=I(SUPER N0+K-1)ERFC(X)           X.LE.0
C                              =I(SUPER N0+K-1)ERFC(X)*EXP(X**2) X.GT.0
C                          FOR K=1,2,...,N 
C           N      - NUMBER OF FUNCTIONS IN THE SEQUENCE, N.GE.1
C           REL    - RELATIVE ERROR PARAMETER, REL=1.D-S FOR S
C                    SIGNIFICANT DIGITS, 2.LE.D.LE.18 AND
C                    REL.GT.D1MACH(4)
C 
C         OUTPUT
C           Y      - A VECTOR OF DIMENSION N OR GREATER CONTAINING
C                    THE N VALUES REQUESTED, DEPENDING ON KODE
C           NZ     - NZ.EQ.0 - A NORMAL RETURN, COMPUTATION COMPLETED 
C                    NZ.NE.0 - LAST NZ COMPONENTS OF Y UNDERFLOWED
C                              Y(K)=0.0, K=N-NZ+1,...,N RETURNED
C 
C     ERROR CONDITIONS
C         IMPROPER INPUT ARGUMENT- A FATAL ERROR
C         UNDERFLOW FOR X.GT.0 AND KODE=1 - A NON-FATAL ERROR,Y(K)=0.0, 
C         K=N-NZ+1,...,N RETURNED 
C         PROCESS FAILS TO CONVERGE AFTER 25 ITERATIONS 
C 
C     INERFC USES SUBROUTINES DGAMLN,I1MACH,D1MACH,XERROR
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION Y(N),SR1(25),SR2(25)
      DATA TORP, MAXI /1.12837916709551D0, 24/
C 
      IF(REL.GT.1.0D-2) GO TO 91
      IF(N.LT.1) GO TO 92 
      IF(KODE.LT.1 .OR. KODE.GT.2) GO TO 93 
      IF(N0.LT.0) GO TO 94
      NZ=0
      R1M4=D1MACH(4)
      R1M5=D1MACH(5)
      TOL=REL 
      IF(REL.LT.R1M4) TOL=R1M4
      KK=MIN(IABS(I1MACH(15)),IABS(I1MACH(16)))
      AA=FLOAT(KK)*R1M5-3.0D0
      XLIM=SQRT(D1MACH(2)*1.0D-3)
      ELIM=2.303D0*AA
      IF(ABS(X).GT.XLIM) GO TO 95
      X2=X*X
      IF(X.LE.0.0D0) GO TO 80
      IFLAG=1 
      CX=TORP 
      IF(KODE.EQ.1) CX=CX*DEXP(-X2)
      XX=X
      NN=N
      NN0=N0
  100 CONTINUE
      NM1=NN-1
      NF=NN0+NN-1 
      NSAVE=NF
      IF(NF.GT.24) GO TO 4
      IF(XX.GE.2.0D0) GO TO 4
      NF=25 
    4 CONTINUE
      IF(X2.GT.ELIM .AND. KODE.EQ.1) GO TO 800
      SK=NF 
      CK=SK 
      SKP1=SK+1.0D0
      TSKP1=SKP1+SKP1 
      EARG=DGAMLN(0.5D0*(SK+2.0D0),IERR)-DGAMLN(0.5D0*(SK+3.0D0),IERR)
      RK1=0.5D0*DEXP(EARG)
      FAC=TSKP1*RK1 
      RSAVE=1.0D0/(XX+SQRT(X2+FAC*FAC))
      SR1(1)=RSAVE
      K=0 
      SKM1=SK-1.0D0
      EARG=DGAMLN(0.5D0*(SKM1+2.0D0),IERR)-DGAMLN(0.5D0*(SKM1+3.0D0),IER
     *R)
      RK2=0.5D0*DEXP(EARG)
C 
   10 CONTINUE
      K=K+1 
      KP1=K+1 
      CK=CK+1.0D0
      TSKP1=TSKP1+2.0D0
      IF(MOD(K,2).EQ.0)GO TO 13 
C     K ODD 
      RK2=RK2*CK/(CK+1.0D0)
      RK=RK2
      GO TO 15
C     K EVEN
   13 RK1=RK1*CK/(CK+1.0D0)
      RK=RK1
   15 CONTINUE
      FAC=TSKP1*RK
      SR2(1)=1.0D0/(XX+DSQRT(X2+FAC*FAC))
      IF(K.GT.MAXI) GO TO 90
C 
      DO 12 J=1,K 
        RA=SR2(J)/SR1(J)
        AD=K-J+1
        AKP1=SK+AD
        TAKP1=AKP1+AKP1
        SR2(J+1)=1.0D0/(XX+DSQRT(X2+TAKP1*RA))
   12 CONTINUE
      RBAR=0.5D0/(XX+AKP1*SR2(K))
      IF(ABS(SR2(KP1)-RBAR) .LE. TOL*RBAR) GO TO 30 
      DO 14 I=1,KP1 
        SR1(I)=SR2(I)
   14 CONTINUE
      GO TO 10
C 
   30 CONTINUE
      AK=SK 
      RK=SR2(KP1) 
      IF(NF.EQ.NSAVE) GO TO 35
      NI=NF-NSAVE 
      DO 31 I=1,NI
        RK=0.5D0/(XX+AK*RK)
        AK=AK-1.0D0
   31 CONTINUE
C-----------------------------------------------------------------------
C     GENERATING RATIOS FOR X .GE. 0 BY BACKWARD RECURRENCE 
C-----------------------------------------------------------------------
   35 CONTINUE
      Y(NN)=RK
      IF(NN.EQ.1) GO TO 37
      DO 40 K=1,NM1 
        RK=0.5D0/(XX+AK*RK)
        Y(NN-K)=RK
        AK=AK-1.0D0
   40 CONTINUE
C 
   37 CONTINUE
C 
      PROD=CX
      IF(NN0.EQ.0) GO TO 45 
      DO 41 K=1,NN0 
        RK=0.5D0/(XX+AK*RK)
        AK=AK-1.0D0
        PROD=PROD*RK
   41 CONTINUE
C 
   45 CONTINUE
      DO 43 K=1,NN
        PROD=PROD*Y(K)
        Y(K)=PROD
   43 CONTINUE
      IF(IFLAG.EQ.2) GO TO 83
   44 CONTINUE
      KK=NN 
      DO 46 K=1,NN
        IF(Y(KK).NE.0.0D0) GO TO 47
        NZ=NZ+1
        KK=KK-1
   46 CONTINUE
   47 CONTINUE
      RETURN
C-----------------------------------------------------------------------
C     NEGATIVE X VALUES 
C-----------------------------------------------------------------------
   80 CONTINUE
      IFLAG=2 
      XX=-X 
      NN0=0 
      NN=1
      KK=IABS(I1MACH(14))+1
      URLIM=2.303D0*DBLE(FLOAT(KK))*R1M5
      IF(X2.GT.URLIM) GO TO 86
      CX=TORP*DEXP(-X2)
      GO TO 100 
C 
   86 CONTINUE
      Y(1)=0.0D0
      CX=0.0D0
   83 CONTINUE
      Y2 = 2.0D0 - Y(1)
      Y1=CX 
C-----------------------------------------------------------------------
C     FORWARD RECURSION 
C-----------------------------------------------------------------------
      TX=X+X
      TWAN = 2.0D0
      IF(N0.EQ.0) GO TO 85
      DO 82 K=1,N0
        YS=Y2
        Y2=(-TX*Y2+Y1)/TWAN
        Y1=YS
        TWAN = TWAN+2.0D0
   82 CONTINUE
   85 CONTINUE
      Y(1)=Y2
      IF(N.EQ.1) RETURN
      DO 84 K=2,N 
        YS=Y2
        Y2=(-TX*Y2+Y1)/TWAN
        Y(K)=Y2
        Y1=YS
        TWAN = TWAN+2.0D0
   84 CONTINUE
      RETURN
C 
  800 CONTINUE
      NZ=N
      DO 805 I=1,N
        Y(I)=0.0D0
  805 CONTINUE
      RETURN
C 
C 
   90 CALL XERROR(' IN DINERFC, NO CONVERGENCE AFTER 25 ITERATIONS',47,
     & 1,2)
      RETURN
   91 CALL XERROR(' IN DINERFC, REL.GT.1.0D-2',26,1,2)
      RETURN
   92 CALL XERROR(' IN DINERFC, N IS ZERO OR NEGATIVE',34,1,2)
      RETURN
   93 CALL XERROR(' IN DINERFC, KODE NOT 1 OR 2',28,1,2)
      RETURN
   94 CALL XERROR(' IN DINERFC, N0 IS NEGATIVE',27,1,2)
      RETURN
   95 CONTINUE
      IF(KODE.EQ.1) GO TO 800
      CALL XERROR(' IN DINERFC, X**2 OVERFLOWS',27,1,2)
      RETURN
      END
      SUBROUTINE DQUAD8(DQFUN,INIT,X1,SIG,REL,X2,QANS,IERR)
C
C     A DOUBLE PRECISION ROUTINE
C
C     INFINITE INTERVALS
C-----------------------
C     DQUAD8 EVALUATES INFINITE INTEGRALS ON (X1,INF) BY SUMMING
C     QUADRATURES (WITH DGAUS8) OF LENGTH SIG STARTING AT X1. TESTS
C     FOR CONVERGENCE ARE MADE IN A RELATIVE ERROR TEST USING REL FOR
C     BOTH DGAUS8 AND DQUAD8. THE TRUNCATION ERROR TEST FOR DQAUD8 IS
C
C     DABS(LAST QUADRATURE OF LENGTH SIG).LE.REL*DABS(ACCUMULATED SUM)
C
C     THE QUADRATURE ON (X1,X2) IS RETURNED IN QANS WITH AN ERROR
C     INDICATOR IERR. IERR=1 IS NORMAL AND IERR=5 MEANS THAT THE
C     CONVERGENCE OF THE INTEGRAL WAS NOT OBTAINED IN 20 SIG STEPS.
C     OTHER VALUES OF IERR ARE DEFINED IN THE PROLOGUE OF DGAUS8. X2
C     CAN BE USED TO COMPUTE A RIGOROUS TRUNCATION ESTIMATE IN THE
C     CALLING PROGRAM IF ONE IS KNOWN. IF THIS TRUNCATION ESTIMATE IS
C     NOT SATISFIED, THEN DQUAD8 CAN BE CALLED AGAIN WITH NO CHNAGE IN
C     THE CALL LIST.
C
C     INIT=0 ON THE FIRST CALL. IF DQUAD8 INDICATES THAT X2 IS TOO
C     SMALL (IERR=5) OR ONE WANTS MORE ASSURANCE THAT THE ANSWER IS
C     ACCURATE, THEN DQUAD8 CAN BE CALLED AGAIN WITH NO CHANGES IN
C     PARAMETERS. AFTER THE INITIAL CALL, INIT = THE NUMBER OF SIG
C     STEPS. ON SUBSEQUENT CALLS, INIT.NE.0 SIGNALS A CONTINUATION OF
C     THE STEPPING PROCESS UNTIL THE TRUNCATION ERROR CRITERION IS MET
C     AGAIN, ETC. SIG CAN BE CHANGED ON SUCCESSIVE CALLS IF DESIRED.
C     MOST LIKELY, DQUAD8'S TRUNCATION TEST WILL BE MET IN ONE
C     QUADRATURE STEP ON EACH CALL AFTER THE FIRST IF IERR=1 WAS
C     RETURNED ON THE PREVIOUS CALL.(I.E. HAVING SEEN CONVERGENCE ONCE,
C     DQUAD8 WILL SEE CONVERGENCE THEREAFTER ON EACH SIG STEP.)
C
C     FINITE INTERVALS
C---------------------
C     FOR MQ QUADRATURES ON A FINITE INTERVAL (A,B) SET THE PARAMETERS
C     AS FOLLOWS:
C                INIT=-MQ
C                X1=A
C                SIG=(B-A)/DBLE(FLOAT(MQ))                  
C     IN THIS CASE, IERR=5 CANNOT OCCUR SINCE THE NUMBER OF QUADRATURES
C     IS PREDETERMINED. ANY OTHER VALUE OF IERR SIGNALS AN ERROR ON SOME
C     INTERVAL FROM DGAUS8.
C
C     CODING THE INTEGRAND
C-------------------------
C     A DOUBLE PRECISION FUNCTION NAME FOR THE INTEGRAND IS SUPPLIED AS
C     THE FIRST ARGUMENT IN THE CALL TO DQUAD8. (DQFUN IS THE PLACE
C     HOLDER NAME FOR THIS FUNCTION). THIS FUNCTION, WHICH COMPUTES THE
C     INTEGRAND, MUST BE DECLARED EXTERNAL IN THE CALLING PROGRAM.
C
C     PARAMETERS FOR THE INTEGRAND ARE SUPPLIED IN A COMMMON STATEMENT.
C
C------------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DATA MCALLS/20/
      EXTERNAL DQFUN
      IERR=1
      IFLAG=0
      IF (INIT.EQ.0) THEN
        XA=X1
        S=0.0D0
        SS=0.0D0
        JMAX=MCALLS
      ELSE
        IF(INIT.LT.0) THEN
          XA=X1
          S=0.0D0
          SS=0.0D0
          JMAX=IABS(INIT)
          IFLAG=1
          INIT=JMAX
        ELSE 
          XA=X2
          S=0.0D0
          SS=QANS
          JMAX=MCALLS
        ENDIF
      ENDIF
      XB=XA+SIG
      DO 35 J=1,JMAX
        CREL=REL
        CALL DGAUS8(DQFUN,XA,XB,CREL,ANS,KERR)
        IF(KERR.NE.1) THEN
          IERR=KERR
        ENDIF
        S=S+ANS
        IF(IFLAG.EQ.0) THEN
          INIT=INIT+1        
          IF (DABS(ANS).LE.REL*DABS(S+SS)) THEN
            X2=XB          
            GOTO 36
          ENDIF
        ENDIF
        XA=XB
        XB=XA+SIG
  35  CONTINUE
      X2=XA
      IF(IFLAG.EQ.0) THEN
        IF(KERR.EQ.1) THEN
          IERR=5
        ENDIF
      ENDIF
  36  CONTINUE
      QANS=S+SS
      RETURN
      END

      DOUBLE PRECISION FUNCTION DFERF(X,REL,IERR)
C
C     DONALD E. AMOS, JANUARY, 2001  
C
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     REFERENCES: FOLDER 16, FOLDER 11, FOLDER 6
C
C     DFERF COMPUTES
C
C                 DFERF = INT ON (0,X] OF ERF(W)/W
C
C     BY THE POWER SERIES FOR SMALL X.LE.1.0D0 AND
C
C                 DFERF = DGERFC(X)+0.5D0*EULER+DLOG(2*X)
C
C     FOR X.GT.1.0D0 WHERE DGERFC IS THE INTEGRAL OF ERFC(W)/W ON
C     [X,INF) AND EULER IS THE EULER CONSTANT = 0.5772156649015329D0
C
C     REL SHOULD SATISFY 0.5D-15 .LE. REL .LE. 0.5D-6.
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DATA EULER /0.577215664901532861D0/
      DATA RTPI  /1.772453850905516027D0/
      IERR=0
      IF(X.LE.0.0D0)THEN
        IERR=1
        DFERF=0.0D0
        RETURN
      ENDIF
      IF(REL.GT.0.5D-3) THEN
        IERR=1
        REL=0.5D-6
      ENDIF
      IF (REL.LT.0.5D-15) THEN
        IERR=1
        REL=0.5D-15
      ENDIF
      X2=X*X
      IF (X.LE.1.0D0) THEN
C-----------------------------------------------------------------------
C       POWER SERIES FOR ERF(X)/X ON [0,1]
C-----------------------------------------------------------------------   
        S=1.0D0
        AK=1.0D0
        BK=3.0D0
        CK=-X2
        DO 10 K=1,50
          TRM=CK/(BK*BK)
          S=S+TRM
          IF (DABS(TRM).LT.REL*DABS(S)) GOTO 11
          AK=AK+1.0D0
          BK=BK+2.0D0
          CK=-CK*X2/AK
  10    CONTINUE
  11    CONTINUE
        SUM=(S+S)*X/RTPI      
        DFERF=SUM
        RETURN
      ENDIF
C-----------------------------------------------------------------------
C     DFERF=DGERFC(X)+0.5D0*EULER+DLOG(2*X)
C-----------------------------------------------------------------------
      KODE=1
      DFERF=DGERFC(X,KODE,REL,IERR)+0.5D0*EULER+DLOG(X+X)
      RETURN
      END
      DOUBLE PRECISION FUNCTION DGERFC(X,KODE,REL,IERR)
C
C     DONALD E. AMOS, JANUARY, 2001  
C
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C
C     REFERENCES: FOLDER 6, FOLDER 11, FOLDER 16
C
C     ON KODE=1, DGERFC COMPUTES
C
C              DGERFC= INTEGRAL ON  [X,INF) OF ERFC(W)/W.
C
C     ON KODE=2, THIS RESULT IS SCALED BY DEXP(X**2).
C     REL IS THE RELATIVE ERROR DESIRED, 0.5D-14 .LE. REL .LE. 0.5D-3
C
C     IERR=0 IS A NORMAL RETURN
C     IERR=1 IS AN INPUT ERROR
C     IERR=2 IS AN UNDERFLOW ON KODE=1 WITH DGERFC=0.0D0 RETURNED
C
C     THE SERIES FOR
C
C                 DFERF=  INTEGRAL ON [0,1] OF ERF(W)/W
C     ALONG WITH     
C                 DGERFC=DFERF-0.5D0*EULER-DLOG(2.0D0*X)
C
C     ARE USED FOR SMALL X.LE.1.125D0 AND THE ASYMPTOTIC EXPANSION IS USED
C     FOR X.GE.7.0D0. CHEBYSHEV EXPANSIONS FOR (X**3)*EXP(X**2) ARE
C     EVALUATED ELSEWHERE WITH RELATIVE ERRORS 0.5D-12 OR BETTER. EULER
C     IS THE EULER CONSTANT = 0.577215664901532861D0
C
C     REL SHOULD SATISFY 0.5D-15 .LE. REL .LE. 0.5D-6.
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION COE(24,3),ENH(24)
C     CHEBYSHEV COEFFICIENTS FOR INTERVAL (1,3]
      DATA (COE(I,1), I=1,24)/
     1      0.183936201622926800D+00,      0.602481282921201200D-01,
     2     -0.119525542040734300D-01,      0.163048183862326200D-02,
     3     -0.916761486984451800D-04,     -0.300245124943356100D-04,
     4      0.136020773297374100D-04,     -0.357151573002627300D-05,
     5      0.743773093967212500D-06,     -0.131331376113140100D-06,
     6      0.196950958343117800D-07,     -0.233463166825802300D-08,
     7      0.141611588786531800D-09,      0.304836446286498300D-10,
     8     -0.158873383783807400D-10,      0.471064076171147000D-11,
     9     -0.117254478451858600D-11,      0.268672719869292500D-12,
     A     -0.589487969074253200D-13,      0.126554771091031300D-13,
     B     -0.269424003935865400D-14,      0.573660397938811300D-15,
     C     -0.122809738646140600D-15,      0.265136030774818600D-16/
C     CHEBYSHEV COEFFICIENTS FOR INTERVAL (3,5]
      DATA (COE(I,2), I=1,18)/
     1      0.249911966143169100D+00,      0.137407615980377800D-01,
     2     -0.211076375427525400D-02,      0.274149770820318000D-03,
     3     -0.315160070674318100D-04,      0.324067726462838300D-05,
     4     -0.293779756592443700D-06,      0.221272952269653000D-07,
     5     -0.108159106183739300D-08,     -0.393712006838136900D-10,
     6      0.211188488938613800D-10,     -0.394325155773269300D-11,
     7      0.574358099846202900D-12,     -0.736872227670002800D-13,
     8      0.868815019030334600D-14,     -0.959589743808029100D-15,
     9      0.100178090811985800D-15,     -0.991516948990920300D-17/
C     CHEBYSHEV COEFFICIENTS FOR INTERVAL (5,7)
      DATA (COE(I,3), I=1,14)/
     1      0.267032022166309300D+00,      0.465764615292050200D-02,
     2     -0.529406889059992700D-03,      0.522543466857855500D-04,
     3     -0.471680853238733800D-05,      0.398010531483489200D-06,
     4     -0.317099625639186200D-07,      0.239340951085099200D-08,
     5     -0.170820332342142600D-09,      0.114336576815315400D-10,
     6     -0.703928286525811000D-12,      0.380529299822365100D-13,
     7     -0.156156454824444000D-14,      0.110122861980293900D-16/
      DATA EULER /0.577215664901532861D0/
      DATA RTPI  /1.772453850905516027D0/
      IERR=0
      IF(X.LE.0.0D0)THEN
        IERR=1
        DGERFC=0.0D0
        RETURN
      ENDIF
      IF((KODE.NE.1).AND.(KODE.NE.2))THEN
        IERR=1
        DGERFC=0.0D0
        RETURN
      ENDIF
      IF(REL.GT.0.5D-3) THEN
        IERR=1
        REL=0.5D-6
      ENDIF
      IF (REL.LT.0.5D-15) THEN
        IERR=1
        REL=0.5D-15
      ENDIF
      X2=X*X
      IF (X.LE.1.0D0) THEN
C-----------------------------------------------------------------------
C       DGERFC=POWER SERIES FOR ERF(X)/X-0.5*EULER-LOG(2*X) ON (0,1]
C-----------------------------------------------------------------------   
        S=1.0D0
        AK=1.0D0
        BK=3.0D0
        CK=-X2
        DO 10 K=1,50
          TRM=CK/(BK*BK)
          S=S+TRM
          IF (DABS(TRM).LT.REL*DABS(S)) GOTO 11
          AK=AK+1.0D0
          BK=BK+2.0D0
          CK=-CK*X2/AK
  10    CONTINUE
  11    CONTINUE
        SUM=(S+S)*X/RTPI      
        DGERFC=SUM-0.5D0*EULER-DLOG(X+X)
        IF(KODE.EQ.2) THEN
          DGERFC=DGERFC*DEXP(X2)
        ENDIF
        RETURN
      ENDIF
C-----------------------------------------------------------------------
C     CHEBYSHEV EXPANSIONS ON INTERVALS OF LENGTH 2 ON (1,7)
C-----------------------------------------------------------------------     
      IF (X.LE.3.0D0) THEN
        N=24
        BMA=2.0D0
        APB=4.0D0
        CALL DCHBYS(APB,BMA,X,N,COE(1,1),SUM)
        X3=X2*X
        IF (KODE.EQ.1) THEN          
          CC=DEXP(-X2)
        ELSE
          CC=1.0D0
        ENDIF
        DGERFC=SUM*CC/X3
        RETURN
      ENDIF
      IF (X.LE.5.0D0) THEN
        N=18
        BMA=2.0D0
        APB=8.0D0
        CALL DCHBYS(APB,BMA,X,N,COE(1,2),SUM)
        X3=X2*X
        IF (KODE.EQ.1) THEN          
          CC=DEXP(-X2)
        ELSE
          CC=1.0D0
        ENDIF
        DGERFC=SUM*CC/X3
        RETURN
      ENDIF
      IF (X.LT.7.0D0) THEN
        N=14
        BMA=2.0D0
        APB=12.0D0
        CALL DCHBYS(APB,BMA,X,N,COE(1,3),SUM)
        X3=X2*X
        IF (KODE.EQ.1) THEN          
          CC=DEXP(-X2)
        ELSE
          CC=1.0D0
        ENDIF
        DGERFC=SUM*CC/X3
        RETURN
      ENDIF
C-----------------------------------------------------------------------
C     ASYMPTOTIC EXPANSION FOR X.GE.7.0D0
C-----------------------------------------------------------------------
C     ENH(*) CONTAINS THE EXPONENTIAL INTEGRALS OF HALF ODD ORDERS.
C     THEY ARE USED IN THE ASYMPTOTIC SERIES WITH REL=0.5D-14 FOR THE
C     HIGHEST POSSIBLE ACCURACY IN DOUBLE PRECISION ARITHMETIC. THE
C     EXPONENTIAL INTEGRALS ENH(K)=E[HK+(K-1),ARG], K=1,MM STARTING
C     AT HALF ODD ORDER HK ARE USED AND ARE COMPUTED BY
C
C          CALL DHEXINT(ARG, FNH, KODE, MM, TOL, ENH, NZ, IERR)
C
C     WHERE  FNH=HK, KODE=1, TOL=REL. THE RETURN VARIABLES ARE ENH, NZ,
C     AND IERR. NZ=NUMBER OF TAIL VALUES IN THE SEQUENCE SET TO ZERO DUE
C     TO UNDERFLOW, ENH(MM-J+1)=0.0D0, J=1,NZ AND IERR= ERROR INDICATOR.
C     IERR=0 IS A NORMAL RETURN. KODE=2 RETURNS ENH(K)*DEXP(ARG), K=1,MM
C
C     IN THIS CONTEXT, ARG=X^2, FNH=HK=0.5D0, KODE=1, MM=20, TOL=REL.
C-----------------------------------------------------------------------
      IF(KODE.EQ.1) THEN
        IF (X2.GT.667.0D0) THEN
          DGERFC=0.0D0
          IERR=2
          RETURN
        ENDIF
      ENDIF
      ARG=X2
      FNH=0.5D0
      MM=24
      TOL=REL
      CALL DHEXINT(ARG, FNH, KODE, MM, TOL, ENH, NZ, IERR)
      IF (KODE.EQ.1) THEN
        IF(NZ.NE.0) THEN
          DGERFC=0.0D0
          IERR=2
          RETURN
        ENDIF
      ENDIF
      S=ENH(2)
      TRM1=S
      DTRM1=DABS(TRM1)
      AK=0.5D0
      CK=-AK/X2
      DO 20 K=1,MM-2
        TRM2=CK*ENH(K+2)
        DTRM2=DABS(TRM2)
        IF (DTRM2.LE.REL*DABS(S)) GOTO 22
        IF (DTRM2.GE.DTRM1) GOTO 21
        S=S+TRM2
        TRM1=TRM2
        AK=AK+1.0D0
        CK=-CK*AK/X2
  20  CONTINUE
C      WRITE(7,301)
C  301 FORMAT(" DROP THROUGH THE LOOP IN THE ASYMPTOTIC SERIES" )
  21  CONTINUE
C      WRITE (7,302)
C  302 FORMAT('REQUESTED ERROR NOT MET IN ASYMPTOTIC SERIES')
  22  CONTINUE
C      WRITE(7,303) K,REL
C  303 FORMAT (I5,D13.4)      
      DGERFC=0.5D0*S/(X*RTPI)
      RETURN
      END
      DOUBLE PRECISION FUNCTION DHERFC(X)
C
C     DONALD E. AMOS, OCTOBER, 2002
C-----------------------------------------------------------------------
C     A DOUBLE PRECISION ROUTINE
C-----------------------------------------------------------------------
C     REFS: FOLDER 23,
C
C           C. W. CLENSHAW, CHEBYSHEV SERIES FOR MATHEMATICAL FUNCTIONS,
C           NATIONAL PHYSICAL LABORATORY MATHEMATICAL TABLES, VOLUME 5,
C           HER MAJESTY'S STATIONERY OFFICE, LONDON, 1963, TABLE 9
C     
C
C     DHERFC COMPUTES
C
C              DHERFC=INT ON (0,X) OF EXP(W*W)*ERFC(W)
C
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION Y(50),AR(18)
      DATA RTPI /1.772453850905516D0/
C
C     CHEBYSHEV COEFFICIENTS FOR EXP(X*X)*ERFC(X)*(X*SQRT(PI))
C     FOR X.GE.4.  TABLE 9 OF THE REFERENCE.
C
      DATA (AR(I), I=1,18)/
     &       1.97070527225754492387D0,  -0.01433974027177497552D0,
     &       0.00029736169220261895D0,  -0.00000980351604336237D0,
     &       0.00000043313342034728D0,  -0.00000002362150026241D0,
     &       0.00000000151549676581D0,  -0.00000000011084939856D0,
     &       0.00000000000904259014D0,  -0.00000000000080947054D0,
     &       0.00000000000007853856D0,  -0.00000000000000817918D0,
     &       0.00000000000000090715D0,  -0.00000000000000010646D0,
     &       0.00000000000000001315D0,  -0.00000000000000000170D0,
     &       0.00000000000000000023D0,  -0.00000000000000000003D0/
C
C           S2=INTEGRAL ON (0,2) OF EXP(W*W)*ERFC(W)
C           S4=INTEGRAL ON (0,4) OF EXP(W*W)*ERFC(W)
C
      DATA S2, S4/ 0.9753620874841564D+00,   0.1344468257503159D+01/
      IF(X.LT.4.0D0) THEN
C       POWER SERIES FOR X.LT.4
        IF(X.LE.2.0D0) THEN
          IF(X.LE.1.0D0) THEN
            X0=0.0D0
            SUM=0.0D0
          ELSE
            X0=2.0D0
            SUM=S2
          ENDIF
        ELSE
          IF(X.LE.3.0D0) THEN
            X0=2.0D0
            SUM=S2
          ELSE
            X0=4.0D0
            SUM=S4
          ENDIF
        ENDIF
        REL=0.5D-14
        N0=0
        KODE=2
        N=40
        TOL=REL
        CALL DINERFC(X0,N0,KODE,N,TOL,Y,NZ)
        TRMK=1.0D0
        AK=2.0D0
        DX=X-X0
        SS=Y(1)
        DO 10 K=1,N-1
          TRMK=-2.0D0*TRMK*DX
          TRM=TRMK*Y(K+1)/AK
          SS=SS+TRM
          IF(DABS(TRM).LE.TOL*DABS(SS)) GOTO 20
          AK=AK+1.0D0  
 10     CONTINUE
 20     CONTINUE
        DHERFC=SUM+SS*DX
        RETURN
      ENDIF
C     CHEBYSHEV EXPANSION FOR X.GE.4
      RX=4.0D0/X
      TX=RX+RX
      CAPA=(1.0D0-RX)*(1.0D0+RX)
      SS=0.0D0
      TK=1.0D0
      TP=RX
      AK=1.0D0
      DO 30 J=1,3
        TM=TK
        TK=TP
        TP=TX*TK-TM
 30   CONTINUE
      DO 40 K=1,17
        SS=SS+AR(K+1)*CAPA        
        RK=-0.5D0/(AK*(AK+1.0D0))-0.5D0*(TP/(AK+1.0D0)-TM/AK)
        CAPA=RK-CAPA
        TM=TK
        TK=TP
        TP=TX*TK-TM
        TM=TK
        TK=TP
        TP=TX*TK-TM
        AK=AK+1.0D0
 40   CONTINUE
      FX=X/4.0D0
      DHERFC=S4+(SS+DLOG(FX))/RTPI
      RETURN
      END

